@@ -2,10 +2,10 @@
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 from polyglot.builtins import map, unicode_type, environ_item, hasenv, getenv
-import sys, locale, codecs, os, collections
+import sys, locale, codecs, os, collections, collections.abc
 
 __appname__   = 'calibre'
-numeric_version = (5, 14, 0)
+numeric_version = (5, 25, 0)
 __version__   = '.'.join(map(unicode_type, numeric_version))
 git_version   = None
 __author__    = "Kovid Goyal <kovid@kovidgoyal.net>"
@@ -167,6 +167,8 @@ def cache_dir():
 
 # plugins {{{
 plugins_loc = sys.extensions_location
+system_plugins_loc = getattr(sys, 'system_plugins_location', None)
+
 from importlib.machinery import ModuleSpec, EXTENSION_SUFFIXES, ExtensionFileLoader
 from importlib.util import find_spec
 from importlib import import_module
@@ -249,6 +251,7 @@ class ExtensionsImporter:
             'matcher',
             'tokenizer',
             'certgen',
+            'sqlite_extension',
         )
         if iswindows:
             extra = ('winutil', 'wpd', 'winfonts', 'winsapi')
@@ -290,7 +293,7 @@ if iswindows:
     from calibre_extensions import winutil
 
 
-class Plugins(collections.Mapping):
+class Plugins(collections.abc.Mapping):
 
     def __iter__(self):
         from importlib.resources import contents
@@ -319,6 +322,15 @@ class Plugins(collections.Mapping):
         except Exception as err:
             return None, str(err)
 
+    def load_apsw_extension(self, conn, name):
+        conn.enableloadextension(True)
+        try:
+            ext = 'pyd' if iswindows else 'so'
+            path = os.path.join(plugins_loc, f'{name}.{ext}')
+            conn.loadextension(path, f'calibre_{name}_init')
+        finally:
+            conn.enableloadextension(False)
+
 
 plugins = None
 if plugins is None:
@@ -394,6 +406,13 @@ def get_version():
     return v
 
 
+def get_appname_for_display():
+    ans = __appname__
+    if isportable:
+        ans = _('{} Portable').format(ans)
+    return ans
+
+
 def get_portable_base():
     'Return path to the directory that contains calibre-portable.exe or None'
     if isportable:
