@@ -8,7 +8,7 @@ __docformat__ = 'restructuredtext en'
 
 import weakref
 from functools import wraps
-from collections import MutableMapping, MutableSequence
+from collections.abc import MutableMapping, MutableSequence
 from copy import deepcopy
 
 from calibre.ebooks.metadata.book.base import Metadata, SIMPLE_GET, TOP_LEVEL_IDENTIFIERS, NULL_VALUES, ALL_METADATA_FIELDS
@@ -258,6 +258,11 @@ def composite_getter(mi, field, dbref, book_id, cache, formatter, template_cache
 
 
 def virtual_libraries_getter(dbref, book_id, cache):
+    '''
+    This method is deprecated because it doesn't (and can't) return virtual
+    library names when the VL search references marked books. It is replaced
+    by db.view.get_virtual_libraries_for_books()
+    '''
     try:
         return cache['virtual_libraries']
     except KeyError:
