@@ -1,13 +1,13 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, traceback, types
-from future_builtins import zip
+import os, traceback, weakref
+from polyglot.builtins import iteritems, zip
+from collections.abc import MutableMapping
 
 from calibre import force_unicode, isbytestring
 from calibre.constants import preferred_encoding
@@ -15,7 +15,7 @@ from calibre.db import _get_next_series_num_for_list, _get_series_values, get_da
 from calibre.db.adding import (
     find_books_in_directory, import_book_directory_multiple,
     import_book_directory, recursive_import, add_catalog, add_news)
-from calibre.db.backend import DB
+from calibre.db.backend import DB, set_global_state as backend_set_global_state
 from calibre.db.cache import Cache
 from calibre.db.errors import NoSuchFormat
 from calibre.db.categories import CATEGORY_SORTS
@@ -30,7 +30,7 @@ def cleanup_tags(tags):
     tags = [x.decode(preferred_encoding, 'replace')
                 if isbytestring(x) else x for x in tags]
     tags = [u' '.join(x.split()) for x in tags]
-    ans, seen = [], set([])
+    ans, seen = [], set()
     for tag in tags:
         if tag.lower() not in seen:
             seen.add(tag.lower())
@@ -48,6 +48,119 @@ def create_backend(
                      load_user_formatter_functions=load_user_formatter_functions)
 
 
+def set_global_state(db):
+    backend_set_global_state(db.backend)
+    set_saved_searches(db, 'saved_searches')
+
+
+class ThreadSafePrefs(MutableMapping):
+
+    def __init__(self, db):
+        self.db = weakref.ref(db)
+
+    def has_setting(self, key):
+        prefs = self.db().backend.prefs
+        return prefs.has_setting(key)
+
+    def __getitem__(self, key):
+        prefs = self.db().backend.prefs
+        return prefs.__getitem__(key)
+
+    def __delitem__(self, key):
+        db = self.db()
+        with db.write_lock:
+            prefs = db.backend.prefs
+            prefs.__delitem__(key)
+
+    def __setitem__(self, key, val):
+        db = self.db()
+        with db.write_lock:
+            prefs = db.backend.prefs
+            prefs.__setitem__(key, val)
+
+    def __contains__(self, key):
+        prefs = self.db().backend.prefs
+        return prefs.__contains__(key)
+
+    def __iter__(self):
+        prefs = self.db().backend.prefs
+        return prefs.__iter__()
+
+    def __len__(self):
+        prefs = self.db().backend.prefs
+        return prefs.__len__()
+
+    def __bool__(self):
+        prefs = self.db().backend.prefs
+        return prefs.__bool__()
+
+    def copy(self):
+        prefs = self.db().backend.prefs
+        return prefs.copy()
+
+    def items(self):
+        prefs = self.db().backend.prefs
+        return prefs.items()
+    iteritems = items
+
+    def keys(self):
+        prefs = self.db().backend.prefs
+        return prefs.keys()
+    iterkeys = keys
+
+    def values(self):
+        prefs = self.db().backend.prefs
+        return prefs.values()
+    itervalues = values
+
+    @property
+    def defaults(self):
+        prefs = self.db().backend.prefs
+        return prefs.defaults
+
+    @property
+    def disable_setting(self):
+        prefs = self.db().backend.prefs
+        return prefs.disable_setting
+
+    @disable_setting.setter
+    def disable_setting(self, val):
+        prefs = self.db().backend.prefs
+        prefs.disable_setting = val
+
+    def get(self, key, default=None):
+        prefs = self.db().backend.prefs
+        return prefs.get(key, default)
+
+    def set(self, key, val):
+        self.__setitem__(key, val)
+
+    def get_namespaced(self, namespace, key, default=None):
+        prefs = self.db().backend.prefs
+        return prefs.get_namespaced(namespace, key, default)
+
+    def set_namespaced(self, namespace, key, val):
+        db = self.db()
+        with db.write_lock:
+            prefs = db.backend.prefs
+            return prefs.set_namespaced(namespace, key, val)
+
+    def write_serialized(self, library_path):
+        prefs = self.db().backend.prefs
+        prefs.write_serialized(library_path)
+
+    def to_raw(self, val):
+        prefs = self.db().backend.prefs
+        return prefs.to_raw(val)
+
+    def raw_to_object(self, raw):
+        if isinstance(raw, bytes):
+            raw = raw.decode(preferred_encoding)
+        import json
+        from calibre.utils.config import from_json
+        return json.loads(raw, object_hook=from_json)
+
+
 class LibraryDatabase(object):
 
     ''' Emulate the old LibraryDatabase2 interface '''
@@ -65,9 +178,11 @@ class LibraryDatabase(object):
 
     def __init__(self, library_path,
             default_prefs=None, read_only=False, is_second_db=False,
-            progress_callback=lambda x, y:True, restore_all_prefs=False):
+            progress_callback=None, restore_all_prefs=False, row_factory=False):
 
         self.is_second_db = is_second_db
+        if progress_callback is None:
+            progress_callback = lambda x, y:True
         self.listeners = set()
 
         backend = self.backend = create_backend(library_path, default_prefs=default_prefs,
@@ -92,19 +207,21 @@ class LibraryDatabase(object):
             set_saved_searches(self, 'saved_searches')
 
     def close(self):
-        self.new_api.close()
+        if hasattr(self, 'new_api'):
+            self.new_api.close()
 
     def break_cycles(self):
-        delattr(self.backend, 'field_metadata')
-        self.data.cache.backend = None
-        self.data.cache = None
-        for x in ('data', 'backend', 'new_api', 'listeners',):
-            delattr(self, x)
+        if hasattr(self, 'backend'):
+            delattr(self.backend, 'field_metadata')
+            self.data.cache.backend = None
+            self.data.cache = None
+            for x in ('data', 'backend', 'new_api', 'listeners',):
+                delattr(self, x)
 
     # Library wide properties {{{
     @property
     def prefs(self):
-        return self.new_api.backend.prefs
+        return ThreadSafePrefs(self.new_api)
 
     @property
     def field_metadata(self):
@@ -164,14 +281,14 @@ class LibraryDatabase(object):
             return not bool(self.new_api.fields['title'].table.book_col_map)
 
     def get_usage_count_by_id(self, field):
-        return [[k, v] for k, v in self.new_api.get_usage_count_by_id(field).iteritems()]
+        return [[k, v] for k, v in iteritems(self.new_api.get_usage_count_by_id(field))]
 
     def field_id_map(self, field):
-        return [(k, v) for k, v in self.new_api.get_id_map(field).iteritems()]
+        return [(k, v) for k, v in iteritems(self.new_api.get_id_map(field))]
 
     def get_custom_items_with_ids(self, label=None, num=None):
         try:
-            return [[k, v] for k, v in self.new_api.get_id_map(self.custom_field_name(label, num)).iteritems()]
+            return [[k, v] for k, v in iteritems(self.new_api.get_id_map(self.custom_field_name(label, num)))]
         except ValueError:
             return []
 
@@ -226,7 +343,7 @@ class LibraryDatabase(object):
             paths, formats, metadata = [], [], []
             for mi, format_map in duplicates:
                 metadata.append(mi)
-                for fmt, path in format_map.iteritems():
+                for fmt, path in iteritems(format_map):
                     formats.append(fmt)
                     paths.append(path)
             duplicates = (paths, formats, metadata)
@@ -409,7 +526,7 @@ class LibraryDatabase(object):
         ans = set()
         if title:
             title = icu_lower(force_unicode(title))
-            for book_id, x in self.new_api.get_id_map('title').iteritems():
+            for book_id, x in iteritems(self.new_api.get_id_map('title')):
                 if icu_lower(x) == title:
                     ans.add(book_id)
                     if not all_matches:
@@ -514,7 +631,7 @@ class LibraryDatabase(object):
 
     def delete_tags(self, tags):
         with self.new_api.write_lock:
-            tag_map = {icu_lower(v):k for k, v in self.new_api._get_id_map('tags').iteritems()}
+            tag_map = {icu_lower(v):k for k, v in iteritems(self.new_api._get_id_map('tags'))}
             tag_ids = (tag_map.get(icu_lower(tag), None) for tag in tags)
             tag_ids = tuple(tid for tid in tag_ids if tid is not None)
             if tag_ids:
@@ -540,7 +657,7 @@ class LibraryDatabase(object):
 
     def format_files(self, index, index_is_id=False):
         book_id = index if index_is_id else self.id(index)
-        return [(v, k) for k, v in self.new_api.format_files(book_id).iteritems()]
+        return [(v, k) for k, v in iteritems(self.new_api.format_files(book_id))]
 
     def format_metadata(self, book_id, fmt, allow_cache=True, update_db=False, commit=False):
         return self.new_api.format_metadata(book_id, fmt, allow_cache=allow_cache, update_db=update_db)
@@ -625,7 +742,7 @@ class LibraryDatabase(object):
     def delete_item_from_multiple(self, item, label=None, num=None):
         field = self.custom_field_name(label, num)
         existing = self.new_api.get_id_map(field)
-        rmap = {icu_lower(v):k for k, v in existing.iteritems()}
+        rmap = {icu_lower(v):k for k, v in iteritems(existing)}
         item_id = rmap.get(icu_lower(item), None)
         if item_id is None:
             return []
@@ -688,7 +805,7 @@ class LibraryDatabase(object):
         self.new_api.delete_custom_column(label, num)
 
     def create_custom_column(self, label, name, datatype, is_multiple, editable=True, display={}):
-        self.new_api.create_custom_column(label, name, datatype, is_multiple, editable=editable, display=display)
+        return self.new_api.create_custom_column(label, name, datatype, is_multiple, editable=editable, display=display)
 
     def set_custom_column_metadata(self, num, name=None, label=None, is_editable=None, display=None,
                                    notify=True, update_last_modified=False):
@@ -742,7 +859,6 @@ class LibraryDatabase(object):
 
     # }}}
 
-MT = lambda func: types.MethodType(func, None, LibraryDatabase)
 
 # Legacy getter API {{{
 for prop in ('author_sort', 'authors', 'comment', 'comments', 'publisher', 'max_size',
@@ -755,7 +871,7 @@ for prop in ('author_sort', 'authors', 'comment', 'comments', 'publisher', 'max_
         def func(self, index, index_is_id=False):
             return self.get_property(index, index_is_id=index_is_id, loc=self.FIELD_MAP[fm])
         return func
-    setattr(LibraryDatabase, prop, MT(getter(prop)))
+    setattr(LibraryDatabase, prop, getter(prop))
 
 for prop in ('series', 'publisher'):
     def getter(field):
@@ -767,22 +883,19 @@ for prop in ('series', 'publisher'):
             except IndexError:
                 pass
         return func
-    setattr(LibraryDatabase, prop + '_id', MT(getter(prop)))
+    setattr(LibraryDatabase, prop + '_id', getter(prop))
 
-LibraryDatabase.format_hash = MT(lambda self, book_id, fmt:self.new_api.format_hash(book_id, fmt))
-LibraryDatabase.index = MT(lambda self, book_id, cache=False:self.data.id_to_index(book_id))
-LibraryDatabase.has_cover = MT(lambda self, book_id:self.new_api.field_for('cover', book_id))
-LibraryDatabase.get_tags = MT(lambda self, book_id:set(self.new_api.field_for('tags', book_id)))
-LibraryDatabase.get_categories = MT(lambda self, sort='name', ids=None:self.new_api.get_categories(sort=sort, book_ids=ids))
-LibraryDatabase.get_identifiers = MT(
-    lambda self, index, index_is_id=False: self.new_api.field_for('identifiers', index if index_is_id else self.id(index)))
-LibraryDatabase.isbn = MT(
-    lambda self, index, index_is_id=False: self.get_identifiers(index, index_is_id=index_is_id).get('isbn', None))
-LibraryDatabase.get_books_for_category = MT(
-    lambda self, category, id_:self.new_api.get_books_for_category(category, id_))
-LibraryDatabase.get_data_as_dict = MT(get_data_as_dict)
-LibraryDatabase.find_identical_books = MT(lambda self, mi:self.new_api.find_identical_books(mi))
-LibraryDatabase.get_top_level_move_items = MT(lambda self:self.new_api.get_top_level_move_items())
+LibraryDatabase.format_hash = lambda self, book_id, fmt:self.new_api.format_hash(book_id, fmt)
+LibraryDatabase.index = lambda self, book_id, cache=False:self.data.id_to_index(book_id)
+LibraryDatabase.has_cover = lambda self, book_id:self.new_api.field_for('cover', book_id)
+LibraryDatabase.get_tags = lambda self, book_id:set(self.new_api.field_for('tags', book_id))
+LibraryDatabase.get_categories = lambda self, sort='name', ids=None:self.new_api.get_categories(sort=sort, book_ids=ids)
+LibraryDatabase.get_identifiers = lambda self, index, index_is_id=False: self.new_api.field_for('identifiers', index if index_is_id else self.id(index))
+LibraryDatabase.isbn = lambda self, index, index_is_id=False: self.get_identifiers(index, index_is_id=index_is_id).get('isbn', None)
+LibraryDatabase.get_books_for_category = lambda self, category, id_:self.new_api.get_books_for_category(category, id_)
+LibraryDatabase.get_data_as_dict = get_data_as_dict
+LibraryDatabase.find_identical_books = lambda self, mi:self.new_api.find_identical_books(mi)
+LibraryDatabase.get_top_level_move_items = lambda self:self.new_api.get_top_level_move_items()
 # }}}
 
 # Legacy setter API {{{
@@ -818,7 +931,7 @@ for field in (
                     self.notify([book_id])
                 return ret if field == 'languages' else retval
         return func
-    setattr(LibraryDatabase, 'set_%s' % field.replace('!', ''), MT(setter(field)))
+    setattr(LibraryDatabase, 'set_%s' % field.replace('!', ''), setter(field))
 
 for field in ('authors', 'tags', 'publisher'):
     def renamer(field):
@@ -828,10 +941,9 @@ for field in ('authors', 'tags', 'publisher'):
                 return id_map[old_id]
         return func
     fname = field[:-1] if field in {'tags', 'authors'} else field
-    setattr(LibraryDatabase, 'rename_%s' % fname, MT(renamer(field)))
+    setattr(LibraryDatabase, 'rename_%s' % fname, renamer(field))
 
-LibraryDatabase.update_last_modified = MT(
-    lambda self, book_ids, commit=False, now=None: self.new_api.update_last_modified(book_ids, now=now))
+LibraryDatabase.update_last_modified = lambda self, book_ids, commit=False, now=None: self.new_api.update_last_modified(book_ids, now=now)
 
 # }}}
 
@@ -842,32 +954,30 @@ for field in ('authors', 'tags', 'publisher', 'series'):
             return self.new_api.all_field_names(field)
         return func
     name = field[:-1] if field in {'authors', 'tags'} else field
-    setattr(LibraryDatabase, 'all_%s_names' % name, MT(getter(field)))
-LibraryDatabase.all_formats = MT(lambda self:self.new_api.all_field_names('formats'))
-LibraryDatabase.all_custom = MT(lambda self, label=None, num=None:self.new_api.all_field_names(self.custom_field_name(label, num)))
+    setattr(LibraryDatabase, 'all_%s_names' % name, getter(field))
+LibraryDatabase.all_formats = lambda self:self.new_api.all_field_names('formats')
+LibraryDatabase.all_custom = lambda self, label=None, num=None:self.new_api.all_field_names(self.custom_field_name(label, num))
 
-for func, field in {'all_authors':'authors', 'all_titles':'title', 'all_tags2':'tags', 'all_series':'series', 'all_publishers':'publisher'}.iteritems():
+for func, field in iteritems({'all_authors':'authors', 'all_titles':'title', 'all_tags2':'tags', 'all_series':'series', 'all_publishers':'publisher'}):
     def getter(field):
         def func(self):
             return self.field_id_map(field)
         return func
-    setattr(LibraryDatabase, func, MT(getter(field)))
+    setattr(LibraryDatabase, func, getter(field))
 
-LibraryDatabase.all_tags = MT(lambda self: list(self.all_tag_names()))
-LibraryDatabase.get_all_identifier_types = MT(lambda self: list(self.new_api.fields['identifiers'].table.all_identifier_types()))
-LibraryDatabase.get_authors_with_ids = MT(
-    lambda self: [[aid, adata['name'], adata['sort'], adata['link']] for aid, adata in self.new_api.author_data().iteritems()])
-LibraryDatabase.get_author_id = MT(
-    lambda self, author: {icu_lower(v):k for k, v in self.new_api.get_id_map('authors').iteritems()}.get(icu_lower(author), None))
+LibraryDatabase.all_tags = lambda self: list(self.all_tag_names())
+LibraryDatabase.get_all_identifier_types = lambda self: list(self.new_api.fields['identifiers'].table.all_identifier_types())
+LibraryDatabase.get_authors_with_ids = lambda self: [[aid, adata['name'], adata['sort'], adata['link']] for aid, adata in iteritems(self.new_api.author_data())]
+LibraryDatabase.get_author_id = lambda self, author: {icu_lower(v):k for k, v in iteritems(self.new_api.get_id_map('authors'))}.get(icu_lower(author), None)
 
 for field in ('tags', 'series', 'publishers', 'ratings', 'languages'):
     def getter(field):
         fname = field[:-1] if field in {'publishers', 'ratings'} else field
 
         def func(self):
-            return [[tid, tag] for tid, tag in self.new_api.get_id_map(fname).iteritems()]
+            return [[tid, tag] for tid, tag in iteritems(self.new_api.get_id_map(fname))]
         return func
-    setattr(LibraryDatabase, 'get_%s_with_ids' % field, MT(getter(field)))
+    setattr(LibraryDatabase, 'get_%s_with_ids' % field, getter(field))
 
 for field in ('author', 'tag', 'series'):
     def getter(field):
@@ -876,7 +986,7 @@ for field in ('author', 'tag', 'series'):
         def func(self, item_id):
             return self.new_api.get_item_name(field, item_id)
         return func
-    setattr(LibraryDatabase, '%s_name' % field, MT(getter(field)))
+    setattr(LibraryDatabase, '%s_name' % field, getter(field))
 
 for field in ('publisher', 'series', 'tag'):
     def getter(field):
@@ -885,7 +995,7 @@ for field in ('publisher', 'series', 'tag'):
         def func(self, item_id):
             self.new_api.remove_items(fname, (item_id,))
         return func
-    setattr(LibraryDatabase, 'delete_%s_using_id' % field, MT(getter(field)))
+    setattr(LibraryDatabase, 'delete_%s_using_id' % field, getter(field))
 # }}}
 
 # Legacy field API {{{
@@ -907,8 +1017,8 @@ for func in (
             def meth(self):
                 return getattr(self.field_metadata, func)()
         return meth
-    setattr(LibraryDatabase, func.replace('!', ''), MT(getter(func)))
-LibraryDatabase.metadata_for_field = MT(lambda self, field:self.field_metadata.get(field))
+    setattr(LibraryDatabase, func.replace('!', ''), getter(func))
+LibraryDatabase.metadata_for_field = lambda self, field:self.field_metadata.get(field)
 
 # }}}
 
@@ -918,22 +1028,19 @@ for meth in ('get_next_series_num_for', 'has_book',):
         def func(self, x):
             return getattr(self.new_api, meth)(x)
         return func
-    setattr(LibraryDatabase, meth, MT(getter(meth)))
+    setattr(LibraryDatabase, meth, getter(meth))
 
-LibraryDatabase.saved_search_names = MT(lambda self:self.new_api.saved_search_names())
-LibraryDatabase.saved_search_lookup = MT(lambda self, x:self.new_api.saved_search_lookup(x))
-LibraryDatabase.saved_search_set_all = MT(lambda self, smap:self.new_api.saved_search_set_all(smap))
-LibraryDatabase.saved_search_delete = MT(lambda self, x:self.new_api.saved_search_delete(x))
-LibraryDatabase.saved_search_add = MT(lambda self, x, y:self.new_api.saved_search_add(x, y))
-LibraryDatabase.saved_search_rename = MT(lambda self, x, y:self.new_api.saved_search_rename(x, y))
-LibraryDatabase.commit_dirty_cache = MT(lambda self: self.new_api.commit_dirty_cache())
-LibraryDatabase.author_sort_from_authors = MT(lambda self, x: self.new_api.author_sort_from_authors(x))
+LibraryDatabase.saved_search_names = lambda self:self.new_api.saved_search_names()
+LibraryDatabase.saved_search_lookup = lambda self, x:self.new_api.saved_search_lookup(x)
+LibraryDatabase.saved_search_set_all = lambda self, smap:self.new_api.saved_search_set_all(smap)
+LibraryDatabase.saved_search_delete = lambda self, x:self.new_api.saved_search_delete(x)
+LibraryDatabase.saved_search_add = lambda self, x, y:self.new_api.saved_search_add(x, y)
+LibraryDatabase.saved_search_rename = lambda self, x, y:self.new_api.saved_search_rename(x, y)
+LibraryDatabase.commit_dirty_cache = lambda self: self.new_api.commit_dirty_cache()
+LibraryDatabase.author_sort_from_authors = lambda self, x: self.new_api.author_sort_from_authors(x)
 # Cleaning is not required anymore
-LibraryDatabase.clean = LibraryDatabase.clean_custom = MT(lambda self:None)
-LibraryDatabase.clean_standard_field = MT(lambda self, field, commit=False:None)
+LibraryDatabase.clean = LibraryDatabase.clean_custom = lambda self:None
+LibraryDatabase.clean_standard_field = lambda self, field, commit=False:None
 # apsw operates in autocommit mode
-LibraryDatabase.commit = MT(lambda self:None)
+LibraryDatabase.commit = lambda self:None
 # }}}
-
-del MT
-
