@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -38,7 +37,8 @@ def main():
     pr.disable()
     pr.dump_stats(stats)
     show_stats(stats)
-    print ('Stats saved to', stats)
+    print('Stats saved to', stats)
+
 
 if __name__ == '__main__':
     main()
