@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john at nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
@@ -8,11 +9,13 @@ __docformat__ = 'restructuredtext en'
 Device driver for Amazon's Kindle
 '''
 
-import datetime, os, re, sys, json, hashlib
+import datetime, os, re, json, hashlib, errno
 
+from calibre.constants import DEBUG, filesystem_encoding
 from calibre.devices.kindle.bookmark import Bookmark
 from calibre.devices.usbms.driver import USBMS
-from calibre import strftime, fsync
+from calibre import strftime, fsync, prints
+from polyglot.builtins import unicode_type, as_bytes, as_unicode
 
 '''
 Notes on collections:
@@ -36,12 +39,27 @@ Adding a book to a collection on the Kindle does not change the book file at all
 file metadata.
 '''
 
+
+def get_files_in(path):
+    if hasattr(os, 'scandir'):
+        for dir_entry in os.scandir(path):
+            if dir_entry.is_file(follow_symlinks=False):
+                yield dir_entry.name, dir_entry.stat(follow_symlinks=False)
+    else:
+        import stat
+        for x in os.listdir(path):
+            xp = os.path.join(path, x)
+            s = os.lstat(xp)
+            if stat.S_ISREG(s.st_mode):
+                yield x, s
+
+
 class KINDLE(USBMS):
 
     name           = 'Kindle Device Interface'
     gui_name       = 'Amazon Kindle'
-    icon           = I('devices/kindle.jpg')
-    description    = _('Communicate with the Kindle eBook reader.')
+    icon           = I('devices/kindle.png')
+    description    = _('Communicate with the Kindle e-book reader.')
     author         = 'John Schember'
     supported_platforms = ['windows', 'osx', 'linux']
 
@@ -71,24 +89,54 @@ class KINDLE(USBMS):
     WIRELESS_FILE_NAME_PATTERN = re.compile(
     r'(?P<title>[^-]+)-asin_(?P<asin>[a-zA-Z\d]{10,})-type_(?P<type>\w{4})-v_(?P<index>\d+).*')
 
+    VIRTUAL_BOOK_EXTENSIONS = frozenset({'kfx'})
+    VIRTUAL_BOOK_EXTENSION_MESSAGE = _(
+        'The following books are in KFX format. KFX is a virtual book format, and cannot'
+        ' be transferred from the device. Instead, you should go to your "Manage my'
+        ' content and devices" page on the Amazon homepage and download the book to your computer from there.'
+        ' That will give you a regular AZW3 file that you can add to calibre normally.'
+        ' Click "Show details" to see the list of books.'
+    )
+
+    def is_allowed_book_file(self, filename, path, prefix):
+        lpath = os.path.join(path, filename).partition(self.normalize_path(prefix))[2].replace('\\', '/')
+        return '.sdr/' not in lpath
+
     @classmethod
     def metadata_from_path(cls, path):
-        mi = cls.metadata_from_formats([path])
+        if path.endswith('.kfx'):
+            from calibre.ebooks.metadata.kfx import read_metadata_kfx
+            try:
+                kfx_path = path
+                with lopen(kfx_path, 'rb') as f:
+                    if f.read(8) != b'\xeaDRMION\xee':
+                        f.seek(0)
+                        mi = read_metadata_kfx(f)
+                    else:
+                        kfx_path = os.path.join(path.rpartition('.')[0] + '.sdr', 'assets', 'metadata.kfx')
+                        with lopen(kfx_path, 'rb') as mf:
+                            mi = read_metadata_kfx(mf)
+            except Exception:
+                import traceback
+                traceback.print_exc()
+                if DEBUG:
+                    prints('failed kfx path:', kfx_path)
+                mi = cls.metadata_from_formats([path])
+        else:
+            mi = cls.metadata_from_formats([path])
         if mi.title == _('Unknown') or ('-asin' in mi.title and '-type' in mi.title):
+            path = as_unicode(path, filesystem_encoding, 'replace')
             match = cls.WIRELESS_FILE_NAME_PATTERN.match(os.path.basename(path))
             if match is not None:
                 mi.title = match.group('title')
-                if not isinstance(mi.title, unicode):
-                    mi.title = mi.title.decode(sys.getfilesystemencoding(),
-                                               'replace')
         return mi
 
     def get_annotations(self, path_map):
-        MBP_FORMATS = [u'azw', u'mobi', u'prc', u'txt']
+        MBP_FORMATS = ['azw', 'mobi', 'prc', 'txt']
         mbp_formats = set(MBP_FORMATS)
-        PDR_FORMATS = [u'pdf']
+        PDR_FORMATS = ['pdf']
         pdr_formats = set(PDR_FORMATS)
-        TAN_FORMATS = [u'tpz', u'azw1']
+        TAN_FORMATS = ['tpz', 'azw1']
         tan_formats = set(TAN_FORMATS)
 
         def get_storage():
@@ -165,7 +213,7 @@ class KINDLE(USBMS):
         return bookmarked_books
 
     def generate_annotation_html(self, bookmark):
-        from calibre.ebooks.BeautifulSoup import BeautifulSoup, Tag, NavigableString
+        from calibre.ebooks.BeautifulSoup import BeautifulSoup
         # Returns <div class="user_annotations"> ... </div>
         last_read_location = bookmark.last_read_location
         timestamp = datetime.datetime.utcfromtimestamp(bookmark.timestamp)
@@ -173,28 +221,25 @@ class KINDLE(USBMS):
 
         ka_soup = BeautifulSoup()
         dtc = 0
-        divTag = Tag(ka_soup,'div')
+        divTag = ka_soup.new_tag('div')
         divTag['class'] = 'user_annotations'
 
         # Add the last-read location
-        spanTag = Tag(ka_soup, 'span')
-        spanTag['style'] = 'font-weight:bold'
         if bookmark.book_format == 'pdf':
-            spanTag.insert(0,NavigableString(
-                _("%(time)s<br />Last Page Read: %(loc)d (%(pr)d%%)") % dict(
-                    time=strftime(u'%x', timestamp.timetuple()),
+            markup = _("%(time)s<br />Last page read: %(loc)d (%(pr)d%%)") % dict(
+                    time=strftime('%x', timestamp.timetuple()),
                     loc=last_read_location,
-                    pr=percent_read)))
+                    pr=percent_read)
         else:
-            spanTag.insert(0,NavigableString(
-                _("%(time)s<br />Last Page Read: Location %(loc)d (%(pr)d%%)") % dict(
+            markup = _("%(time)s<br />Last page read: Location %(loc)d (%(pr)d%%)") % dict(
                     time=strftime(u'%x', timestamp.timetuple()),
                     loc=last_read_location,
-                    pr=percent_read)))
+                    pr=percent_read)
+        spanTag = BeautifulSoup('<span style="font-weight:bold">' + markup + '</span>').find('span')
 
         divTag.insert(dtc, spanTag)
         dtc += 1
-        divTag.insert(dtc, Tag(ka_soup,'br'))
+        divTag.insert(dtc, ka_soup.new_tag('br'))
         dtc += 1
 
         if bookmark.user_notes:
@@ -225,18 +270,19 @@ class KINDLE(USBMS):
                                     typ=user_notes[location]['type']))
 
             for annotation in annotations:
-                divTag.insert(dtc, annotation)
+                annot = BeautifulSoup('<span>' + annotation + '</span>').find('span')
+                divTag.insert(dtc, annot)
                 dtc += 1
 
         ka_soup.insert(0,divTag)
         return ka_soup
 
     def add_annotation_to_library(self, db, db_id, annotation):
-        from calibre.ebooks.BeautifulSoup import Tag
         from calibre.ebooks.metadata import MetaInformation
+        from calibre.ebooks.BeautifulSoup import prettify
 
         bm = annotation
-        ignore_tags = set(['Catalog', 'Clippings'])
+        ignore_tags = {'Catalog', 'Clippings'}
 
         if bm.type == 'kindle_bookmark':
             mi = db.get_metadata(db_id, index_is_id=True)
@@ -252,13 +298,13 @@ class KINDLE(USBMS):
                 if set(mi.tags).intersection(ignore_tags):
                     return
                 if mi.comments:
-                    hrTag = Tag(user_notes_soup,'hr')
+                    hrTag = user_notes_soup.new_tag('hr')
                     hrTag['class'] = 'annotations_divider'
                     user_notes_soup.insert(0, hrTag)
 
-                mi.comments += unicode(user_notes_soup.prettify())
+                mi.comments += prettify(user_notes_soup)
             else:
-                mi.comments = unicode(user_notes_soup.prettify())
+                mi.comments = prettify(user_notes_soup)
             # Update library comments
             db.set_comment(db_id, mi.comments)
 
@@ -281,19 +327,20 @@ class KINDLE(USBMS):
                 mi.comments = last_update
                 db.add_books([bm.value['path']], ['txt'], [mi])
 
+
 class KINDLE2(KINDLE):
 
-    name           = 'Kindle 2/3/4/Touch/PaperWhite Device Interface'
-    description    = _('Communicate with the Kindle 2/3/4/Touch/PaperWhite eBook reader.')
+    name           = 'Kindle 2/3/4/Touch/PaperWhite/Voyage Device Interface'
+    description    = _('Communicate with the Kindle 2/3/4/Touch/PaperWhite/Voyage e-book reader.')
 
-    FORMATS     = ['azw', 'mobi', 'azw3', 'prc', 'azw1', 'tpz', 'azw4', 'pobi', 'pdf', 'txt']
+    FORMATS     = ['azw', 'mobi', 'azw3', 'prc', 'azw1', 'tpz', 'azw4', 'kfx', 'pobi', 'pdf', 'txt']
     DELETE_EXTS    = KINDLE.DELETE_EXTS + ['.mbp1', '.mbs', '.sdr', '.han']
     # On the Touch, there's also .asc files, but not using the same basename
     # (for X-Ray & End Actions), azw3f & azw3r files, but all of them are in
     # the .sdr sidecar folder
 
-    PRODUCT_ID = [0x0002, 0x0004]
-    BCD        = [0x0100]
+    PRODUCT_ID = [0x0002, 0x0004, 0x0324]
+    BCD        = [0x0100, 0x0310, 0x401]
     # SUPPORTS_SUB_DIRS = False # Apparently the Paperwhite doesn't like files placed in subdirectories
     # SUPPORTS_SUB_DIRS_FOR_SCAN = True
 
@@ -323,21 +370,41 @@ class KINDLE2(KINDLE):
             ' store the page count of books, you can have calibre use that'
             ' information, instead of calculating a page count. Specify the'
             ' name of the custom column here, for example, #pages.'),
+        _('Custom column name to retrieve calculation method from') + ':::' + _(
+            'If you have a custom column in your library that you use to'
+            ' store the preferred method for calculating the number of pages'
+            ' for a book, you can have calibre use that method instead of the'
+            ' default one selected above.  Specify the name of the custom'
+            ' column here, for example, #pagemethod. The custom column should have the '
+            ' values: fast, accurate or pagebreak.'),
+        _('Overwrite existing APNX on device') + ':::' + _(
+            'Uncheck this option to allow an APNX file existing on the device'
+            ' to have priority over the version which calibre would send.'
+            ' Since APNX files are usually deleted when a book is removed from'
+            ' the Kindle, this is mostly useful when resending a book to the'
+            ' device which is already on the device (e.g. after making a'
+            ' modification.)'),
 
     ]
     EXTRA_CUSTOMIZATION_DEFAULT = [
         True,
         'fast',
         '',
+        '',
+        True,
     ]
     OPT_APNX                 = 0
     OPT_APNX_METHOD          = 1
     OPT_APNX_CUST_COL        = 2
+    OPT_APNX_METHOD_COL      = 3
+    OPT_APNX_OVERWRITE       = 4
     EXTRA_CUSTOMIZATION_CHOICES = {OPT_APNX_METHOD:{'fast', 'accurate', 'pagebreak'}}
 
     # x330 on the PaperWhite
-    THUMBNAIL_HEIGHT         = 330
     # x262 on the Touch. Doesn't choke on x330, though.
+    # x470 on the Voyage, checked that it works on PW, Touch checked by eschwartz.
+    # x500 on the Oasis 2017. checked that it works on the PW3
+    THUMBNAIL_HEIGHT         = 500
 
     @classmethod
     def migrate_extra_customization(cls, vals):
@@ -347,7 +414,7 @@ class KINDLE2(KINDLE):
         return vals
 
     def formats_to_scan_for(self):
-        ans = USBMS.formats_to_scan_for(self) | {'azw3'}
+        ans = USBMS.formats_to_scan_for(self) | {'azw3', 'kfx'}
         return ans
 
     def books(self, oncard=None, end_session=True):
@@ -363,7 +430,7 @@ class KINDLE2(KINDLE):
         return bl
 
     def kindle_update_booklist(self, bl, collections):
-        with open(collections, 'rb') as f:
+        with lopen(collections, 'rb') as f:
             collections = f.read()
         collections = json.loads(collections)
         path_map = {}
@@ -373,17 +440,23 @@ class KINDLE2(KINDLE):
             for x in items:
                 x = x[-40:]
                 if x not in path_map:
-                    path_map[x] = set([])
+                    path_map[x] = set()
                 path_map[x].add(col)
         if path_map:
             for book in bl:
                 path = '/mnt/us/'+book.lpath
-                h = hashlib.sha1(path).hexdigest()
+                h = hashlib.sha1(as_bytes(path)).hexdigest()
                 if h in path_map:
                     book.device_collections = list(sorted(path_map[h]))
 
-    # Detect if the product family needs .apnx files uploaded to sidecar folder
     def post_open_callback(self):
+        try:
+            self.sync_cover_thumbnails()
+        except Exception:
+            import traceback
+            traceback.print_exc()
+
+        # Detect if the product family needs .apnx files uploaded to sidecar folder
         product_id = self.device_being_opened[1]
         self.sidecar_apnx = False
         if product_id > 0x3:
@@ -407,26 +480,100 @@ class KINDLE2(KINDLE):
         # Upload the apnx file
         self.upload_apnx(path, filename, metadata, filepath)
 
-    def upload_kindle_thumbnail(self, metadata, filepath):
+    def amazon_system_thumbnails_dir(self):
+        return os.path.join(self._main_prefix, 'system', 'thumbnails')
+
+    def thumbpath_from_filepath(self, filepath):
+        from calibre.ebooks.metadata.kfx import (CONTAINER_MAGIC, read_book_key_kfx)
+        from calibre.ebooks.mobi.reader.headers import MetadataHeader
         from calibre.utils.logging import default_log
+        thumb_dir = self.amazon_system_thumbnails_dir()
+        if not os.path.exists(thumb_dir):
+            return
+        with lopen(filepath, 'rb') as f:
+            is_kfx = f.read(4) == CONTAINER_MAGIC
+            f.seek(0)
+            uuid = cdetype = None
+            if is_kfx:
+                uuid, cdetype = read_book_key_kfx(f)
+            else:
+                mh = MetadataHeader(f, default_log)
+                if mh.exth is not None:
+                    uuid = mh.exth.uuid
+                    cdetype = mh.exth.cdetype
+        if not uuid or not cdetype:
+            return
+        return os.path.join(thumb_dir,
+                'thumbnail_{uuid}_{cdetype}_portrait.jpg'.format(
+                    uuid=uuid, cdetype=cdetype))
+
+    def amazon_cover_bug_cache_dir(self):
+        # see https://www.mobileread.com/forums/showthread.php?t=329945
+        return os.path.join(self._main_prefix, 'amazon-cover-bug')
+
+    def upload_kindle_thumbnail(self, metadata, filepath):
         coverdata = getattr(metadata, 'thumbnail', None)
         if not coverdata or not coverdata[2]:
             return
-        thumb_dir = os.path.join(self._main_prefix, 'system', 'thumbnails')
-        if not os.path.exists(thumb_dir):
-            return
 
-        from calibre.ebooks.mobi.reader.headers import MetadataHeader
-        with lopen(filepath, 'rb') as f:
-            mh = MetadataHeader(f, default_log)
-        if mh.exth is None or not mh.exth.uuid or not mh.exth.cdetype:
+        tp = self.thumbpath_from_filepath(filepath)
+        if tp:
+            with lopen(tp, 'wb') as f:
+                f.write(coverdata[2])
+                fsync(f)
+            cache_dir = self.amazon_cover_bug_cache_dir()
+            try:
+                os.mkdir(cache_dir)
+            except EnvironmentError:
+                pass
+            with lopen(os.path.join(cache_dir, os.path.basename(tp)), 'wb') as f:
+                f.write(coverdata[2])
+                fsync(f)
+
+    def sync_cover_thumbnails(self):
+        import shutil
+        # See https://www.mobileread.com/forums/showthread.php?t=329945
+        # for why this is needed
+        if DEBUG:
+            prints('Syncing cover thumbnails to workaround amazon cover bug')
+        dest_dir = self.amazon_system_thumbnails_dir()
+        src_dir = self.amazon_cover_bug_cache_dir()
+        if not os.path.exists(dest_dir) or not os.path.exists(src_dir):
             return
-        thumbfile = os.path.join(thumb_dir,
-                'thumbnail_{uuid}_{cdetype}_portrait.jpg'.format(
-                    uuid=mh.exth.uuid, cdetype=mh.exth.cdetype))
-        with open(thumbfile, 'wb') as f:
-            f.write(coverdata[2])
-            fsync(f)
+        count = 0
+        for name, src_stat_result in get_files_in(src_dir):
+            dest_path = os.path.join(dest_dir, name)
+            try:
+                dest_stat_result = os.lstat(dest_path)
+            except EnvironmentError:
+                needs_sync = True
+            else:
+                needs_sync = src_stat_result.st_size != dest_stat_result.st_size
+            if needs_sync:
+                count += 1
+                if DEBUG:
+                    prints('Restoring cover thumbnail:', name)
+                with lopen(os.path.join(src_dir, name), 'rb') as src, lopen(dest_path, 'wb') as dest:
+                    shutil.copyfileobj(src, dest)
+                    fsync(dest)
+        if DEBUG:
+            prints('Restored {} cover thumbnails that were destroyed by Amazon'.format(count))
+
+    def delete_single_book(self, path):
+        try:
+            tp1 = self.thumbpath_from_filepath(path)
+            if tp1:
+                tp2 = os.path.join(self.amazon_cover_bug_cache_dir(), os.path.basename(tp1))
+                for tp in (tp1, tp2):
+                    try:
+                        os.remove(tp)
+                    except EnvironmentError as err:
+                        if err.errno != errno.ENOENT:
+                            prints('Failed to delete thumbnail for {!r} at {!r} with error: {}'.format(path, tp, err))
+        except Exception:
+            import traceback
+            traceback.print_exc()
+        USBMS.delete_single_book(self, path)
 
     def upload_apnx(self, path, filename, metadata, filepath):
         from calibre.devices.kindle.apnx import APNXBuilder
@@ -456,19 +603,31 @@ class KINDLE2(KINDLE):
 
         apnx_path = '%s.apnx' % os.path.join(path, filename)
         apnx_builder = APNXBuilder()
-        try:
-            method = opts.extra_customization[self.OPT_APNX_METHOD]
-            apnx_builder.write_apnx(filepath, apnx_path, method=method, page_count=custom_page_count)
-        except:
-            print 'Failed to generate APNX'
-            import traceback
-            traceback.print_exc()
+        # Check to see if there is an existing apnx file on Kindle we should keep.
+        if opts.extra_customization[self.OPT_APNX_OVERWRITE] or not os.path.exists(apnx_path):
+            try:
+                method = opts.extra_customization[self.OPT_APNX_METHOD]
+                cust_col_name = opts.extra_customization[self.OPT_APNX_METHOD_COL]
+                if cust_col_name:
+                    try:
+                        temp = unicode_type(metadata.get(cust_col_name)).lower()
+                        if temp in self.EXTRA_CUSTOMIZATION_CHOICES[self.OPT_APNX_METHOD]:
+                            method = temp
+                        else:
+                            print("Invalid method choice for this book (%r), ignoring." % temp)
+                    except:
+                        print('Could not retrieve override method choice, using default.')
+                apnx_builder.write_apnx(filepath, apnx_path, method=method, page_count=custom_page_count)
+            except:
+                print('Failed to generate APNX')
+                import traceback
+                traceback.print_exc()
 
 
 class KINDLE_DX(KINDLE2):
 
     name           = 'Kindle DX Device Interface'
-    description    = _('Communicate with the Kindle DX eBook reader.')
+    description    = _('Communicate with the Kindle DX e-book reader.')
 
     FORMATS = ['azw', 'mobi', 'prc', 'azw1', 'tpz', 'azw4', 'pobi', 'pdf', 'txt']
     PRODUCT_ID = [0x0003]
@@ -477,12 +636,16 @@ class KINDLE_DX(KINDLE2):
     def upload_kindle_thumbnail(self, metadata, filepath):
         pass
 
+    def delete_single_book(self, path):
+        USBMS.delete_single_book(self, path)
+
+
 class KINDLE_FIRE(KINDLE2):
 
     name = 'Kindle Fire Device Interface'
     description = _('Communicate with the Kindle Fire')
     gui_name = 'Fire'
-    FORMATS = ['azw3', 'azw', 'mobi', 'prc', 'azw1', 'tpz', 'azw4', 'pobi', 'pdf', 'txt']
+    FORMATS = ['azw3', 'azw', 'mobi', 'prc', 'azw1', 'tpz', 'azw4', 'kfx', 'pobi', 'pdf', 'txt']
 
     PRODUCT_ID = [0x0006]
     BCD = [0x216, 0x100]
@@ -497,3 +660,5 @@ class KINDLE_FIRE(KINDLE2):
     def upload_kindle_thumbnail(self, metadata, filepath):
         pass
 
+    def delete_single_book(self, path):
+        USBMS.delete_single_book(self, path)
