@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,6 +9,7 @@ __docformat__ = 'restructuredtext en'
 import traceback, re
 
 from calibre.constants import iswindows
+from polyglot.builtins import iteritems
 
 
 class DeviceDefaults(object):
@@ -20,7 +20,7 @@ class DeviceDefaults(object):
                 ({'vendor':0x1949}, {
                     'format_map': ['azw3', 'mobi', 'azw',
                                     'azw1', 'azw4', 'kfx', 'pdf'],
-                    'send_to': ['documents', 'books', 'kindle'],
+                    'send_to': ['documents', 'kindle', 'books'],
                     }
                 ),
                 # B&N devices
@@ -47,7 +47,7 @@ class DeviceDefaults(object):
         for rule in self.rules:
             tests = rule[0]
             matches = True
-            for k, v in tests.iteritems():
+            for k, v in iteritems(tests):
                 if k == 'vendor' and v != vid:
                     matches = False
                     break
