@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2009, John Schember <john at nachtimwald.com> ' \
                 '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -11,20 +12,24 @@ intended to be subclassed with the relevant parts implemented for a particular
 device. This class handles device detection.
 '''
 
-import os, subprocess, time, re, sys, glob
-from itertools import repeat
+import glob
+import os
+import re
+import subprocess
+import sys
+import time
 from collections import namedtuple
+from itertools import repeat
 
 from calibre import prints
-from calibre.constants import DEBUG
-from calibre.devices.interface import DevicePlugin
+from calibre.constants import DEBUG, isfreebsd, islinux, ismacos, iswindows
 from calibre.devices.errors import DeviceError
+from calibre.devices.interface import DevicePlugin
 from calibre.devices.usbms.deviceconfig import DeviceConfig
-from calibre.constants import iswindows, islinux, isosx, isfreebsd, plugins
 from calibre.utils.filenames import ascii_filename as sanitize
+from polyglot.builtins import iteritems, map, string_or_bytes
 
-if isosx:
-    usbobserver, usbobserver_err = plugins['usbobserver']
+if ismacos:
     osx_sanitize_name_pat = re.compile(r'[.-]')
 
 if iswindows:
@@ -62,7 +67,7 @@ class USBDevice:
             return False
         if man == self.manufacturer and prod == self.product:
             return True
-        # As of OS X 10.11.4 Apple started mangling the names returned via the
+        # As of macOS 10.11.4 Apple started mangling the names returned via the
         # IOKit registry. See
         # https://www.mobileread.com/forums/showthread.php?t=273213
         m = osx_sanitize_name_pat.sub('_', (self.manufacturer or ''))
@@ -84,30 +89,24 @@ class Device(DeviceConfig, DevicePlugin):
 
     VENDOR_NAME = None
 
-    #: String identifying the main memory of the device in the windows PnP id
+    #: String identifying the main memory of the device in the Windows PnP id
     #: strings
     #: This can be None, string, list of strings or compiled regex
     WINDOWS_MAIN_MEM = None
 
-    #: String identifying the first card of the device in the windows PnP id
+    #: String identifying the first card of the device in the Windows PnP id
     #: strings
     #: This can be None, string, list of strings or compiled regex
     WINDOWS_CARD_A_MEM = None
 
-    #: String identifying the second card of the device in the windows PnP id
+    #: String identifying the second card of the device in the Windows PnP id
     #: strings
     #: This can be None, string, list of strings or compiled regex
     WINDOWS_CARD_B_MEM = None
 
-    # The following are used by the check_ioreg_line method and can be either:
-    # None, a string, a list of strings or a compiled regular expression
-    OSX_MAIN_MEM = None
-    OSX_CARD_A_MEM = None
-    OSX_CARD_B_MEM = None
-
     #: Used by the new driver detection to disambiguate main memory from
     #: storage cards. Should be a regular expression that matches the
-    #: main memory mount point assigned by OS X
+    #: main memory mount point assigned by macOS
     OSX_MAIN_MEM_VOL_PAT = None
     OSX_EJECT_COMMAND = ['diskutil', 'eject']
 
@@ -150,20 +149,17 @@ class Device(DeviceConfig, DevicePlugin):
         if not prefix:
             return 0, 0
         prefix = prefix[:-1]
-        import win32file, winerror
+        from calibre_extensions import winutil
         try:
-            sectors_per_cluster, bytes_per_sector, free_clusters, total_clusters = \
-                win32file.GetDiskFreeSpace(prefix)
-        except Exception as err:
-            if getattr(err, 'args', [None])[0] == winerror.ERROR_NOT_READY:
+            available_space, total_space, free_space = winutil.get_disk_free_space(prefix)
+        except OSError as err:
+            if err.winerror == winutil.ERROR_NOT_READY:
                 # Disk not ready
                 time.sleep(3)
-                sectors_per_cluster, bytes_per_sector, free_clusters, total_clusters = \
-                    win32file.GetDiskFreeSpace(prefix)
+                available_space, total_space, free_space = winutil.get_disk_free_space(prefix)
             else:
                 raise
-        mult = sectors_per_cluster * bytes_per_sector
-        return total_clusters * mult, free_clusters * mult
+        return total_space, available_space
 
     def total_space(self, end_session=True):
         msz = casz = cbsz = 0
@@ -215,8 +211,7 @@ class Device(DeviceConfig, DevicePlugin):
         return drives
 
     def can_handle_windows(self, usbdevice, debug=False):
-        from calibre.devices.interface import DevicePlugin
-        if self.can_handle.im_func is DevicePlugin.can_handle.im_func:
+        if hasattr(self.can_handle, 'is_base_class_implementation'):
             # No custom can_handle implementation
             return True
         # Delegate to the unix can_handle function, creating a unix like
@@ -248,9 +243,9 @@ class Device(DeviceConfig, DevicePlugin):
         try:
             dlmap = get_drive_letters_for_device(usbdev, debug=debug)
         except Exception:
-            dlmap = []
+            dlmap = {}
 
-        if not dlmap['drive_letters']:
+        if not dlmap.get('drive_letters'):
             time.sleep(7)
             dlmap = get_drive_letters_for_device(usbdev, debug=debug)
 
@@ -309,49 +304,6 @@ class Device(DeviceConfig, DevicePlugin):
     def osx_sort_names(self, names):
         return names
 
-    def check_ioreg_line(self, line, pat):
-        if pat is None:
-            return False
-        if not line.strip().endswith('<class IOMedia>'):
-            return False
-        if hasattr(pat, 'search'):
-            return pat.search(line) is not None
-        if isinstance(pat, basestring):
-            pat = [pat]
-        for x in pat:
-            if x in line:
-                return True
-        return False
-
-    def get_osx_mountpoints(self, raw=None):
-        raw = self.run_ioreg(raw)
-        lines = raw.splitlines()
-        names = {}
-
-        def get_dev_node(lines, loc):
-            for line in lines:
-                line = line.strip()
-                if line.endswith('}'):
-                    break
-                match = re.search(r'"BSD Name"\s+=\s+"(.*?)"', line)
-                if match is not None:
-                    names[loc] = match.group(1)
-                    break
-
-        for i, line in enumerate(lines):
-            if 'main' not in names and self.check_ioreg_line(line, self.OSX_MAIN_MEM):
-                get_dev_node(lines[i+1:], 'main')
-                continue
-            if 'carda' not in names and self.check_ioreg_line(line, self.OSX_CARD_A_MEM):
-                get_dev_node(lines[i+1:], 'carda')
-                continue
-            if 'cardb' not in names and self.check_ioreg_line(line, self.OSX_CARD_B_MEM):
-                get_dev_node(lines[i+1:], 'cardb')
-                continue
-            if len(names.keys()) == 3:
-                break
-        return self.osx_sort_names(names)
-
     @classmethod
     def osx_run_mount(cls):
         for i in range(3):
@@ -365,9 +317,8 @@ class Device(DeviceConfig, DevicePlugin):
 
     @classmethod
     def osx_get_usb_drives(cls):
-        if usbobserver_err:
-            raise RuntimeError('Failed to load usbobserver: '+usbobserver_err)
-        return usbobserver.get_usb_drives()
+        from calibre_extensions.usbobserver import get_usb_drives
+        return get_usb_drives()
 
     def _osx_bsd_names(self):
         drives = self.osx_get_usb_drives()
@@ -403,9 +354,9 @@ class Device(DeviceConfig, DevicePlugin):
             g = m.groupdict()
             if g['p'] is None:
                 g['p'] = 0
-            return map(int, (g.get('m'), g.get('p')))
+            return list(map(int, (g.get('m'), g.get('p'))))
 
-        def dcmp(x, y):
+        def cmp_key(x):
             '''
             Sorting based on the following scheme:
                 - disks without partitions are first
@@ -414,18 +365,11 @@ class Device(DeviceConfig, DevicePlugin):
                   disk number, then on partition number
             '''
             x = x.rpartition('/')[-1]
-            y = y.rpartition('/')[-1]
-            x, y = nums(x), nums(y)
-            if x[1] == 0 and y[1] > 0:
-                return cmp(1, 2)
-            if x[1] > 0 and y[1] == 0:
-                return cmp(2, 1)
-            ans = cmp(x[0], y[0])
-            if ans == 0:
-                ans = cmp(x[1], y[1])
-            return ans
+            disk_num, part_num = nums(x)
+            has_part = 1 if part_num > 0 else 0
+            return has_part, disk_num, part_num
 
-        matches.sort(cmp=dcmp)
+        matches.sort(key=cmp_key)
         drives = {'main':matches[0]}
         if len(matches) > 1:
             drives['carda'] = matches[1]
@@ -435,27 +379,35 @@ class Device(DeviceConfig, DevicePlugin):
         return drives
 
     def osx_bsd_names(self):
-        drives = []
+        drives = {}
         for i in range(3):
             try:
                 drives = self._osx_bsd_names()
-                if len(drives) > 1:
+                if len(drives) > 1:  # wait for device to settle and SD card (if any) to become available
                     return drives
-            except:
+            except Exception:
                 if i == 2:
                     raise
             time.sleep(3)
         return drives
 
     def open_osx(self):
-        drives = self.osx_bsd_names()
-        bsd_drives = dict(**drives)
-        drives = self.osx_sort_names(drives)
-        mount_map = usbobserver.get_mounted_filesystems()
-        for k, v in drives.items():
-            drives[k] = mount_map.get(v, None)
-        if drives['main'] is None:
-            print bsd_drives, mount_map, drives
+        from calibre_extensions.usbobserver import get_mounted_filesystems
+        bsd_drives = self.osx_bsd_names()
+        drives = self.osx_sort_names(bsd_drives.copy())
+        mount_map = get_mounted_filesystems()
+        drives = {k: mount_map.get(v) for k, v in iteritems(drives)}
+        if DEBUG:
+            print()
+            from pprint import pprint
+            pprint({'bsd_drives': bsd_drives, 'mount_map': mount_map, 'drives': drives})
+        if drives.get('carda') is None and drives.get('cardb') is not None:
+            drives['carda'] = drives.pop('cardb')
+        if drives.get('main') is None and drives.get('carda') is not None:
+            drives['main'] = drives.pop('carda')
+        if drives.get('carda') is None and drives.get('cardb') is not None:
+            drives['carda'] = drives.pop('cardb')
+        if drives.get('main') is None:
             raise DeviceError(_('Unable to detect the %s mount point. Try rebooting.')%self.__class__.__name__)
         pat = self.OSX_MAIN_MEM_VOL_PAT
         if pat is not None and len(drives) > 1 and 'main' in drives:
@@ -508,15 +460,22 @@ class Device(DeviceConfig, DevicePlugin):
             q = getattr(detected_device, attr)
             return q == val
 
+        def getnum(usb_dir):
+            def rc(q):
+                with open(j(usb_dir, q), 'rb') as f:
+                    return raw2num(f.read().decode('utf-8'))
+            return rc
+
         for x, isfile in walk('/sys/devices'):
             if isfile and x.endswith('idVendor'):
                 usb_dir = d(x)
                 for y in ('idProduct', 'idVendor', 'bcdDevice'):
                     if not os.access(j(usb_dir, y), os.R_OK):
                         usb_dir = None
-                        continue
-                e = lambda q : raw2num(open(j(usb_dir, q)).read())
-                ven, prod, bcd = map(e, ('idVendor', 'idProduct', 'bcdDevice'))
+                        break
+                if usb_dir is None:
+                    continue
+                ven, prod, bcd = map(getnum(usb_dir), ('idVendor', 'idProduct', 'bcdDevice'))
                 if not (test(ven, 'idVendor') and test(prod, 'idProduct') and
                         test(bcd, 'bcdDevice')):
                     usb_dir = None
@@ -537,7 +496,8 @@ class Device(DeviceConfig, DevicePlugin):
                     sz = j(x, 'size')
                     node = parts[idx+1]
                     try:
-                        exists = int(open(sz).read()) > 0
+                        with open(sz, 'rb') as szf:
+                            exists = int(szf.read().decode('utf-8')) > 0
                         if exists:
                             node = self.find_largest_partition(x)
                             ok[node] = True
@@ -546,14 +506,14 @@ class Device(DeviceConfig, DevicePlugin):
                     except:
                         ok[node] = False
                     if DEBUG and not ok[node]:
-                        print '\nIgnoring the node: %s as could not read size from: %s' % (node, sz)
+                        print('\nIgnoring the node: %s as could not read size from: %s' % (node, sz))
 
                     devnodes.append(node)
 
         devnodes += list(repeat(None, 3))
-        ans = ['/dev/'+x if ok.get(x, False) else None for x in devnodes[:3]]
+        ans = ['/dev/'+x if ok.get(x, False) else None for x in devnodes]
         ans.sort(key=lambda x: x[5:] if x else 'zzzzz')
-        return self.linux_swap_drives(ans)
+        return self.linux_swap_drives(ans[:3])
 
     def linux_swap_drives(self, drives):
         return drives
@@ -571,13 +531,13 @@ class Device(DeviceConfig, DevicePlugin):
             if not os.access(sz, os.R_OK):
                 continue
             try:
-                sz = int(open(sz).read())
+                sz = int(open(sz, 'rb').read().decode('utf-8'))
             except:
                 continue
             if sz > 0:
                 nodes.append((x.split('/')[-1], sz))
 
-        nodes.sort(cmp=lambda x, y: cmp(x[1], y[1]))
+        nodes.sort(key=lambda x: x[1])
         if not nodes:
             return node
         return nodes[-1][0]
@@ -595,7 +555,7 @@ class Device(DeviceConfig, DevicePlugin):
                     mount(node)
                     return 0
                 except:
-                    print 'Udisks mount call failed:'
+                    print('Udisks mount call failed:')
                     import traceback
                     traceback.print_exc()
                     return 1
@@ -612,7 +572,7 @@ class Device(DeviceConfig, DevicePlugin):
             'kernel is exporting a deprecated version of SYSFS.')
                     %self.__class__.__name__)
         if DEBUG:
-            print '\nFound device nodes:', main, carda, cardb
+            print('\nFound device nodes:', main, carda, cardb)
 
         self._linux_mount_map = {}
         mp, ret = mount(main, 'main')
@@ -631,7 +591,7 @@ class Device(DeviceConfig, DevicePlugin):
                 continue
             mp, ret = mount(card, typ)
             if mp is None:
-                print >>sys.stderr, 'Unable to mount card (Error code: %d)'%ret
+                print('Unable to mount card (Error code: %d)'%ret, file=sys.stderr)
             else:
                 if not mp.endswith('/'):
                     mp += '/'
@@ -658,7 +618,7 @@ class Device(DeviceConfig, DevicePlugin):
                 except:
                     pass
             if DEBUG and ro:
-                print '\nThe mountpoint', mp, 'is readonly, ignoring it'
+                print('\nThe mountpoint', mp, 'is readonly, ignoring it')
             return ro
 
         for mp in ('_main_prefix', '_card_a_prefix', '_card_b_prefix'):
@@ -693,7 +653,6 @@ class Device(DeviceConfig, DevicePlugin):
 #              4.  when finished, we have a list of mount points and associated dbus nodes
 #
     def open_freebsd(self):
-        import dbus
         # There should be some way to access the -v arg...
         verbose = False
 
@@ -703,115 +662,17 @@ class Device(DeviceConfig, DevicePlugin):
 
         if not d.serial:
             raise DeviceError("Device has no S/N.  Can't continue")
-            return False
-
-        vols=[]
-
-        bus = dbus.SystemBus()
-        manager = dbus.Interface(bus.get_object('org.freedesktop.Hal',
-                      '/org/freedesktop/Hal/Manager'), 'org.freedesktop.Hal.Manager')
-        paths = manager.FindDeviceStringMatch('usb.serial',d.serial)
-        for path in paths:
-            objif = dbus.Interface(bus.get_object('org.freedesktop.Hal', path), 'org.freedesktop.Hal.Device')
-            # Extra paranoia...
-            try:
-                if d.idVendor == objif.GetProperty('usb.vendor_id') and \
-                        d.idProduct == objif.GetProperty('usb.product_id') and \
-                        d.manufacturer == objif.GetProperty('usb.vendor') and \
-                        d.product == objif.GetProperty('usb.product') and \
-                        d.serial == objif.GetProperty('usb.serial'):
-                    midpath = manager.FindDeviceStringMatch('info.parent', path)
-                    dpaths = manager.FindDeviceStringMatch(
-                        'storage.originating_device', path) + manager.FindDeviceStringMatch('storage.originating_device', midpath[0])
-                    for dpath in dpaths:
-                        # devif = dbus.Interface(bus.get_object('org.freedesktop.Hal', dpath), 'org.freedesktop.Hal.Device')
-                        try:
-                            vpaths = manager.FindDeviceStringMatch('block.storage_device', dpath)
-                            for vpath in vpaths:
-                                try:
-                                    vdevif = dbus.Interface(bus.get_object('org.freedesktop.Hal', vpath), 'org.freedesktop.Hal.Device')
-                                    if not vdevif.GetProperty('block.is_volume'):
-                                        continue
-                                    if vdevif.GetProperty('volume.fsusage') != 'filesystem':
-                                        continue
-                                    volif = dbus.Interface(bus.get_object('org.freedesktop.Hal', vpath), 'org.freedesktop.Hal.Device.Volume')
-                                    pdevif = dbus.Interface(bus.get_object('org.freedesktop.Hal', vdevif.GetProperty('info.parent')),
-                                                            'org.freedesktop.Hal.Device')
-                                    vol = {'node': pdevif.GetProperty('block.device'),
-                                            'dev': vdevif,
-                                            'vol': volif,
-                                            'label': vdevif.GetProperty('volume.label')}
-                                    vols.append(vol)
-                                except dbus.exceptions.DBusException as e:
-                                    print e
-                                    continue
-                        except dbus.exceptions.DBusException as e:
-                            print e
-                            continue
-            except dbus.exceptions.DBusException as e:
-                continue
-
-        def ocmp(x,y):
-            if x['node'] < y['node']:
-                return -1
-            if x['node'] > y['node']:
-                return 1
-            return 0
-
-        vols.sort(cmp=ocmp)
-
+        from .hal import get_hal
+        hal = get_hal()
+        vols = hal.get_volumes(d)
         if verbose:
-            print "FBSD:	", vols
+            print("FBSD:	", vols)
 
-        mtd=0
-
-        for vol in vols:
-            mp = ''
-            if vol['dev'].GetProperty('volume.is_mounted'):
-                mp = vol['dev'].GetProperty('volume.mount_point')
-            else:
-                try:
-                    vol['vol'].Mount('Calibre-'+vol['label'],
-                            vol['dev'].GetProperty('volume.fstype'), [])
-                    loops = 0
-                    while not vol['dev'].GetProperty('volume.is_mounted'):
-                        time.sleep(1)
-                        loops += 1
-                        if loops > 100:
-                            print "ERROR: Timeout waiting for mount to complete"
-                            continue
-                    mp = vol['dev'].GetProperty('volume.mount_point')
-                except dbus.exceptions.DBusException as e:
-                    print "Failed to mount ", e
-                    continue
-
-            # Mount Point becomes Mount Path
-            mp += '/'
-
-            if verbose:
-                print "FBSD:	  mounted", vol['label'], "on", mp
-            if mtd == 0:
-                self._main_prefix = mp
-                self._main_vol = vol['vol']
-                if verbose:
-                    print "FBSD:	main = ", self._main_prefix
-            if mtd == 1:
-                self._card_a_prefix = mp
-                self._card_a_vol = vol['vol']
-                if verbose:
-                    print "FBSD:	card a = ", self._card_a_prefix
-            if mtd == 2:
-                self._card_b_prefix = mp
-                self._card_b_vol = vol['vol']
-                if verbose:
-                    print "FBSD:	card b = ", self._card_b_prefix
-                # Note that mtd is used as a bool... not incrementing is fine.
-                break
-            mtd += 1
-
-        if mtd > 0:
-            return True
-        raise DeviceError(_('Unable to mount the device'))
+        ok, mv = hal.mount_volumes(vols)
+        if not ok:
+            raise DeviceError(_('Unable to mount the device'))
+        for k, v in mv.items():
+            setattr(self, k, v)
 
 #
 # ------------------------------------------------------
@@ -821,37 +682,18 @@ class Device(DeviceConfig, DevicePlugin):
 #        mounted filesystems, using the stored volume object
 #
     def eject_freebsd(self):
-        import dbus
-        # There should be some way to access the -v arg...
-        verbose = False
-
+        from .hal import get_hal
+        hal = get_hal()
         if self._main_prefix:
-            if verbose:
-                print "FBSD:	umount main:", self._main_prefix
-            try:
-                self._main_vol.Unmount([])
-            except dbus.exceptions.DBusException as e:
-                print 'Unable to eject ', e
-
+            hal.unmount(self._main_vol)
         if self._card_a_prefix:
-            if verbose:
-                print "FBSD:	umount card a:", self._card_a_prefix
-            try:
-                self._card_a_vol.Unmount([])
-            except dbus.exceptions.DBusException as e:
-                print 'Unable to eject ', e
-
+            hal.unmount(self._card_a_vol)
         if self._card_b_prefix:
-            if verbose:
-                print "FBSD:	umount card b:", self._card_b_prefix
-            try:
-                self._card_b_vol.Unmount([])
-            except dbus.exceptions.DBusException as e:
-                print 'Unable to eject ', e
+            hal.unmount(self._card_b_vol)
 
-        self._main_prefix = None
-        self._card_a_prefix = None
-        self._card_b_prefix = None
+        self._main_prefix = self._main_vol = None
+        self._card_a_prefix = self._card_a_vol = None
+        self._card_b_prefix = self._card_b_vol = None
 # ------------------------------------------------------
 
     def open(self, connected_device, library_uuid):
@@ -874,7 +716,7 @@ class Device(DeviceConfig, DevicePlugin):
                     self.open_freebsd()
             if iswindows:
                 self.open_windows()
-            if isosx:
+            if ismacos:
                 try:
                     self.open_osx()
                 except DeviceError:
@@ -898,8 +740,7 @@ class Device(DeviceConfig, DevicePlugin):
                 drives.append(x[0].upper())
 
         def do_it(drives):
-            import win32process
-            subprocess.Popen([eject_exe()] + drives, creationflags=win32process.CREATE_NO_WINDOW).wait()
+            subprocess.Popen([eject_exe()] + drives, creationflags=subprocess.CREATE_NO_WINDOW).wait()
 
         t = Thread(target=do_it, args=[drives])
         t.daemon = True
@@ -927,8 +768,8 @@ class Device(DeviceConfig, DevicePlugin):
             try:
                 eject(d)
             except Exception as e:
-                print 'Udisks eject call for:', d, 'failed:'
-                print '\t', e
+                print('Udisks eject call for:', d, 'failed:')
+                print('\t', e)
 
     def eject(self):
         if islinux:
@@ -946,7 +787,7 @@ class Device(DeviceConfig, DevicePlugin):
                 self.eject_windows()
             except:
                 pass
-        if isosx:
+        if ismacos:
             try:
                 self.eject_osx()
             except:
@@ -979,7 +820,7 @@ class Device(DeviceConfig, DevicePlugin):
         sanity_check(on_card, files, self.card_prefix(), self.free_space())
 
         def get_dest_dir(prefix, candidates):
-            if isinstance(candidates, basestring):
+            if isinstance(candidates, string_or_bytes):
                 candidates = [candidates]
             if not candidates:
                 candidates = ['']
