@@ -1,4 +1,5 @@
-from __future__ import with_statement
+
+
 __license__ = 'GPL 3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -7,6 +8,7 @@ import os, re, posixpath
 from itertools import cycle
 
 from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
+from polyglot.builtins import getcwd
 
 ADOBE_OBFUSCATION =  'http://ns.adobe.com/pdf/enc#RC'
 IDPF_OBFUSCATION = 'http://www.idpf.org/2008/embedding'
@@ -17,12 +19,12 @@ def decrypt_font_data(key, data, algorithm):
     crypt_len = 1024 if is_adobe else 1040
     crypt = bytearray(data[:crypt_len])
     key = cycle(iter(bytearray(key)))
-    decrypt = bytes(bytearray(x^key.next() for x in crypt))
+    decrypt = bytes(bytearray(x^next(key) for x in crypt))
     return decrypt + data[crypt_len:]
 
 
 def decrypt_font(key, path, algorithm):
-    with open(path, 'r+b') as f:
+    with lopen(path, 'r+b') as f:
         data = decrypt_font_data(key, f.read(), algorithm)
         f.seek(0), f.truncate(), f.write(data)
 
@@ -31,18 +33,19 @@ class EPUBInput(InputFormatPlugin):
 
     name        = 'EPUB Input'
     author      = 'Kovid Goyal'
-    description = 'Convert EPUB files (.epub) to HTML'
-    file_types  = set(['epub'])
+    description = _('Convert EPUB files (.epub) to HTML')
+    file_types  = {'epub'}
     output_encoding = None
+    commit_name = 'epub_input'
 
-    recommendations = set([('page_breaks_before', '/', OptionRecommendation.MED)])
+    recommendations = {('page_breaks_before', '/', OptionRecommendation.MED)}
 
     def process_encryption(self, encfile, opf, log):
         from lxml import etree
         import uuid, hashlib
         idpf_key = opf.raw_unique_identifier
         if idpf_key:
-            idpf_key = re.sub(u'[\u0020\u0009\u000d\u000a]', u'', idpf_key)
+            idpf_key = re.sub('[\u0020\u0009\u000d\u000a]', '', idpf_key)
             idpf_key = hashlib.sha1(idpf_key.encode('utf-8')).digest()
         key = None
         for item in opf.identifier_iter():
@@ -53,7 +56,7 @@ class EPUBInput(InputFormatPlugin):
             if (scheme and scheme.lower() == 'uuid') or \
                     (item.text and item.text.startswith('urn:uuid:')):
                 try:
-                    key = bytes(item.text).rpartition(':')[-1]
+                    key = item.text.rpartition(':')[-1]
                     key = uuid.UUID(key).bytes
                 except:
                     import traceback
@@ -80,14 +83,14 @@ class EPUBInput(InputFormatPlugin):
         return False
 
     def set_guide_type(self, opf, gtype, href=None, title=''):
-        # Set the titlepage guide entry
+        # Set the specified guide entry
         for elem in list(opf.iterguide()):
-            if elem.get('type', '').lower() == 'titlepage':
+            if elem.get('type', '').lower() == gtype:
                 elem.getparent().remove(elem)
 
         if href is not None:
             t = opf.create_guide_item(gtype, title, href)
-            for guide in opf.root.iterchildren('guide'):
+            for guide in opf.root.xpath('./*[local-name()="guide"]'):
                 guide.append(t)
                 return
             guide = opf.create_guide_element()
@@ -99,8 +102,25 @@ class EPUBInput(InputFormatPlugin):
         ''' If there is a reference to the cover/titlepage via manifest properties, convert to
         entries in the <guide> so that the rest of the pipeline picks it up. '''
         from calibre.ebooks.metadata.opf3 import items_with_property
-        removed = None
-        raster_cover_href = opf.epub3_raster_cover
+        removed = guide_titlepage_href = guide_titlepage_id = None
+
+        # Look for titlepages incorrectly marked in the <guide> as covers
+        guide_cover, guide_elem = None, None
+        for guide_elem in opf.iterguide():
+            if guide_elem.get('type', '').lower() == 'cover':
+                guide_cover = guide_elem.get('href', '').partition('#')[0]
+                break
+        if guide_cover:
+            spine = list(opf.iterspine())
+            if spine:
+                idref = spine[0].get('idref', '')
+                for x in opf.itermanifest():
+                    if x.get('id') == idref and x.get('href') == guide_cover:
+                        guide_titlepage_href = guide_cover
+                        guide_titlepage_id = idref
+                        break
+
+        raster_cover_href = opf.epub3_raster_cover or opf.raster_cover
         if raster_cover_href:
             self.set_guide_type(opf, 'cover', raster_cover_href, 'Cover Image')
         titlepage_id = titlepage_href = None
@@ -109,8 +129,10 @@ class EPUBInput(InputFormatPlugin):
             if href and tid:
                 titlepage_id, titlepage_href = tid, href.partition('#')[0]
                 break
+        if titlepage_href is None:
+            titlepage_href, titlepage_id = guide_titlepage_href, guide_titlepage_id
         if titlepage_href is not None:
-            self.set_guide_type(opf, 'titlepage', titlepage_href, 'Title Page')
+            self.set_guide_type(opf, 'titlepage', titlepage_href, 'Title page')
             spine = list(opf.iterspine())
             if len(spine) > 1:
                 for item in spine:
@@ -201,33 +223,33 @@ class EPUBInput(InputFormatPlugin):
             if os.path.exists(guide_cover):
                 renderer = render_html_svg_workaround(guide_cover, log)
                 if renderer is not None:
-                    open('calibre_raster_cover.jpg', 'wb').write(
-                        renderer)
+                    with lopen('calibre_raster_cover.jpg', 'wb') as f:
+                        f.write(renderer)
 
         # Set the titlepage guide entry
-        self.set_guide_type(opf, 'titlepage', guide_cover, 'Title Page')
+        self.set_guide_type(opf, 'titlepage', guide_cover, 'Title page')
         return removed
 
     def find_opf(self):
-        from lxml import etree
+        from calibre.utils.xml_parse import safe_xml_fromstring
 
         def attr(n, attr):
             for k, v in n.attrib.items():
                 if k.endswith(attr):
                     return v
         try:
-            with open('META-INF/container.xml') as f:
-                root = etree.fromstring(f.read())
+            with lopen('META-INF/container.xml', 'rb') as f:
+                root = safe_xml_fromstring(f.read())
                 for r in root.xpath('//*[local-name()="rootfile"]'):
                     if attr(r, 'media-type') != "application/oebps-package+xml":
                         continue
                     path = attr(r, 'full-path')
                     if not path:
                         continue
-                    path = os.path.join(os.getcwdu(), *path.split('/'))
+                    path = os.path.join(getcwd(), *path.split('/'))
                     if os.path.exists(path):
                         return path
-        except:
+        except Exception:
             import traceback
             traceback.print_exc()
 
@@ -238,7 +260,7 @@ class EPUBInput(InputFormatPlugin):
         from calibre.ebooks.metadata.opf2 import OPF
         try:
             zf = ZipFile(stream)
-            zf.extractall(os.getcwdu())
+            zf.extractall(getcwd())
         except:
             log.exception('EPUB appears to be invalid ZIP file, trying a'
                     ' more forgiving ZIP parser')
@@ -248,7 +270,7 @@ class EPUBInput(InputFormatPlugin):
         encfile = os.path.abspath(os.path.join('META-INF', 'encryption.xml'))
         opf = self.find_opf()
         if opf is None:
-            for f in walk(u'.'):
+            for f in walk('.'):
                 if f.lower().endswith('.opf') and '__MACOSX' not in f and \
                         not os.path.basename(f).startswith('.'):
                     opf = os.path.abspath(f)
@@ -258,7 +280,7 @@ class EPUBInput(InputFormatPlugin):
         if opf is None:
             raise ValueError('%s is not a valid EPUB file (could not find opf)'%path)
 
-        opf = os.path.relpath(opf, os.getcwdu())
+        opf = os.path.relpath(opf, getcwd())
         parts = os.path.split(opf)
         opf = OPF(opf, os.path.dirname(os.path.abspath(opf)))
 
@@ -268,10 +290,6 @@ class EPUBInput(InputFormatPlugin):
                 raise DRMError(os.path.basename(path))
         self.encrypted_fonts = self._encrypted_font_uris
 
-        epub3_nav = opf.epub3_nav
-        if epub3_nav is not None:
-            self.convert_epub3_nav(epub3_nav, opf, log)
-
         if len(parts) > 1 and parts[0]:
             delta = '/'.join(parts[:-1])+'/'
 
@@ -285,6 +303,11 @@ class EPUBInput(InputFormatPlugin):
 
         f = self.rationalize_cover3 if opf.package_version >= 3.0 else self.rationalize_cover2
         self.removed_cover = f(opf, log)
+        if self.removed_cover:
+            self.removed_items_to_ignore = (self.removed_cover,)
+        epub3_nav = opf.epub3_nav
+        if epub3_nav is not None:
+            self.convert_epub3_nav(epub3_nav, opf, log, options)
 
         for x in opf.itermanifest():
             if x.get('media-type', '') == 'application/x-dtbook+xml':
@@ -294,11 +317,22 @@ class EPUBInput(InputFormatPlugin):
         not_for_spine = set()
         for y in opf.itermanifest():
             id_ = y.get('id', None)
-            if id_ and y.get('media-type', None) in {
-                    'application/vnd.adobe-page-template+xml', 'application/vnd.adobe.page-template+xml',
-                    'application/adobe-page-template+xml', 'application/adobe.page-template+xml',
-                    'application/text'}:
-                not_for_spine.add(id_)
+            if id_:
+                mt = y.get('media-type', None)
+                if mt in {
+                        'application/vnd.adobe-page-template+xml',
+                        'application/vnd.adobe.page-template+xml',
+                        'application/adobe-page-template+xml',
+                        'application/adobe.page-template+xml',
+                        'application/text'
+                }:
+                    not_for_spine.add(id_)
+                ext = y.get('href', '').rpartition('.')[-1].lower()
+                if mt == 'text/plain' and ext in {'otf', 'ttf'}:
+                    # some epub authoring software sets font mime types to
+                    # text/plain
+                    not_for_spine.add(id_)
+                    y.set('media-type', 'application/font')
 
         seen = set()
         for x in list(opf.iterspine()):
@@ -314,20 +348,21 @@ class EPUBInput(InputFormatPlugin):
         with lopen('content.opf', 'wb') as nopf:
             nopf.write(opf.render())
 
-        return os.path.abspath(u'content.opf')
+        return os.path.abspath('content.opf')
 
-    def convert_epub3_nav(self, nav_path, opf, log):
+    def convert_epub3_nav(self, nav_path, opf, log, opts):
         from lxml import etree
         from calibre.ebooks.chardet import xml_to_unicode
         from calibre.ebooks.oeb.polish.parsing import parse
-        from calibre.ebooks.oeb.base import EPUB_NS, XHTML, NCX_MIME, NCX
+        from calibre.ebooks.oeb.base import EPUB_NS, XHTML, NCX_MIME, NCX, urlnormalize, urlunquote, serialize
         from calibre.ebooks.oeb.polish.toc import first_child
+        from calibre.utils.xml_parse import safe_xml_fromstring
         from tempfile import NamedTemporaryFile
         with lopen(nav_path, 'rb') as f:
             raw = f.read()
         raw = xml_to_unicode(raw, strip_encoding_pats=True, assume_utf8=True)[0]
         root = parse(raw, log=log)
-        ncx = etree.fromstring('<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1" xml:lang="eng"><navMap/></ncx>')
+        ncx = safe_xml_fromstring('<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1" xml:lang="eng"><navMap/></ncx>')
         navmap = ncx[0]
         et = '{%s}type' % EPUB_NS
         bn = os.path.basename(nav_path)
@@ -335,7 +370,9 @@ class EPUBInput(InputFormatPlugin):
         def add_from_li(li, parent):
             href = text = None
             for x in li.iterchildren(XHTML('a'), XHTML('span')):
-                text = etree.tostring(x, method='text', encoding=unicode, with_tail=False).strip() or ' '.join(x.xpath('descendant-or-self::*/@title')).strip()
+                text = etree.tostring(
+                    x, method='text', encoding='unicode', with_tail=False).strip() or ' '.join(
+                            x.xpath('descendant-or-self::*/@title')).strip()
                 href = x.get('href')
                 if href:
                     if href.startswith('#'):
@@ -368,9 +405,25 @@ class EPUBInput(InputFormatPlugin):
 
         with NamedTemporaryFile(suffix='.ncx', dir=os.path.dirname(nav_path), delete=False) as f:
             f.write(etree.tostring(ncx, encoding='utf-8'))
-        ncx_id = opf.add_path_to_manifest(f.name, NCX_MIME)
+        ncx_href = os.path.relpath(f.name, getcwd()).replace(os.sep, '/')
+        ncx_id = opf.create_manifest_item(ncx_href, NCX_MIME, append=True).get('id')
         for spine in opf.root.xpath('//*[local-name()="spine"]'):
             spine.set('toc', ncx_id)
+        opts.epub3_nav_href = urlnormalize(os.path.relpath(nav_path).replace(os.sep, '/'))
+        opts.epub3_nav_parsed = root
+        if getattr(self, 'removed_cover', None):
+            changed = False
+            base_path = os.path.dirname(nav_path)
+            for elem in root.xpath('//*[@href]'):
+                href, frag = elem.get('href').partition('#')[::2]
+                link_path = os.path.relpath(os.path.join(base_path, urlunquote(href)), base_path)
+                abs_href = urlnormalize(link_path)
+                if abs_href == self.removed_cover:
+                    changed = True
+                    elem.set('data-calibre-removed-titlepage', '1')
+            if changed:
+                with lopen(nav_path, 'wb') as f:
+                    f.write(serialize(root, 'application/xhtml+xml'))
 
     def postprocess_book(self, oeb, opts, log):
         rc = getattr(self, 'removed_cover', None)
