@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -10,6 +10,7 @@ import sys, os
 
 from calibre.customize.conversion import OutputFormatPlugin
 from calibre.customize.conversion import OptionRecommendation
+from polyglot.builtins import unicode_type
 
 
 class LRFOptions(object):
@@ -17,7 +18,7 @@ class LRFOptions(object):
     def __init__(self, output, opts, oeb):
         def f2s(f):
             try:
-                return unicode(f[0])
+                return unicode_type(f[0])
             except:
                 return ''
         m = oeb.metadata
@@ -31,13 +32,13 @@ class LRFOptions(object):
         self.title_sort = self.author_sort = ''
         for x in m.creator:
             if x.role == 'aut':
-                self.author = unicode(x)
-                fa = unicode(getattr(x, 'file_as', ''))
+                self.author = unicode_type(x)
+                fa = unicode_type(getattr(x, 'file_as', ''))
                 if fa:
                     self.author_sort = fa
         for x in m.title:
-            if unicode(x.file_as):
-                self.title_sort = unicode(x.file_as)
+            if unicode_type(x.file_as):
+                self.title_sort = unicode_type(x.file_as)
         self.freetext = f2s(m.description)
         self.category = f2s(m.subject)
         self.cover = None
@@ -54,7 +55,7 @@ class LRFOptions(object):
         self.ignore_colors = False
         from calibre.ebooks.lrf import PRS500_PROFILE
         self.profile = PRS500_PROFILE
-        self.link_levels = sys.maxint
+        self.link_levels = sys.maxsize
         self.link_exclude = '@'
         self.no_links_in_toc = True
         self.disable_chapter_detection = True
@@ -90,10 +91,11 @@ class LRFOutput(OutputFormatPlugin):
     name = 'LRF Output'
     author = 'Kovid Goyal'
     file_type = 'lrf'
+    commit_name = 'lrf_output'
 
-    options = set([
+    options = {
         OptionRecommendation(name='enable_autorotation', recommended_value=False,
-            help=_('Enable autorotation of images that are wider than the screen width.')
+            help=_('Enable auto-rotation of images that are wider than the screen width.')
         ),
         OptionRecommendation(name='wordspace',
             recommended_value=2.5, level=OptionRecommendation.LOW,
@@ -115,8 +117,7 @@ class LRFOutput(OutputFormatPlugin):
         ),
         OptionRecommendation(name='render_tables_as_images',
             recommended_value=False,
-            help=_('Render tables in the HTML as images (useful if the '
-                'document has large or complex tables)')
+            help=_('This option has no effect')
         ),
         OptionRecommendation(name='text_size_multiplier_for_rendered_tables',
             recommended_value=1.0,
@@ -133,11 +134,10 @@ class LRFOutput(OutputFormatPlugin):
             help=_('The monospace family of fonts to embed')
         ),
 
-    ])
+    }
 
-    recommendations = set([
-        ('change_justification', 'original', OptionRecommendation.HIGH),
-        ])
+    recommendations = {
+        ('change_justification', 'original', OptionRecommendation.HIGH)}
 
     def convert_images(self, pages, opts, wide):
         from calibre.ebooks.lrf.pylrs.pylrs import Book, BookSetting, ImageStream, ImageBlock
@@ -187,11 +187,10 @@ class LRFOutput(OutputFormatPlugin):
         self.flatten_toc()
 
         from calibre.ptempfile import TemporaryDirectory
-        with TemporaryDirectory(u'_lrf_output') as tdir:
+        with TemporaryDirectory('_lrf_output') as tdir:
             from calibre.customize.ui import plugin_for_output_format
             oeb_output = plugin_for_output_format('oeb')
             oeb_output.convert(oeb, tdir, input_plugin, opts, log)
             opf = [x for x in os.listdir(tdir) if x.endswith('.opf')][0]
             from calibre.ebooks.lrf.html.convert_from import process_file
             process_file(os.path.join(tdir, opf), lrf_opts, self.log)
-
