@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -106,15 +105,6 @@ def read_letter_spacing(parent, dest, XPath, get):
     setattr(dest, 'letter_spacing', ans)
 
 
-def read_sz(parent, dest, XPath, get):
-    ans = inherit
-    for col in XPath('./w:sz[@w:val]')(parent):
-        val = simple_float(get(col, 'w:val'), 0.5)
-        if val is not None:
-            ans = val
-    setattr(dest, 'font_size', ans)
-
-
 def read_underline(parent, dest, XPath, get):
     ans = inherit
     for col in XPath('./w:u[@w:val]')(parent):
@@ -144,8 +134,8 @@ def read_position(parent, dest, XPath, get):
     setattr(dest, 'position', ans)
 
 
-def read_font_family(parent, dest, XPath, get):
-    ans = inherit
+def read_font(parent, dest, XPath, get):
+    ff = inherit
     for col in XPath('./w:rFonts')(parent):
         val = get(col, 'w:asciiTheme')
         if val:
@@ -153,8 +143,34 @@ def read_font_family(parent, dest, XPath, get):
         else:
             val = get(col, 'w:ascii')
         if val:
-            ans = val
-    setattr(dest, 'font_family', ans)
+            ff = val
+    setattr(dest, 'font_family', ff)
+    for col in XPath('./w:sz[@w:val]')(parent):
+        val = simple_float(get(col, 'w:val'), 0.5)
+        if val is not None:
+            setattr(dest, 'font_size', val)
+            return
+    setattr(dest, 'font_size', inherit)
+
+
+def read_font_cs(parent, dest, XPath, get):
+    ff = inherit
+    for col in XPath('./w:rFonts')(parent):
+        val = get(col, 'w:csTheme')
+        if val:
+            val = '|%s|' % val
+        else:
+            val = get(col, 'w:cs')
+        if val:
+            ff = val
+    setattr(dest, 'cs_font_family', ff)
+    for col in XPath('./w:szCS[@w:val]')(parent):
+        val = simple_float(get(col, 'w:val'), 0.5)
+        if val is not None:
+            setattr(dest, 'font_size', val)
+            return
+    setattr(dest, 'cs_font_size', inherit)
+
 # }}}
 
 
@@ -166,6 +182,7 @@ class RunStyle(object):
 
         'border_color', 'border_style', 'border_width', 'padding', 'color', 'highlight', 'background_color',
         'letter_spacing', 'font_size', 'text_decoration', 'vert_align', 'lang', 'font_family', 'position',
+        'cs_font_size', 'cs_font_family'
     }
 
     toggle_properties = {
@@ -179,18 +196,27 @@ class RunStyle(object):
             for p in self.all_properties:
                 setattr(self, p, inherit)
         else:
+            X, g = namespace.XPath, namespace.get
             for p in (
                 'b', 'bCs', 'caps', 'cs', 'dstrike', 'emboss', 'i', 'iCs', 'imprint', 'rtl', 'shadow',
                 'smallCaps', 'strike', 'vanish', 'webHidden',
             ):
-                setattr(self, p, binary_property(rPr, p, namespace.XPath, namespace.get))
+                setattr(self, p, binary_property(rPr, p, X, g))
 
-            for x in ('text_border', 'color', 'highlight', 'shd', 'letter_spacing', 'sz', 'underline', 'vert_align', 'position', 'lang', 'font_family'):
-                f = globals()['read_%s' % x]
-                f(rPr, self, namespace.XPath, namespace.get)
+            read_font(rPr, self, X, g)
+            read_font_cs(rPr, self, X, g)
+            read_text_border(rPr, self, X, g)
+            read_color(rPr, self, X, g)
+            read_highlight(rPr, self, X, g)
+            read_shd(rPr, self, X, g)
+            read_letter_spacing(rPr, self, X, g)
+            read_underline(rPr, self, X, g)
+            read_vert_align(rPr, self, X, g)
+            read_position(rPr, self, X, g)
+            read_lang(rPr, self, X, g)
 
-            for s in namespace.XPath('./w:rStyle[@w:val]')(rPr):
-                self.linked_style = namespace.get(s, 'w:val')
+            for s in X('./w:rStyle[@w:val]')(rPr):
+                self.linked_style = g(s, 'w:val')
 
         self._css = None
 
@@ -274,4 +300,3 @@ class RunStyle(object):
 
     def same_border(self, other):
         return self.get_border_css({}) == other.get_border_css({})
-
