@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -19,11 +18,11 @@ from calibre.ptempfile import PersistentTemporaryDirectory
 from calibre.utils.localization import canonicalize_lang
 from calibre.utils.logging import default_log
 from calibre.utils.zipfile import ZipFile
-from calibre.ebooks.oeb.parse_utils import RECOVER_PARSER
+from calibre.utils.xml_parse import safe_xml_fromstring
 
 
-def fromstring(raw, parser=RECOVER_PARSER):
-    return etree.fromstring(raw, parser=parser)
+def fromstring(raw, parser=None):
+    return safe_xml_fromstring(raw)
 
 # Read metadata {{{
 
@@ -56,7 +55,7 @@ def read_doc_props(raw, mi, XPath):
 
     desc = XPath('//dc:description')(root)
     if desc:
-        raw = etree.tostring(desc[0], method='text', encoding=unicode)
+        raw = etree.tostring(desc[0], method='text', encoding='unicode')
         raw = raw.replace('_x000d_', '')  # Word 2007 mangles newlines in the summary
         mi.comments = raw.strip()
 
@@ -263,6 +262,7 @@ class DOCX(object):
             except EnvironmentError:
                 pass
 
+
 if __name__ == '__main__':
     d = DOCX(sys.argv[-1], extract=False)
-    print (d.metadata)
+    print(d.metadata)
