@@ -1,6 +1,7 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -14,6 +15,7 @@ from calibre.ebooks.metadata.book import (SC_COPYABLE_FIELDS,
         TOP_LEVEL_IDENTIFIERS, ALL_METADATA_FIELDS)
 from calibre.library.field_metadata import FieldMetadata
 from calibre.utils.icu import sort_key
+from polyglot.builtins import iteritems, unicode_type, filter, map, string_or_bytes
 
 # Special sets used to optimize the performance of getting and setting
 # attributes on Metadata objects
@@ -23,7 +25,11 @@ SIMPLE_SET = frozenset(SIMPLE_GET - {'identifiers'})
 
 def human_readable(size, precision=2):
     """ Convert a size in bytes into megabytes """
-    return ('%.'+str(precision)+'f'+ 'MB') % ((size/(1024.*1024.)),)
+    ans = size/(1024*1024)
+    if ans < 0.1:
+        return '<0.1MB'
+    return ('%.'+unicode_type(precision)+'f'+ 'MB') % ans
+
 
 NULL_VALUES = {
                 'user_metadata': {},
@@ -48,6 +54,7 @@ def reset_field_metadata():
     global field_metadata
     field_metadata = FieldMetadata()
 
+
 ck = lambda typ: icu_lower(typ).strip().replace(':', '').replace(',', '')
 cv = lambda val: val.strip().replace(',', '|')
 
@@ -72,6 +79,7 @@ class Metadata(object):
     Please keep the method based API of this class to a minimum. Every method
     becomes a reserved field name.
     '''
+    __calibre_serializable__ = True
 
     def __init__(self, title, authors=(_('Unknown'),), other=None, template_cache=None,
                  formatter=None):
@@ -114,6 +122,10 @@ class Metadata(object):
         except:
             return True
 
+    def set_null(self, field):
+        null_val = copy.copy(NULL_VALUES.get(field))
+        setattr(self, field, null_val)
+
     def __getattribute__(self, field):
         _data = object.__getattribute__(self, '_data')
         if field in SIMPLE_GET:
@@ -129,7 +141,7 @@ class Metadata(object):
             return object.__getattribute__(self, field)
         except AttributeError:
             pass
-        if field in _data['user_metadata'].iterkeys():
+        if field in _data['user_metadata']:
             d = _data['user_metadata'][field]
             val = d['#value#']
             if d['datatype'] != 'composite':
@@ -172,7 +184,7 @@ class Metadata(object):
             if val and val.lower() != 'und':
                 langs = [val]
             _data['languages'] = langs
-        elif field in _data['user_metadata'].iterkeys():
+        elif field in _data['user_metadata']:
             _data['user_metadata'][field]['#value#'] = val
             _data['user_metadata'][field]['#extra#'] = extra
         else:
@@ -182,7 +194,7 @@ class Metadata(object):
             self.__dict__[field] = val
 
     def __iter__(self):
-        return object.__getattribute__(self, '_data').iterkeys()
+        return iter(object.__getattribute__(self, '_data'))
 
     def has_key(self, key):
         return key in object.__getattribute__(self, '_data')
@@ -211,7 +223,7 @@ class Metadata(object):
 
     def get_extra(self, field, default=None):
         _data = object.__getattribute__(self, '_data')
-        if field in _data['user_metadata'].iterkeys():
+        if field in _data['user_metadata']:
             try:
                 return _data['user_metadata'][field]['#extra#']
             except:
@@ -247,7 +259,7 @@ class Metadata(object):
         Set all identifiers. Note that if you previously set ISBN, calling
         this method will delete it.
         '''
-        cleaned = {ck(k):cv(v) for k, v in identifiers.iteritems() if k and v}
+        cleaned = {ck(k):cv(v) for k, v in iteritems(identifiers) if k and v}
         object.__getattribute__(self, '_data')['identifiers'] = cleaned
 
     def set_identifier(self, typ, val):
@@ -279,14 +291,14 @@ class Metadata(object):
         '''
         return a list of the custom fields in this book
         '''
-        return object.__getattribute__(self, '_data')['user_metadata'].iterkeys()
+        return iter(object.__getattribute__(self, '_data')['user_metadata'])
 
     def all_field_keys(self):
         '''
         All field keys known by this instance, even if their value is None
         '''
         _data = object.__getattribute__(self, '_data')
-        return frozenset(ALL_METADATA_FIELDS.union(_data['user_metadata'].iterkeys()))
+        return frozenset(ALL_METADATA_FIELDS.union(frozenset(_data['user_metadata'])))
 
     def metadata_for_field(self, key):
         '''
@@ -312,7 +324,7 @@ class Metadata(object):
             v = self.get(attr, None)
             if v is not None:
                 result[attr] = v
-        for attr in _data['user_metadata'].iterkeys():
+        for attr in _data['user_metadata']:
             v = self.get(attr, None)
             if v is not None:
                 result[attr] = v
@@ -388,7 +400,7 @@ class Metadata(object):
             return
 
         um = {}
-        for key, meta in metadata.iteritems():
+        for key, meta in iteritems(metadata):
             m = meta.copy()
             if '#value#' not in m:
                 if m['datatype'] == 'text' and m['is_multiple']:
@@ -414,7 +426,7 @@ class Metadata(object):
             m = dict(metadata)
             # Copying the elements should not be necessary. The objects referenced
             # in the dict should not change. Of course, they can be replaced.
-            # for k,v in metadata.iteritems():
+            # for k,v in iteritems(metadata):
             #     m[k] = copy.copy(v)
             if '#value#' not in m:
                 if m['datatype'] == 'text' and m['is_multiple']:
@@ -424,6 +436,19 @@ class Metadata(object):
             _data = object.__getattribute__(self, '_data')
             _data['user_metadata'][field] = m
 
+    def remove_stale_user_metadata(self, other_mi):
+        '''
+        Remove user metadata keys (custom column keys) if they
+        don't exist in 'other_mi', which must be a metadata object
+        '''
+        me = self.get_all_user_metadata(make_copy=False)
+        other = set(other_mi.custom_field_keys())
+        new = {}
+        for k,v in me.items():
+            if k in other:
+                new[k] = v
+        self.set_all_user_metadata(new)
+
     def template_to_attribute(self, other, ops):
         '''
         Takes a list [(src,dest), (src,dest)], evaluates the template in the
@@ -522,11 +547,11 @@ class Metadata(object):
 
             if getattr(other, 'cover_data', False):
                 other_cover = other.cover_data[-1]
-                self_cover = self.cover_data[-1] if self.cover_data else ''
+                self_cover = self.cover_data[-1] if self.cover_data else b''
                 if not self_cover:
-                    self_cover = ''
+                    self_cover = b''
                 if not other_cover:
-                    other_cover = ''
+                    other_cover = b''
                 if len(other_cover) > len(self_cover):
                     self.cover_data = other.cover_data
 
@@ -535,6 +560,8 @@ class Metadata(object):
                     meta = other.get_user_metadata(x, make_copy=True)
                     if meta is not None:
                         self_tags = self.get(x, [])
+                        if isinstance(self_tags, string_or_bytes):
+                            self_tags = []
                         self.set_user_metadata(x, meta)  # get... did the deepcopy
                         other_tags = other.get(x, [])
                         if meta['datatype'] == 'text' and meta['is_multiple']:
@@ -568,7 +595,7 @@ class Metadata(object):
             if callable(getattr(other, 'get_identifiers', None)):
                 d = self.get_identifiers()
                 s = other.get_identifiers()
-                d.update([v for v in s.iteritems() if v[1] is not None])
+                d.update([v for v in iteritems(s) if v[1] is not None])
                 self.set_identifiers(d)
             else:
                 # other structure not Metadata. Copy the top-level identifiers
@@ -586,7 +613,7 @@ class Metadata(object):
         v = self.series_index if val is None else val
         try:
             x = float(v)
-        except (ValueError, TypeError):
+        except Exception:
             x = 1
         return fmt_sidx(x)
 
@@ -599,14 +626,14 @@ class Metadata(object):
         return authors_to_string(self.authors)
 
     def format_tags(self):
-        return u', '.join([unicode(t) for t in sorted(self.tags, key=sort_key)])
+        return ', '.join([unicode_type(t) for t in sorted(self.tags, key=sort_key)])
 
-    def format_rating(self, v=None, divide_by=1.0):
+    def format_rating(self, v=None, divide_by=1):
         if v is None:
             if self.rating is not None:
-                return unicode(self.rating/divide_by)
-            return u'None'
-        return unicode(v/divide_by)
+                return unicode_type(self.rating/divide_by)
+            return 'None'
+        return unicode_type(v/divide_by)
 
     def format_field(self, key, series_with_index=True):
         '''
@@ -630,15 +657,15 @@ class Metadata(object):
             if cmeta and cmeta['datatype'] == 'series':
                 if self.get(tkey):
                     res = self.get_extra(tkey)
-                    return (unicode(cmeta['name']+'_index'),
+                    return (unicode_type(cmeta['name']+'_index'),
                             self.format_series_index(res), res, cmeta)
                 else:
-                    return (unicode(cmeta['name']+'_index'), '', '', cmeta)
+                    return (unicode_type(cmeta['name']+'_index'), '', '', cmeta)
 
         if key in self.custom_field_keys():
             res = self.get(key, None)       # get evaluates all necessary composites
             cmeta = self.get_user_metadata(key, make_copy=False)
-            name = unicode(cmeta['name'])
+            name = unicode_type(cmeta['name'])
             if res is None or res == '':    # can't check "not res" because of numeric fields
                 return (name, res, None, None)
             orig_res = res
@@ -654,14 +681,14 @@ class Metadata(object):
             elif datatype == 'bool':
                 res = _('Yes') if res else _('No')
             elif datatype == 'rating':
-                res = u'%.2g'%(res/2.0)
+                res = '%.2g'%(res/2)
             elif datatype in ['int', 'float']:
                 try:
                     fmt = cmeta['display'].get('number_format', None)
                     res = fmt.format(res)
                 except:
                     pass
-            return (name, unicode(res), orig_res, cmeta)
+            return (name, unicode_type(res), orig_res, cmeta)
 
         # convert top-level ids into their value
         if key in TOP_LEVEL_IDENTIFIERS:
@@ -675,11 +702,11 @@ class Metadata(object):
         if fmkey in field_metadata and field_metadata[fmkey]['kind'] == 'field':
             res = self.get(key, None)
             fmeta = field_metadata[fmkey]
-            name = unicode(fmeta['name'])
+            name = unicode_type(fmeta['name'])
             if res is None or res == '':
                 return (name, res, None, None)
             orig_res = res
-            name = unicode(fmeta['name'])
+            name = unicode_type(fmeta['name'])
             datatype = fmeta['datatype']
             if key == 'authors':
                 res = authors_to_string(res)
@@ -694,14 +721,14 @@ class Metadata(object):
             elif datatype == 'datetime':
                 res = format_date(res, fmeta['display'].get('date_format','dd MMM yyyy'))
             elif datatype == 'rating':
-                res = u'%.2g'%(res/2.0)
+                res = '%.2g'%(res/2)
             elif key == 'size':
                 res = human_readable(res)
-            return (name, unicode(res), orig_res, fmeta)
+            return (name, unicode_type(res), orig_res, fmeta)
 
         return (None, None, None, None)
 
-    def __unicode__(self):
+    def __unicode__representation__(self):
         '''
         A string representation of this object, suitable for printing to
         console
@@ -711,7 +738,7 @@ class Metadata(object):
         ans = []
 
         def fmt(x, y):
-            ans.append(u'%-20s: %s'%(unicode(x), unicode(y)))
+            ans.append('%-20s: %s'%(unicode_type(x), unicode_type(y)))
 
         fmt('Title', self.title)
         if self.title_sort:
@@ -725,23 +752,23 @@ class Metadata(object):
         if getattr(self, 'book_producer', False):
             fmt('Book Producer', self.book_producer)
         if self.tags:
-            fmt('Tags', u', '.join([unicode(t) for t in self.tags]))
+            fmt('Tags', ', '.join([unicode_type(t) for t in self.tags]))
         if self.series:
             fmt('Series', self.series + ' #%s'%self.format_series_index())
         if not self.is_null('languages'):
             fmt('Languages', ', '.join(self.languages))
         if self.rating is not None:
-            fmt('Rating', (u'%.2g'%(float(self.rating)/2.0)) if self.rating
-                    else u'')
+            fmt('Rating', ('%.2g'%(float(self.rating)/2)) if self.rating
+                    else '')
         if self.timestamp is not None:
             fmt('Timestamp', isoformat(self.timestamp))
         if self.pubdate is not None:
             fmt('Published', isoformat(self.pubdate))
         if self.rights is not None:
-            fmt('Rights', unicode(self.rights))
+            fmt('Rights', unicode_type(self.rights))
         if self.identifiers:
-            fmt('Identifiers', u', '.join(['%s:%s'%(k, v) for k, v in
-                self.identifiers.iteritems()]))
+            fmt('Identifiers', ', '.join(['%s:%s'%(k, v) for k, v in
+                iteritems(self.identifiers)]))
         if self.comments:
             fmt('Comments', self.comments)
 
@@ -749,8 +776,8 @@ class Metadata(object):
             val = self.get(key, None)
             if val:
                 (name, val) = self.format_field(key)
-                fmt(name, unicode(val))
-        return u'\n'.join(ans)
+                fmt(name, unicode_type(val))
+        return '\n'.join(ans)
 
     def to_html(self):
         '''
@@ -758,36 +785,36 @@ class Metadata(object):
         '''
         from calibre.ebooks.metadata import authors_to_string
         from calibre.utils.date import isoformat
-        ans = [(_('Title'), unicode(self.title))]
+        ans = [(_('Title'), unicode_type(self.title))]
         ans += [(_('Author(s)'), (authors_to_string(self.authors) if self.authors else _('Unknown')))]
-        ans += [(_('Publisher'), unicode(self.publisher))]
-        ans += [(_('Producer'), unicode(self.book_producer))]
-        ans += [(_('Comments'), unicode(self.comments))]
-        ans += [('ISBN', unicode(self.isbn))]
-        ans += [(_('Tags'), u', '.join([unicode(t) for t in self.tags]))]
+        ans += [(_('Publisher'), unicode_type(self.publisher))]
+        ans += [(_('Producer'), unicode_type(self.book_producer))]
+        ans += [(_('Comments'), unicode_type(self.comments))]
+        ans += [('ISBN', unicode_type(self.isbn))]
+        ans += [(_('Tags'), ', '.join([unicode_type(t) for t in self.tags]))]
         if self.series:
-            ans += [(_('Series'), unicode(self.series) + ' #%s'%self.format_series_index())]
-        ans += [(_('Languages'), u', '.join(self.languages))]
+            ans += [(ngettext('Series', 'Series', 1), unicode_type(self.series) + ' #%s'%self.format_series_index())]
+        ans += [(_('Languages'), ', '.join(self.languages))]
         if self.timestamp is not None:
-            ans += [(_('Timestamp'), unicode(isoformat(self.timestamp, as_utc=False, sep=' ')))]
+            ans += [(_('Timestamp'), unicode_type(isoformat(self.timestamp, as_utc=False, sep=' ')))]
         if self.pubdate is not None:
-            ans += [(_('Published'), unicode(isoformat(self.pubdate, as_utc=False, sep=' ')))]
+            ans += [(_('Published'), unicode_type(isoformat(self.pubdate, as_utc=False, sep=' ')))]
         if self.rights is not None:
-            ans += [(_('Rights'), unicode(self.rights))]
+            ans += [(_('Rights'), unicode_type(self.rights))]
         for key in self.custom_field_keys():
             val = self.get(key, None)
             if val:
                 (name, val) = self.format_field(key)
                 ans += [(name, val)]
         for i, x in enumerate(ans):
-            ans[i] = u'<tr><td><b>%s</b></td><td>%s</td></tr>'%x
-        return u'<table>%s</table>'%u'\n'.join(ans)
+            ans[i] = '<tr><td><b>%s</b></td><td>%s</td></tr>'%x
+        return '<table>%s</table>'%'\n'.join(ans)
 
-    def __str__(self):
-        return self.__unicode__().encode('utf-8')
+    __str__ = __unicode__representation__
 
     def __nonzero__(self):
         return bool(self.title or self.author or self.comments or self.tags)
+    __bool__ = __nonzero__
 
     # }}}
 
@@ -802,8 +829,12 @@ def field_from_string(field, raw, field_metadata):
     elif dt == 'rating':
         val = float(raw) * 2
     elif dt == 'datetime':
-        from calibre.utils.date import parse_only_date
-        val = parse_only_date(raw)
+        from calibre.utils.iso8601 import parse_iso8601
+        try:
+            val = parse_iso8601(raw, require_aware=True)
+        except Exception:
+            from calibre.utils.date import parse_only_date
+            val = parse_only_date(raw)
     elif dt == 'bool':
         if raw.lower() in {'true', 'yes', 'y'}:
             val = True
@@ -824,6 +855,3 @@ def field_from_string(field, raw, field_metadata):
     if val is object:
         val = raw
     return val
-
-
-
