@@ -325,7 +325,7 @@ class MetadataUpdater(object):
             stop, = unpack('>I', self.data[offoff + 8:offoff + 12])
         return StreamSlicer(self.stream, start, stop)
 
-    def update(self, mi):
+    def update(self, mi, asin=None):
         mi.title = normalize(mi.title)
 
         def update_exth_record(rec):
@@ -395,10 +395,16 @@ class MetadataUpdater(object):
             update_exth_record((202, pack('>I', self.thumbnail_rindex)))
         # Add a 113 record if not present to allow Amazon syncing
         if (113 not in self.original_exth_records and
-                self.original_exth_records.get(501, None) == 'EBOK' and
+                self.original_exth_records.get(501, None) == b'EBOK' and
                 not added_501 and not share_not_sync):
             from uuid import uuid4
             update_exth_record((113, unicode_type(uuid4()).encode(self.codec)))
+
+        if asin is not None:
+            update_exth_record((113, asin.encode(self.codec)))
+            update_exth_record((501, b'EBOK'))
+            update_exth_record((504, asin.encode(self.codec)))
+
         # Add a 112 record with actual UUID
         if getattr(mi, 'uuid', None):
             update_exth_record((112,
