@@ -1,9 +1,7 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
 
 from lxml import etree
 
@@ -11,6 +9,7 @@ from calibre.ebooks.metadata.opf2 import OPF, pretty_print
 from calibre.ebooks.metadata.opf3 import apply_metadata, read_metadata
 from calibre.ebooks.metadata.utils import parse_opf, normalize_languages, create_manifest_item, parse_opf_version
 from calibre.ebooks.metadata import MetaInformation
+from polyglot.builtins import iteritems
 
 
 class DummyFile(object):
@@ -61,7 +60,7 @@ def set_metadata_opf2(root, cover_prefix, mi, opf_version,
     else:
         orig = opf.get_identifiers()
         orig.update(mi.get_identifiers())
-        opf.set_identifiers({k:v for k, v in orig.iteritems() if k and v})
+        opf.set_identifiers({k:v for k, v in iteritems(orig) if k and v})
     if update_timestamp and mi.timestamp is not None:
         opf.timestamp = mi.timestamp
     raster_cover = opf.raster_cover
