@@ -971,6 +971,7 @@ class OPF(object):  # {{{
 
     def get_identifiers(self):
         identifiers = {}
+        schemeless = []
         for x in self.XPath(
             'descendant::*[local-name() = "identifier" and text()]')(
                     self.metadata):
@@ -993,6 +994,15 @@ class OPF(object):  # {{{
                     val = check_isbn(val.split(':')[-1])
                     if val is not None:
                         identifiers['isbn'] = val
+                else:
+                    schemeless.append(val)
+
+        if schemeless and 'isbn' not in identifiers:
+            for val in schemeless:
+                if check_isbn(val, simple_sanitize=True) is not None:
+                    identifiers['isbn'] = check_isbn(val)
+                    break
+
         return identifiers
 
     def set_identifiers(self, identifiers):
