@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -9,19 +8,48 @@ __docformat__ = 'restructuredtext en'
 
 from collections import OrderedDict
 
+from calibre import random_user_agent
 from calibre.ebooks.metadata.sources.base import Source, Option
 
-USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64; rv:45.0) Gecko/20100101 Firefox/45.0'
+
+def parse_html(raw):
+    try:
+        from html5_parser import parse
+    except ImportError:
+        # Old versions of calibre
+        import html5lib
+        return html5lib.parse(raw, treebuilder='lxml', namespaceHTMLElements=False)
+    else:
+        return parse(raw)
+
+
+def imgurl_from_id(raw, tbnid):
+    from json import JSONDecoder
+    q = '"{}",['.format(tbnid)
+    start_pos = raw.index(q)
+    if start_pos < 100:
+        return
+    jd = JSONDecoder()
+    data = jd.raw_decode('[' + raw[start_pos:])[0]
+    # from pprint import pprint
+    # pprint(data)
+    url_num = 0
+    for x in data:
+        if isinstance(x, list) and len(x) == 3:
+            q = x[0]
+            if hasattr(q, 'lower') and q.lower().startswith('http'):
+                url_num += 1
+                if url_num > 1:
+                    return q
 
 
 class GoogleImages(Source):
 
     name = 'Google Images'
-    version = (1, 0, 0)
+    version = (1, 0, 2)
     minimum_calibre_version = (2, 80, 0)
     description = _('Downloads covers from a Google Image search. Useful to find larger/alternate covers.')
     capabilities = frozenset(['cover'])
-    config_help_message = _('Configure the Google Image Search plugin')
     can_get_multiple_covers = True
     supports_gzip_transfer_encoding = True
     options = (Option('max_covers', 'number', 5, _('Maximum number of covers to get'),
@@ -52,17 +80,20 @@ class GoogleImages(Source):
 
     @property
     def user_agent(self):
-        return USER_AGENT
+        return random_user_agent(allow_ie=False)
 
     def get_image_urls(self, title, author, log, abort, timeout):
         from calibre.utils.cleantext import clean_ascii_chars
-        from urllib import urlencode
-        import html5lib
-        import json
+        try:
+            from urllib.parse import urlencode
+        except ImportError:
+            from urllib import urlencode
         from collections import OrderedDict
         ans = OrderedDict()
         br = self.browser
-        q = urlencode({'as_q': ('%s %s'%(title, author)).encode('utf-8')}).decode('utf-8')
+        q = urlencode({'as_q': ('%s %s'%(title, author)).encode('utf-8')})
+        if isinstance(q, bytes):
+            q = q.decode('utf-8')
         sz = self.prefs['size']
         if sz == 'any':
             sz = ''
@@ -74,20 +105,26 @@ class GoogleImages(Source):
         # URL scheme
         url = 'https://www.google.com/search?as_st=y&tbm=isch&{}&as_epq=&as_oq=&as_eq=&cr=&as_sitesearch=&safe=images&tbs={}iar:t,ift:jpg'.format(q, sz)
         log('Search URL: ' + url)
-        raw = br.open(url).read().decode('utf-8')
-        root = html5lib.parse(clean_ascii_chars(raw), treebuilder='lxml', namespaceHTMLElements=False)
-        for div in root.xpath('//div[@class="rg_meta"]'):
+        raw = clean_ascii_chars(br.open(url).read().decode('utf-8'))
+        root = parse_html(raw)
+        results = root.xpath('//div/@data-tbnid')  # could also use data-id
+        # from calibre.utils.ipython import ipython
+        # ipython({'root': root, 'raw': raw, 'url': url, 'results': results})
+        for tbnid in results:
             try:
-                data = json.loads(div.text)
+                imgurl = imgurl_from_id(raw, tbnid)
             except Exception:
                 continue
-            if 'ou' in data:
-                ans[data['ou']] = True
-        return list(ans.iterkeys())
+            if imgurl:
+                ans[imgurl] = True
+        return list(ans)
 
 
 def test():
-    from Queue import Queue
+    try:
+        from queue import Queue
+    except ImportError:
+        from Queue import Queue
     from threading import Event
     from calibre.utils.logging import default_log
     p = GoogleImages(None)
@@ -95,7 +132,7 @@ def test():
     rq = Queue()
     p.download_cover(default_log, rq, Event(), title='The Heroes',
                      authors=('Joe Abercrombie',))
-    print ('Downloaded', rq.qsize(), 'covers')
+    print('Downloaded', rq.qsize(), 'covers')
 
 
 if __name__ == '__main__':
