@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,6 +8,7 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 from struct import unpack_from
 
 from calibre.ebooks.mobi.debug.headers import EXTHHeader
+from polyglot.builtins import filter, unicode_type
 
 
 class ContainerHeader(object):
@@ -43,7 +43,7 @@ class ContainerHeader(object):
 
     def add_hrefs(self, data):
         # kindlegen inserts a trailing | after the last href
-        self.hrefs = filter(None, data.decode('utf-8').split('|'))
+        self.hrefs = list(filter(None, data.decode('utf-8').split('|')))
 
     def __str__(self):
         ans = [('*'*10) + ' Container Header ' + ('*'*10)]
@@ -63,5 +63,4 @@ class ContainerHeader(object):
         a('Null bytes after EXTH: %d' % self.null_bytes_after_exth)
         if len(self.bytes_after_exth) != self.null_bytes_after_exth:
             a('Non-null bytes present after EXTH header!!!!')
-        return '\n'.join(ans) + '\n\n' + str(self.exth) + '\n\n' + ('Title: %s' % self.title)
-
+        return '\n'.join(ans) + '\n\n' + unicode_type(self.exth) + '\n\n' + ('Title: %s' % self.title)
