@@ -88,6 +88,8 @@ class Resources(object):
         for item in self.oeb.manifest.values():
             if item.media_type not in OEB_RASTER_IMAGES:
                 continue
+            if item.media_type.lower() == 'image/webp':
+                self.convert_webp(item)
             try:
                 data = self.process_image(item.data)
             except:
@@ -129,6 +131,14 @@ class Resources(object):
                     self.item_map[item.href] = len(self.records)
                     self.has_fonts = True
 
+    def convert_webp(self, item):
+        from calibre.utils.img import image_and_format_from_data, image_to_data
+        img, fmt = image_and_format_from_data(item.data)
+        if fmt == 'webp' and not img.isNull():
+            self.log.info(f'Converting WebP image {item.href} to PNG')
+            item.data = image_to_data(img, fmt='PNG')
+            item.media_type = 'image/png'
+
     def add_extra_images(self):
         '''
         Add any images that were created after the call to add_resources()
