@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -23,6 +22,7 @@ from operator import attrgetter
 
 from calibre.ebooks.mobi.utils import (encode_trailing_data,
         encode_tbs)
+from polyglot.builtins import iteritems, itervalues
 
 Entry = namedtuple('IndexEntry', 'index start length depth parent '
         'first_child last_child title action start_offset length_offset '
@@ -122,7 +122,7 @@ def encode_strands_as_sequences(strands, tbs_type=8):
     max_length_offset = 0
     first_entry = None
     for strand in strands:
-        for entries in strand.itervalues():
+        for entries in itervalues(strand):
             for entry in entries:
                 if first_entry is None:
                     first_entry = entry
@@ -131,7 +131,7 @@ def encode_strands_as_sequences(strands, tbs_type=8):
 
     for strand in strands:
         strand_seqs = []
-        for depth, entries in strand.iteritems():
+        for depth, entries in iteritems(strand):
             extra = {}
             if entries[-1].action == 'spans':
                 extra[0b1] = 0
@@ -207,9 +207,7 @@ def apply_trailing_byte_sequences(index_table, records, text_record_lengths):
     except NegativeStrandIndex:
         rmap = calculate_all_tbs(indexing_data, tbs_type=5)
 
-    for i, tbs_bytes in rmap.iteritems():
+    for i, tbs_bytes in iteritems(rmap):
         records[i] += encode_trailing_data(tbs_bytes)
 
     return True
-
-
