@@ -1,30 +1,30 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import re
 
-from lxml.etree import XMLParser, fromstring, XMLSyntaxError
-import cssutils
+from lxml.etree import XMLSyntaxError
 
 from calibre import force_unicode, human_readable, prepare_string_for_xml
 from calibre.ebooks.chardet import replace_encoding_declarations, find_declared_encoding
+from calibre.utils.xml_parse import safe_xml_fromstring
 from calibre.ebooks.html_entities import html5_entities
 from calibre.ebooks.oeb.polish.pretty import pretty_script_or_style as fix_style_tag
 from calibre.ebooks.oeb.polish.utils import PositionFinder, guess_type
 from calibre.ebooks.oeb.polish.check.base import BaseError, WARN, ERROR, INFO
 from calibre.ebooks.oeb.base import OEB_DOCS, XHTML_NS, urlquote, URL_SAFE, XHTML
+from polyglot.builtins import iteritems, unicode_type, error_message
 
 HTML_ENTITTIES = frozenset(html5_entities)
 XML_ENTITIES = {'lt', 'gt', 'amp', 'apos', 'quot'}
 ALL_ENTITIES = HTML_ENTITTIES | XML_ENTITIES
 
 replace_pat = re.compile('&(%s);' % '|'.join(re.escape(x) for x in sorted((HTML_ENTITTIES - XML_ENTITIES))))
-mismatch_pat = re.compile('tag mismatch:.+?line (\d+).+?line \d+')
+mismatch_pat = re.compile(r'tag mismatch:.+?line (\d+).+?line \d+')
 
 
 class EmptyFile(BaseError):
@@ -80,6 +80,13 @@ class HTMLParseError(XMLParseError):
              ' however, automatic fixing can sometimes "do the wrong thing".')
 
 
+class PrivateEntities(XMLParseError):
+
+    HELP = _('This HTML file uses private entities.'
+    ' These are not supported. You can try running "Fix HTML" from the Tools menu,'
+    ' which will try to automatically resolve the private entities.')
+
+
 class NamedEntities(BaseError):
 
     level = WARN
@@ -95,7 +102,7 @@ class NamedEntities(BaseError):
         changed = False
         from calibre.ebooks.oeb.polish.check.main import XML_TYPES
         check_types = XML_TYPES | OEB_DOCS
-        for name, mt in container.mime_map.iteritems():
+        for name, mt in iteritems(container.mime_map):
             if mt in check_types:
                 raw = container.raw_data(name)
                 nraw = replace_pat.sub(lambda m:html5_entities[m.group(1)], raw)
@@ -196,7 +203,7 @@ class NonUTF8(BaseError):
 
     def __call__(self, container):
         raw = container.raw_data(self.name)
-        if isinstance(raw, type('')):
+        if isinstance(raw, unicode_type):
             raw, changed = replace_encoding_declarations(raw)
             if changed:
                 container.open(self.name, 'wb').write(raw.encode('utf-8'))
@@ -255,14 +262,20 @@ def check_encoding_declarations(name, container):
     return errors
 
 
+def check_for_private_entities(name, raw):
+    if re.search(br'<!DOCTYPE\s+.+?<!ENTITY\s+.+?]>', raw, flags=re.DOTALL) is not None:
+        return True
+
+
 def check_xml_parsing(name, mt, raw):
     if not raw:
         return [EmptyFile(name)]
+    if check_for_private_entities(name, raw):
+        return [PrivateEntities(_('Private entities found'), name)]
     raw = raw.replace(b'\r\n', b'\n').replace(b'\r', b'\n')
     # Get rid of entities as named entities trip up the XML parser
     eproc = EntitityProcessor(mt)
     eraw = entity_pat.sub(eproc, raw)
-    parser = XMLParser(recover=False)
     errcls = HTMLParseError if mt in OEB_DOCS else XMLParseError
     errors = []
     if eproc.ok_named_entities:
@@ -274,7 +287,7 @@ def check_xml_parsing(name, mt, raw):
             errors.append(BadEntity(ent, name, lnum, col))
 
     try:
-        root = fromstring(eraw, parser=parser)
+        root = safe_xml_fromstring(eraw, recover=False)
     except UnicodeDecodeError:
         return errors + [DecodeError(name)]
     except XMLSyntaxError as err:
@@ -282,9 +295,9 @@ def check_xml_parsing(name, mt, raw):
             line, col = err.position
         except:
             line = col = None
-        return errors + [errcls(err.message, name, line, col)]
+        return errors + [errcls(error_message(err), name, line, col)]
     except Exception as err:
-        return errors + [errcls(err.message, name)]
+        return errors + [errcls(error_message(err), name)]
 
     if mt in OEB_DOCS:
         if root.nsmap.get(root.prefix, None) != XHTML_NS:
@@ -418,7 +431,7 @@ class BareTextInBody(BaseError):
 
 class ErrorHandler(object):
 
-    ' Replacement logger to get useful error/warning info out of cssutils during parsing '
+    ' Replacement logger to get useful error/warning info out of css_parser during parsing '
 
     def __init__(self, name):
         # may be disabled during setting of known valid items
@@ -430,7 +443,7 @@ class ErrorHandler(object):
     info = debug = setLevel = getEffectiveLevel = addHandler = removeHandler = __noop
 
     def __handle(self, level, *args):
-        msg = ' '.join(map(unicode, args))
+        msg = ' '.join(map(unicode_type, args))
         line = col = None
         for pat in pos_pats:
             m = pat.search(msg)
@@ -451,21 +464,6 @@ class ErrorHandler(object):
     warning = warn
 
 
-def check_css_parsing(name, raw, line_offset=0, is_declaration=False):
-    log = ErrorHandler(name)
-    parser = cssutils.CSSParser(fetcher=lambda x: (None, None), log=log)
-    if is_declaration:
-        parser.parseStyle(raw, validate=True)
-    else:
-        try:
-            parser.parseString(raw, validate=True)
-        except UnicodeDecodeError:
-            return [DecodeError(name)]
-    for err in log.errors:
-        err.line += line_offset
-    return log.errors
-
-
 def check_filenames(container):
     errors = []
     all_names = set(container.name_path_map) - container.names_that_must_not_be_changed
@@ -481,7 +479,7 @@ valid_id = re.compile(r'^[a-zA-Z][a-zA-Z0-9_:.-]*$')
 def check_ids(container):
     errors = []
     mts = set(OEB_DOCS) | {guess_type('a.opf'), guess_type('a.ncx')}
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in mts:
             root = container.parsed(name)
             seen_ids = {}
@@ -496,13 +494,13 @@ def check_ids(container):
                     seen_ids[eid] = elem.sourceline
                 if eid and valid_id.match(eid) is None:
                     errors.append(InvalidId(name, elem.sourceline, eid))
-            errors.extend(DuplicateId(name, eid, locs) for eid, locs in dups.iteritems())
+            errors.extend(DuplicateId(name, eid, locs) for eid, locs in iteritems(dups))
     return errors
 
 
 def check_markup(container):
     errors = []
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in OEB_DOCS:
             lines = []
             root = container.parsed(name)
