@@ -1,14 +1,15 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
+from calibre.ebooks.oeb.base import css_text
 from calibre.ebooks.oeb.polish.container import OEB_STYLES, OEB_DOCS
 from calibre.ebooks.oeb.normalize_css import normalize_font
 from tinycss.fonts3 import parse_font_family, parse_font, serialize_font_family, serialize_font
+from polyglot.builtins import iteritems, filter
 
 
 def unquote(x):
@@ -26,7 +27,7 @@ def font_family_data_from_declaration(style, families):
             font_families = [unquote(x) for x in f]
     f = style.getProperty('font-family')
     if f is not None:
-        font_families = parse_font_family(f.propertyValue.cssText)
+        font_families = parse_font_family(css_text(f.propertyValue))
 
     for f in font_families:
         families[f] = families.get(f, False)
@@ -39,13 +40,13 @@ def font_family_data_from_sheet(sheet, families):
         elif rule.type == rule.FONT_FACE_RULE:
             ff = rule.style.getProperty('font-family')
             if ff is not None:
-                for f in parse_font_family(ff.propertyValue.cssText):
+                for f in parse_font_family(css_text(ff.propertyValue)):
                     families[f] = True
 
 
 def font_family_data(container):
     families = {}
-    for name, mt in container.mime_map.iteritems():
+    for name, mt in iteritems(container.mime_map):
         if mt in OEB_STYLES:
             sheet = container.parsed(name)
             font_family_data_from_sheet(sheet, families)
@@ -66,8 +67,8 @@ def change_font_in_declaration(style, old_name, new_name=None):
     changed = False
     ff = style.getProperty('font-family')
     if ff is not None:
-        fams = parse_font_family(ff.propertyValue.cssText)
-        nfams = filter(None, [new_name if x == old_name else x for x in fams])
+        fams = parse_font_family(css_text(ff.propertyValue))
+        nfams = list(filter(None, [new_name if x == old_name else x for x in fams]))
         if fams != nfams:
             if nfams:
                 ff.propertyValue.cssText = serialize_font_family(nfams)
@@ -76,9 +77,9 @@ def change_font_in_declaration(style, old_name, new_name=None):
             changed = True
     ff = style.getProperty('font')
     if ff is not None:
-        props = parse_font(ff.propertyValue.cssText)
+        props = parse_font(css_text(ff.propertyValue))
         fams = props.get('font-family') or []
-        nfams = filter(None, [new_name if x == old_name else x for x in fams])
+        nfams = list(filter(None, [new_name if x == old_name else x for x in fams]))
         if fams != nfams:
             props['font-family'] = nfams
             if nfams:
@@ -90,7 +91,7 @@ def change_font_in_declaration(style, old_name, new_name=None):
 
 
 def remove_embedded_font(container, sheet, rule, sheet_name):
-    src = getattr(rule.style.getProperty('src'), 'value')
+    src = getattr(rule.style.getProperty('src'), 'value', None)
     if src is not None:
         if src.startswith('url('):
             src = src[4:-1]
@@ -111,7 +112,7 @@ def change_font_in_sheet(container, sheet, old_name, new_name, sheet_name):
         elif rule.type == rule.FONT_FACE_RULE:
             ff = rule.style.getProperty('font-family')
             if ff is not None:
-                families = {x for x in parse_font_family(ff.propertyValue.cssText)}
+                families = {x for x in parse_font_family(css_text(ff.propertyValue))}
                 if old_name in families:
                     changed = True
                     removals.append(rule)
@@ -128,7 +129,7 @@ def change_font(container, old_name, new_name=None):
     new_name to None to remove the font family instead of changing it.
     '''
     changed = False
-    for name, mt in tuple(container.mime_map.iteritems()):
+    for name, mt in tuple(iteritems(container.mime_map)):
         if mt in OEB_STYLES:
             sheet = container.parsed(name)
             if change_font_in_sheet(container, sheet, old_name, new_name, name):
@@ -147,7 +148,7 @@ def change_font(container, old_name, new_name=None):
                 if style:
                     style = container.parse_css(style, is_declaration=True)
                     if change_font_in_declaration(style, old_name, new_name):
-                        style = style.cssText.strip().rstrip(';').strip()
+                        style = css_text(style).strip().rstrip(';').strip()
                         if style:
                             elem.set('style', style)
                         else:
