@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,12 +9,13 @@ __docformat__ = 'restructuredtext en'
 import os, sys
 
 from calibre import prints, as_unicode
-from calibre.ebooks.oeb.base import OEB_STYLES, OEB_DOCS, XPath
+from calibre.ebooks.oeb.base import OEB_STYLES, OEB_DOCS, XPath, css_text
 from calibre.ebooks.oeb.polish.container import OEB_FONTS
 from calibre.ebooks.oeb.polish.utils import guess_type
 from calibre.utils.fonts.sfnt.subset import subset
 from calibre.utils.fonts.sfnt.errors import UnsupportedFont
 from calibre.utils.fonts.utils import get_font_names
+from polyglot.builtins import iteritems, itervalues
 
 
 def remove_font_face_rules(container, sheet, remove_names, base):
@@ -34,59 +34,64 @@ def remove_font_face_rules(container, sheet, remove_names, base):
     return changed
 
 
+def iter_subsettable_fonts(container):
+    for name, mt in iteritems(container.mime_map):
+        if (mt in OEB_FONTS or name.rpartition('.')[-1].lower() in {'otf', 'ttf'}) and mt != guess_type('a.woff'):
+            yield name, mt
+
+
 def subset_all_fonts(container, font_stats, report):
     remove = set()
     total_old = total_new = 0
     changed = False
-    for name, mt in container.mime_map.iteritems():
-        if (mt in OEB_FONTS or name.rpartition('.')[-1].lower() in {'otf', 'ttf'}) and mt != guess_type('a.woff'):
-            chars = font_stats.get(name, set())
-            with container.open(name, 'rb') as f:
-                f.seek(0, os.SEEK_END)
-                total_old += f.tell()
-            if not chars:
-                remove.add(name)
-                report(_('Removed unused font: %s')%name)
+    for name, mt in iter_subsettable_fonts(container):
+        chars = font_stats.get(name, set())
+        with container.open(name, 'rb') as f:
+            f.seek(0, os.SEEK_END)
+            total_old += f.tell()
+        if not chars:
+            remove.add(name)
+            report(_('Removed unused font: %s')%name)
+            continue
+        with container.open(name, 'r+b') as f:
+            raw = f.read()
+            try:
+                font_name = get_font_names(raw)[-1]
+            except Exception as e:
+                container.log.warning(
+                    'Corrupted font: %s, ignoring.  Error: %s'%(
+                        name, as_unicode(e)))
+                continue
+            warnings = []
+            container.log('Subsetting font: %s'%(font_name or name))
+            try:
+                nraw, old_sizes, new_sizes = subset(raw, chars,
+                                                warnings=warnings)
+            except UnsupportedFont as e:
+                container.log.warning(
+                    'Unsupported font: %s, ignoring.  Error: %s'%(
+                        name, as_unicode(e)))
                 continue
-            with container.open(name, 'r+b') as f:
-                raw = f.read()
-                try:
-                    font_name = get_font_names(raw)[-1]
-                except Exception as e:
-                    container.log.warning(
-                        'Corrupted font: %s, ignoring.  Error: %s'%(
-                            name, as_unicode(e)))
-                    continue
-                warnings = []
-                container.log('Subsetting font: %s'%(font_name or name))
-                try:
-                    nraw, old_sizes, new_sizes = subset(raw, chars,
-                                                   warnings=warnings)
-                except UnsupportedFont as e:
-                    container.log.warning(
-                        'Unsupported font: %s, ignoring.  Error: %s'%(
-                            name, as_unicode(e)))
-                    continue
 
-                for w in warnings:
-                    container.log.warn(w)
-                olen = sum(old_sizes.itervalues())
-                nlen = sum(new_sizes.itervalues())
-                total_new += len(nraw)
-                if nlen == olen:
-                    report(_('The font %s was already subset')%font_name)
-                else:
-                    report(_('Decreased the font {0} to {1} of its original size').format(
-                        font_name, ('%.1f%%' % (nlen/olen * 100))))
-                    changed = True
-                f.seek(0), f.truncate(), f.write(nraw)
+            for w in warnings:
+                container.log.warn(w)
+            olen = sum(itervalues(old_sizes))
+            nlen = sum(itervalues(new_sizes))
+            total_new += len(nraw)
+            if nlen == olen:
+                report(_('The font %s was already subset')%font_name)
+            else:
+                report(_('Decreased the font {0} to {1} of its original size').format(
+                    font_name, ('%.1f%%' % (nlen/olen * 100))))
+                changed = True
+            f.seek(0), f.truncate(), f.write(nraw)
 
     for name in remove:
         container.remove_item(name)
         changed = True
 
     if remove:
-        for name, mt in container.mime_map.iteritems():
+        for name, mt in iteritems(container.mime_map):
             if mt in OEB_STYLES:
                 sheet = container.parsed(name)
                 if remove_font_face_rules(container, sheet, remove, name):
@@ -96,7 +101,7 @@ def subset_all_fonts(container, font_stats, report):
                     if style.get('type', 'text/css') == 'text/css' and style.text:
                         sheet = container.parse_css(style.text, name)
                         if remove_font_face_rules(container, sheet, remove, name):
-                            style.text = sheet.cssText
+                            style.text = css_text(sheet)
                             container.dirty(name)
     if total_old > 0:
         report(_('Reduced total font size to %.1f%% of original')%(
