@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -16,6 +15,7 @@ from calibre.ebooks.oeb.polish.replace import rename_files, rationalize_folders
 from calibre.ebooks.oeb.polish.split import split, merge
 from calibre.utils.filenames import nlinks_file
 from calibre.ptempfile import TemporaryFile, TemporaryDirectory
+from polyglot.builtins import iteritems, itervalues, unicode_type
 
 
 def get_container(*args, **kwargs):
@@ -38,7 +38,7 @@ class ContainerTests(BaseTest):
             c2 = clone_container(c1, tdir)
 
             for c in (c1, c2):
-                for name, path in c.name_path_map.iteritems():
+                for name, path in iteritems(c.name_path_map):
                     self.assertEqual(2, nlinks_file(path), 'The file %s is not linked' % name)
 
             for name in c1.name_path_map:
@@ -98,7 +98,7 @@ class ContainerTests(BaseTest):
 
         def new_container():
             count[0] += 1
-            tdir = os.mkdir(os.path.join(self.tdir, str(count[0])))
+            tdir = os.mkdir(os.path.join(self.tdir, unicode_type(count[0])))
             return get_container(book, tdir=tdir)
 
         # Test simple opf rename
@@ -178,13 +178,13 @@ class ContainerTests(BaseTest):
         name = 'folder/added file.html'
         c.add_file(name, b'xxx')
         self.assertEqual('xxx', c.raw_data(name))
-        self.assertIn(name, set(c.manifest_id_map.itervalues()))
+        self.assertIn(name, set(itervalues(c.manifest_id_map)))
         self.assertIn(name, {x[0] for x in c.spine_names})
 
         name = 'added.css'
         c.add_file(name, b'xxx')
         self.assertEqual('xxx', c.raw_data(name))
-        self.assertIn(name, set(c.manifest_id_map.itervalues()))
+        self.assertIn(name, set(itervalues(c.manifest_id_map)))
         self.assertNotIn(name, {x[0] for x in c.spine_names})
         self.assertEqual(c.make_name_unique(name), 'added-1.css')
         c.add_file('added-1.css', b'xxx')
