@@ -163,7 +163,7 @@ class CoverManager(object):
         if item is not None:
             self.oeb.spine.insert(0, item, True)
             if 'cover' not in self.oeb.guide.refs:
-                self.oeb.guide.add('cover', 'Title Page', 'a')
+                self.oeb.guide.add('cover', 'Title page', 'a')
             self.oeb.guide.refs['cover'].href = item.href
             if 'titlepage' in self.oeb.guide.refs:
                 self.oeb.guide.refs['titlepage'].href = item.href
