@@ -1,18 +1,19 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import re
 from calibre.ebooks.oeb.base import XPath, urlunquote
+from polyglot.builtins import as_bytes
 
 
 class DataURL(object):
 
     def __call__(self, oeb, opts):
+        from calibre.utils.imghdr import what
         self.log = oeb.log
         attr_path = XPath('//h:img[@src]')
         for item in oeb.spine:
@@ -28,15 +29,15 @@ class DataURL(object):
                     continue
                 if ';base64' in header:
                     data = re.sub(r'\s+', '', data)
-                    from base64 import b64decode
+                    from polyglot.binary import from_base64_bytes
                     try:
-                        data = b64decode(data)
+                        data = from_base64_bytes(data)
                     except Exception:
                         self.log.error('Found invalid base64 encoded data URI, ignoring it')
                         continue
                 else:
                     data = urlunquote(data)
-                from imghdr import what
+                data = as_bytes(data)
                 fmt = what(None, data)
                 if not fmt:
                     self.log.warn('Image encoded as data URL has unknown format, ignoring')
