@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,7 +8,7 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 import logging
 from collections import defaultdict
 
-import cssutils
+import css_parser
 from lxml import etree
 
 from calibre import guess_type
@@ -18,10 +17,11 @@ from calibre.ebooks.oeb.transforms.subset import get_font_properties, find_font_
 from calibre.utils.filenames import ascii_filename
 from calibre.utils.fonts.scanner import font_scanner, NoFonts
 from calibre.ebooks.oeb.polish.embed import font_key
+from polyglot.builtins import iteritems, unicode_type
 
 
 def font_families_from_style(style):
-    return [unicode(f) for f in style.get('font-family', []) if unicode(f).lower() not in {
+    return [unicode_type(f) for f in style.get('font-family', []) if unicode_type(f).lower() not in {
         'serif', 'sansserif', 'sans-serif', 'fantasy', 'cursive', 'monospace'}]
 
 
@@ -39,7 +39,7 @@ def used_font(style, embedded_fonts):
     ff = font_families_from_style(style)
     if not ff:
         return False, None
-    lnames = {unicode(x).lower() for x in ff}
+    lnames = {unicode_type(x).lower() for x in ff}
 
     matching_set = []
 
@@ -97,7 +97,7 @@ class EmbedFonts(object):
         self.sheet_cache = {}
         self.find_style_rules()
         self.find_embedded_fonts()
-        self.parser = cssutils.CSSParser(loglevel=logging.CRITICAL, log=logging.getLogger('calibre.css'))
+        self.parser = css_parser.CSSParser(loglevel=logging.CRITICAL, log=logging.getLogger('calibre.css'))
         self.warned = set()
         self.warned2 = set()
         self.newly_embedded_fonts = set()
@@ -137,7 +137,7 @@ class EmbedFonts(object):
                 if rule.type != rule.STYLE_RULE:
                     continue
                 props = {k:v for k,v in
-                        get_font_properties(rule).iteritems() if v}
+                        iteritems(get_font_properties(rule)) if v}
                 if not props:
                     continue
                 for sel in rule.selectorList:
@@ -156,7 +156,7 @@ class EmbedFonts(object):
             manifest = self.oeb.manifest
             id_, href = manifest.generate('page_css', 'page_styles.css')
             self.page_sheet = manifest.add(id_, href, CSS_MIME, data=self.parser.parseString('', validate=False))
-            head = self.current_item.xpath('//*[local-name()="head"][1]')
+            head = self.current_item.data.xpath('//*[local-name()="head"][1]')
             if head:
                 href = self.current_item.relhref(href)
                 l = etree.SubElement(head[0], XHTML('link'),
