@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,7 +9,7 @@ __docformat__ = 'restructuredtext en'
 from math import sqrt
 from collections import namedtuple
 
-from PyQt5.Qt import (
+from qt.core import (
     QBrush, QPen, Qt, QPointF, QTransform, QPaintEngine, QImage)
 
 from calibre.ebooks.pdf.render.common import (
@@ -44,6 +43,7 @@ def convert_path(path):  # {{{
     return p
 # }}}
 
+
 Brush = namedtuple('Brush', 'origin brush color')
 
 
@@ -235,8 +235,8 @@ class TexturePattern(TilingPattern):
             image = pixmap.toImage()
             cache_key = pixmap.cacheKey()
             imgref = pdf.add_image(image, cache_key)
-            paint_type = (2 if image.format() in {QImage.Format_MonoLSB,
-                                                QImage.Format_Mono} else 1)
+            paint_type = (2 if image.format() in {QImage.Format.Format_MonoLSB,
+                                                QImage.Format.Format_Mono} else 1)
             super(TexturePattern, self).__init__(
                 cache_key, matrix, w=image.width(), h=image.height(),
                 paint_type=paint_type)
@@ -257,7 +257,7 @@ class GraphicsState(object):
                   'clip_updated', 'do_fill', 'do_stroke')
 
     def __init__(self):
-        self.fill = QBrush(Qt.white)
+        self.fill = QBrush(Qt.GlobalColor.white)
         self.stroke = QPen()
         self.opacity = 1.0
         self.transform = QTransform()
@@ -303,22 +303,22 @@ class Graphics(object):
 
         s = self.pending_state
 
-        if flags & QPaintEngine.DirtyTransform:
+        if flags & QPaintEngine.DirtyFlag.DirtyTransform:
             s.transform = state.transform()
 
-        if flags & QPaintEngine.DirtyBrushOrigin:
+        if flags & QPaintEngine.DirtyFlag.DirtyBrushOrigin:
             s.brush_origin = state.brushOrigin()
 
-        if flags & QPaintEngine.DirtyBrush:
+        if flags & QPaintEngine.DirtyFlag.DirtyBrush:
             s.fill = state.brush()
 
-        if flags & QPaintEngine.DirtyPen:
+        if flags & QPaintEngine.DirtyFlag.DirtyPen:
             s.stroke = state.pen()
 
-        if flags & QPaintEngine.DirtyOpacity:
+        if flags & QPaintEngine.DirtyFlag.DirtyOpacity:
             s.opacity = state.opacity()
 
-        if flags & QPaintEngine.DirtyClipPath or flags & QPaintEngine.DirtyClipRegion:
+        if flags & QPaintEngine.DirtyFlag.DirtyClipPath or flags & QPaintEngine.DirtyFlag.DirtyClipRegion:
             s.clip_updated = True
 
     def reset(self):
@@ -354,8 +354,8 @@ class Graphics(object):
             path = painter.clipPath()
             if not path.isEmpty():
                 p = convert_path(path)
-                fill_rule = {Qt.OddEvenFill:'evenodd',
-                            Qt.WindingFill:'winding'}[path.fillRule()]
+                fill_rule = {Qt.FillRule.OddEvenFill:'evenodd',
+                            Qt.FillRule.WindingFill:'winding'}[path.fillRule()]
                 pdf.add_clip(p, fill_rule=fill_rule)
 
         self.current_state = self.pending_state
@@ -375,26 +375,26 @@ class Graphics(object):
         vals = list(brush.color().getRgbF())
         self.brushobj = None
 
-        if style <= Qt.DiagCrossPattern:
+        if style <= Qt.BrushStyle.DiagCrossPattern:
             opacity *= vals[-1]
             color = vals[:3]
 
-            if style > Qt.SolidPattern:
+            if style > Qt.BrushStyle.SolidPattern:
                 pat = QtPattern(style, matrix)
 
-        elif style == Qt.TexturePattern:
+        elif style == Qt.BrushStyle.TexturePattern:
             pat = TexturePattern(brush.texture(), matrix, pdf)
             if pat.paint_type == 2:
                 opacity *= vals[-1]
                 color = vals[:3]
 
-        elif style == Qt.LinearGradientPattern:
+        elif style == Qt.BrushStyle.LinearGradientPattern:
             pat = LinearGradientPattern(brush, matrix, pdf, self.page_width_px,
                                         self.page_height_px)
             opacity *= pat.const_opacity
         # TODO: Add support for radial/conical gradient fills
 
-        if opacity < 1e-4 or style == Qt.NoBrush:
+        if opacity < 1e-4 or style == Qt.BrushStyle.NoBrush:
             do_fill = False
         self.brushobj = Brush(brush_origin, pat, color)
 
@@ -420,19 +420,22 @@ class Graphics(object):
         pdf.current_page.write(' w ')
 
         # Line cap
-        cap = {Qt.FlatCap:0, Qt.RoundCap:1, Qt.SquareCap:
+        cap = {Qt.PenCapStyle.FlatCap:0, Qt.PenCapStyle.RoundCap:1, Qt.PenCapStyle.SquareCap:
                2}.get(pen.capStyle(), 0)
         pdf.current_page.write('%d J '%cap)
 
         # Line join
-        join = {Qt.MiterJoin:0, Qt.RoundJoin:1,
-                Qt.BevelJoin:2}.get(pen.joinStyle(), 0)
+        join = {Qt.PenJoinStyle.MiterJoin:0, Qt.PenJoinStyle.RoundJoin:1,
+                Qt.PenJoinStyle.BevelJoin:2}.get(pen.joinStyle(), 0)
         pdf.current_page.write('%d j '%join)
 
         # Dash pattern
-        ps = {Qt.DashLine:[3], Qt.DotLine:[1,2], Qt.DashDotLine:[3,2,1,2],
-              Qt.DashDotDotLine:[3, 2, 1, 2, 1, 2]}.get(pen.style(), [])
-        if ps:
+        if pen.style() == Qt.PenStyle.CustomDashLine:
+            pdf.serialize(Array(pen.dashPattern()))
+            pdf.current_page.write(' %d d ' % pen.dashOffset())
+        else:
+            ps = {Qt.PenStyle.DashLine:[3], Qt.PenStyle.DotLine:[1,2], Qt.PenStyle.DashDotLine:[3,2,1,2],
+                  Qt.PenStyle.DashDotDotLine:[3, 2, 1, 2, 1, 2]}.get(pen.style(), [])
             pdf.serialize(Array(ps))
             pdf.current_page.write(' 0 d ')
 
@@ -441,7 +444,7 @@ class Graphics(object):
             pen.brush(), state.brush_origin, state.opacity, pdf_system,
             painter.transform())
         self.pdf.apply_stroke(color, pattern, opacity)
-        if pen.style() == Qt.NoPen:
+        if pen.style() == Qt.PenStyle.NoPen:
             self.pending_state.do_stroke = False
 
     def apply_fill(self, state, pdf_system, painter):
@@ -482,5 +485,3 @@ class Graphics(object):
             pat = TexturePattern(None, matrix, self.pdf, clone=self.last_fill.brush)
             pattern = self.pdf.add_pattern(pat)
             self.pdf.apply_fill(self.last_fill.color, pattern)
-
-
