@@ -1,16 +1,15 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import hashlib
-from future_builtins import map
+import hashlib, numbers
+from polyglot.builtins import map, iteritems
 
-from PyQt5.Qt import QBuffer, QByteArray, QImage, Qt, QColor, qRgba, QPainter
+from qt.core import QBuffer, QByteArray, QImage, Qt, QColor, qRgba, QPainter
 
 from calibre.constants import (__appname__, __version__)
 from calibre.ebooks.pdf.render.common import (
@@ -19,6 +18,7 @@ from calibre.ebooks.pdf.render.common import (
 from calibre.ebooks.pdf.render.fonts import FontManager
 from calibre.ebooks.pdf.render.links import Links
 from calibre.utils.date import utcnow
+from polyglot.builtins import as_unicode
 
 PDFVER = b'%PDF-1.4'  # 1.4 is needed for XMP metadata
 
@@ -56,7 +56,7 @@ class IndirectObjects(object):
 
     def __getitem__(self, o):
         try:
-            return self._map[id(self._list[o] if isinstance(o, int) else o)]
+            return self._map[id(self._list[o] if isinstance(o, numbers.Integral) else o)]
         except (KeyError, IndexError):
             raise KeyError('The object %r was not found'%o)
 
@@ -119,24 +119,24 @@ class Page(Stream):
         r = Dictionary()
         if self.opacities:
             extgs = Dictionary()
-            for opref, name in self.opacities.iteritems():
+            for opref, name in iteritems(self.opacities):
                 extgs[name] = opref
             r['ExtGState'] = extgs
         if self.fonts:
             fonts = Dictionary()
-            for ref, name in self.fonts.iteritems():
+            for ref, name in iteritems(self.fonts):
                 fonts[name] = ref
             r['Font'] = fonts
         if self.xobjects:
             xobjects = Dictionary()
-            for ref, name in self.xobjects.iteritems():
+            for ref, name in iteritems(self.xobjects):
                 xobjects[name] = ref
             r['XObject'] = xobjects
         if self.patterns:
             r['ColorSpace'] = Dictionary({'PCSp':Array(
                 [Name('Pattern'), Name('DeviceRGB')])})
             patterns = Dictionary()
-            for ref, name in self.patterns.iteritems():
+            for ref, name in iteritems(self.patterns):
                 patterns[name] = ref
             r['Pattern'] = patterns
         if r:
@@ -278,7 +278,7 @@ class PDFStream(object):
         self.stream = HashingStream(stream)
         self.compress = compress
         self.write_line(PDFVER)
-        self.write_line(b'%íì¦"')
+        self.write_line(u'%íì¦"'.encode('utf-8'))
         creator = ('%s %s [https://calibre-ebook.com]'%(__appname__,
                                     __version__))
         self.write_line('%% Created by %s'%creator)
@@ -296,8 +296,9 @@ class PDFStream(object):
         self.image_cache = {}
         self.pattern_cache, self.shader_cache = {}, {}
         self.debug = debug
+        self.page_size = page_size
         self.links = Links(self, mark_links, page_size)
-        i = QImage(1, 1, QImage.Format_ARGB32)
+        i = QImage(1, 1, QImage.Format.Format_ARGB32)
         i.fill(qRgba(0, 0, 0, 255))
         self.alpha_bit = i.constBits().asstring(4).find(b'\xff')
 
@@ -355,7 +356,7 @@ class PDFStream(object):
                 self.current_page.write_line()
             for x in op:
                 self.current_page.write(
-                (fmtnum(x) if isinstance(x, (int, long, float)) else x) + ' ')
+                (fmtnum(x) if isinstance(x, numbers.Number) else x) + ' ')
 
     def draw_path(self, path, stroke=True, fill=False, fill_rule='winding'):
         if not path.ops:
@@ -415,6 +416,9 @@ class PDFStream(object):
         self.objects.commit(r, self.stream)
         return r
 
+    def add_jpeg_image(self, img_data, w, h, cache_key=None):
+        return self.write_image(img_data, w, h, 32, dct=True)
+
     def add_image(self, img, cache_key):
         ref = self.get_image(cache_key)
         if ref is not None:
@@ -423,21 +427,21 @@ class PDFStream(object):
         fmt = img.format()
         image = QImage(img)
         if (image.depth() == 1 and img.colorTable().size() == 2 and
-            img.colorTable().at(0) == QColor(Qt.black).rgba() and
-            img.colorTable().at(1) == QColor(Qt.white).rgba()):
-            if fmt == QImage.Format_MonoLSB:
-                image = image.convertToFormat(QImage.Format_Mono)
-            fmt = QImage.Format_Mono
+            img.colorTable().at(0) == QColor(Qt.GlobalColor.black).rgba() and
+            img.colorTable().at(1) == QColor(Qt.GlobalColor.white).rgba()):
+            if fmt == QImage.Format.Format_MonoLSB:
+                image = image.convertToFormat(QImage.Format.Format_Mono)
+            fmt = QImage.Format.Format_Mono
         else:
-            if (fmt != QImage.Format_RGB32 and fmt != QImage.Format_ARGB32):
-                image = image.convertToFormat(QImage.Format_ARGB32)
-                fmt = QImage.Format_ARGB32
+            if (fmt != QImage.Format.Format_RGB32 and fmt != QImage.Format.Format_ARGB32):
+                image = image.convertToFormat(QImage.Format.Format_ARGB32)
+                fmt = QImage.Format.Format_ARGB32
 
         w = image.width()
         h = image.height()
         d = image.depth()
 
-        if fmt == QImage.Format_Mono:
+        if fmt == QImage.Format.Format_Mono:
             bytes_per_line = (w + 7) >> 3
             data = image.constBits().asstring(bytes_per_line * h)
             return self.write_image(data, w, h, d, cache_key=cache_key)
@@ -445,7 +449,7 @@ class PDFStream(object):
         has_alpha = False
         soft_mask = None
 
-        if fmt == QImage.Format_ARGB32:
+        if fmt == QImage.Format.Format_ARGB32:
             tmask = image.constBits().asstring(4*w*h)[self.alpha_bit::4]
             sdata = bytearray(tmask)
             vals = set(sdata)
@@ -454,8 +458,8 @@ class PDFStream(object):
             if has_alpha:
                 # Blend image onto a white background as otherwise Qt will render
                 # transparent pixels as black
-                background = QImage(image.size(), QImage.Format_ARGB32_Premultiplied)
-                background.fill(Qt.white)
+                background = QImage(image.size(), QImage.Format.Format_ARGB32_Premultiplied)
+                background.fill(Qt.GlobalColor.white)
                 painter = QPainter(background)
                 painter.drawImage(0, 0, image)
                 painter.end()
@@ -464,7 +468,7 @@ class PDFStream(object):
         ba = QByteArray()
         buf = QBuffer(ba)
         image.save(buf, 'jpeg', 94)
-        data = bytes(ba.data())
+        data = ba.data()
 
         if has_alpha:
             soft_mask = self.write_image(tmask, w, h, 8)
@@ -483,9 +487,11 @@ class PDFStream(object):
         return self.shader_cache[shader.cache_key]
 
     def draw_image(self, x, y, width, height, imgref):
+        self.draw_image_with_transform(imgref, scaling=(width, -height), translation=(x, y + height))
+
+    def draw_image_with_transform(self, imgref, translation=(0, 0), scaling=(1, 1)):
         name = self.current_page.add_image(imgref)
-        self.current_page.write('q %s 0 0 %s %s %s cm '%(fmtnum(width),
-                            fmtnum(-height), fmtnum(x), fmtnum(y+height)))
+        self.current_page.write('q {} 0 0 {} {} {} cm '.format(*(tuple(scaling) + tuple(translation))))
         serialize(Name(name), self.current_page)
         self.current_page.write_line(' Do Q')
 
@@ -520,7 +526,7 @@ class PDFStream(object):
         self.objects.pdf_serialize(self.stream)
         self.write_line()
         startxref = self.objects.write_xref(self.stream)
-        file_id = String(self.stream.hashobj.hexdigest().decode('ascii'))
+        file_id = String(as_unicode(self.stream.hashobj.hexdigest()))
         self.write_line('trailer')
         trailer = Dictionary({'Root':self.catalog, 'Size':len(self.objects)+1,
                               'ID':Array([file_id, file_id]), 'Info':inforef})
@@ -528,5 +534,3 @@ class PDFStream(object):
         self.write_line('startxref')
         self.write_line('%d'%startxref)
         self.stream.write('%%EOF')
-
-
