@@ -1,12 +1,12 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 import re, sys
 from collections import defaultdict
 
-from lxml.etree import tostring
+from polyglot.builtins import reraise, unicode_type
+
 from lxml.html import (fragment_fromstring, document_fromstring,
         tostring as htostring)
 
@@ -15,7 +15,7 @@ from calibre.ebooks.readability.cleaners import html_cleaner, clean_attributes
 
 
 def tounicode(tree_or_node, **kwargs):
-    kwargs['encoding'] = unicode
+    kwargs['encoding'] = unicode_type
     return htostring(tree_or_node, **kwargs)
 
 
@@ -62,7 +62,7 @@ def to_int(x):
 
 
 def clean(text):
-    text = re.sub('\s*\n\s*', '\n', text)
+    text = re.sub('\\s*\n\\s*', '\n', text)
     text = re.sub('[ \t]{2,}', ' ', text)
     return text.strip()
 
@@ -154,9 +154,9 @@ class Document:
                     continue  # try again
                 else:
                     return cleaned_article
-        except StandardError, e:
+        except Exception as e:
             self.log.exception('error getting summary: ')
-            raise Unparseable(str(e)), None, sys.exc_info()[2]
+            reraise(Unparseable, Unparseable(unicode_type(e)), sys.exc_info()[2])
 
     def get_article(self, candidates, best_candidate):
         # Now that we have the top candidate, look through its siblings for content that might also be related.
@@ -183,7 +183,7 @@ class Document:
 
                 if node_length > 80 and link_density < 0.25:
                     append = True
-                elif node_length < 80 and link_density == 0 and re.search('\.( |$)', node_content):
+                elif node_length < 80 and link_density == 0 and re.search(r'\.( |$)', node_content):
                     append = True
 
             if append:
@@ -216,7 +216,7 @@ class Document:
     def score_paragraphs(self, ):
         MIN_LEN = self.options.get('min_text_length', self.TEXT_LENGTH_THRESHOLD)
         candidates = {}
-        # self.debug(str([describe(node) for node in self.tags(self.html, "div")]))
+        # self.debug(unicode_type([describe(node) for node in self.tags(self.html, "div")]))
 
         ordered = []
         for elem in self.tags(self.html, "p", "pre", "td"):
@@ -313,10 +313,10 @@ class Document:
     def transform_misused_divs_into_paragraphs(self):
         for elem in self.tags(self.html, 'div'):
             # transform <div>s that do not contain other block elements into <p>s
-            if not REGEXES['divToPElementsRe'].search(unicode(''.join(map(tostring, list(elem))))):
+            if not REGEXES['divToPElementsRe'].search(unicode_type(''.join(map(tounicode, list(elem))))):
                 # self.debug("Altering %s to p" % (describe(elem)))
                 elem.tag = "p"
-                # print "Fixed element "+describe(elem)
+                # print("Fixed element "+describe(elem))
 
         for elem in self.tags(self.html, 'div'):
             if elem.text and elem.text.strip():
@@ -324,7 +324,7 @@ class Document:
                 p.text = elem.text
                 elem.text = None
                 elem.insert(0, p)
-                # print "Appended "+tounicode(p)+" to "+describe(elem)
+                # print("Appended "+tounicode(p)+" to "+describe(elem))
 
             for pos, child in reversed(list(enumerate(elem))):
                 if child.tail and child.tail.strip():
@@ -332,9 +332,9 @@ class Document:
                     p.text = child.tail
                     child.tail = None
                     elem.insert(pos + 1, p)
-                    # print "Inserted "+tounicode(p)+" to "+describe(elem)
+                    # print("Inserted "+tounicode(p)+" to "+describe(elem))
                 if child.tag == 'br':
-                    # print 'Dropped <br> at '+describe(elem)
+                    # print('Dropped <br> at '+describe(elem))
                     child.drop_tree()
 
     def tags(self, node, *tag_names):
@@ -363,7 +363,7 @@ class Document:
             weight = self.class_weight(el)
             if el in candidates:
                 content_score = candidates[el]['content_score']
-                # print '!',el, '-> %6.3f' % content_score
+                # print('!',el, '-> %6.3f' % content_score)
             else:
                 content_score = 0
             tag = el.tag
@@ -445,7 +445,7 @@ class Document:
                         # self.debug(sib.text_content())
                         sib_content_length = text_length(sib)
                         if sib_content_length:
-                            i =+ 1
+                            i += 1
                             siblings.append(sib_content_length)
                             if i == x:
                                 break
@@ -457,7 +457,7 @@ class Document:
                             siblings.append(sib_content_length)
                             if j == x:
                                 break
-                    # self.debug(str(siblings))
+                    # self.debug(unicode_type(siblings))
                     if siblings and sum(siblings) > 1000 :
                         to_remove = False
                         self.debug("Allowing %s" % describe(el))
@@ -467,7 +467,7 @@ class Document:
                 if to_remove:
                     self.debug("Cleaned %6.3f %s with weight %s cause it has %s." %
                         (content_score, describe(el), weight, reason))
-                    # print tounicode(el)
+                    # print(tounicode(el))
                     # self.debug("pname %s pweight %.3f" %(pname, pweight))
                     el.drop_tree()
 
@@ -502,10 +502,11 @@ def main():
     enc = sys.__stdout__.encoding or 'utf-8'
     if options.verbose:
         default_log.filter_level = default_log.DEBUG
-    print (Document(raw, default_log,
+    print(Document(raw, default_log,
             debug=options.verbose,
             keep_elements=options.keep_elements).summary().encode(enc,
                 'replace'))
 
+
 if __name__ == '__main__':
     main()
