@@ -1,6 +1,10 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
+
+
 import sys
 
+from polyglot.builtins import unicode_type
+
 
 class CheckEncoding:
 
@@ -13,17 +17,17 @@ class CheckEncoding:
             char_position +=1
             try:
                 char.decode(encoding)
-            except UnicodeError, msg:
-                sys.stderr.write('line: %s char: %s\n%s\n' %  (line_num, char_position, str(msg)))
+            except ValueError as msg:
+                sys.stderr.write('line: %s char: %s\n%s\n' %  (line_num, char_position, unicode_type(msg)))
 
     def check_encoding(self, path, encoding='us-ascii', verbose=True):
         line_num = 0
-        with open(path, 'r') as read_obj:
+        with open(path, 'rb') as read_obj:
             for line in read_obj:
                 line_num += 1
                 try:
                     line.decode(encoding)
-                except UnicodeError:
+                except ValueError:
                     if verbose:
                         if len(line) < 1000:
                             self.__get_position_error(line, encoding, line_num)
@@ -32,6 +36,7 @@ class CheckEncoding:
                     return True
         return False
 
+
 if __name__ == '__main__':
     check_encoding_obj = CheckEncoding()
     check_encoding_obj.check_encoding(sys.argv[1])
