@@ -1,17 +1,16 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import sys, os, shlex, subprocess, shutil, unicodedata
+import sys, os, unicodedata
 
 from calibre import prints, as_unicode, walk
 from calibre.constants import iswindows, __appname__
-from calibre.ptempfile import TemporaryDirectory, TemporaryFile
+from calibre.ptempfile import TemporaryDirectory
 from calibre.libunzip import extract as zipextract
 from calibre.utils.zipfile import ZipFile, ZIP_DEFLATED, ZIP_STORED
 from calibre.utils.ipc.simple_worker import WorkerError
@@ -102,6 +101,61 @@ def get_tools(fmt):
     return ans
 
 
+def explode(ebook_file, output_dir):
+    if not os.path.exists(output_dir):
+        os.makedirs(output_dir)
+    if not os.path.isdir(output_dir):
+        raise SystemExit('%s is not a directory' % output_dir)
+    output_dir = os.path.abspath(output_dir)
+    fmt = ebook_file.rpartition('.')[-1].lower()
+    exploder, rebuilder = get_tools(fmt)
+    if exploder is None:
+        raise SystemExit('Cannot tweak %s files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI, DOCX' % fmt.upper())
+    try:
+        opf = exploder(ebook_file, output_dir, question=ask_cli_question)
+    except WorkerError as e:
+        prints('Failed to unpack', ebook_file)
+        prints(e.orig_tb)
+        raise SystemExit(1)
+    except Error as e:
+        prints(as_unicode(e), file=sys.stderr)
+        raise SystemExit(1)
+
+    if opf is None:
+        # The question was answered with No
+        return
+    h = '_' if iswindows else '.'
+    with lopen(os.path.join(output_dir, h + '__explode_fmt__'), 'wb') as f:
+        f.write(fmt.encode('utf-8'))
+    prints('Book extracted to', output_dir)
+    prints('Make your changes and once you are done, use --implode-book to rebuild')
+
+
+def implode(output_dir, ebook_file):
+    output_dir = os.path.abspath(output_dir)
+    fmt = ebook_file.rpartition('.')[-1].lower()
+    exploder, rebuilder = get_tools(fmt)
+    if rebuilder is None:
+        raise SystemExit('Cannot tweak %s files. Supported formats are: EPUB, HTMLZ, AZW3, MOBI, DOCX' % fmt.upper())
+    h = '_' if iswindows else '.'
+    efmt_path = os.path.join(output_dir, h + '__explode_fmt__')
+    try:
+        with lopen(efmt_path, 'rb') as f:
+            efmt = f.read().decode('utf-8')
+    except Exception:
+        raise SystemExit('The folder %s does not seem to have been created by --explode-book' % output_dir)
+    if efmt != fmt:
+        raise SystemExit('You must use the same format of file as was used when exploding the book')
+    os.remove(efmt_path)
+    try:
+        rebuilder(output_dir, ebook_file)
+    except WorkerError as e:
+        prints('Failed to rebuild', ebook_file)
+        prints(e.orig_tb)
+        raise SystemExit(1)
+    prints(ebook_file, 'successfully rebuilt')
+
+
 def tweak(ebook_file):
     ''' Command line interface to the Tweak Book tool '''
     fmt = ebook_file.rpartition('.')[-1].lower()
@@ -127,33 +181,11 @@ def tweak(ebook_file):
             # The question was answered with No
             return
 
-        ed = os.environ.get('EDITOR', 'dummy')
-        cmd = shlex.split(ed)
-        isvim = bool([x for x in cmd[0].split('/') if x.endswith('vim')])
-
-        proceed = False
         prints('Book extracted to', tdir)
-
-        if not isvim:
-            prints('Make your tweaks and once you are done,', __appname__,
-                    'will rebuild', ebook_file, 'from', tdir)
-            print()
-            proceed = ask_cli_question('Rebuild ' + ebook_file + '?')
-        else:
-            base = os.path.basename(ebook_file)
-            with TemporaryFile(base+'.zip') as zipf:
-                with ZipFile(zipf, 'w') as zf:
-                    zf.add_dir(tdir)
-                try:
-                    subprocess.check_call(cmd + [zipf])
-                except:
-                    prints(ed, 'failed, aborting...')
-                    raise SystemExit(1)
-                with ZipFile(zipf, 'r') as zf:
-                    shutil.rmtree(tdir)
-                    os.mkdir(tdir)
-                    zf.extractall(path=tdir)
-            proceed = True
+        prints('Make your tweaks and once you are done,', __appname__,
+                'will rebuild', ebook_file, 'from', tdir)
+        print()
+        proceed = ask_cli_question('Rebuild ' + ebook_file + '?')
 
         if proceed:
             prints('Rebuilding', ebook_file, 'please wait ...')
@@ -164,4 +196,3 @@ def tweak(ebook_file):
                 prints(e.orig_tb)
                 raise SystemExit(1)
             prints(ebook_file, 'successfully tweaked')
-
