@@ -124,7 +124,8 @@ def create_defs():
             'Edit Metadata', 'Send To Device', 'Save To Disk',
             'Connect Share', 'Copy To Library', None,
             'Convert Books', 'View', 'Open Folder', 'Show Book Details',
-            'Similar Books', 'Tweak ePub', None, 'Remove Books',
+            'Similar Books', 'Tweak ePub', None, 'Remove Books', None,
+            'Autoscroll Books'
             )
 
     defs['show_splash_screen'] = True
@@ -174,6 +175,7 @@ def create_defs():
     defs['show_highlight_toggle_button'] = False
     defs['add_comments_to_email'] = False
     defs['cb_preserve_aspect_ratio'] = False
+    defs['cb_double_click_to_activate'] = False
     defs['gpm_template_editor_font_size'] = 10
     defs['show_emblems'] = False
     defs['emblem_size'] = 32
@@ -198,6 +200,7 @@ def create_defs():
     defs['browse_annots_restrict_to_type'] = None
     defs['browse_annots_use_stemmer'] = True
     defs['annots_export_format'] = 'txt'
+    defs['books_autoscroll_time'] = 2.0
 
 
 create_defs()
@@ -233,7 +236,7 @@ def _config():  # {{{
     c.add_opt('LRF_ebook_viewer_options', default=None,
               help=_('Options for the LRF e-book viewer'))
     c.add_opt('internally_viewed_formats', default=['LRF', 'EPUB', 'LIT',
-        'MOBI', 'PRC', 'POBI', 'AZW', 'AZW3', 'HTML', 'FB2', 'PDB', 'RB',
+        'MOBI', 'PRC', 'POBI', 'AZW', 'AZW3', 'HTML', 'FB2', 'FBZ', 'PDB', 'RB',
         'SNB', 'HTMLZ', 'KEPUB'], help=_(
             'Formats that are viewed using the internal viewer'))
     c.add_opt('column_map', default=ALL_COLUMNS,
@@ -307,7 +310,7 @@ def _config():  # {{{
     # This option is no longer used. It remains for compatibility with upgrades
     # so the value can be migrated
     c.add_opt('tag_browser_hidden_categories', default=set(),
-            help=_('tag browser categories not to display'))
+            help=_('Tag browser categories not to display'))
 
     c.add_opt
     return ConfigProxy(c)
@@ -398,6 +401,10 @@ def error_dialog(parent, title, msg, det_msg='', show=False,
     return d
 
 
+class Aborted(Exception):
+    pass
+
+
 def question_dialog(parent, title, msg, det_msg='', show_copy_button=False,
     default_yes=True,
     # Skippable dialogs
@@ -410,13 +417,17 @@ def question_dialog(parent, title, msg, det_msg='', show_copy_button=False,
     # Change the text/icons of the yes and no buttons.
     # The icons must be QIcon objects or strings for I()
     yes_text=None, no_text=None, yes_icon=None, no_icon=None,
+    # Add an Abort button which if clicked will cause this function to raise
+    # the Aborted exception
+    add_abort_button=False,
 ):
     from calibre.gui2.dialogs.message_box import MessageBox
+    prefs = gui_prefs()
 
     if not isinstance(skip_dialog_name, unicode_type):
         skip_dialog_name = None
     try:
-        auto_skip = set(gprefs.get('questions_to_auto_skip', ()))
+        auto_skip = set(prefs.get('questions_to_auto_skip', ()))
     except Exception:
         auto_skip = set()
     if (skip_dialog_name is not None and skip_dialog_name in auto_skip):
@@ -425,7 +436,7 @@ def question_dialog(parent, title, msg, det_msg='', show_copy_button=False,
     d = MessageBox(MessageBox.QUESTION, title, msg, det_msg, parent=parent,
                    show_copy_button=show_copy_button, default_yes=default_yes,
                    q_icon=override_icon, yes_text=yes_text, no_text=no_text,
-                   yes_icon=yes_icon, no_icon=no_icon)
+                   yes_icon=yes_icon, no_icon=no_icon, add_abort_button=add_abort_button)
 
     if skip_dialog_name is not None and skip_dialog_msg:
         tc = d.toggle_checkbox
@@ -435,19 +446,21 @@ def question_dialog(parent, title, msg, det_msg='', show_copy_button=False,
         d.resize_needed.emit()
 
     ret = d.exec_() == QDialog.DialogCode.Accepted
+    if add_abort_button and d.aborted:
+        raise Aborted()
 
     if skip_dialog_name is not None and not d.toggle_checkbox.isChecked():
         auto_skip.add(skip_dialog_name)
-        gprefs.set('questions_to_auto_skip', list(auto_skip))
+        prefs.set('questions_to_auto_skip', list(auto_skip))
 
     return ret
 
 
 def info_dialog(parent, title, msg, det_msg='', show=False,
-        show_copy_button=True):
+        show_copy_button=True, only_copy_details=False):
     from calibre.gui2.dialogs.message_box import MessageBox
     d = MessageBox(MessageBox.INFO, title, msg, det_msg, parent=parent,
-                    show_copy_button=show_copy_button)
+                    show_copy_button=show_copy_button, only_copy_details=only_copy_details)
 
     if show:
         return d.exec_()
@@ -993,6 +1006,11 @@ class Application(QApplication):
 
     def ensure_window_on_screen(self, widget):
         screen_rect = self.desktop().availableGeometry(widget)
+        g = widget.geometry()
+        w = min(screen_rect.width(), g.width())
+        h = min(screen_rect.height(), g.height())
+        if w != g.width() or h != g.height():
+            widget.resize(w, h)
         if not widget.geometry().intersects(screen_rect):
             w = min(widget.width(), screen_rect.width() - 10)
             h = min(widget.height(), screen_rect.height() - 10)
@@ -1204,7 +1222,7 @@ class Application(QApplication):
 
     def signal_received(self):
         try:
-            os.read(self.signal_notifier.socket(), 1024)
+            os.read(int(self.signal_notifier.socket()), 1024)
         except OSError:
             return
         self.shutdown_signal_received.emit()
