@@ -39,8 +39,8 @@ class InterfaceAction(QObject):
     priority takes precedence.
 
     Sub-classes should implement the :meth:`genesis`, :meth:`library_changed`,
-    :meth:`location_selected` :meth:`shutting_down`
-    and :meth:`initialization_complete` methods.
+    :meth:`location_selected`, :meth:`shutting_down`,
+    :meth:`initialization_complete` and :meth:`tag_browser_context_action` methods.
 
     Once initialized, this plugin has access to the main calibre GUI via the
     :attr:`gui` member. You can access other plugins by name, for example::
@@ -349,6 +349,17 @@ class InterfaceAction(QObject):
         '''
         pass
 
+    def tag_browser_context_action(self, index):
+        '''
+        Called when displaying the context menu in the Tag browser. ``index`` is
+        the QModelIndex that points to the Tag browser item that was right clicked.
+        Test it for validity with index.valid() and get the underlying TagTreeItem
+        object with index.data(Qt.ItemDataRole.UserRole). Any action objects
+        yielded by this method will be added to the context menu.
+        '''
+        if False:
+            yield QAction()
+
     def shutting_down(self):
         '''
         Called once per plugin when the main GUI is in the process of shutting
