@@ -1,6 +1,7 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -8,7 +9,7 @@ __docformat__ = 'restructuredtext en'
 import os
 from functools import partial
 
-from PyQt5.Qt import QModelIndex, QTimer
+from qt.core import QModelIndex, QTimer
 
 from calibre.gui2 import error_dialog, Dispatcher, gprefs
 from calibre.gui2.tools import convert_single_ebook, convert_bulk_ebook
@@ -21,7 +22,7 @@ class ConvertAction(InterfaceAction):
 
     name = 'Convert Books'
     action_spec = (_('Convert books'), 'convert.png', _('Convert books between different e-book formats'), _('C'))
-    dont_add_to = frozenset(['context-menu-device'])
+    dont_add_to = frozenset(('context-menu-device',))
     action_type = 'current'
     action_add_menu = True
 
@@ -40,7 +41,7 @@ class ConvertAction(InterfaceAction):
     def drop_event(self, event, mime_data):
         mime = 'application/calibre+from_library'
         if mime_data.hasFormat(mime):
-            self.dropped_ids = tuple(map(int, str(mime_data.data(mime)).split()))
+            self.dropped_ids = tuple(map(int, mime_data.data(mime).data().split()))
             QTimer.singleShot(1, self.do_drop)
             return True
         return False
@@ -69,6 +70,9 @@ class ConvertAction(InterfaceAction):
     def location_selected(self, loc):
         enabled = loc == 'library'
         self.qaction.setEnabled(enabled)
+        self.menuless_qaction.setEnabled(enabled)
+        for action in list(self.convert_menu.actions()):
+            action.setEnabled(enabled)
 
     def auto_convert(self, book_ids, on_card, format):
         previous = self.gui.library_view.currentIndex()
