@@ -1,11 +1,12 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-from PyQt5.Qt import QUrl
+from qt.core import QUrl
 
 from calibre.gui2 import open_url
 from calibre.gui2.actions import InterfaceAction
@@ -22,6 +23,3 @@ class HelpAction(InterfaceAction):
 
     def show_help(self, *args):
         open_url(QUrl(localize_user_manual_link('https://manual.calibre-ebook.com')))
-
-
-
