@@ -1,13 +1,12 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Grant Drake <grant.drake@gmail.com>'
 __docformat__ = 'restructuredtext en'
 
-from PyQt5.Qt import QApplication, Qt, QIcon
+from qt.core import QApplication, Qt, QIcon
 from calibre.gui2.actions import InterfaceAction
 from calibre.gui2.dialogs.plugin_updater import (PluginUpdaterDialog,
                                                  FILTER_ALL, FILTER_UPDATE_AVAILABLE)
@@ -27,7 +26,7 @@ class PluginUpdaterAction(InterfaceAction):
         # Get the user to choose a plugin to install
         initial_filter = FILTER_UPDATE_AVAILABLE
         mods = QApplication.keyboardModifiers()
-        if mods & Qt.ControlModifier or mods & Qt.ShiftModifier:
+        if mods & Qt.KeyboardModifier.ControlModifier or mods & Qt.KeyboardModifier.ShiftModifier:
             initial_filter = FILTER_ALL
 
         d = PluginUpdaterDialog(self.gui, initial_filter=initial_filter)
