@@ -1,11 +1,12 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-from PyQt5.Qt import Qt
+from qt.core import Qt, sip
 
 from calibre.gui2.actions import InterfaceAction
 from calibre.gui2.dialogs.book_info import BookInfo
@@ -17,7 +18,7 @@ class ShowBookDetailsAction(InterfaceAction):
     name = 'Show Book Details'
     action_spec = (_('Show Book details'), 'dialog_information.png',
                    _('Show the detailed metadata for the current book in a separate window'), _('I'))
-    dont_add_to = frozenset(['context-menu-device'])
+    dont_add_to = frozenset(('context-menu-device',))
     action_type = 'current'
 
     def genesis(self):
@@ -34,8 +35,9 @@ class ShowBookDetailsAction(InterfaceAction):
         if index.isValid():
             d = BookInfo(self.gui, self.gui.library_view, index,
                     self.gui.book_details.handle_click)
+            d.open_cover_with.connect(self.gui.bd_open_cover_with, type=Qt.ConnectionType.QueuedConnection)
             self.memory.append(d)
-            d.closed.connect(self.closed, type=Qt.QueuedConnection)
+            d.closed.connect(self.closed, type=Qt.ConnectionType.QueuedConnection)
             d.show()
 
     def closed(self, d):
@@ -45,6 +47,5 @@ class ShowBookDetailsAction(InterfaceAction):
         except ValueError:
             pass
         else:
-            import sip
             sip.delete(d)
             del d
