@@ -107,7 +107,7 @@ class Rules(RulesBase):
     RuleItemClass = RuleItem
     RuleEditDialogClass = RuleEditDialog
     MSG = _('You can specify rules to add/ignore files here. They will be used'
-            ' when recursively adding files from directories/archives and also'
+            ' when recursively adding files from folders/archives and also'
             ' when auto-adding. Click the "Add Rule" button'
             ' below to get started. The rules will be processed in order for every file until either an'
             ' "add" or an "ignore" rule matches. If no rules match, the file will be added only'
