@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -11,6 +11,8 @@ import os
 from calibre.gui2.convert.debug_ui import Ui_Form
 from calibre.gui2.convert import Widget
 from calibre.gui2 import error_dialog, choose_dir
+from calibre.ebooks.conversion.config import OPTIONS
+from polyglot.builtins import unicode_type
 
 
 class DebugWidget(Widget, Ui_Form):
@@ -21,16 +23,10 @@ class DebugWidget(Widget, Ui_Form):
     COMMIT_NAME = 'debug'
 
     def __init__(self, parent, get_option, get_help, db=None, book_id=None):
-        Widget.__init__(self, parent,
-                ['debug_pipeline']
-                )
+        Widget.__init__(self, parent, OPTIONS['pipe']['debug'])
         self.db, self.book_id = db, book_id
         self.initialize_options(get_option, get_help, db, book_id)
         self.button_debug_dir.clicked.connect(self.set_debug_dir)
-        self.button_clear.clicked.connect(self.clear_debug_dir)
-
-    def clear_debug_dir(self):
-        self.opt_debug_pipeline.setText('')
 
     def set_debug_dir(self):
         x = choose_dir(self, 'conversion debug dir', _('Choose debug folder'))
@@ -39,7 +35,7 @@ class DebugWidget(Widget, Ui_Form):
 
     def pre_commit_check(self):
         try:
-            x = unicode(self.opt_debug_pipeline.text()).strip()
+            x = unicode_type(self.opt_debug_pipeline.text()).strip()
             if not x:
                 return True
             x = os.path.abspath(x)
@@ -52,10 +48,8 @@ class DebugWidget(Widget, Ui_Form):
         except:
             import traceback
             det_msg = traceback.format_exc()
-            error_dialog(self, _('Invalid debug directory'),
-                    _('Failed to create debug directory')+': '+
-                        unicode(self.opt_debug_pipeline.text()),
+            error_dialog(self, _('Invalid debug folder'),
+                    _('Failed to create debug folder')+': '+ unicode_type(self.opt_debug_pipeline.text()),
                         det_msg=det_msg, show=True)
             return False
         return True
-
