@@ -20,7 +20,7 @@ class LookAndFeelWidget(Widget, Ui_Form):
 
     TITLE = _('Look & feel')
     ICON  = I('lookfeel.png')
-    HELP  = _('Control the look and feel of the output')
+    HELP  = _('Control the look and feel of the output.')
     COMMIT_NAME = 'look_and_feel'
 
     FILTER_CSS = {
