@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -8,7 +8,7 @@ __docformat__ = 'restructuredtext en'
 
 import os, re, errno
 
-from PyQt5.Qt import QPixmap
+from qt.core import QPixmap
 
 from calibre.gui2 import choose_images, error_dialog
 from calibre.gui2.convert.metadata_ui import Ui_Form
@@ -20,6 +20,8 @@ from calibre.gui2.convert import Widget
 from calibre.utils.icu import sort_key
 from calibre.library.comments import comments_to_html
 from calibre.utils.config import tweaks
+from calibre.ebooks.conversion.config import OPTIONS
+from polyglot.builtins import unicode_type
 
 
 def create_opf_file(db, book_id, opf_file=None):
@@ -55,7 +57,7 @@ class MetadataWidget(Widget, Ui_Form):
     COMMIT_NAME = 'metadata'
 
     def __init__(self, parent, get_option, get_help, db=None, book_id=None):
-        Widget.__init__(self, parent, ['prefer_metadata_cover'])
+        Widget.__init__(self, parent, OPTIONS['pipe']['metadata'])
         self.db, self.book_id = db, book_id
         self.cover_changed = False
         self.cover_data = None
@@ -66,29 +68,35 @@ class MetadataWidget(Widget, Ui_Form):
         self.comment.hide_toolbars()
         self.cover.cover_changed.connect(self.change_cover)
         self.series.currentTextChanged.connect(self.series_changed)
+        self.cover.draw_border = False
 
     def change_cover(self, data):
         self.cover_changed = True
         self.cover_data = data
 
     def deduce_author_sort(self, *args):
-        au = unicode(self.author.currentText())
+        au = unicode_type(self.author.currentText())
         au = re.sub(r'\s+et al\.$', '', au)
         authors = string_to_authors(au)
         self.author_sort.setText(self.db.author_sort_from_authors(authors))
+        self.author_sort.home(False)
 
     def initialize_metadata_options(self):
         self.initialize_combos()
         self.author.editTextChanged.connect(self.deduce_author_sort)
 
         mi = self.db.get_metadata(self.book_id, index_is_id=True)
-        self.title.setText(mi.title)
+        self.title.setText(mi.title), self.title.home(False)
         self.publisher.show_initial_value(mi.publisher if mi.publisher else '')
+        self.publisher.home(False)
         self.author_sort.setText(mi.author_sort if mi.author_sort else '')
+        self.author_sort.home(False)
         self.tags.setText(', '.join(mi.tags if mi.tags else []))
-        self.tags.update_items_cache(self.db.all_tags())
+        self.tags.update_items_cache(self.db.new_api.all_field_names('tags'))
+        self.tags.home(False)
         self.comment.html = comments_to_html(mi.comments) if mi.comments else ''
         self.series.show_initial_value(mi.series if mi.series else '')
+        self.series.home(False)
         if mi.series_index is not None:
             try:
                 self.series_index.setValue(mi.series_index)
@@ -133,51 +141,48 @@ class MetadataWidget(Widget, Ui_Form):
         self.author.set_separator('&')
         self.author.set_space_before_sep(True)
         self.author.set_add_separator(tweaks['authors_completer_append_separator'])
-        self.author.update_items_cache(self.db.all_author_names())
+        self.author.update_items_cache(self.db.new_api.all_field_names('authors'))
 
         au = self.db.authors(self.book_id, True)
         if not au:
             au = _('Unknown')
         au = ' & '.join([a.strip().replace('|', ',') for a in au.split(',')])
         self.author.show_initial_value(au)
+        self.author.home(False)
 
     def initialize_series(self):
-        all_series = self.db.all_series()
-        all_series.sort(key=lambda x : sort_key(x[1]))
         self.series.set_separator(None)
-        self.series.update_items_cache([x[1] for x in all_series])
+        self.series.update_items_cache(self.db.new_api.all_field_names('series'))
 
     def initialize_publisher(self):
-        all_publishers = self.db.all_publishers()
-        all_publishers.sort(key=lambda x : sort_key(x[1]))
         self.publisher.set_separator(None)
-        self.publisher.update_items_cache([x[1] for x in all_publishers])
+        self.publisher.update_items_cache(self.db.new_api.all_field_names('publisher'))
 
     def get_title_and_authors(self):
-        title = unicode(self.title.text()).strip()
+        title = unicode_type(self.title.text()).strip()
         if not title:
             title = _('Unknown')
-        authors = unicode(self.author.text()).strip()
+        authors = unicode_type(self.author.text()).strip()
         authors = string_to_authors(authors) if authors else [_('Unknown')]
         return title, authors
 
     def get_metadata(self):
         title, authors = self.get_title_and_authors()
         mi = MetaInformation(title, authors)
-        publisher = unicode(self.publisher.text()).strip()
+        publisher = unicode_type(self.publisher.text()).strip()
         if publisher:
             mi.publisher = publisher
-        author_sort = unicode(self.author_sort.text()).strip()
+        author_sort = unicode_type(self.author_sort.text()).strip()
         if author_sort:
             mi.author_sort = author_sort
         comments = self.comment.html
         if comments:
             mi.comments = comments
         mi.series_index = float(self.series_index.value())
-        series = unicode(self.series.currentText()).strip()
+        series = unicode_type(self.series.currentText()).strip()
         if series:
             mi.series = series
-        tags = [t.strip() for t in unicode(self.tags.text()).strip().split(',')]
+        tags = [t.strip() for t in unicode_type(self.tags.text()).strip().split(',')]
         if tags:
             mi.tags = tags
 
@@ -185,7 +190,7 @@ class MetadataWidget(Widget, Ui_Form):
 
     def select_cover(self):
         files = choose_images(self, 'change cover dialog',
-                             _('Choose cover for ') + unicode(self.title.text()))
+                             _('Choose cover for ') + unicode_type(self.title.text()))
         if not files:
             return
         _file = files[0]
@@ -196,13 +201,13 @@ class MetadataWidget(Widget, Ui_Form):
                         _('You do not have permission to read the file: ') + _file)
                 d.exec_()
                 return
-            cf, cover = None, None
+            cover = None
             try:
-                cf = open(_file, "rb")
-                cover = cf.read()
+                with open(_file, "rb") as f:
+                    cover = f.read()
             except IOError as e:
                 d = error_dialog(self.parent(), _('Error reading file'),
-                        _("<p>There was an error reading from file: <br /><b>") + _file + "</b></p><br />"+str(e))
+                        _("<p>There was an error reading from file: <br /><b>") + _file + "</b></p><br />"+unicode_type(e))
                 d.exec_()
             if cover:
                 pix = QPixmap()
