@@ -15,7 +15,7 @@ from qt.core import (Qt, QComboBox, QLabel, QSpinBox, QDoubleSpinBox,
         QMessageBox, QToolButton, QPlainTextEdit, QApplication, QStyle, QDialog)
 
 from calibre.utils.date import qt_to_dt, now, as_local_time, as_utc, internal_iso_format_string
-from calibre.gui2.complete2 import EditWithComplete
+from calibre.gui2.complete2 import EditWithComplete as EWC
 from calibre.gui2.comments_editor import Editor as CommentsEditor
 from calibre.gui2 import UNDEFINED_QDATETIME, error_dialog, elided_text
 from calibre.gui2.dialogs.tag_editor import TagEditor
@@ -27,6 +27,13 @@ from calibre.gui2.widgets2 import RatingEditor, DateTimeEdit as DateTimeEditBase
 from polyglot.builtins import unicode_type
 
 
+class EditWithComplete(EWC):
+
+    def __init__(self, *a, **kw):
+        super().__init__(*a, **kw)
+        self.set_clear_button_enabled(False)
+
+
 def safe_disconnect(signal):
     try:
         signal.disconnect()
@@ -56,6 +63,7 @@ class Base(object):
 
     def __init__(self, db, col_id, parent=None):
         self.db, self.col_id = db, col_id
+        self.book_id = None
         self.col_metadata = db.custom_column_num_map[col_id]
         self.initial_val = self.widgets = None
         self.signals_to_disconnect = []
@@ -86,6 +94,7 @@ class Base(object):
         l.addWidget(self.clear_button)
 
     def initialize(self, book_id):
+        self.book_id = book_id
         val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
         val = self.normalize_db_val(val)
         self.setter(val)
@@ -437,7 +446,7 @@ class MultipleWidget(QWidget):
         self.tags_box = EditWithComplete(parent)
         layout.addWidget(self.tags_box, stretch=1000)
         self.editor_button = QToolButton(self)
-        self.editor_button.setToolTip(_('Open item editor'))
+        self.editor_button.setToolTip(_('Open Item editor. If CTRL or SHIFT is pressed, open Manage items'))
         self.editor_button.setIcon(QIcon(I('chapters.png')))
         layout.addWidget(self.editor_button)
         self.setLayout(layout)
@@ -547,6 +556,8 @@ class Text(Base):
         return val
 
     def edit(self):
+        ctrl_or_shift_pressed = (QApplication.keyboardModifiers() &
+                (Qt.KeyboardModifier.ControlModifier + Qt.KeyboardModifier.ShiftModifier))
         if (self.getter() != self.initial_val and (self.getter() or self.initial_val)):
             d = _save_dialog(self.parent, _('Values changed'),
                     _('You have changed the values. In order to use this '
@@ -560,9 +571,14 @@ class Text(Base):
                 self.initial_val = self.current_val
             else:
                 self.setter(self.initial_val)
-        d = TagEditor(self.parent, self.db, self.book_id, self.key)
-        if d.exec_() == QDialog.DialogCode.Accepted:
-            self.setter(d.tags)
+        if ctrl_or_shift_pressed:
+            from calibre.gui2.ui import get_gui
+            get_gui().do_tags_list_edit(None, self.key)
+            self.initialize(self.book_id)
+        else:
+            d = TagEditor(self.parent, self.db, self.book_id, self.key)
+            if d.exec_() == QDialog.DialogCode.Accepted:
+                self.setter(d.tags)
 
     def connect_data_changed(self, slot):
         if self.col_metadata['is_multiple']:
@@ -918,7 +934,7 @@ class BulkBase(Base):
             l.insertWidget(1, self.set_yes_button)
         if add_edit_tags_button[0]:
             self.edit_tags_button = QToolButton(parent)
-            self.edit_tags_button.setToolTip(_('Open item editor'))
+            self.edit_tags_button.setToolTip(_('Open Item editor'))
             self.edit_tags_button.setIcon(QIcon(I('chapters.png')))
             self.edit_tags_button.clicked.connect(add_edit_tags_button[1])
             l.insertWidget(1, self.edit_tags_button)
@@ -1349,7 +1365,7 @@ class RemoveTags(QWidget):
         self.tags_box.update_items_cache(values)
         layout.addWidget(self.tags_box, stretch=3)
         self.remove_tags_button = QToolButton(parent)
-        self.remove_tags_button.setToolTip(_('Open item editor'))
+        self.remove_tags_button.setToolTip(_('Open Item editor'))
         self.remove_tags_button.setIcon(QIcon(I('chapters.png')))
         layout.addWidget(self.remove_tags_button)
         self.checkbox = QCheckBox(_('Remove all tags'), parent)
