@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,7 +8,7 @@ __docformat__ = 'restructuredtext en'
 
 import weakref
 
-from PyQt5.Qt import (QWidget, QListWidgetItem, Qt, QToolButton, QLabel,
+from qt.core import (QWidget, QListWidgetItem, Qt, QToolButton, QLabel,
         QTabWidget, QGridLayout, QListWidget, QIcon, QLineEdit, QVBoxLayout,
         QPushButton, QGroupBox, QScrollArea, QHBoxLayout, QComboBox,
         pyqtSignal, QSizePolicy, QDialog, QDialogButtonBox, QPlainTextEdit,
@@ -20,6 +19,7 @@ from calibre.gui2 import error_dialog
 from calibre.gui2.dialogs.template_dialog import TemplateDialog
 from calibre.utils.date import parse_date
 from calibre.gui2.device_drivers.mtp_folder_browser import Browser, IgnoredFolders
+from polyglot.builtins import iteritems, unicode_type, range
 
 
 class FormatsConfig(QWidget):  # {{{
@@ -34,9 +34,9 @@ class FormatsConfig(QWidget):  # {{{
         unchecked_formats = sorted(all_formats - set(format_map))
         for fmt in format_map + unchecked_formats:
             item = QListWidgetItem(fmt, f)
-            item.setData(Qt.UserRole, fmt)
-            item.setFlags(Qt.ItemIsEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsSelectable)
-            item.setCheckState(Qt.Checked if fmt in format_map else Qt.Unchecked)
+            item.setData(Qt.ItemDataRole.UserRole, fmt)
+            item.setFlags(Qt.ItemFlag.ItemIsEnabled|Qt.ItemFlag.ItemIsUserCheckable|Qt.ItemFlag.ItemIsSelectable)
+            item.setCheckState(Qt.CheckState.Checked if fmt in format_map else Qt.CheckState.Unchecked)
 
         self.button_up = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-up.png')))
@@ -50,8 +50,8 @@ class FormatsConfig(QWidget):  # {{{
 
     @property
     def format_map(self):
-        return [unicode(self.f.item(i).data(Qt.UserRole) or '') for i in
-                xrange(self.f.count()) if self.f.item(i).checkState()==Qt.Checked]
+        return [unicode_type(self.f.item(i).data(Qt.ItemDataRole.UserRole) or '') for i in
+                range(self.f.count()) if self.f.item(i).checkState()==Qt.CheckState.Checked]
 
     def validate(self):
         if not self.format_map:
@@ -97,7 +97,7 @@ class TemplateConfig(QWidget):  # {{{
 
     @property
     def template(self):
-        return unicode(self.t.text()).strip()
+        return unicode_type(self.t.text()).strip()
 
     def edit_template(self):
         t = TemplateDialog(self, self.template)
@@ -114,7 +114,7 @@ class TemplateConfig(QWidget):  # {{{
         except Exception as err:
             error_dialog(self, _('Invalid template'),
                     '<p>'+_('The template %s is invalid:')%tmpl +
-                    '<br>'+unicode(err), show=True)
+                    '<br>'+unicode_type(err), show=True)
 
             return False
 # }}}
@@ -149,13 +149,13 @@ class SendToConfig(QWidget):  # {{{
     def browse(self):
         b = Browser(self.device.filesystem_cache, show_files=False,
                 parent=self)
-        if b.exec_() == b.Accepted and b.current_item is not None:
+        if b.exec_() == QDialog.DialogCode.Accepted and b.current_item is not None:
             sid, path = b.current_item
             self.t.setText('/'.join(path[1:]))
 
     @property
     def value(self):
-        ans = [x.strip() for x in unicode(self.t.text()).strip().split(',')]
+        ans = [x.strip() for x in unicode_type(self.t.text()).strip().split(',')]
         return [x for x in ans if x]
 
 # }}}
@@ -176,26 +176,26 @@ class IgnoredDevices(QWidget):  # {{{
         l.addWidget(f)
 
         devs = [(snum, (x[0], parse_date(x[1]))) for snum, x in
-                devs.iteritems()]
+                iteritems(devs)]
         for dev, x in sorted(devs, key=lambda x:x[1][1], reverse=True):
             name = x[0]
             name = '%s [%s]'%(name, dev)
             item = QListWidgetItem(name, f)
-            item.setData(Qt.UserRole, dev)
-            item.setFlags(Qt.ItemIsEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsSelectable)
-            item.setCheckState(Qt.Checked if dev in blacklist else Qt.Unchecked)
+            item.setData(Qt.ItemDataRole.UserRole, dev)
+            item.setFlags(Qt.ItemFlag.ItemIsEnabled|Qt.ItemFlag.ItemIsUserCheckable|Qt.ItemFlag.ItemIsSelectable)
+            item.setCheckState(Qt.CheckState.Checked if dev in blacklist else Qt.CheckState.Unchecked)
 
     @property
     def blacklist(self):
-        return [unicode(self.f.item(i).data(Qt.UserRole) or '') for i in
-                xrange(self.f.count()) if self.f.item(i).checkState()==Qt.Checked]
+        return [unicode_type(self.f.item(i).data(Qt.ItemDataRole.UserRole) or '') for i in
+                range(self.f.count()) if self.f.item(i).checkState()==Qt.CheckState.Checked]
 
     def ignore_device(self, snum):
-        for i in xrange(self.f.count()):
+        for i in range(self.f.count()):
             i = self.f.item(i)
-            c = unicode(i.data(Qt.UserRole) or '')
+            c = unicode_type(i.data(Qt.ItemDataRole.UserRole) or '')
             if c == snum:
-                i.setCheckState(Qt.Checked)
+                i.setCheckState(Qt.CheckState.Checked)
                 break
 
 # }}}
@@ -255,7 +255,7 @@ class Rule(QWidget):
     def browse(self):
         b = Browser(self.device.filesystem_cache, show_files=False,
                 parent=self)
-        if b.exec_() == b.Accepted and b.current_item is not None:
+        if b.exec_() == QDialog.DialogCode.Accepted and b.current_item is not None:
             sid, path = b.current_item
             self.folder.setText('/'.join(path[1:]))
 
@@ -264,10 +264,10 @@ class Rule(QWidget):
 
     @property
     def rule(self):
-        folder = unicode(self.folder.text()).strip()
+        folder = unicode_type(self.folder.text()).strip()
         if folder:
             return (
-                unicode(self.fmt.itemData(self.fmt.currentIndex()) or ''),
+                unicode_type(self.fmt.itemData(self.fmt.currentIndex()) or ''),
                 folder
                 )
         return None
@@ -306,7 +306,7 @@ class FormatRules(QGroupBox):
         self.b = b = QPushButton(QIcon(I('plus.png')), _('Add a &new rule'))
         l.addWidget(b)
         b.clicked.connect(self.add_rule)
-        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Ignored)
+        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Ignored)
 
     @property
     def device(self):
@@ -421,11 +421,11 @@ class MTPConfig(QTabWidget):
         v.setMinimumWidth(400)
         v.setMinimumHeight(350)
         l.addWidget(v)
-        bb = d.bb = QDialogButtonBox(QDialogButtonBox.Close)
+        bb = d.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
         bb.accepted.connect(d.accept)
         bb.rejected.connect(d.reject)
         l.addWidget(bb)
-        bb.addButton(_('Copy to clipboard'), bb.ActionRole)
+        bb.addButton(_('Copy to clipboard'), QDialogButtonBox.ButtonRole.ActionRole)
         bb.clicked.connect(lambda :
                 QApplication.clipboard().setText(v.toPlainText()))
         d.exec_()
@@ -433,7 +433,7 @@ class MTPConfig(QTabWidget):
     def change_ignored_folders(self):
         d = IgnoredFolders(self.device,
                      self.current_ignored_folders, parent=self)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.current_ignored_folders = d.ignored_folders
 
     def ignore_device(self):
@@ -446,7 +446,7 @@ class MTPConfig(QTabWidget):
 
     def get_pref(self, key):
         p = self.device.prefs.get(self.current_device_key, {})
-        if not p:
+        if not p and self.current_device_key is not None:
             self.device.prefs[self.current_device_key] = p
         return self.device.get_pref(key)
 
@@ -490,7 +490,8 @@ class MTPConfig(QTabWidget):
             if self.current_ignored_folders != self.initial_ignored_folders:
                 p['ignored_folders'] = self.current_ignored_folders
 
-            self.device.prefs[self.current_device_key] = p
+            if self.current_device_key is not None:
+                self.device.prefs[self.current_device_key] = p
 
 
 class SendError(QDialog):
@@ -508,8 +509,8 @@ class SendError(QDialog):
         la.setWordWrap(True)
         la.setMinimumWidth(500)
         l.addWidget(la)
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Close)
-        self.b = bb.addButton(_('Configure'), bb.AcceptRole)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
+        self.b = bb.addButton(_('Configure'), QDialogButtonBox.ButtonRole.AcceptRole)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
         l.addWidget(bb)
@@ -542,10 +543,10 @@ if __name__ == '__main__':
     d.l = QVBoxLayout()
     d.setLayout(d.l)
     d.l.addWidget(cw)
-    bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+    bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
     d.l.addWidget(bb)
     bb.accepted.connect(d.accept)
     bb.rejected.connect(d.reject)
-    if d.exec_() == d.Accepted:
+    if d.exec_() == QDialog.DialogCode.Accepted:
         cw.commit()
     dev.shutdown()
