@@ -1,7 +1,5 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals,  # division, absolute_import,
-                        print_function)
 
 __license__   = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,7 +7,7 @@ __docformat__ = 'restructuredtext en'
 
 import weakref, textwrap
 
-from PyQt5.Qt import (
+from qt.core import (
     QWidget, QLabel, QTabWidget, QGridLayout, QLineEdit, QVBoxLayout,
     QGroupBox, QComboBox, QSizePolicy, QDialog, QDialogButtonBox, QCheckBox,
     QSpacerItem)
@@ -17,6 +15,7 @@ from PyQt5.Qt import (
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.gui2.device_drivers.mtp_config import (FormatsConfig, TemplateConfig)
 from calibre.devices.usbms.driver import debug_print
+from polyglot.builtins import unicode_type, range
 
 
 def wrap_msg(msg):
@@ -90,8 +89,8 @@ class TabbedDeviceConfig(QTabWidget):
             self.formats.hide()
 
         self.opt_use_subdirs = create_checkbox(
-                                           _("Use sub-directories"),
-                                           _('Place files in sub-directories if the device supports them'),
+                                           _("Use sub-folders"),
+                                           _('Place files in sub-folders if the device supports them'),
                                            device_settings.use_subdirs
                                            )
         self.opt_read_metadata = create_checkbox(
@@ -163,7 +162,7 @@ class TabbedDeviceConfig(QTabWidget):
                     return getattr(atab, attr_name)
                 except AttributeError:
                     pass
-        raise ae
+            raise ae
 
     @property
     def device(self):
@@ -244,7 +243,7 @@ class DeviceConfigTab(QWidget):  # {{{
                     return getattr(awidget, attr_name)
                 except AttributeError:
                     pass
-        raise ae
+            raise ae
 
 
 class ExtraCustomization(DeviceConfigTab):  # {{{
@@ -277,7 +276,7 @@ class ExtraCustomization(DeviceConfigTab):  # {{{
             if isinstance(extra_customization_message, list):
                 self.opt_extra_customization = []
                 if len(extra_customization_message) > 6:
-                    row_func = lambda x, y: ((x/2) * 2) + y
+                    row_func = lambda x, y: ((x//2) * 2) + y
                     col_func = lambda x: x%2
                 else:
                     row_func = lambda x, y: x*2 + y
@@ -312,7 +311,7 @@ class ExtraCustomization(DeviceConfigTab):  # {{{
                         self.extra_layout.addWidget(l, row_func(i + 2, 0), col_func(i))
                     self.extra_layout.addWidget(self.opt_extra_customization[i],
                                                 row_func(i + 2, 1), col_func(i))
-                spacerItem1 = QSpacerItem(10, 10, QSizePolicy.Minimum, QSizePolicy.Expanding)
+                spacerItem1 = QSpacerItem(10, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)
                 self.extra_layout.addItem(spacerItem1, row_func(i + 2 + 2, 1), 0, 1, 2)
                 self.extra_layout.setRowStretch(row_func(i + 2 + 2, 1), 2)
             else:
@@ -340,11 +339,11 @@ class ExtraCustomization(DeviceConfigTab):  # {{{
                     if hasattr(self.opt_extra_customization[i], 'isChecked'):
                         ec.append(self.opt_extra_customization[i].isChecked())
                     elif hasattr(self.opt_extra_customization[i], 'currentText'):
-                        ec.append(unicode(self.opt_extra_customization[i].currentText()).strip())
+                        ec.append(unicode_type(self.opt_extra_customization[i].currentText()).strip())
                     else:
-                        ec.append(unicode(self.opt_extra_customization[i].text()).strip())
+                        ec.append(unicode_type(self.opt_extra_customization[i].text()).strip())
             else:
-                ec = unicode(self.opt_extra_customization.text()).strip()
+                ec = unicode_type(self.opt_extra_customization.text()).strip()
                 if not ec:
                     ec = None
 
@@ -387,10 +386,10 @@ if __name__ == '__main__':
     d.l = QVBoxLayout()
     d.setLayout(d.l)
     d.l.addWidget(cw)
-    bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+    bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
     d.l.addWidget(bb)
     bb.accepted.connect(d.accept)
     bb.rejected.connect(d.reject)
-    if d.exec_() == d.Accepted:
+    if d.exec_() == QDialog.DialogCode.Accepted:
         cw.commit()
     dev.shutdown()
