@@ -1,16 +1,19 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
+
+
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
 __license__   = 'GPL v3'
 
 
-from PyQt5.Qt import (
+from qt.core import (
     QDialog, QGridLayout, QLabel, QDialogButtonBox,  QApplication, QSpinBox,
     QToolButton, QIcon, QLineEdit, QComboBox, QCheckBox)
 from calibre.ebooks.metadata import string_to_authors
 from calibre.gui2.complete2 import EditWithComplete
 from calibre.utils.config import tweaks
 from calibre.gui2 import gprefs
+from polyglot.builtins import unicode_type
 
 
 class AddEmptyBookDialog(QDialog):
@@ -37,7 +40,7 @@ class AddEmptyBookDialog(QDialog):
 
         self.authors_combo = EditWithComplete(self)
         self.authors_combo.setSizeAdjustPolicy(
-                self.authors_combo.AdjustToMinimumContentsLengthWithIcon)
+                QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
         self.authors_combo.setEditable(True)
         self._layout.addWidget(self.authors_combo, 3, 0, 1, 1)
         self.initialize_authors(db, author)
@@ -53,7 +56,7 @@ class AddEmptyBookDialog(QDialog):
 
         self.series_combo = EditWithComplete(self)
         self.series_combo.setSizeAdjustPolicy(
-                self.authors_combo.AdjustToMinimumContentsLengthWithIcon)
+                QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
         self.series_combo.setEditable(True)
         self._layout.addWidget(self.series_combo, 5, 0, 1, 1)
         self.initialize_series(db, series)
@@ -102,12 +105,12 @@ class AddEmptyBookDialog(QDialog):
         cf.setChecked(gprefs.get('create_empty_copy_dup_formats', False))
         self._layout.addWidget(cf, 10, 0, 1, -1)
 
-        button_box = self.bb = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        button_box = self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
         button_box.accepted.connect(self.accept)
         button_box.rejected.connect(self.reject)
         self._layout.addWidget(button_box, 11, 0, 1, -1)
         if dup_title:
-            self.dup_button = b = button_box.addButton(_('&Duplicate current book'), button_box.ActionRole)
+            self.dup_button = b = button_box.addButton(_('&Duplicate current book'), QDialogButtonBox.ButtonRole.ActionRole)
             b.clicked.connect(self.do_duplicate_book)
             b.setIcon(QIcon(I('edit-copy.png')))
             b.setToolTip(_(
@@ -161,11 +164,11 @@ class AddEmptyBookDialog(QDialog):
 
     @property
     def selected_authors(self):
-        return string_to_authors(unicode(self.authors_combo.text()))
+        return string_to_authors(unicode_type(self.authors_combo.text()))
 
     @property
     def selected_series(self):
-        return unicode(self.series_combo.text())
+        return unicode_type(self.series_combo.text())
 
     @property
     def selected_title(self):
