@@ -115,13 +115,13 @@ class ChooseLibrary(QDialog, Ui_Dialog):
             if not empty:
                 error_dialog(self, _('Not empty'),
                     _('The folder %s is not empty. Please choose an empty'
-                       ' folder')%loc,
+                       ' folder.')%loc,
                     show=True)
                 return False
             if (iswindows and len(loc) >
                     LibraryDatabase.WINDOWS_LIBRARY_PATH_LIMIT):
                 error_dialog(self, _('Too long'),
-                    _('Path to library too long. Must be less than'
+                    _('Path to library too long. It must be less than'
                     ' %d characters.')%LibraryDatabase.WINDOWS_LIBRARY_PATH_LIMIT,
                     show=True)
                 return False
