@@ -8,12 +8,12 @@ __docformat__ = 'restructuredtext en'
 
 import functools
 from qt.core import (
-    QAction, QApplication, QIcon, QLabel, QMenu, QPainter, QSizePolicy, QSplitter,
-    QStackedWidget, QStatusBar, QStyle, QStyleOption, Qt, QTabBar, QTimer,
-    QToolButton, QVBoxLayout, QWidget, QDialog, QEvent
+    QAction, QApplication, QDialog, QEvent, QIcon, QLabel, QMenu, QPainter,
+    QSizePolicy, QSplitter, QStackedWidget, QStatusBar, QStyle, QStyleOption, Qt,
+    QTabBar, QTimer, QToolButton, QVBoxLayout, QWidget
 )
 
-from calibre.constants import __appname__, get_version, ismacos
+from calibre.constants import get_appname_for_display, get_version, ismacos
 from calibre.customize.ui import find_plugin
 from calibre.gui2 import (
     config, error_dialog, gprefs, is_widescreen, open_local_file, open_url
@@ -266,7 +266,7 @@ class StatusBar(QStatusBar):  # {{{
     def __init__(self, parent=None):
         QStatusBar.__init__(self, parent)
         self.version = get_version()
-        self.base_msg = '%s %s' % (__appname__, self.version)
+        self.base_msg = f'{get_appname_for_display()} {self.version}'
         self.device_string = ''
         self.update_label = UpdateLabel('')
         self.total = self.current = self.selected = self.library_total = 0
@@ -664,7 +664,7 @@ class LayoutMixin(object):  # {{{
                 type=Qt.ConnectionType.QueuedConnection)
         self.book_details.open_containing_folder.connect(self.iactions['View'].view_folder_for_id)
         self.book_details.view_specific_format.connect(self.iactions['View'].view_format_by_id)
-        self.book_details.search_requested.connect(self.search.set_search_string)
+        self.book_details.search_requested.connect(self.set_search_string_with_append)
         self.book_details.remove_specific_format.connect(
                 self.iactions['Remove Books'].remove_format_by_id)
         self.book_details.remove_metadata_item.connect(
@@ -691,6 +691,14 @@ class LayoutMixin(object):  # {{{
                     self.library_view.currentIndex())
         self.library_view.setFocus(Qt.FocusReason.OtherFocusReason)
 
+    def set_search_string_with_append(self, expression, append=''):
+        current = self.search.text().strip()
+        if append:
+            expr = f'{current} {append} {expression}' if current else expression
+        else:
+            expr = expression
+        self.search.set_search_string(expr)
+
     def edit_identifiers_triggerred(self):
         book_id = self.library_view.current_book
         db = self.current_db.new_api
