@@ -1,23 +1,24 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 from functools import partial
+from qt.core import (
+    QActionGroup, QCoreApplication, QFrame, QHBoxLayout, QIcon, QLabel, QLineEdit,
+    QMenu, QObject, QSizePolicy, Qt, QToolButton, QVBoxLayout, QWidget, pyqtSignal
+)
 
-from PyQt5.Qt import (QIcon, Qt, QWidget, QSize,
-    pyqtSignal, QToolButton, QMenu, QAction, QCoreApplication,
-    QObject, QVBoxLayout, QSizePolicy, QLabel, QHBoxLayout, QActionGroup)
-
-
+from calibre import human_readable
 from calibre.constants import __appname__
-from calibre.gui2.search_box import SearchBox2, SavedSearchBox
 from calibre.gui2.bars import BarsManager
+from calibre.gui2.search_box import SavedSearchBox, SearchBox2
 from calibre.gui2.widgets2 import RightClickButton
 from calibre.utils.config_base import tweaks
-from calibre import human_readable
+from polyglot.builtins import unicode_type
 
 
 class LocationManager(QObject):  # {{{
@@ -128,7 +129,7 @@ class LocationManager(QObject):  # {{{
         had_device = self.has_device
         if cp is None:
             cp = (None, None)
-        if isinstance(cp, (str, unicode)):
+        if isinstance(cp, (bytes, unicode_type)):
             cp = (cp, None)
         if len(fs) < 3:
             fs = list(fs) + [0]
@@ -149,7 +150,7 @@ class LocationManager(QObject):  # {{{
         for i, loc in enumerate(('main', 'carda', 'cardb')):
             t = self.tooltips[loc]
             if self.free[i] > -1:
-                t += u'\n\n%s '%human_readable(self.free[i]) + _('available')
+                t += '\n\n%s '%human_readable(self.free[i]) + _('available')
             ac = getattr(self, 'location_'+loc)
             ac.setToolTip(t)
             ac.setWhatsThis(t)
@@ -170,91 +171,145 @@ class LocationManager(QObject):  # {{{
 # }}}
 
 
-class SearchBar(QWidget):  # {{{
+def search_as_url(text):
+    if text:
+        from calibre.gui2.ui import get_gui
+        db = get_gui().current_db
+        lid = db.new_api.server_library_id
+        lid = lid.encode('utf-8').hex()
+        eq = text.encode('utf-8').hex()
+        vl = db.data.get_base_restriction_name()
+        ans = f'calibre://search/_hex_-{lid}?eq={eq}'
+        if vl:
+            vl = vl.encode('utf-8').hex()
+            ans += '&encoded_virtual_library=' + vl
+        return ans
+
+
+class SearchBar(QFrame):  # {{{
 
     def __init__(self, parent):
-        QWidget.__init__(self, parent)
-        self._layout = l = QHBoxLayout()
-        self.setLayout(self._layout)
-        self._layout.setContentsMargins(0,5,0,0)
+        QFrame.__init__(self, parent)
+        self.setFrameStyle(QFrame.Shape.NoFrame)
+        self.setObjectName('search_bar')
+        self._layout = l = QHBoxLayout(self)
+        l.setContentsMargins(0, 4, 0, 4)
 
-        x = QToolButton(self)
-        x.setText(_('Vi&rtual library'))
-        x.setIcon(QIcon(I('lt.png')))
+        x = parent.virtual_library = QToolButton(self)
+        x.setCursor(Qt.CursorShape.PointingHandCursor)
+        x.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
+        x.setText(_('Virtual library'))
+        x.setAutoRaise(True)
+        x.setIcon(QIcon(I('vl.png')))
         x.setObjectName("virtual_library")
-        x.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        x.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         l.addWidget(x)
-        parent.virtual_library = x
 
         x = QToolButton(self)
+        x.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
+        x.setAutoRaise(True)
         x.setIcon(QIcon(I('minus.png')))
         x.setObjectName('clear_vl')
         l.addWidget(x)
         x.setVisible(False)
         x.setToolTip(_('Close the Virtual library'))
         parent.clear_vl = x
+        self.vl_sep = QFrame(self)
+        self.vl_sep.setFrameStyle(QFrame.Shape.VLine | QFrame.Shadow.Sunken)
+        l.addWidget(self.vl_sep)
 
-        x = QLabel(self)
-        x.setObjectName("search_count")
-        l.addWidget(x)
-        parent.search_count = x
-        x.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
+        parent.sort_sep = QFrame(self)
+        parent.sort_sep.setFrameStyle(QFrame.Shape.VLine | QFrame.Shadow.Sunken)
+        parent.sort_sep.setVisible(False)
+        parent.sort_button = self.sort_button = sb = QToolButton(self)
+        sb.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
+        sb.setToolTip(_('Change how the displayed books are sorted'))
+        sb.setCursor(Qt.CursorShape.PointingHandCursor)
+        sb.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
+        sb.setAutoRaise(True)
+        sb.setText(_('Sort'))
+        sb.setIcon(QIcon(I('sort.png')))
+        sb.setMenu(QMenu(sb))
+        sb.menu().aboutToShow.connect(self.populate_sort_menu)
+        sb.setVisible(False)
+        l.addWidget(sb)
+        l.addWidget(parent.sort_sep)
 
-        parent.advanced_search_button = x = QToolButton(self)
-        parent.advanced_search_toggle_action = ac = QAction(parent)
-        parent.addAction(ac)
-        parent.keyboard.register_shortcut('advanced search toggle',
-                _('Advanced search'), default_keys=("Shift+Ctrl+F",),
-                action=ac)
-        ac.triggered.connect(x.click)
-        x.setIcon(QIcon(I('search.png')))
-        l.addWidget(x)
-        x.setToolTip(_("Advanced search"))
-
-        x = parent.search = SearchBox2(self)
-        x.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
+        x = parent.search = SearchBox2(self, as_url=search_as_url)
+        x.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
         x.setObjectName("search")
         x.setToolTip(_("<p>Search the list of books by title, author, publisher, "
                        "tags, comments, etc.<br><br>Words separated by spaces are ANDed"))
         x.setMinimumContentsLength(10)
         l.addWidget(x)
 
+        parent.advanced_search_toggle_action = ac = parent.search.add_action('gear.png', QLineEdit.ActionPosition.LeadingPosition)
+        parent.addAction(ac)
+        ac.setToolTip(_('Advanced search'))
+        parent.keyboard.register_shortcut('advanced search toggle',
+                _('Advanced search'), default_keys=("Shift+Ctrl+F",),
+                action=ac)
+
         self.search_button = QToolButton()
-        self.search_button.setToolButtonStyle(Qt.ToolButtonTextOnly)
-        self.search_button.setText(_('&Go!'))
+        self.search_button.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextOnly)
+        self.search_button.setIcon(QIcon(I('search.png')))
+        self.search_button.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
+        self.search_button.setText(_('Search'))
+        self.search_button.setAutoRaise(True)
+        self.search_button.setCursor(Qt.CursorShape.PointingHandCursor)
         l.addWidget(self.search_button)
-        self.search_button.setSizePolicy(QSizePolicy.Minimum,
-                QSizePolicy.Minimum)
+        self.search_button.setSizePolicy(QSizePolicy.Policy.Minimum,
+                QSizePolicy.Policy.Minimum)
         self.search_button.clicked.connect(parent.do_search_button)
         self.search_button.setToolTip(
-            _('Do Quick Search (you can also press the Enter key)'))
-
-        x = parent.clear_button = QToolButton(self)
-        x.setIcon(QIcon(I('clear_left.png')))
-        x.setObjectName("clear_button")
-        l.addWidget(x)
-        x.setToolTip(_("Reset Quick Search"))
+            _('Do quick search (you can also press the Enter key)'))
 
         x = parent.highlight_only_button = QToolButton(self)
+        x.setAutoRaise(True)
+        x.setText(_('Highlight'))
+        x.setCursor(Qt.CursorShape.PointingHandCursor)
+        x.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         x.setIcon(QIcon(I('arrow-down.png')))
         l.addWidget(x)
 
         x = parent.saved_search = SavedSearchBox(self)
-        x.setMaximumSize(QSize(150, 16777215))
-        x.setMinimumContentsLength(10)
         x.setObjectName("saved_search")
         l.addWidget(x)
+        x.setVisible(tweaks['show_saved_search_box'])
 
         x = parent.copy_search_button = QToolButton(self)
+        x.setAutoRaise(True)
+        x.setCursor(Qt.CursorShape.PointingHandCursor)
         x.setIcon(QIcon(I("search_copy_saved.png")))
         x.setObjectName("copy_search_button")
         l.addWidget(x)
         x.setToolTip(_("Copy current search text (instead of search name)"))
+        x.setVisible(tweaks['show_saved_search_box'])
 
         x = parent.save_search_button = RightClickButton(self)
+        x.setAutoRaise(True)
+        x.setCursor(Qt.CursorShape.PointingHandCursor)
         x.setIcon(QIcon(I("search_add_saved.png")))
         x.setObjectName("save_search_button")
         l.addWidget(x)
+        x.setVisible(tweaks['show_saved_search_box'])
+
+        x = parent.add_saved_search_button = RightClickButton(self)
+        x.setToolTip(_(
+            'Use an existing Saved search or create a new one'
+        ))
+        x.setText(_('Saved search'))
+        x.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
+        x.setCursor(Qt.CursorShape.PointingHandCursor)
+        x.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
+        x.setAutoRaise(True)
+        x.setIcon(QIcon(I("bookmarks.png")))
+        l.addWidget(x)
+        x.setVisible(not tweaks['show_saved_search_box'])
+
+    def populate_sort_menu(self):
+        from calibre.gui2.ui import get_gui
+        get_gui().iactions['Sort By'].update_menu(self.sort_button.menu())
 
 # }}}
 
@@ -279,11 +334,12 @@ class MainWindowMixin(object):  # {{{
         self.setWindowIcon(QIcon(I('lt.png')))
         self.setWindowTitle(__appname__)
 
-        self.setContextMenuPolicy(Qt.NoContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.NoContextMenu)
         self.centralwidget = QWidget(self)
         self.setCentralWidget(self.centralwidget)
-        self._central_widget_layout = QVBoxLayout()
-        self.centralwidget.setLayout(self._central_widget_layout)
+        self._central_widget_layout = l = QVBoxLayout(self.centralwidget)
+        l.setContentsMargins(0, 0, 0, 0)
+        l.setSpacing(0)
         self.resize(1012, 740)
         self.location_manager = LocationManager(self)
 
@@ -293,9 +349,11 @@ class MainWindowMixin(object):  # {{{
         self.bars_manager = BarsManager(self.donate_action,
                 self.location_manager, self)
         for bar in self.bars_manager.main_bars:
-            self.addToolBar(Qt.TopToolBarArea, bar)
+            self.addToolBar(Qt.ToolBarArea.TopToolBarArea, bar)
+            bar.setStyleSheet('QToolBar { border: 0px }')
         for bar in self.bars_manager.child_bars:
-            self.addToolBar(Qt.BottomToolBarArea, bar)
+            self.addToolBar(Qt.ToolBarArea.BottomToolBarArea, bar)
+            bar.setStyleSheet('QToolBar { border: 0px }')
         self.bars_manager.update_bars()
         # This is disabled because it introduces various toolbar related bugs
         # The width of the toolbar becomes the sum of both toolbars
@@ -303,9 +361,7 @@ class MainWindowMixin(object):  # {{{
             try:
                 self.setUnifiedTitleAndToolBarOnMac(True)
             except AttributeError:
-                pass  # PyQt5 seems to be missing this property
-
-        l = self.centralwidget.layout()
+                pass  # PyQt seems to be missing this property
 
         # And now, start adding the real widgets
         l.addWidget(self.search_bar)
@@ -313,7 +369,7 @@ class MainWindowMixin(object):  # {{{
         # Add in the widget for the shutdown messages. It is invisible until a
         # message is shown
         smw = self.shutdown_message_widget = QLabel(self)
-        smw.setAlignment(Qt.AlignCenter)
+        smw.setAlignment(Qt.AlignmentFlag.AlignCenter)
         smw.setVisible(False)
         smw.setAutoFillBackground(True)
         smw.setStyleSheet('QLabel { background-color: rgba(200, 200, 200, 200); color: black }')
