@@ -5,7 +5,8 @@
 import codecs
 import json
 import os
-from functools import partial
+import re
+from functools import lru_cache, partial
 from qt.core import (
     QAbstractItemView, QApplication, QCheckBox, QComboBox, QCursor, QDateTime,
     QDialog, QDialogButtonBox, QFont, QFormLayout, QFrame, QHBoxLayout, QIcon,
@@ -16,16 +17,21 @@ from qt.core import (
 from urllib.parse import quote
 
 from calibre import prepare_string_for_xml
-from calibre.ebooks.metadata import authors_to_string, fmt_sidx
+from calibre.constants import (
+    builtin_colors_dark, builtin_colors_light, builtin_decorations
+)
 from calibre.db.backend import FTSQueryError
-from calibre.gui2 import Application, choose_save_file, config, error_dialog, gprefs
+from calibre.ebooks.metadata import authors_to_string, fmt_sidx
+from calibre.gui2 import (
+    Application, choose_save_file, config, error_dialog, gprefs, is_dark_theme,
+    safe_open_url
+)
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.viewer.widgets import ResultsDelegate, SearchBox
-from calibre.gui2.widgets2 import Dialog
+from calibre.gui2.widgets2 import Dialog, RightClickButton
 
 
 # rendering {{{
-
 def render_highlight_as_text(hl, lines, as_markdown=False, link_prefix=None):
     lines.append(hl['highlighted_text'])
     date = QDateTime.fromString(hl['timestamp'], Qt.DateFormat.ISODate).toLocalTime().toString(Qt.DateFormat.SystemLocaleShortDate)
@@ -62,11 +68,57 @@ def render_bookmark_as_text(b, lines, as_markdown=False, link_prefix=None):
     lines.append('')
 
 
+url_prefixes = 'http', 'https'
+url_delimiters = (
+    '\x00-\x09\x0b-\x20\x7f-\xa0\xad\u0600-\u0605\u061c\u06dd\u070f\u08e2\u1680\u180e\u2000-\u200f\u2028-\u202f'
+    '\u205f-\u2064\u2066-\u206f\u3000\ud800-\uf8ff\ufeff\ufff9-\ufffb\U000110bd\U000110cd\U00013430-\U00013438'
+    '\U0001bca0-\U0001bca3\U0001d173-\U0001d17a\U000e0001\U000e0020-\U000e007f\U000f0000-\U000ffffd\U00100000-\U0010fffd'
+)
+url_pattern = r'\b(?:{})://[^{}]{{3,}}'.format('|'.join(url_prefixes), url_delimiters)
+
+
+@lru_cache(maxsize=2)
+def url_pat():
+    return re.compile(url_pattern, flags=re.I)
+
+
+closing_bracket_map = {'(': ')', '[': ']', '{': '}', '<': '>', '*': '*', '"': '"', "'": "'"}
+
+
+def url(text: str, s: int, e: int):
+    while text[e - 1] in '.,?!' and e > 1:  # remove trailing punctuation
+        e -= 1
+    # truncate url at closing bracket/quote
+    if s > 0 and e <= len(text) and text[s-1] in closing_bracket_map:
+        q = closing_bracket_map[text[s-1]]
+        idx = text.find(q, s)
+        if idx > s:
+            e = idx
+    return s, e
+
+
+def render_note_line(line):
+    urls = []
+    for m in url_pat().finditer(line):
+        s, e = url(line, m.start(), m.end())
+        urls.append((s, e))
+    if not urls:
+        yield prepare_string_for_xml(line)
+        return
+    pos = 0
+    for (s, e) in urls:
+        if s > pos:
+            yield prepare_string_for_xml(line[pos:s])
+        yield '<a href="{0}">{0}</a>'.format(prepare_string_for_xml(line[s:e], True))
+    if urls[-1][1] < len(line):
+        yield prepare_string_for_xml(line[urls[-1][1]:])
+
+
 def render_notes(notes, tag='p'):
     current_lines = []
     for line in notes.splitlines():
         if line:
-            current_lines.append(line)
+            current_lines.append(''.join(render_note_line(line)))
         else:
             if current_lines:
                 yield '<{0}>{1}</{0}>'.format(tag, '\n'.join(current_lines))
@@ -114,6 +166,57 @@ class AnnotsResultsDelegate(ResultsDelegate):
 # }}}
 
 
+def sorted_items(items):
+    from calibre.ebooks.epub.cfi.parse import cfi_sort_key
+    def_spine = 999999999
+    defval = cfi_sort_key(f'/{def_spine}')
+
+    def sort_key(x):
+        x = x['annotation']
+        atype = x['type']
+        if atype == 'highlight':
+            cfi = x.get('start_cfi')
+            if cfi:
+                spine_idx = x.get('spine_index', def_spine)
+                cfi = f'/{spine_idx}{cfi}'
+                return cfi_sort_key(cfi)
+        elif atype == 'bookmark':
+            if x.get('pos_type') == 'epubcfi':
+                return cfi_sort_key(x['pos'], only_path=False)
+        return defval
+
+    return sorted(items, key=sort_key)
+
+
+def css_for_highlight_style(style):
+    is_dark = is_dark_theme()
+    kind = style.get('kind')
+    ans = ''
+    if kind == 'color':
+        key = 'dark' if is_dark else 'light'
+        val = style.get(key)
+        if val is None:
+            which = style.get('which')
+            val = (builtin_colors_dark if is_dark else builtin_colors_light).get(which)
+        if val is None:
+            val = style.get('background-color')
+        if val is not None:
+            ans = f'background-color: {val}'
+    elif 'background-color' in style:
+        ans = 'background-color: ' + style['background-color']
+        if 'color' in style:
+            ans += '; color: ' + style["color"]
+    elif kind == 'decoration':
+        which = style.get('which')
+        if which is not None:
+            q = builtin_decorations.get(which)
+            if q is not None:
+                ans = q
+        else:
+            ans = '; '.join(f'{k}: {v}' for k, v in style.items())
+    return ans
+
+
 class Export(Dialog):  # {{{
 
     prefs = gprefs
@@ -158,10 +261,10 @@ class Export(Dialog):  # {{{
         self.accept()
 
     def save_to_file(self):
-        filters = [(self.export_format.currentText(), self.export_format.currentData())]
+        filters = [(self.export_format.currentText(), [self.export_format.currentData()])]
         path = choose_save_file(
             self, 'annots-export-save', _('File for exports'), filters=filters,
-            initial_filename=self.initial_filename() + '.' + filters[0][1])
+            initial_filename=self.initial_filename() + '.' + filters[0][1][0])
         if path:
             data = self.exported_data().encode('utf-8')
             with open(path, 'wb') as f:
@@ -187,18 +290,29 @@ class Export(Dialog):  # {{{
         for a in self.annotations:
             bid_groups.setdefault(a['book_id'], []).append(a)
         for book_id, group in bid_groups.items():
+            chapter_groups = {}
+            def_chap = (_('Unknown chapter'),)
+            for a in group:
+                toc_titles = a.get('toc_family_titles', def_chap)
+                chapter_groups.setdefault(toc_titles[0], []).append(a)
+
             lines.append('## ' + db.field_for('title', book_id))
             lines.append('')
-            for a in group:
-                atype = a['type']
-                if library_id:
-                    link_prefix = f'calibre://view-book/{library_id}/{book_id}/{a["format"]}?open_at='
-                else:
-                    link_prefix = None
-                if atype == 'highlight':
-                    render_highlight_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
-                elif atype == 'bookmark':
-                    render_bookmark_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
+
+            for chapter, group in chapter_groups.items():
+                if len(chapter_groups) > 1:
+                    lines.append('### ' + chapter)
+                    lines.append('')
+                for a in group:
+                    atype = a['type']
+                    if library_id:
+                        link_prefix = f'calibre://view-book/{library_id}/{book_id}/{a["format"]}?open_at='
+                    else:
+                        link_prefix = None
+                    if atype == 'highlight':
+                        render_highlight_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
+                    elif atype == 'bookmark':
+                        render_bookmark_as_text(a, lines, as_markdown=as_markdown, link_prefix=link_prefix)
             lines.append('')
         return '\n'.join(lines).strip()
 # }}}
@@ -295,9 +409,10 @@ class ResultsList(QTreeWidget):
             section = QTreeWidgetItem([entry['title']], 1)
             section.setFlags(Qt.ItemFlag.ItemIsEnabled)
             section.setFont(0, self.section_font)
+            section.setData(0, Qt.ItemDataRole.UserRole, book_id)
             self.addTopLevelItem(section)
             section.setExpanded(True)
-            for result in entry['matches']:
+            for result in sorted_items(entry['matches']):
                 item = QTreeWidgetItem(section, [' '], 2)
                 self.item_map.append(item)
                 item.setFlags(Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemNeverHasChildren)
@@ -343,8 +458,39 @@ class ResultsList(QTreeWidget):
             self.delete_requested.emit()
             ev.accept()
             return
+        if ev.key() == Qt.Key.Key_F2:
+            item = self.currentItem()
+            if item:
+                self.edit_notes(item)
+                ev.accept()
+                return
         return QTreeWidget.keyPressEvent(self, ev)
 
+    @property
+    def tree_state(self):
+        ans = {'closed': set()}
+        item = self.currentItem()
+        if item is not None:
+            ans['current'] = item.data(0, Qt.ItemDataRole.UserRole)
+        for item in (self.topLevelItem(i) for i in range(self.topLevelItemCount())):
+            if not item.isExpanded():
+                ans['closed'].add(item.data(0, Qt.ItemDataRole.UserRole))
+        return ans
+
+    @tree_state.setter
+    def tree_state(self, state):
+        closed = state['closed']
+        for item in (self.topLevelItem(i) for i in range(self.topLevelItemCount())):
+            if item.data(0, Qt.ItemDataRole.UserRole) in closed:
+                item.setExpanded(False)
+
+        cur = state.get('current')
+        if cur is not None:
+            for item in self.item_map:
+                if item.data(0, Qt.ItemDataRole.UserRole) == cur:
+                    self.setCurrentItem(item)
+                    break
+
 
 class Restrictions(QWidget):
 
@@ -360,7 +506,7 @@ class Restrictions(QWidget):
         v.addLayout(h)
         self.rla = QLabel(_('Restrict to') + ': ')
         h.addWidget(self.rla)
-        la = QLabel(_('Types:'))
+        la = QLabel(_('Type:'))
         h.addWidget(la)
         self.types_box = tb = QComboBox(self)
         tb.la = la
@@ -595,6 +741,12 @@ class BrowsePanel(QWidget):
     def selected_annotations(self):
         return self.results_list.selected_annotations
 
+    def save_tree_state(self):
+        return self.results_list.tree_state
+
+    def restore_tree_state(self, state):
+        self.results_list.tree_state = state
+
 
 class Details(QTextBrowser):
 
@@ -626,7 +778,10 @@ class DetailsPanel(QWidget):
         self.show_result(None)
 
     def link_clicked(self, qurl):
-        getattr(self, qurl.host())()
+        if qurl.scheme() == 'calibre':
+            getattr(self, qurl.host())()
+        else:
+            safe_open_url(qurl)
 
     def open_result(self):
         if self.current_result is not None:
@@ -678,6 +833,7 @@ class DetailsPanel(QWidget):
         book_format = r['format']
         annot_text = ''
         a = prepare_string_for_xml
+        highlight_css = ''
 
         paras = []
 
@@ -687,7 +843,8 @@ class DetailsPanel(QWidget):
         if annot['type'] == 'bookmark':
             p(annot['title'])
         elif annot['type'] == 'highlight':
-            p(annot['highlighted_text'])
+            for line in annot['highlighted_text'].splitlines():
+                p(line)
             notes = annot.get('notes')
             if notes:
                 paras.append('<h4>{} (<a title="{}" href="calibre://edit_result">{}</a>)</h4>'.format(
@@ -696,6 +853,8 @@ class DetailsPanel(QWidget):
             else:
                 paras.append('<p><a title="{}" href="calibre://edit_result">{}</a></p>'.format(
                     _('Add notes to this highlight'), _('Add notes')))
+            if 'style' in annot:
+                highlight_css = css_for_highlight_style(annot['style'])
 
         annot_text += '\n'.join(paras)
         date = QDateTime.fromString(annot['timestamp'], Qt.DateFormat.ISODate).toLocalTime().toString(Qt.DateFormat.SystemLocaleShortDate)
@@ -707,6 +866,7 @@ class DetailsPanel(QWidget):
         <div style="text-align: center">{series}</div>
         <div>&nbsp;</div>
         <div>&nbsp;</div>
+
         <div>{dt}: {date}</div>
         <div>{ut}: {user}</div>
         <div>
@@ -714,12 +874,12 @@ class DetailsPanel(QWidget):
             <span>\xa0\xa0\xa0</span>
             <a title="{sictt}" href="calibre://show_in_library">{sic}</a>
         </div>
-        <h3 style="text-align: left">{atype}</h3>
+        <h3 style="text-align: left; {highlight_css}">{atype}</h3>
         {text}
         '''.format(
             title=a(title), authors=a(authors), series=a(series_text), book_format=a(book_format),
             atype=a(atype), text=annot_text, dt=_('Date'), date=a(date), ut=a(_('User')),
-            user=a(friendly_username(r['user_type'], r['user'])),
+            user=a(friendly_username(r['user_type'], r['user'])), highlight_css=highlight_css,
             ov=a(_('Open in viewer')), sic=a(_('Show in calibre')),
             ovtt=a(_('Open the book at this annotation in the calibre E-book viewer')),
             sictt=(_('Show this book in the main calibre book list')),
@@ -731,7 +891,8 @@ class EditNotes(Dialog):
 
     def __init__(self, notes, parent=None):
         self.initial_notes = notes
-        Dialog.__init__(self, _('Edit notes for highlight'), 'library-annotations-browser-edit-notes', parent=parent)
+        Dialog.__init__(
+            self, _('Edit notes for highlight'), 'library-annotations-browser-edit-notes', parent=parent)
 
     def setup_ui(self):
         self.notes_edit = QPlainTextEdit(self)
@@ -754,7 +915,8 @@ class AnnotationsBrowser(Dialog):
     show_book = pyqtSignal(object, object)
 
     def __init__(self, parent=None):
-        Dialog.__init__(self, _('Annotations browser'), 'library-annotations-browser', parent=parent)
+        self.current_restriction = None
+        Dialog.__init__(self, _('Annotations browser'), 'library-annotations-browser', parent=parent, default_buttons=QDialogButtonBox.StandardButton.Close)
         self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, False)
         self.setWindowIcon(QIcon(I('highlight.png')))
 
@@ -812,6 +974,17 @@ class AnnotationsBrowser(Dialog):
         b.setToolTip(_('Export the selected annotations'))
         b.setIcon(QIcon(I('save.png')))
         b.clicked.connect(self.export_selected)
+        self.refresh_button = b = RightClickButton(self.bb)
+        self.bb.addButton(b, QDialogButtonBox.ButtonRole.ActionRole)
+        b.setText(_('Refresh'))
+        b.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
+        self.refresh_menu = m = QMenu(self)
+        m.addAction(_('Rebuild search index')).triggered.connect(self.rebuild)
+        b.setMenu(m)
+        b.setToolTip(_('Refresh annotations in case they have been changed since this window was opened'))
+        b.setIcon(QIcon(I('restart.png')))
+        b.setPopupMode(QToolButton.ToolButtonPopupMode.DelayedPopup)
+        b.clicked.connect(self.refresh)
 
     def delete_selected(self):
         ids = frozenset(self.browse_panel.selected_annot_ids)
@@ -872,8 +1045,20 @@ class AnnotationsBrowser(Dialog):
             self.browse_panel.selection_changed(gui.library_view.get_selected_ids(as_set=True))
 
     def reinitialize(self, restrict_to_book_ids=None):
+        self.current_restriction = restrict_to_book_ids
         self.browse_panel.re_initialize(restrict_to_book_ids or set())
 
+    def refresh(self):
+        state = self.browse_panel.save_tree_state()
+        self.browse_panel.re_initialize(self.current_restriction)
+        self.browse_panel.effective_query_changed()
+        self.browse_panel.restore_tree_state(state)
+
+    def rebuild(self):
+        with BusyCursor():
+            current_db().reindex_annotations()
+        self.refresh()
+
 
 if __name__ == '__main__':
     from calibre.library import db
