@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,9 +8,10 @@ __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 from threading import Lock, current_thread
 from collections import OrderedDict
 
-from PyQt5.Qt import QImage, QPixmap
+from qt.core import QImage, QPixmap
 
 from calibre.db.utils import ThumbnailCache as TC
+from polyglot.builtins import itervalues
 
 
 class ThumbnailCache(TC):
@@ -45,7 +45,7 @@ class CoverCache(dict):
 
     def _pop(self, book_id):
         val = self.items.pop(book_id, None)
-        if type(val) is QPixmap and current_thread() is not self.gui_thread:
+        if isinstance(val, QPixmap) and current_thread() is not self.gui_thread:
             self.pixmap_staging.append(val)
 
     def __getitem__(self, key):
@@ -54,7 +54,7 @@ class CoverCache(dict):
             self.clear_staging()
             ans = self.items.pop(key, False)  # pop() so that item is moved to the top
             if ans is not False:
-                if type(ans) is QImage:
+                if isinstance(ans, QImage):
                     # Convert to QPixmap, since rendering QPixmap is much
                     # faster
                     ans = QPixmap.fromImage(ans)
@@ -67,12 +67,12 @@ class CoverCache(dict):
             self._pop(key)  # pop() so that item is moved to the top
             self.items[key] = val
             if len(self.items) > self.limit:
-                del self.items[next(self.items.iterkeys())]
+                del self.items[next(iter(self.items))]
 
     def clear(self):
         with self.lock:
             if current_thread() is not self.gui_thread:
-                pixmaps = (x for x in self.items.itervalues() if type(x) is QPixmap)
+                pixmaps = (x for x in itervalues(self.items) if isinstance(x, QPixmap))
                 self.pixmap_staging.extend(pixmaps)
             self.items.clear()
 
@@ -84,8 +84,6 @@ class CoverCache(dict):
             self.limit = limit
             if len(self.items) > self.limit:
                 extra = len(self.items) - self.limit
-                remove = tuple(self.iterkeys())[:extra]
+                remove = tuple(self)[:extra]
                 for k in remove:
                     self._pop(k)
-
-
