@@ -51,8 +51,8 @@ class HeaderView(QHeaderView):  # {{{
         if self.orientation() == Qt.Orientation.Horizontal:
             self.setSectionsMovable(True)
             self.setSectionsClickable(True)
-            self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
             self.setTextElideMode(Qt.TextElideMode.ElideRight)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.hover = -1
         self.current_font = QFont(self.font())
         self.current_font.setBold(True)
@@ -237,14 +237,20 @@ class BooksView(QTableView):  # {{{
         for wv in self, self.pin_view:
             if not tweaks['horizontal_scrolling_per_column']:
                 wv.setHorizontalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
+            if not tweaks['vertical_scrolling_per_row']:
+                wv.setVerticalScrollMode(QAbstractItemView.ScrollMode.ScrollPerPixel)
 
             wv.setEditTriggers(QAbstractItemView.EditTrigger.EditKeyPressed)
-            if tweaks['doubleclick_on_library_view'] == 'edit_cell':
+            tval = tweaks['doubleclick_on_library_view']
+            if tval == 'edit_cell':
                 wv.setEditTriggers(QAbstractItemView.EditTrigger.DoubleClicked|wv.editTriggers())
-            elif tweaks['doubleclick_on_library_view'] == 'open_viewer':
+            elif tval == 'open_viewer':
                 wv.setEditTriggers(QAbstractItemView.EditTrigger.SelectedClicked|wv.editTriggers())
                 wv.doubleClicked.connect(parent.iactions['View'].view_triggered)
-            elif tweaks['doubleclick_on_library_view'] == 'edit_metadata':
+            elif tval == 'show_book_details':
+                wv.setEditTriggers(QAbstractItemView.EditTrigger.SelectedClicked|wv.editTriggers())
+                wv.doubleClicked.connect(parent.iactions['Show Book Details'].show_book_info)
+            elif tval == 'edit_metadata':
                 # Must not enable single-click to edit, or the field will remain
                 # open in edit mode underneath the edit metadata dialog
                 if use_edit_metadata_dialog:
@@ -315,6 +321,7 @@ class BooksView(QTableView):  # {{{
             self.pin_view.column_header.customContextMenuRequested.connect(partial(self.show_column_header_context_menu, view=self.pin_view))
         self.row_header = HeaderView(Qt.Orientation.Vertical, self)
         self.row_header.setSectionResizeMode(QHeaderView.ResizeMode.Fixed)
+        self.row_header.customContextMenuRequested.connect(self.show_row_header_context_menu)
         self.setVerticalHeader(self.row_header)
         # }}}
 
@@ -494,6 +501,15 @@ class BooksView(QTableView):  # {{{
                     QIcon(I('column.png')), _('Add your own columns'), partial(handler, action='addcustcol'))
         return ans
 
+    def show_row_header_context_menu(self, pos):
+        menu = QMenu(self)
+        menu.addAction(_('Hide row numbers'), self.hide_row_numbers)
+        menu.popup(self.mapToGlobal(pos))
+
+    def hide_row_numbers(self):
+        gprefs['row_numbers_in_book_list'] = False
+        self.set_row_header_visibility()
+
     def show_column_header_context_menu(self, pos, view=None):
         view = view or self
         idx = view.column_header.logicalIndexAt(pos)
@@ -1276,6 +1292,14 @@ class BooksView(QTableView):  # {{{
                 self.set_current_row(row, select=False)
                 break
 
+    def show_next_book(self):
+        ci = self.currentIndex()
+        if not ci.isValid():
+            self.set_current_row()
+            return
+        n = (ci.row() + 1) % self.model().rowCount(QModelIndex())
+        self.set_current_row(n)
+
     @property
     def next_id(self):
         '''
