@@ -99,7 +99,7 @@ def find_portable_library():
     if len(lib) > 74:
         error_dialog(None, _('Path too long'),
             _("Path to Calibre Portable (%s) "
-                'too long. Must be less than 59 characters.')%base, show=True)
+                'too long. It must be less than 59 characters.')%base, show=True)
         raise AbortInit()
 
     prefs.set('library_path', lib)
@@ -318,7 +318,7 @@ class GuiRunner(QObject):
                 details = ''
             self.show_error(_('Startup error'), _(
                 'There was an error during {0} startup. Parts of {0} may not function.'
-                ' Click Show details to learn more.').format(__appname__), det_msg=details)
+                ' Click "Show details" to learn more.').format(__appname__), det_msg=details)
 
     def initialize_db(self):
         from calibre.db.legacy import LibraryDatabase
@@ -447,14 +447,18 @@ singleinstance_name = 'GUI'
 def send_message(msg):
     try:
         send_message_in_process(msg)
-    except Exception as err:
-        print(_('Failed to contact running instance of calibre'), file=sys.stderr, flush=True)
-        print(err, file=sys.stderr, flush=True)
-        if Application.instance():
-            error_dialog(None, _('Contacting calibre failed'), _(
-                'Failed to contact running instance of calibre, try restarting calibre'),
-                det_msg=str(err) + '\n\n' + repr(msg), show=True)
-        return False
+    except Exception:
+        time.sleep(2)
+        try:
+            send_message_in_process(msg)
+        except Exception as err:
+            print(_('Failed to contact running instance of calibre'), file=sys.stderr, flush=True)
+            print(err, file=sys.stderr, flush=True)
+            if Application.instance():
+                error_dialog(None, _('Contacting calibre failed'), _(
+                    'Failed to contact running instance of calibre, try restarting calibre'),
+                    det_msg=str(err) + '\n\n' + repr(msg), show=True)
+            return False
     return True
 
 
@@ -482,7 +486,9 @@ def communicate(opts, args):
 
 
 def restart_after_quit():
-    if iswindows:
+    e = sys.executable if getattr(sys, 'frozen', False) else sys.argv[0]
+    is_calibre_debug_exe = os.path.splitext(e)[0].endswith('-debug')
+    if iswindows and not is_calibre_debug_exe:
         # detach the stdout/stderr/stdin handles
         winutil.prepare_for_restart()
     if after_quit_actions['debug_on_restart']:
@@ -502,9 +508,8 @@ def restart_after_quit():
             else:
                 cmd.append('calibre')
         else:
-            e = sys.executable if getattr(sys, 'frozen', False) else sys.argv[0]
             cmd = [e]
-            if os.path.splitext(e)[0].endswith('-debug'):
+            if is_calibre_debug_exe:
                 cmd.append('-g')
         prints('Restarting with:', ' '.join(cmd))
         subprocess.Popen(cmd)
