@@ -251,7 +251,7 @@ class MetadataSingleDialogBase(QDialog):
         self.paste_isbn_button = b = RightClickButton(self)
         b.setToolTip('<p>' +
                     _('Paste the contents of the clipboard into the '
-                      'identifiers prefixed with isbn:. Or right click, '
+                      'identifiers prefixed with isbn: or url:. Or right click, '
                       'to choose a different prefix.') + '</p>')
         b.setIcon(QIcon(I('edit-paste.png')))
         b.clicked.connect(self.identifiers.paste_identifier)
@@ -468,7 +468,7 @@ class MetadataSingleDialogBase(QDialog):
             cdata = mi.cover_data[1]
         if cdata is None:
             error_dialog(self, _('Could not read cover'),
-                         _('Could not read cover from %s format')%ext).exec_()
+                         _('Could not read cover from %s format')%ext.upper()).exec_()
             return
         self.update_cover(cdata, ext)
 
