@@ -1,19 +1,21 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import copy, sys
 
-from PyQt5.Qt import Qt, QTableWidgetItem, QIcon
+from qt.core import Qt, QTableWidgetItem, QIcon
 
 from calibre.gui2 import gprefs, Application
 from calibre.gui2.preferences import ConfigWidgetBase, test_widget
 from calibre.gui2.preferences.columns_ui import Ui_Form
 from calibre.gui2.preferences.create_custom_column import CreateCustomColumn
 from calibre.gui2 import error_dialog, question_dialog, ALL_COLUMNS
+from polyglot.builtins import unicode_type, range, map
 
 
 class ConfigWidget(ConfigWidgetBase, Ui_Form):
@@ -68,7 +70,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         state = self.columns_state(defaults)
         self.hidden_cols = state['hidden_columns']
         positions = state['column_positions']
-        colmap.sort(cmp=lambda x,y: cmp(positions[x], positions[y]))
+        colmap.sort(key=lambda x: positions[x])
         self.opt_columns.clear()
 
         db = model.db
@@ -115,7 +117,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.opt_columns.resizeRowsToContents()
 
     def setup_row(self, field_metadata, row, col, oldkey=None):
-        flags = Qt.ItemIsEnabled | Qt.ItemIsSelectable
+        flags = Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsSelectable
 
         item = QTableWidgetItem(col)
         item.setFlags(flags)
@@ -152,18 +154,18 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.opt_columns.setItem(row, 3, item)
 
         item = QTableWidgetItem(fm['name'])
-        item.setData(Qt.UserRole, (col))
+        item.setData(Qt.ItemDataRole.UserRole, (col))
         item.setFlags(flags)
         self.opt_columns.setItem(row, 0, item)
 
         if col.startswith('#'):
-            item.setData(Qt.DecorationRole, (QIcon(I('column.png'))))
+            item.setData(Qt.ItemDataRole.DecorationRole, (QIcon(I('column.png'))))
         if col != 'ondevice':
-            flags |= Qt.ItemIsUserCheckable
+            flags |= Qt.ItemFlag.ItemIsUserCheckable
         item.setFlags(flags)
         if col != 'ondevice':
-            item.setCheckState(Qt.Unchecked if col in self.hidden_cols else
-                    Qt.Checked)
+            item.setCheckState(Qt.CheckState.Unchecked if col in self.hidden_cols else
+                    Qt.CheckState.Checked)
 
     def up_column(self):
         idx = self.opt_columns.currentRow()
@@ -192,7 +194,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         if idx < 0:
             return error_dialog(self, '', _('You must select a column to delete it'),
                     show=True)
-        col = unicode(self.opt_columns.item(idx, 0).data(Qt.UserRole) or '')
+        col = unicode_type(self.opt_columns.item(idx, 0).data(Qt.ItemDataRole.UserRole) or '')
         if col not in self.custcols:
             return error_dialog(self, '',
                     _('The selected column is not a custom column'), show=True)
@@ -221,7 +223,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         model = self.gui.library_view.model()
         row = self.opt_columns.currentRow()
         try:
-            key = unicode(self.opt_columns.item(row, 0).data(Qt.UserRole))
+            key = unicode_type(self.opt_columns.item(row, 0).data(Qt.ItemDataRole.UserRole))
         except:
             key = ''
         CreateCustomColumn(self, row, key, model.orig_headers, ALL_COLUMNS)
@@ -234,25 +236,23 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
     def apply_custom_column_changes(self):
         model = self.gui.library_view.model()
         db = model.db
-        config_cols = [unicode(self.opt_columns.item(i, 0).data(Qt.UserRole) or '')
+        config_cols = [unicode_type(self.opt_columns.item(i, 0).data(Qt.ItemDataRole.UserRole) or '')
                  for i in range(self.opt_columns.rowCount())]
         if not config_cols:
             config_cols = ['title']
         removed_cols = set(model.column_map) - set(config_cols)
-        hidden_cols = set([unicode(self.opt_columns.item(i, 0).data(Qt.UserRole) or '')
+        hidden_cols = {unicode_type(self.opt_columns.item(i, 0).data(Qt.ItemDataRole.UserRole) or '')
                  for i in range(self.opt_columns.rowCount())
-                 if self.opt_columns.item(i, 0).checkState()==Qt.Unchecked])
+                 if self.opt_columns.item(i, 0).checkState()==Qt.CheckState.Unchecked}
         hidden_cols = hidden_cols.union(removed_cols)  # Hide removed cols
         hidden_cols = list(hidden_cols.intersection(set(model.column_map)))
         if 'ondevice' in hidden_cols:
             hidden_cols.remove('ondevice')
 
-        def col_pos(x, y):
-            xidx = config_cols.index(x) if x in config_cols else sys.maxint
-            yidx = config_cols.index(y) if y in config_cols else sys.maxint
-            return cmp(xidx, yidx)
+        def col_pos(x):
+            return config_cols.index(x) if x in config_cols else sys.maxsize
         positions = {}
-        for i, col in enumerate((sorted(model.column_map, cmp=col_pos))):
+        for i, col in enumerate((sorted(model.column_map, key=col_pos))):
             positions[col] = i
         state = {'hidden_columns': hidden_cols, 'column_positions':positions}
         self.gui.library_view.apply_state(state)
