@@ -451,7 +451,7 @@ class CreateCustomColumn(QDialog):
                     '</ul></p>'
                     "<p>For example:\n"
                     "<ul>\n"
-                    "<li>ddd, d MMM yyyy gives Mon, 5 Jan 2010<li>\n"
+                    "<li>ddd, d MMM yyyy gives Mon, 5 Jan 2010</li>\n"
                     "<li>dd MMMM yy gives 05 January 10</li>\n"
                     "</ul> "))
             else:
