@@ -118,6 +118,8 @@ class IdLinksRuleEdit(Dialog):
         title = _('Edit rule') if key else _('Create a new rule')
         Dialog.__init__(self, title=title, name='id-links-rule-editor', parent=parent)
         self.key.setText(key), self.nw.setText(name), self.template.setText(template or 'https://example.com/{id}')
+        if self.size().height() < self.sizeHint().height():
+            self.resize(self.sizeHint())
 
     @property
     def rule(self):
@@ -135,7 +137,10 @@ class IdLinksRuleEdit(Dialog):
         self.nw = n = QLineEdit(self)
         l.addRow(_('&Name:'), n)
         la = QLabel(_(
-            'The template used to create the link. The placeholder {id} in the template will be replaced with the actual identifier value.'))
+            'The template used to create the link.'
+            ' The placeholder {0} in the template will be replaced'
+            ' with the actual identifier value. Use {1} to avoid the value'
+            ' being quoted.').format('{id}', '{id_unquoted}'))
         la.setWordWrap(True)
         l.addRow(la)
         self.template = t = QLineEdit(self)
@@ -425,6 +430,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('cover_grid_show_title', gprefs)
         r('tag_browser_show_counts', gprefs)
         r('tag_browser_item_padding', gprefs)
+        r('books_autoscroll_time', gprefs)
 
         r('qv_respects_vls', gprefs)
         r('qv_dclick_changes_column', gprefs)
@@ -475,6 +481,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('separate_cover_flow', config, restart_required=True)
         r('cb_fullscreen', gprefs)
         r('cb_preserve_aspect_ratio', gprefs)
+        r('cb_double_click_to_activate', gprefs)
 
         choices = [(_('Off'), 'off'), (_('Small'), 'small'),
             (_('Medium'), 'medium'), (_('Large'), 'large')]
@@ -549,8 +556,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         keys = [QKeySequence('F11', QKeySequence.SequenceFormat.PortableText), QKeySequence(
             'Ctrl+Shift+F', QKeySequence.SequenceFormat.PortableText)]
         keys = [unicode_type(x.toString(QKeySequence.SequenceFormat.NativeText)) for x in keys]
-        self.fs_help_msg.setText(unicode_type(self.fs_help_msg.text())%(
-            _(' or ').join(keys)))
+        self.fs_help_msg.setText(self.fs_help_msg.text()%(
+            QKeySequence(QKeySequence.StandardKey.FullScreen).toString(QKeySequence.SequenceFormat.NativeText)))
         self.size_calculated.connect(self.update_cg_cache_size, type=Qt.ConnectionType.QueuedConnection)
         self.tabWidget.currentChanged.connect(self.tab_changed)
 
@@ -565,7 +572,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         b.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         l.addWidget(b, 1, 1)
         b.clicked.connect(self.change_cover_grid_texture)
-        self.cover_grid_default_appearance_button = b = QPushButton(_('Restore &default appearance'), self)
+        self.cover_grid_default_appearance_button = b = QPushButton(_('Restore default &appearance'), self)
         b.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         l.addWidget(b, 2, 1)
         b.clicked.connect(self.restore_cover_grid_appearance)
@@ -800,12 +807,14 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         gui.library_view.set_row_header_visibility()
         gui.cover_flow.setShowReflections(gprefs['cover_browser_reflections'])
         gui.cover_flow.setPreserveAspectRatio(gprefs['cb_preserve_aspect_ratio'])
+        gui.cover_flow.setActivateOnDoubleClick(gprefs['cb_double_click_to_activate'])
         gui.update_cover_flow_subtitle_font()
         gui.cover_flow.template_inited = False
         for view in 'library memory card_a card_b'.split():
             getattr(gui, view + '_view').set_row_header_visibility()
         gui.library_view.refresh_row_sizing()
         gui.grid_view.refresh_settings()
+        gui.update_auto_scroll_timeout()
         qv = get_quickview_action_plugin()
         if qv:
             qv.refill_quickview()
