@@ -1,23 +1,26 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 from operator import attrgetter
+from qt.core import (
+    QAbstractListModel, QAbstractTableModel, QDialogButtonBox, QFrame, QIcon, QLabel,
+    QScrollArea, Qt, QVBoxLayout, QWidget, pyqtSignal, QDialog
+)
 
-from PyQt5.Qt import (QAbstractTableModel, Qt, QAbstractListModel, QWidget,
-        pyqtSignal, QVBoxLayout, QDialogButtonBox, QFrame, QLabel, QIcon)
-
+from calibre.customize.ui import (
+    all_metadata_plugins, default_disabled_plugins, disable_plugin, enable_plugin,
+    is_disabled
+)
+from calibre.ebooks.metadata.sources.prefs import msprefs
+from calibre.gui2 import error_dialog, question_dialog
 from calibre.gui2.preferences import ConfigWidgetBase, test_widget
 from calibre.gui2.preferences.metadata_sources_ui import Ui_Form
-from calibre.ebooks.metadata.sources.prefs import msprefs
-from calibre.customize.ui import (all_metadata_plugins, is_disabled,
-        enable_plugin, disable_plugin, default_disabled_plugins)
-from calibre.gui2 import error_dialog, question_dialog
+from polyglot.builtins import iteritems
 
 
 class SourcesModel(QAbstractTableModel):  # {{{
@@ -45,7 +48,7 @@ class SourcesModel(QAbstractTableModel):  # {{{
         return 2
 
     def headerData(self, section, orientation, role):
-        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+        if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
             if section == 0:
                 return _('Source')
             if section == 1:
@@ -59,21 +62,21 @@ class SourcesModel(QAbstractTableModel):  # {{{
             return None
         col = index.column()
 
-        if role in (Qt.DisplayRole, Qt.EditRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             if col == 0:
                 return plugin.name
             elif col == 1:
                 orig = msprefs['cover_priorities'].get(plugin.name, 1)
                 return self.cover_overrides.get(plugin, orig)
-        elif role == Qt.CheckStateRole and col == 0:
-            orig = Qt.Unchecked if is_disabled(plugin) else Qt.Checked
+        elif role == Qt.ItemDataRole.CheckStateRole and col == 0:
+            orig = Qt.CheckState.Unchecked if is_disabled(plugin) else Qt.CheckState.Checked
             return self.enabled_overrides.get(plugin, orig)
-        elif role == Qt.UserRole:
+        elif role == Qt.ItemDataRole.UserRole:
             return plugin
-        elif (role == Qt.DecorationRole and col == 0 and not
+        elif (role == Qt.ItemDataRole.DecorationRole and col == 0 and not
                     plugin.is_configured()):
             return QIcon(I('list_remove.png'))
-        elif role == Qt.ToolTipRole:
+        elif role == Qt.ItemDataRole.ToolTipRole:
             base = plugin.description + '\n\n'
             if plugin.is_configured():
                 return base + _('This source is configured and ready to go')
@@ -87,8 +90,8 @@ class SourcesModel(QAbstractTableModel):  # {{{
             return False
         col = index.column()
         ret = False
-        if col == 0 and role == Qt.CheckStateRole:
-            if val == Qt.Checked and 'Douban' in plugin.name:
+        if col == 0 and role == Qt.ItemDataRole.CheckStateRole:
+            if val == Qt.CheckState.Checked and 'Douban' in plugin.name:
                 if not question_dialog(self.gui_parent,
                     _('Are you sure?'), '<p>'+
                     _('This plugin is useful only for <b>Chinese</b>'
@@ -99,7 +102,7 @@ class SourcesModel(QAbstractTableModel):  # {{{
                     return ret
             self.enabled_overrides[plugin] = int(val)
             ret = True
-        if col == 1 and role == Qt.EditRole:
+        if col == 1 and role == Qt.ItemDataRole.EditRole:
             try:
                 self.cover_overrides[plugin] = max(1, int(val))
                 ret = True
@@ -113,19 +116,19 @@ class SourcesModel(QAbstractTableModel):  # {{{
         col = index.column()
         ans = QAbstractTableModel.flags(self, index)
         if col == 0:
-            return ans | Qt.ItemIsUserCheckable
-        return Qt.ItemIsEditable | ans
+            return ans | Qt.ItemFlag.ItemIsUserCheckable
+        return Qt.ItemFlag.ItemIsEditable | ans
 
     def commit(self):
-        for plugin, val in self.enabled_overrides.iteritems():
-            if val == Qt.Checked:
+        for plugin, val in iteritems(self.enabled_overrides):
+            if val == Qt.CheckState.Checked:
                 enable_plugin(plugin)
-            elif val == Qt.Unchecked:
+            elif val == Qt.CheckState.Unchecked:
                 disable_plugin(plugin)
 
         if self.cover_overrides:
             cp = msprefs['cover_priorities']
-            for plugin, val in self.cover_overrides.iteritems():
+            for plugin, val in iteritems(self.cover_overrides):
                 if val == 1:
                     cp.pop(plugin.name, None)
                 else:
@@ -137,8 +140,8 @@ class SourcesModel(QAbstractTableModel):  # {{{
 
     def restore_defaults(self):
         self.beginResetModel()
-        self.enabled_overrides = dict([(p, (Qt.Unchecked if p.name in
-            default_disabled_plugins else Qt.Checked)) for p in self.plugins])
+        self.enabled_overrides = dict([(p, (Qt.CheckState.Unchecked if p.name in
+            default_disabled_plugins else Qt.CheckState.Checked)) for p in self.plugins])
         self.cover_overrides = dict([(p,
             msprefs.defaults['cover_priorities'].get(p.name, 1))
                 for p in self.plugins])
@@ -161,7 +164,7 @@ class FieldsModel(QAbstractListModel):  # {{{
                 'rating' : _('Rating'),
                 'tags' : _('Tags'),
                 'title': _('Title'),
-                'series': _('Series'),
+                'series': ngettext('Series', 'Series', 1),
                 'languages': _('Languages'),
         }
         self.overrides = {}
@@ -186,37 +189,37 @@ class FieldsModel(QAbstractListModel):  # {{{
 
     def state(self, field, defaults=False):
         src = msprefs.defaults if defaults else msprefs
-        return (Qt.Unchecked if field in src['ignore_fields']
-                    else Qt.Checked)
+        return (Qt.CheckState.Unchecked if field in src['ignore_fields']
+                    else Qt.CheckState.Checked)
 
     def data(self, index, role):
         try:
             field = self.fields[index.row()]
         except:
             return None
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return self.descs.get(field, field)
-        if role == Qt.CheckStateRole:
+        if role == Qt.ItemDataRole.CheckStateRole:
             return self.overrides.get(field, self.state(field))
         return None
 
     def flags(self, index):
         ans = QAbstractListModel.flags(self, index)
-        return ans | Qt.ItemIsUserCheckable
+        return ans | Qt.ItemFlag.ItemIsUserCheckable
 
     def restore_defaults(self):
         self.beginResetModel()
-        self.overrides = dict([(f, self.state(f, Qt.Checked)) for f in self.fields])
+        self.overrides = dict([(f, self.state(f, Qt.CheckState.Checked)) for f in self.fields])
         self.endResetModel()
 
     def select_all(self):
         self.beginResetModel()
-        self.overrides = dict([(f, Qt.Checked) for f in self.fields])
+        self.overrides = dict([(f, Qt.CheckState.Checked) for f in self.fields])
         self.endResetModel()
 
     def clear_all(self):
         self.beginResetModel()
-        self.overrides = dict([(f, Qt.Unchecked) for f in self.fields])
+        self.overrides = dict([(f, Qt.CheckState.Unchecked) for f in self.fields])
         self.endResetModel()
 
     def setData(self, index, val, role):
@@ -225,7 +228,7 @@ class FieldsModel(QAbstractListModel):  # {{{
         except:
             return False
         ret = False
-        if role == Qt.CheckStateRole:
+        if role == Qt.ItemDataRole.CheckStateRole:
             self.overrides[field] = int(val)
             ret = True
         if ret:
@@ -233,15 +236,15 @@ class FieldsModel(QAbstractListModel):  # {{{
         return ret
 
     def commit(self):
-        ignored_fields = set([x for x in msprefs['ignore_fields'] if x not in
-            self.overrides])
-        changed = set([k for k, v in self.overrides.iteritems() if v ==
-            Qt.Unchecked])
+        ignored_fields = {x for x in msprefs['ignore_fields'] if x not in
+            self.overrides}
+        changed = {k for k, v in iteritems(self.overrides) if v ==
+            Qt.CheckState.Unchecked}
         msprefs['ignore_fields'] = list(ignored_fields.union(changed))
 
     def user_default_state(self, field):
-        return (Qt.Unchecked if field in msprefs.get('user_default_ignore_fields',[])
-                    else Qt.Checked)
+        return (Qt.CheckState.Unchecked if field in msprefs.get('user_default_ignore_fields',[])
+                    else Qt.CheckState.Checked)
 
     def select_user_defaults(self):
         self.beginResetModel()
@@ -249,10 +252,10 @@ class FieldsModel(QAbstractListModel):  # {{{
         self.endResetModel()
 
     def commit_user_defaults(self):
-        default_ignored_fields = set([x for x in msprefs['user_default_ignore_fields'] if x not in
-            self.overrides])
-        changed = set([k for k, v in self.overrides.iteritems() if v ==
-            Qt.Unchecked])
+        default_ignored_fields = {x for x in msprefs['user_default_ignore_fields'] if x not in
+            self.overrides}
+        changed = {k for k, v in iteritems(self.overrides) if v ==
+            Qt.CheckState.Unchecked}
         msprefs['user_default_ignore_fields'] = list(default_ignored_fields.union(changed))
 
 # }}}
@@ -271,14 +274,17 @@ class PluginConfig(QWidget):  # {{{
         self.setLayout(l)
         self.c = c = QLabel(_('<b>Configure %(name)s</b><br>%(desc)s') % dict(
             name=plugin.name, desc=plugin.description))
-        c.setAlignment(Qt.AlignHCenter)
+        c.setAlignment(Qt.AlignmentFlag.AlignHCenter)
         l.addWidget(c)
 
         self.config_widget = plugin.config_widget()
-        self.l.addWidget(self.config_widget)
+        self.sa = sa = QScrollArea(self)
+        sa.setWidgetResizable(True)
+        sa.setWidget(self.config_widget)
+        l.addWidget(sa)
 
         self.bb = QDialogButtonBox(
-                QDialogButtonBox.Save|QDialogButtonBox.Cancel,
+                QDialogButtonBox.StandardButton.Save|QDialogButtonBox.StandardButton.Cancel,
                 parent=self)
         self.bb.accepted.connect(self.finished)
         self.bb.rejected.connect(self.finished)
@@ -286,7 +292,7 @@ class PluginConfig(QWidget):  # {{{
         l.addWidget(self.bb)
 
         self.f = QFrame(self)
-        self.f.setFrameShape(QFrame.HLine)
+        self.f.setFrameShape(QFrame.Shape.HLine)
         l.addWidget(self.f)
 
     def commit(self):
@@ -304,7 +310,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('wait_after_first_cover_result', msprefs)
         r('swap_author_names', msprefs)
         r('fewer_tags', msprefs)
-        r('find_first_edition_date', msprefs)
+        r('keep_dups', msprefs)
         r('append_comments', msprefs)
 
         self.configure_plugin_button.clicked.connect(self.configure_plugin)
@@ -323,12 +329,16 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.select_default_button.clicked.connect(self.fields_model.select_user_defaults)
         self.select_default_button.clicked.connect(self.changed_signal)
         self.set_as_default_button.clicked.connect(self.fields_model.commit_user_defaults)
-        self.tag_map_rules = None
+        self.tag_map_rules = self.author_map_rules = None
         self.tag_map_rules_button.clicked.connect(self.change_tag_map_rules)
+        self.author_map_rules_button.clicked.connect(self.change_author_map_rules)
+        l = self.page.layout()
+        l.setStretch(0, 1)
+        l.setStretch(1, 1)
 
     def configure_plugin(self):
         for index in self.sources_view.selectionModel().selectedRows():
-            plugin = self.sources_model.data(index, Qt.UserRole)
+            plugin = self.sources_model.data(index, Qt.ItemDataRole.UserRole)
             if plugin is not None:
                 return self.do_config(plugin)
         error_dialog(self, _('No source selected'),
@@ -354,16 +364,25 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         d = RulesDialog(self)
         if msprefs.get('tag_map_rules'):
             d.rules = msprefs['tag_map_rules']
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.tag_map_rules = d.rules
             self.changed_signal.emit()
 
+    def change_author_map_rules(self):
+        from calibre.gui2.author_mapper import RulesDialog
+        d = RulesDialog(self)
+        if msprefs.get('author_map_rules'):
+            d.rules = msprefs['author_map_rules']
+        if d.exec_() == QDialog.DialogCode.Accepted:
+            self.author_map_rules = d.rules
+            self.changed_signal.emit()
+
     def initialize(self):
         ConfigWidgetBase.initialize(self)
         self.sources_model.initialize()
         self.sources_view.resizeColumnsToContents()
         self.fields_model.initialize()
-        self.tag_map_rules = None
+        self.tag_map_rules = self.author_map_rules = None
 
     def restore_defaults(self):
         ConfigWidgetBase.restore_defaults(self)
@@ -375,14 +394,13 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         self.sources_model.commit()
         self.fields_model.commit()
         if self.tag_map_rules is not None:
-            if self.tag_map_rules:
-                msprefs['tag_map_rules'] = self.tag_map_rules
-            else:
-                msprefs.pop('tag_map_rules', None)
+            msprefs['tag_map_rules'] = self.tag_map_rules or []
+        if self.author_map_rules is not None:
+            msprefs['author_map_rules'] = self.author_map_rules or []
         return ConfigWidgetBase.commit(self)
 
-if __name__ == '__main__':
-    from PyQt5.Qt import QApplication
-    app = QApplication([])
-    test_widget('Sharing', 'Metadata download')
 
+if __name__ == '__main__':
+    from calibre.gui2 import Application
+    app = Application([])
+    test_widget('Sharing', 'Metadata download')
