@@ -1,6 +1,7 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -9,8 +10,9 @@ import textwrap
 
 from calibre.gui2.preferences import ConfigWidgetBase, test_widget, Setting
 from calibre.gui2.preferences.misc_ui import Ui_Form
-from calibre.gui2 import (config, open_local_file, gprefs)
+from calibre.gui2 import config, open_local_file, gprefs
 from calibre import get_proxies
+from polyglot.builtins import iteritems
 
 
 class WorkersSetting(Setting):
@@ -45,7 +47,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         txt = _('No proxies used')
         if proxies:
             lines = ['<br><code>%s: %s</code>'%(t, p) for t, p in
-                    proxies.iteritems()]
+                    iteritems(proxies)]
             txt = _('<b>Using proxies:</b>') + ''.join(lines)
         self.proxies.setText(txt)
 
@@ -67,8 +69,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         from calibre.utils.config import config_dir
         open_local_file(config_dir)
 
+
 if __name__ == '__main__':
-    from PyQt5.Qt import QApplication
+    from qt.core import QApplication
     app = QApplication([])
     test_widget('Advanced', 'Misc')
-
