@@ -1,24 +1,34 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-from functools import partial
-
-from PyQt5.Qt import QAbstractListModel, Qt, QIcon, \
-        QItemSelectionModel
+from qt.core import QAbstractListModel, Qt, QIcon, QItemSelectionModel
 
+from calibre import force_unicode
 from calibre.gui2.preferences.toolbar_ui import Ui_Form
 from calibre.gui2 import gprefs, warning_dialog, error_dialog
 from calibre.gui2.preferences import ConfigWidgetBase, test_widget, AbortCommit
+from calibre.utils.icu import primary_sort_key
+from polyglot.builtins import unicode_type
+
+
+def sort_key_for_action(ac):
+    q = getattr(ac, 'action_spec', None)
+    try:
+        q = ac.name if q is None else q[0]
+        return primary_sort_key(force_unicode(q))
+    except Exception:
+        return primary_sort_key('')
 
 
 class FakeAction(object):
 
     def __init__(self, name, gui_name, icon, tooltip=None,
-            dont_add_to=frozenset([]), dont_remove_from=frozenset([])):
+            dont_add_to=frozenset(), dont_remove_from=frozenset()):
         self.name = name
         self.action_spec = (gui_name, icon, tooltip, None)
         self.dont_remove_from = dont_remove_from
@@ -53,13 +63,13 @@ class BaseModel(QAbstractListModel):
     def data(self, index, role):
         row = index.row()
         action = self._data[row].action_spec
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             text = action[0]
             text = text.replace('&', '')
             if text == _('%d books'):
                 text = _('Choose library')
             return (text)
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             if hasattr(self._data[row], 'qaction'):
                 icon = self._data[row].qaction.icon()
                 if not icon.isNull():
@@ -68,7 +78,7 @@ class BaseModel(QAbstractListModel):
             if ic is None:
                 ic = 'blank.png'
             return (QIcon(I(ic)))
-        if role == Qt.ToolTipRole and action[2] is not None:
+        if role == Qt.ItemDataRole.ToolTipRole and action[2] is not None:
             return (action[2])
         return None
 
@@ -104,7 +114,8 @@ class AllModel(BaseModel):
         all = [x for x in all if x not in current] + [None]
         all = [self.name_to_action(x, self.gui) for x in all]
         all = [x for x in all if self.key not in x.dont_add_to]
-        all.sort()
+
+        all.sort(key=sort_key_for_action)
         return all
 
     def add(self, names):
@@ -115,12 +126,12 @@ class AllModel(BaseModel):
             actions.append(self.name_to_action(name, self.gui))
         self.beginResetModel()
         self._data.extend(actions)
-        self._data.sort()
+        self._data.sort(key=sort_key_for_action)
         self.endResetModel()
 
     def remove(self, indices, allowed):
         rows = [i.row() for i in indices]
-        remove = set([])
+        remove = set()
         for row in rows:
             ac = self._data[row]
             if ac.name.startswith('---'):
@@ -155,9 +166,7 @@ class CurrentModel(BaseModel):
 
     def move(self, idx, delta):
         row = idx.row()
-        if row < 0 or row >= len(self._data):
-            return
-        nrow = row + delta
+        nrow = (row + delta + len(self._data)) % len(self._data)
         if nrow < 0 or nrow >= len(self._data):
             return
         t = self._data[row]
@@ -168,9 +177,17 @@ class CurrentModel(BaseModel):
         self.dataChanged.emit(ni, ni)
         return ni
 
+    def move_many(self, indices, delta):
+        indices = sorted(indices, key=lambda i: i.row(), reverse=delta > 0)
+        ans = {}
+        for idx in indices:
+            ni = self.move(idx, delta)
+            ans[idx.row()] = ni
+        return ans
+
     def add(self, names):
         actions = []
-        reject = set([])
+        reject = set()
         for name in names:
             ac = self.name_to_action(name, self.gui)
             if self.key in ac.dont_add_to:
@@ -185,7 +202,7 @@ class CurrentModel(BaseModel):
 
     def remove(self, indices):
         rows = [i.row() for i in indices]
-        remove, rejected = set([]), set([])
+        remove, rejected = set(), set()
         for row in rows:
             ac = self._data[row]
             if self.key in ac.dont_remove_from:
@@ -234,6 +251,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             ('menubar-device', _('The menubar when a device is connected')),
             ('context-menu', _('The context menu for the books in the '
                 'calibre library')),
+            ('context-menu-split', _('The context menu for the split book list')),
             ('context-menu-device', _('The context menu for the books on '
                 'the device')),
             ('context-menu-cover-browser', _('The context menu for the Cover '
@@ -241,6 +259,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             ]
 
     def genesis(self, gui):
+        self.all_actions.doubleClicked.connect(self.add_single_action)
+        self.current_actions.doubleClicked.connect(self.remove_single_action)
         self.models = {}
         self.what.addItem(_('Click to choose toolbar or menu to customize'),
                 'blank')
@@ -255,23 +275,23 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
 
         self.add_action_button.clicked.connect(self.add_action)
         self.remove_action_button.clicked.connect(self.remove_action)
-        self.action_up_button.clicked.connect(partial(self.move, -1))
-        self.action_down_button.clicked.connect(partial(self.move, 1))
+        connect_lambda(self.action_up_button.clicked, self, lambda self: self.move(-1))
+        connect_lambda(self.action_down_button.clicked, self, lambda self: self.move(1))
         self.all_actions.setMouseTracking(True)
         self.current_actions.setMouseTracking(True)
         self.all_actions.entered.connect(self.all_entered)
         self.current_actions.entered.connect(self.current_entered)
 
     def all_entered(self, index):
-        tt = self.all_actions.model().data(index, Qt.ToolTipRole) or ''
+        tt = self.all_actions.model().data(index, Qt.ItemDataRole.ToolTipRole) or ''
         self.help_text.setText(tt)
 
     def current_entered(self, index):
-        tt = self.current_actions.model().data(index, Qt.ToolTipRole) or ''
+        tt = self.current_actions.model().data(index, Qt.ItemDataRole.ToolTipRole) or ''
         self.help_text.setText(tt)
 
     def what_changed(self, idx):
-        key = unicode(self.what.itemData(idx) or '')
+        key = unicode_type(self.what.itemData(idx) or '')
         if key == 'blank':
             self.actions_widget.setVisible(False)
             self.spacer_widget.setVisible(True)
@@ -282,13 +302,18 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             self.current_actions.setModel(self.models[key][1])
 
     def add_action(self, *args):
-        x = self.all_actions.selectionModel().selectedIndexes()
-        names = self.all_actions.model().names(x)
+        self._add_action(self.all_actions.selectionModel().selectedIndexes())
+
+    def add_single_action(self, index):
+        self._add_action([index])
+
+    def _add_action(self, indices):
+        names = self.all_actions.model().names(indices)
         if names:
             not_added = self.current_actions.model().add(names)
-            ns = set([y.name for y in not_added])
+            ns = {y.name for y in not_added}
             added = set(names) - ns
-            self.all_actions.model().remove(x, added)
+            self.all_actions.model().remove(indices, added)
             if not_added:
                 warning_dialog(self, _('Cannot add'),
                         _('Cannot add the actions %s to this location') %
@@ -301,11 +326,16 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 self.changed_signal.emit()
 
     def remove_action(self, *args):
-        x = self.current_actions.selectionModel().selectedIndexes()
-        names = self.current_actions.model().names(x)
+        self._remove_action(self.current_actions.selectionModel().selectedIndexes())
+
+    def remove_single_action(self, index):
+        self._remove_action([index])
+
+    def _remove_action(self, indices):
+        names = self.current_actions.model().names(indices)
         if names:
-            not_removed = self.current_actions.model().remove(x)
-            ns = set([y.name for y in not_removed])
+            not_removed = self.current_actions.model().remove(indices)
+            ns = {y.name for y in not_removed}
             removed = set(names) - ns
             self.all_actions.model().add(removed)
             if not_removed:
@@ -317,15 +347,19 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 self.changed_signal.emit()
 
     def move(self, delta, *args):
-        ci = self.current_actions.currentIndex()
-        m = self.current_actions.model()
-        if ci.isValid():
-            ni = m.move(ci, delta)
-            if ni is not None:
-                self.current_actions.setCurrentIndex(ni)
-                self.current_actions.selectionModel().select(ni,
-                        QItemSelectionModel.ClearAndSelect)
-                self.changed_signal.emit()
+        sm = self.current_actions.selectionModel()
+        x = sm.selectedIndexes()
+        if x and len(x):
+            i = sm.currentIndex().row()
+            m = self.current_actions.model()
+            idx_map = m.move_many(x, delta)
+            newci = idx_map.get(i)
+            if newci is not None:
+                sm.setCurrentIndex(newci, QItemSelectionModel.SelectionFlag.ClearAndSelect)
+            sm.clear()
+            for idx in idx_map.values():
+                sm.select(idx, QItemSelectionModel.SelectionFlag.Select)
+            self.changed_signal.emit()
 
     def commit(self):
         # Ensure preferences are showing in either the toolbar or
@@ -360,6 +394,6 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
 
 
 if __name__ == '__main__':
-    from PyQt5.Qt import QApplication
-    app = QApplication([])
+    from calibre.gui2 import Application
+    app = Application([])
     test_widget('Interface', 'Toolbar')
