@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -11,12 +10,13 @@ import sys, time, io, re
 from zlib import decompressobj
 from collections import OrderedDict
 from threading import Thread
-from urllib import urlencode
 
 from calibre import prints
 from calibre.constants import numeric_version, DEBUG
 from calibre.gui2.store import StorePlugin
 from calibre.utils.config import JSONConfig
+from polyglot.urllib import urlencode
+from polyglot.builtins import iteritems, itervalues, unicode_type
 
 
 class VersionMismatch(ValueError):
@@ -28,7 +28,7 @@ class VersionMismatch(ValueError):
 
 def download_updates(ver_map={}, server='https://code.calibre-ebook.com'):
     from calibre.utils.https import get_https_resource_securely
-    data = {k:type(u'')(v) for k, v in ver_map.iteritems()}
+    data = {k:unicode_type(v) for k, v in iteritems(ver_map)}
     data['ver'] = '1'
     url = '%s/stores?%s'%(server, urlencode(data))
     # We use a timeout here to ensure the non-daemonic update thread does not
@@ -58,7 +58,7 @@ class Stores(OrderedDict):
         self.version_map = {}
         self.cached_version_map = {}
         self.name_rmap = {}
-        for key, val in self.iteritems():
+        for key, val in iteritems(self):
             prefix, name = val.__module__.rpartition('.')[0::2]
             if prefix == 'calibre.gui2.store.stores' and name.endswith('_plugin'):
                 module = sys.modules[val.__module__]
@@ -74,7 +74,7 @@ class Stores(OrderedDict):
         # Load plugins from on disk cache
         remove = set()
         pat = re.compile(r'^store_version\s*=\s*(\d+)', re.M)
-        for name, src in self.cache_file.iteritems():
+        for name, src in iteritems(self.cache_file):
             try:
                 key = self.name_rmap[name]
             except KeyError:
@@ -123,7 +123,7 @@ class Stores(OrderedDict):
 
     def download_updates(self):
         ver_map = {name:max(ver, self.cached_version_map.get(name, -1))
-            for name, ver in self.version_map.iteritems()}
+            for name, ver in iteritems(self.version_map)}
         try:
             updates = download_updates(ver_map)
         except:
@@ -159,7 +159,7 @@ class Stores(OrderedDict):
 
         if replacements:
             with self.cache_file:
-                for name, src in replacements.iteritems():
+                for name, src in iteritems(replacements):
                     self.cache_file[name] = src
 
     def replace_plugin(self, ver, name, obj, source):
@@ -175,10 +175,10 @@ class Stores(OrderedDict):
     def load_object(self, src, key):
         namespace = {}
         builtin = self[key]
-        exec src in namespace
+        exec(src, namespace)
         ver = namespace['store_version']
         cls = None
-        for x in namespace.itervalues():
+        for x in itervalues(namespace):
             if (isinstance(x, type) and issubclass(x, StorePlugin) and x is not
                 StorePlugin):
                 cls = x
@@ -190,6 +190,7 @@ class Stores(OrderedDict):
         return cls(builtin.gui, builtin.name, config=builtin.config,
                    base_plugin=builtin.base_plugin), ver
 
+
 if __name__ == '__main__':
     st = time.time()
     count = 0
@@ -198,6 +199,4 @@ if __name__ == '__main__':
         print(name)
         print(code.encode('utf-8'))
         print('\n', '_'*80, '\n', sep='')
-    print ('Time to download all %d plugins: %.2f seconds'%(count, time.time() - st))
-
-
+    print('Time to download all %d plugins: %.2f seconds'%(count, time.time() - st))
