@@ -1,17 +1,19 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
-store_version = 4  # Needed for dynamic plugin loading
+store_version = 5  # Needed for dynamic plugin loading
 
 from contextlib import closing
-import urllib
+try:
+    from urllib.parse import urlencode
+except ImportError:
+    from urllib import urlencode
 
 from lxml import html
 
-from PyQt5.Qt import QUrl
+from qt.core import QUrl
 
 from calibre import browser
 from calibre.gui2 import open_url
@@ -43,8 +45,8 @@ def search_amazon(query, max_results=10, timeout=60,
         if isinstance(x, type('')):
             x = x.encode('utf-8')
         return x
-    uquery = {asbytes(k):asbytes(v) for k, v in uquery.iteritems()}
-    url = base_url + '?' + urllib.urlencode(uquery).decode('ascii')
+    uquery = {asbytes(k):asbytes(v) for k, v in uquery.items()}
+    url = base_url + '?' + urlencode(uquery)
     br = browser(user_agent=get_user_agent())
 
     counter = max_results
@@ -66,7 +68,8 @@ def search_amazon(query, max_results=10, timeout=60,
             cover_xpath =  "descendant-or-self::img[@class and contains(concat(' ', normalize-space(@class), ' '), ' s-access-image ')]/@src"
             title_xpath = "descendant-or-self::h2[@class and contains(concat(' ', normalize-space(@class), ' '), ' s-access-title ')]//text()"
             author_xpath = './/span[starts-with(text(), "by ")]/following-sibling::span//text()'
-            price_xpath = 'descendant::span[contains(@class, "sx-price")]/../@aria-label'
+            price_xpath = ('descendant::div[@class="a-row a-spacing-none" and'
+                           ' not(span[contains(@class, "kindle-unlimited")])]//span[contains(@class, "s-price")]//text()')
         else:
             return
 
@@ -146,4 +149,4 @@ class AmazonKindleStore(StorePlugin):
 if __name__ == '__main__':
     import sys
     for result in search_amazon(' '.join(sys.argv[1:]), write_html_to='/t/amazon.html'):
-        print (result)
+        print(result)
