@@ -1,20 +1,22 @@
 # -*- coding: utf-8 -*-
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-from __future__ import (unicode_literals, division, absolute_import, print_function)
-store_version = 5 # Needed for dynamic plugin loading
+store_version = 10  # Needed for dynamic plugin loading
 
 __license__ = 'GPL 3'
-__copyright__ = '2011-2014, Tomasz Długosz <tomek3d@gmail.com>'
+__copyright__ = '2011-2019, Tomasz Długosz <tomek3d@gmail.com>'
 __docformat__ = 'restructuredtext en'
 
-import re
-import urllib
 from base64 import b64encode
 from contextlib import closing
+try:
+    from urllib.parse import quote_plus
+except ImportError:
+    from urllib import quote_plus
 
 from lxml import html
 
-from PyQt4.Qt import QUrl
+from qt.core import QUrl
 
 from calibre import browser, url_slash_cleaner
 from calibre.gui2 import open_url
@@ -23,18 +25,28 @@ from calibre.gui2.store.basic_config import BasicStoreConfig
 from calibre.gui2.store.search_result import SearchResult
 from calibre.gui2.store.web_store_dialog import WebStoreDialog
 
+
+def as_base64(data):
+    if not isinstance(data, bytes):
+        data = data.encode('utf-8')
+    ans = b64encode(data)
+    if isinstance(ans, bytes):
+        ans = ans.decode('ascii')
+    return ans
+
+
 class LegimiStore(BasicStoreConfig, StorePlugin):
 
     def open(self, parent=None, detail_item=None, external=False):
         aff_root = 'https://www.a4b-tracking.com/pl/stat-click-text-link/9/58/'
 
-        url = 'http://www.legimi.com/pl/ebooki/'
+        url = 'https://www.legimi.pl/ebooki/'
 
-        aff_url = aff_root + str(b64encode(url))
+        aff_url = aff_root + as_base64(url)
 
         detail_url = None
         if detail_item:
-            detail_url = aff_root + str(b64encode(detail_item))
+            detail_url = aff_root + as_base64(detail_item)
 
         if external or self.config.get('open_external', False):
             open_url(QUrl(url_slash_cleaner(detail_url if detail_url else aff_url)))
@@ -45,48 +57,40 @@ class LegimiStore(BasicStoreConfig, StorePlugin):
             d.exec_()
 
     def search(self, query, max_results=10, timeout=60):
-        url = 'http://www.legimi.com/pl/ebooki/?szukaj=' + urllib.quote_plus(query)
+        url = 'https://www.legimi.pl/ebooki/?sort=score&searchphrase=' + quote_plus(query)
 
         br = browser()
 
         counter = max_results
         with closing(br.open(url, timeout=timeout)) as f:
             doc = html.fromstring(f.read())
-            for data in doc.xpath('//div[@id="listBooks"]/div'):
+            for data in doc.xpath('//div[@class="book-search row auto-clear"]/div'):
                 if counter <= 0:
                     break
 
-                id = ''.join(data.xpath('.//a[1]/@href'))
+                id = ''.join(data.xpath('.//div[@class="panel-body"]/a/@href'))
                 if not id:
                     continue
 
-                cover_url = ''.join(data.xpath('.//img[1]/@src'))
-                title = ''.join(data.xpath('.//span[@class="bookListTitle ellipsis"]/text()'))
-                author = ''.join(data.xpath('.//span[@class="bookListAuthor ellipsis"]/text()'))
-                price = ''.join(data.xpath('.//div[@class="bookListPrice"]/span/text()'))
-
+                cover_url = ''.join(data.xpath('.//div[@class="img-content"]/img/@data-src'))
+                title = ''.join(data.xpath('.//a[@class="book-title clampBookTitle"]/text()'))
+                author = ' '.join(data.xpath('.//div[@class="authors-container clampBookAuthors"]/a/text()'))
                 counter -= 1
 
                 s = SearchResult()
-                s.cover_url = 'http://www.legimi.com/' + cover_url
+                s.cover_url = cover_url
                 s.title = title.strip()
                 s.author = author.strip()
-                s.price = price
-                s.detail_item = 'http://www.legimi.com/' + id.strip()
+                s.detail_item = 'https://www.legimi.pl' + id.strip()
+                s.drm = SearchResult.DRM_UNLOCKED
 
                 yield s
 
     def get_details(self, search_result, timeout):
-        drm_pattern = re.compile("zabezpieczona DRM")
-        formats = []
         br = browser()
-        with closing(br.open(search_result.detail_item, timeout=timeout)) as nf:
+        with closing(br.open(search_result.detail_item, timeout=timeout/2)) as nf:
             idata = html.fromstring(nf.read())
-            formatlist = idata.xpath('.//div[@id="fullBookFormats"]//span[@class="bookFormat"]/text()')
-            for x in formatlist:
-                if x.strip() not in formats:
-                    formats.append(x.strip())
-            drm = drm_pattern.search(''.join(idata.xpath('.//div[@id="fullBookFormats"]/p/text()')))
-            search_result.formats = ', '.join(formats)
-            search_result.drm = SearchResult.DRM_LOCKED if drm else SearchResult.DRM_UNLOCKED
+
+            price = ''.join(idata.xpath('.//section[@class="book-sale-options"]//p[@class="light-text"]/text()'))
+            search_result.price = price.split('bez abonamentu ')[-1]
         return True
