@@ -6,14 +6,14 @@ __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import textwrap
+import copy, textwrap
 from functools import partial
 
 from qt.core import (
     Qt, QIcon, QWidget, QHBoxLayout, QVBoxLayout, QToolButton, QLabel, QFrame, QDialog, QComboBox, QLineEdit,
     QTimer, QMenu, QActionGroup, QAction, QSizePolicy, pyqtSignal)
 
-from calibre.gui2 import error_dialog, question_dialog, gprefs
+from calibre.gui2 import error_dialog, question_dialog, gprefs, config
 from calibre.gui2.widgets import HistoryLineEdit
 from calibre.library.field_metadata import category_icon_map
 from calibre.utils.icu import sort_key
@@ -87,6 +87,7 @@ class TagBrowserMixin(object):  # {{{
         self.tags_view.restriction_error.connect(self.do_restriction_error,
                                                  type=Qt.ConnectionType.QueuedConnection)
         self.tags_view.tag_item_delete.connect(self.do_tag_item_delete)
+        self.tags_view.tag_identifier_delete.connect(self.delete_identifier)
         self.tags_view.apply_tag_to_selected.connect(self.apply_tag_to_selected)
         self.populate_tb_manage_menu(db)
         self.tags_view.model().user_categories_edited.connect(self.user_categories_edited,
@@ -98,9 +99,10 @@ class TagBrowserMixin(object):  # {{{
     def user_categories_edited(self):
         self.library_view.model().refresh()
 
-    def do_restriction_error(self):
+    def do_restriction_error(self, e):
         error_dialog(self.tags_view, _('Invalid search restriction'),
-                         _('The current search restriction is invalid'), show=True)
+                         _('The current search restriction is invalid'),
+                         det_msg=str(e) if e else '', show=True)
 
     def do_add_subcategory(self, on_category_key, new_category_name=None):
         '''
@@ -134,6 +136,9 @@ class TagBrowserMixin(object):  # {{{
         self.tags_view.show_item_at_index(idx)
         # Open the editor on the new item to rename it
         if new_category_name is None:
+            item = m.get_node(idx)
+            item.use_vl = False
+            item.ignore_vl = True
             self.tags_view.edit(idx)
 
     def do_edit_user_categories(self, on_category=None):
@@ -380,6 +385,33 @@ class TagBrowserMixin(object):  # {{{
             self.library_view.model().refresh_ids(set(changes), current_row=self.library_view.currentIndex().row())
             self.tags_view.recount_with_position_based_index()
 
+    def delete_identifier(self, name, in_vl):
+        d = self.current_db.new_api
+        changed = False
+        books_to_use = self.tags_view.model().get_book_ids_to_use() if in_vl else d.all_book_ids()
+        ids = d.all_field_for('identifiers', books_to_use)
+        new_ids = {}
+        for id_ in ids:
+            for identifier_type in ids[id_]:
+                if identifier_type == name:
+                    new_ids[id_] = copy.copy(ids[id_])
+                    new_ids[id_].pop(name)
+                    changed = True
+        if changed:
+            if in_vl:
+                msg = _('The identifier %s will be deleted from books in the '
+                        'current virtual library. Are you sure?')%name
+            else:
+                msg= _('The identifier %s will be deleted from all books. Are you sure?')%name
+            if not question_dialog(self,
+                title=_('Delete identifier'),
+                msg=msg,
+                skip_dialog_name='tag_browser_delete_identifiers',
+                skip_dialog_msg=_('Show this confirmation again')):
+                return
+            d.set_field('identifiers', new_ids)
+            self.tags_view.recount_with_position_based_index()
+
     def edit_enum_values(self, parent, db, key):
         from calibre.gui2.dialogs.enum_values_edit import EnumValuesEdit
         d = EnumValuesEdit(parent, db, key)
@@ -633,8 +665,6 @@ class TagBrowserWidget(QFrame):  # {{{
         self.not_found_label_timer.setSingleShot(True)
         self.not_found_label_timer.timeout.connect(self.not_found_label_timer_event,
                                                    type=Qt.ConnectionType.QueuedConnection)
-        # The Alter Tag Browser button
-        l = self.alter_tb
         self.collapse_all_action = ac = QAction(parent)
         parent.addAction(ac)
         parent.keyboard.register_shortcut('tag browser collapse all',
@@ -642,6 +672,8 @@ class TagBrowserWidget(QFrame):  # {{{
                 action=ac, group=_('Tag browser'))
         connect_lambda(ac.triggered, self, lambda self: self.tags_view.collapseAll())
 
+        # The Configure Tag Browser button
+        l = self.alter_tb
         ac = QAction(parent)
         parent.addAction(ac)
         parent.keyboard.register_shortcut('tag browser alter',
@@ -649,6 +681,11 @@ class TagBrowserWidget(QFrame):  # {{{
                 action=ac, group=_('Tag browser'))
         ac.triggered.connect(l.showMenu)
 
+        l.m.aboutToShow.connect(self.about_to_show_configure_menu)
+        l.m.show_counts_action = ac = l.m.addAction('counts')
+        ac.triggered.connect(self.toggle_counts)
+        l.m.show_avg_rating_action = ac = l.m.addAction('avg rating')
+        ac.triggered.connect(self.toggle_avg_rating)
         sb = l.m.addAction(_('Sort by'))
         sb.m = l.sort_menu = QMenu(l.m)
         sb.setMenu(sb.m)
@@ -707,6 +744,18 @@ class TagBrowserWidget(QFrame):  # {{{
         # self.leak_test_timer.timeout.connect(self.test_for_leak)
         # self.leak_test_timer.start(5000)
 
+    def about_to_show_configure_menu(self):
+        ac = self.alter_tb.m.show_counts_action
+        ac.setText(_('Hide counts') if gprefs['tag_browser_show_counts'] else _('Show counts'))
+        ac = self.alter_tb.m.show_avg_rating_action
+        ac.setText(_('Hide average rating') if config['show_avg_rating'] else _('Show average rating'))
+
+    def toggle_counts(self):
+        gprefs['tag_browser_show_counts'] ^= True
+
+    def toggle_avg_rating(self):
+        config['show_avg_rating'] ^= True
+
     def save_state(self):
         gprefs.set('tag browser search box visible', self.toggle_search_button.isChecked())
 
@@ -826,7 +875,7 @@ class TagBrowserWidget(QFrame):  # {{{
         self.not_found_label.setVisible(False)
 
     def keyPressEvent(self, ev):
-        if ev.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return) and self.find_text:
+        if ev.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return) and self.item_search.hasFocus():
             self.find()
             ev.accept()
             return
