@@ -56,6 +56,7 @@ d['remove_unused_classes'] = False
 d['merge_identical_selectors'] = False
 d['merge_identical_selectors'] = False
 d['merge_rules_with_identical_properties'] = False
+d['remove_unreferenced_sheets'] = True
 d['global_book_toolbar'] = [
 'new-file', 'open-book',  'save-book', None, 'global-undo', 'global-redo', 'create-checkpoint', None, 'donate', 'user-manual']
 d['global_tools_toolbar'] = [
@@ -83,6 +84,7 @@ d['preserve_aspect_ratio_when_inserting_image'] = False
 d['file_list_shows_full_pathname'] = False
 d['auto_link_stylesheets'] = True
 d['check_external_link_anchors'] = True
+d['remove_ncx'] = True
 del d
 
 ucase_map = {l:string.ascii_uppercase[i] for i, l in enumerate(string.ascii_lowercase)}
