@@ -1,27 +1,28 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import unicodedata, re, os, cPickle, textwrap
+import re
+import textwrap
 from bisect import bisect
 from functools import partial
-from collections import defaultdict
+from qt.core import (
+    QAbstractItemModel, QAbstractListModel, QApplication, QCheckBox, QGridLayout,
+    QHBoxLayout, QIcon, QInputMethodEvent, QLabel, QListView, QMenu, QMimeData,
+    QModelIndex, QPen, QPushButton, QSize, QSizePolicy, QSplitter,
+    QStyledItemDelegate, Qt, QToolButton, QTreeView, pyqtSignal, QAbstractItemView, QDialogButtonBox
+)
 
-from PyQt5.Qt import (
-    QAbstractItemModel, QModelIndex, Qt, pyqtSignal, QApplication,
-    QTreeView, QSize, QGridLayout, QAbstractListModel, QListView, QPen, QMenu,
-    QStyledItemDelegate, QSplitter, QLabel, QSizePolicy, QIcon, QMimeData,
-    QPushButton, QToolButton, QInputMethodEvent)
-
-from calibre.constants import plugins, cache_dir
-from calibre.gui2.widgets2 import HistoryLineEdit2
 from calibre.gui2.tweak_book import tprefs
-from calibre.gui2.tweak_book.widgets import Dialog, BusyCursor
-from calibre.utils.icu import safe_chr as chr, icu_unicode_version, character_name_from_code
+from calibre.gui2.tweak_book.widgets import BusyCursor, Dialog
+from calibre.gui2.widgets2 import HistoryLineEdit2
+from calibre.utils.icu import safe_chr as codepoint_to_chr
+from calibre.utils.unicode_names import character_name_from_code, points_for_word
+from calibre_extensions.progress_indicator import set_no_activate_on_click
+from polyglot.builtins import map, range, unicode_type
 
 ROOT = QModelIndex()
 
@@ -33,55 +34,20 @@ non_printing = {
     0x206e: 'nads', 0x206f: 'nods', 0x20: 'sp', 0x7f: 'del', 0x2e3a: '2m', 0x2e3b: '3m', 0xad: 'shy',
 }
 
+
 # Searching {{{
-
-
-def load_search_index():
-    topchar = 0x10ffff
-    ver = (1, topchar, icu_unicode_version or unicodedata.unidata_version)  # Increment this when you make any changes to the index
-    name_map = {}
-    path = os.path.join(cache_dir(), 'unicode-name-index.pickle')
-    if os.path.exists(path):
-        with open(path, 'rb') as f:
-            name_map = cPickle.load(f)
-        if name_map.pop('calibre-nm-version:', None) != ver:
-            name_map = {}
-    if not name_map:
-        name_map = defaultdict(set)
-        for x in xrange(1, topchar + 1):
-            for word in character_name_from_code(x).split():
-                name_map[word.lower()].add(x)
-        from calibre.ebooks.html_entities import html5_entities
-        for name, char in html5_entities.iteritems():
-            try:
-                name_map[name.lower()].add(ord(char))
-            except TypeError:
-                continue
-        name_map['nnbsp'].add(0x202F)
-        name_map['calibre-nm-version:'] = ver
-        cPickle.dump(dict(name_map), open(path, 'wb'), -1)
-        del name_map['calibre-nm-version:']
-    return name_map
-
-
-_index = None
-
-
 def search_for_chars(query, and_tokens=False):
-    global _index
-    if _index is None:
-        _index = load_search_index()
     ans = set()
-    for token in query.split():
+    for i, token in enumerate(query.split()):
         token = token.lower()
         m = re.match(r'(?:[u]\+)([a-f0-9]+)', token)
         if m is not None:
             chars = {int(m.group(1), 16)}
         else:
-            chars = _index.get(token, None)
+            chars = points_for_word(token)
         if chars is not None:
             if and_tokens:
-                ans &= chars
+                ans = chars if i == 0 else (ans & chars)
             else:
                 ans |= chars
     return sorted(ans)
@@ -436,19 +402,19 @@ class CategoryModel(QAbstractItemModel):
             return ROOT
         return self.index(pid - 1, 0)
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         if not index.isValid():
             return None
         pid = index.internalId()
         if pid == 0:
-            if role == Qt.DisplayRole:
+            if role == Qt.ItemDataRole.DisplayRole:
                 return self.categories[index.row()][0]
-            if role == Qt.FontRole:
+            if role == Qt.ItemDataRole.FontRole:
                 return self.bold_font
-            if role == Qt.DecorationRole and index.row() == 0:
+            if role == Qt.ItemDataRole.DecorationRole and index.row() == 0:
                 return self.fav_icon
         else:
-            if role == Qt.DisplayRole:
+            if role == Qt.ItemDataRole.DisplayRole:
                 item = self.categories[pid - 1][1][index.row()]
                 return item[0]
         return None
@@ -461,7 +427,7 @@ class CategoryModel(QAbstractItemModel):
                     return (_('Favorites'), list(tprefs['charmap_favorites']))
             else:
                 item = self.categories[pid - 1][1][index.row()]
-                return (item[0], list(xrange(item[1][0], item[1][1] + 1)))
+                return (item[0], list(range(item[1][0], item[1][1] + 1)))
 
     def get_char_info(self, char_code):
         ipos = bisect(self.starts, char_code) - 1
@@ -469,7 +435,7 @@ class CategoryModel(QAbstractItemModel):
             category, subcategory = self.category_map[self.starts[ipos]]
         except IndexError:
             category = subcategory = _('Unknown')
-        return category, subcategory, (character_name_from_code(char_code) or _('Unknown'))
+        return category, subcategory, character_name_from_code(char_code)
 
 
 class CategoryDelegate(QStyledItemDelegate):
@@ -494,9 +460,7 @@ class CategoryView(QTreeView):
         self.setAnimated(True)
         self.activated.connect(self.item_activated)
         self.clicked.connect(self.item_activated)
-        pi = plugins['progress_indicator'][0]
-        if hasattr(pi, 'set_no_activate_on_click'):
-            pi.set_no_activate_on_click(self)
+        set_no_activate_on_click(self)
         self.initialized = False
         self.setExpandsOnDoubleClick(False)
 
@@ -537,33 +501,33 @@ class CharModel(QAbstractListModel):
         return len(self.chars)
 
     def data(self, index, role):
-        if role == Qt.UserRole and -1 < index.row() < len(self.chars):
+        if role == Qt.ItemDataRole.UserRole and -1 < index.row() < len(self.chars):
             return self.chars[index.row()]
         return None
 
     def flags(self, index):
-        ans = Qt.ItemIsEnabled
+        ans = Qt.ItemFlag.ItemIsEnabled
         if self.allow_dnd:
-            ans |= Qt.ItemIsSelectable
-            ans |= Qt.ItemIsDragEnabled if index.isValid() else Qt.ItemIsDropEnabled
+            ans |= Qt.ItemFlag.ItemIsSelectable
+            ans |= Qt.ItemFlag.ItemIsDragEnabled if index.isValid() else Qt.ItemFlag.ItemIsDropEnabled
         return ans
 
     def supportedDropActions(self):
-        return Qt.MoveAction
+        return Qt.DropAction.MoveAction
 
     def mimeTypes(self):
         return ['application/calibre_charcode_indices']
 
     def mimeData(self, indexes):
-        data = ','.join(str(i.row()) for i in indexes)
+        data = ','.join(unicode_type(i.row()) for i in indexes)
         md = QMimeData()
         md.setData('application/calibre_charcode_indices', data.encode('utf-8'))
         return md
 
     def dropMimeData(self, md, action, row, column, parent):
-        if action != Qt.MoveAction or not md.hasFormat('application/calibre_charcode_indices') or row < 0 or column != 0:
+        if action != Qt.DropAction.MoveAction or not md.hasFormat('application/calibre_charcode_indices') or row < 0 or column != 0:
             return False
-        indices = map(int, bytes(md.data('application/calibre_charcode_indices')).split(','))
+        indices = list(map(int, bytes(md.data('application/calibre_charcode_indices')).decode('ascii').split(',')))
         codes = [self.chars[x] for x in indices]
         for x in indices:
             self.chars[x] = None
@@ -589,7 +553,7 @@ class CharDelegate(QStyledItemDelegate):
     def paint(self, painter, option, index):
         QStyledItemDelegate.paint(self, painter, option, index)
         try:
-            charcode = int(index.data(Qt.UserRole))
+            charcode = int(index.data(Qt.ItemDataRole.UserRole))
         except (TypeError, ValueError):
             return
         painter.save()
@@ -605,12 +569,12 @@ class CharDelegate(QStyledItemDelegate):
         f = option.font
         f.setPixelSize(option.rect.height() - 8)
         painter.setFont(f)
-        painter.drawText(option.rect, Qt.AlignHCenter | Qt.AlignBottom | Qt.TextSingleLine, chr(charcode))
+        painter.drawText(option.rect, Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignBottom | Qt.TextFlag.TextSingleLine, codepoint_to_chr(charcode))
 
     def paint_non_printing(self, painter, option, charcode):
         text = self.np_pat.sub(r'\n\1', non_printing[charcode])
-        painter.drawText(option.rect, Qt.AlignCenter | Qt.TextWordWrap | Qt.TextWrapAnywhere, text)
-        painter.setPen(QPen(Qt.DashLine))
+        painter.drawText(option.rect, Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWordWrap | Qt.TextFlag.TextWrapAnywhere, text)
+        painter.setPen(QPen(Qt.PenStyle.DashLine))
         painter.drawRect(option.rect.adjusted(1, 1, -1, -1))
 
 
@@ -625,29 +589,27 @@ class CharView(QListView):
         self._model = CharModel(self)
         self.setModel(self._model)
         self.delegate = CharDelegate(self)
-        self.setResizeMode(self.Adjust)
+        self.setResizeMode(QListView.ResizeMode.Adjust)
         self.setItemDelegate(self.delegate)
-        self.setFlow(self.LeftToRight)
+        self.setFlow(QListView.Flow.LeftToRight)
         self.setWrapping(True)
         self.setMouseTracking(True)
         self.setSpacing(2)
         self.setUniformItemSizes(True)
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.customContextMenuRequested.connect(self.context_menu)
         self.showing_favorites = False
-        pi = plugins['progress_indicator'][0]
-        if hasattr(pi, 'set_no_activate_on_click'):
-            pi.set_no_activate_on_click(self)
+        set_no_activate_on_click(self)
         self.activated.connect(self.item_activated)
         self.clicked.connect(self.item_activated)
 
     def item_activated(self, index):
         try:
-            char_code = int(self.model().data(index, Qt.UserRole))
+            char_code = int(self.model().data(index, Qt.ItemDataRole.UserRole))
         except (TypeError, ValueError):
             pass
         else:
-            self.char_selected.emit(chr(char_code))
+            self.char_selected.emit(codepoint_to_chr(char_code))
 
     def set_allow_drag_and_drop(self, enabled):
         if not enabled:
@@ -656,7 +618,7 @@ class CharView(QListView):
             self.setDropIndicatorShown(True)
             self._model.allow_dnd = False
         else:
-            self.setSelectionMode(self.ExtendedSelection)
+            self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
             self.viewport().setAcceptDrops(True)
             self.setDragEnabled(True)
             self.setAcceptDrops(True)
@@ -677,14 +639,14 @@ class CharView(QListView):
             if row != self.last_mouse_idx:
                 self.last_mouse_idx = row
                 try:
-                    char_code = int(self.model().data(index, Qt.UserRole))
+                    char_code = int(self.model().data(index, Qt.ItemDataRole.UserRole))
                 except (TypeError, ValueError):
                     pass
                 else:
                     self.show_name.emit(char_code)
-            self.setCursor(Qt.PointingHandCursor)
+            self.setCursor(Qt.CursorShape.PointingHandCursor)
         else:
-            self.setCursor(Qt.ArrowCursor)
+            self.setCursor(Qt.CursorShape.ArrowCursor)
             self.show_name.emit(-1)
             self.last_mouse_idx = -1
         return QListView.mouseMoveEvent(self, ev)
@@ -693,14 +655,14 @@ class CharView(QListView):
         index = self.indexAt(pos)
         if index.isValid():
             try:
-                char_code = int(self.model().data(index, Qt.UserRole))
+                char_code = int(self.model().data(index, Qt.ItemDataRole.UserRole))
             except (TypeError, ValueError):
                 pass
             else:
                 m = QMenu(self)
-                m.addAction(QIcon(I('edit-copy.png')), _('Copy %s to clipboard') % chr(char_code), partial(self.copy_to_clipboard, char_code))
+                m.addAction(QIcon(I('edit-copy.png')), _('Copy %s to clipboard') % codepoint_to_chr(char_code), partial(self.copy_to_clipboard, char_code))
                 m.addAction(QIcon(I('rating.png')),
-                            (_('Remove %s from favorites') if self.showing_favorites else _('Add %s to favorites')) % chr(char_code),
+                            (_('Remove %s from favorites') if self.showing_favorites else _('Add %s to favorites')) % codepoint_to_chr(char_code),
                             partial(self.remove_from_favorites, char_code))
                 if self.showing_favorites:
                     m.addAction(_('Restore favorites to defaults'), self.restore_defaults)
@@ -714,7 +676,7 @@ class CharView(QListView):
 
     def copy_to_clipboard(self, char_code):
         c = QApplication.clipboard()
-        c.setText(chr(char_code))
+        c.setText(codepoint_to_chr(char_code))
 
     def remove_from_favorites(self, char_code):
         existing = tprefs['charmap_favorites']
@@ -741,40 +703,40 @@ class CharSelect(Dialog):
         self.l = l = QGridLayout(self)
         self.setLayout(l)
 
-        self.bb.setStandardButtons(self.bb.Close)
-        self.rearrange_button = b = self.bb.addButton(_('Re-arrange favorites'), self.bb.ActionRole)
+        self.bb.setStandardButtons(QDialogButtonBox.StandardButton.Close)
+        self.rearrange_button = b = self.bb.addButton(_('Re-arrange favorites'), QDialogButtonBox.ButtonRole.ActionRole)
         b.setCheckable(True)
         b.setChecked(False)
         b.setVisible(False)
         b.setDefault(True)
 
         self.splitter = s = QSplitter(self)
-        s.setFocusPolicy(Qt.NoFocus)
+        s.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         s.setChildrenCollapsible(False)
 
         self.search = h = HistoryLineEdit2(self)
         h.setToolTip(textwrap.fill(_(
-            'Search for unicode characters by using the English names or nicknames.'
+            'Search for Unicode characters by using the English names or nicknames.'
             ' You can also search directly using a character code. For example, the following'
             ' searches will all yield the no-break space character: U+A0, nbsp, no-break')))
         h.initialize('charmap_search')
         h.setPlaceholderText(_('Search by name, nickname or character code'))
         self.search_button = b = QPushButton(_('&Search'))
-        b.setFocusPolicy(Qt.NoFocus)
+        b.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         h.returnPressed.connect(self.do_search)
         b.clicked.connect(self.do_search)
         self.clear_button = cb = QToolButton(self)
         cb.setIcon(QIcon(I('clear_left.png')))
-        cb.setFocusPolicy(Qt.NoFocus)
+        cb.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         cb.setText(_('Clear search'))
         cb.clicked.connect(self.clear_search)
         l.addWidget(h), l.addWidget(b, 0, 1), l.addWidget(cb, 0, 2)
 
         self.category_view = CategoryView(self)
-        self.category_view.setFocusPolicy(Qt.NoFocus)
+        self.category_view.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         l.addWidget(s, 1, 0, 1, 3)
         self.char_view = CharView(self)
-        self.char_view.setFocusPolicy(Qt.NoFocus)
+        self.char_view.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         self.rearrange_button.toggled[bool].connect(self.set_allow_drag_and_drop)
         self.category_view.category_selected.connect(self.show_chars)
         self.char_view.show_name.connect(self.show_char_info)
@@ -782,23 +744,30 @@ class CharSelect(Dialog):
         s.addWidget(self.category_view), s.addWidget(self.char_view)
 
         self.char_info = la = QLabel('\xa0')
-        la.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        la.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
         l.addWidget(la, 2, 0, 1, 3)
 
         self.rearrange_msg = la = QLabel(_(
             'Drag and drop characters to re-arrange them. Click the "Re-arrange" button again when you are done.'))
-        la.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        la.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
         la.setVisible(False)
         l.addWidget(la, 3, 0, 1, 3)
-        l.addWidget(self.bb, 4, 0, 1, 3)
-        self.char_view.setFocus(Qt.OtherFocusReason)
+        self.h = h = QHBoxLayout()
+        h.setContentsMargins(0, 0, 0, 0)
+        self.match_any = mm = QCheckBox(_('Match any word'))
+        mm.setToolTip(_('When searching return characters whose names match any of the specified words'))
+        mm.setChecked(tprefs.get('char_select_match_any', True))
+        connect_lambda(mm.stateChanged, self, lambda self: tprefs.set('char_select_match_any', self.match_any.isChecked()))
+        h.addWidget(mm), h.addStretch(), h.addWidget(self.bb)
+        l.addLayout(h, 4, 0, 1, 3)
+        self.char_view.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def do_search(self):
-        text = unicode(self.search.text()).strip()
+        text = unicode_type(self.search.text()).strip()
         if not text:
             return self.clear_search()
         with BusyCursor():
-            chars = search_for_chars(text)
+            chars = search_for_chars(text, and_tokens=not self.match_any.isChecked())
         self.show_chars(_('Search'), chars)
 
     def clear_search(self):
@@ -838,7 +807,7 @@ class CharSelect(Dialog):
         self.raise_()
 
     def char_selected(self, c):
-        if QApplication.keyboardModifiers() & Qt.CTRL:
+        if QApplication.keyboardModifiers() & Qt.Modifier.CTRL:
             self.hide()
         if self.parent() is None or self.parent().focusWidget() is None:
             QApplication.clipboard().setText(c)
