@@ -1,22 +1,21 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
 
 from collections import defaultdict
 from threading import Thread
 
-from PyQt5.Qt import (
-    QVBoxLayout, QTextBrowser, QProgressBar, Qt, QWidget, QStackedWidget,
-    QLabel, QSizePolicy, pyqtSignal, QIcon, QInputDialog
+from qt.core import (
+    QCheckBox, QHBoxLayout, QIcon, QInputDialog, QLabel, QProgressBar, QSizePolicy,
+    QStackedWidget, Qt, QTextBrowser, QVBoxLayout, QWidget, pyqtSignal, QDialogButtonBox
 )
 
 from calibre.gui2 import error_dialog
-from calibre.gui2.tweak_book import current_container, set_current_container, editors
+from calibre.gui2.tweak_book import current_container, editors, set_current_container, tprefs
 from calibre.gui2.tweak_book.boss import get_boss
 from calibre.gui2.tweak_book.widgets import Dialog
+from polyglot.builtins import iteritems
 
 
 def get_data(name):
@@ -31,6 +30,8 @@ def set_data(name, val):
         editors[name].replace_data(val, only_if_different=False)
     else:
         with current_container().open(name, 'wb') as f:
+            if isinstance(val, str):
+                val = val.encode('utf-8')
             f.write(val)
     get_boss().set_modified()
 
@@ -41,7 +42,7 @@ class CheckExternalLinks(Dialog):
 
     def __init__(self, parent=None):
         Dialog.__init__(self, _('Check external links'), 'check-external-links-dialog', parent)
-        self.progress_made.connect(self.on_progress_made, type=Qt.QueuedConnection)
+        self.progress_made.connect(self.on_progress_made, type=Qt.ConnectionType.QueuedConnection)
 
     def show(self):
         if self.rb.isEnabled():
@@ -58,14 +59,14 @@ class CheckExternalLinks(Dialog):
     def setup_ui(self):
         self.pb = pb = QProgressBar(self)
         pb.setTextVisible(True)
-        pb.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        pb.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
         pb.setRange(0, 0)
         self.w = w = QWidget(self)
         self.w.l = l = QVBoxLayout(w)
         l.addStretch(), l.addWidget(pb)
         self.w.la = la = QLabel(_('Checking external links, please wait...'))
         la.setStyleSheet('QLabel { font-size: 20px; font-weight: bold }')
-        l.addWidget(la, 0, Qt.AlignCenter), l.addStretch()
+        l.addWidget(la, 0, Qt.AlignmentFlag.AlignCenter), l.addStretch()
 
         self.l = l = QVBoxLayout(self)
         self.results = QTextBrowser(self)
@@ -74,12 +75,22 @@ class CheckExternalLinks(Dialog):
         self.stack = s = QStackedWidget(self)
         s.addWidget(w), s.addWidget(self.results)
         l.addWidget(s)
-        l.addWidget(self.bb)
-        self.bb.setStandardButtons(self.bb.Close)
-        self.rb = b = self.bb.addButton(_('&Refresh'), self.bb.ActionRole)
+        self.bh = h = QHBoxLayout()
+        self.check_anchors = ca = QCheckBox(_('Check &anchors'))
+        ca.setToolTip(_('Check HTML anchors in links (the part after the #).\n'
+            ' This can be a little slow, since it requires downloading and parsing all the HTML pages.'))
+        ca.setChecked(tprefs.get('check_external_link_anchors', True))
+        ca.stateChanged.connect(self.anchors_changed)
+        h.addWidget(ca), h.addStretch(100), h.addWidget(self.bb)
+        l.addLayout(h)
+        self.bb.setStandardButtons(QDialogButtonBox.StandardButton.Close)
+        self.rb = b = self.bb.addButton(_('&Refresh'), QDialogButtonBox.ButtonRole.ActionRole)
         b.setIcon(QIcon(I('view-refresh.png')))
         b.clicked.connect(self.refresh)
 
+    def anchors_changed(self):
+        tprefs.set('check_external_link_anchors', self.check_anchors.isChecked())
+
     def sizeHint(self):
         ans = Dialog.sizeHint(self)
         ans.setHeight(600)
@@ -91,7 +102,7 @@ class CheckExternalLinks(Dialog):
         self.tb = None
         self.errors = []
         try:
-            self.errors = check_external_links(current_container(), self.progress_made.emit)
+            self.errors = check_external_links(current_container(), self.progress_made.emit, check_anchors=self.check_anchors.isChecked())
         except Exception:
             import traceback
             self.tb = traceback.format_exc()
@@ -105,7 +116,7 @@ class CheckExternalLinks(Dialog):
             self.rb.setEnabled(True)
             if self.tb is not None:
                 return error_dialog(self, _('Checking failed'), _(
-                    'There was an error while checking links, click "Show Details" for more information'),
+                    'There was an error while checking links, click "Show details" for more information'),
                              det_msg=self.tb, show=True)
             if not self.errors:
                 self.results.setText(_('No broken links found'))
@@ -140,7 +151,7 @@ class CheckExternalLinks(Dialog):
             for name, href in {(l[0], l[1]) for l in err[0]}:
                 nmap[name].add(href)
 
-            for name, hrefs in nmap.iteritems():
+            for name, hrefs in iteritems(nmap):
                 raw = oraw = get_data(name)
                 for href in hrefs:
                     raw = raw.replace(href, newurl)
