@@ -1,19 +1,19 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os
 
-from PyQt5.Qt import QTextDocument, QTextCursor, QPlainTextDocumentLayout
+from qt.core import QTextDocument, QTextCursor, QPlainTextDocumentLayout
 
 from calibre.gui2.tweak_book import tprefs
 from calibre.gui2.tweak_book.editor.text import get_highlighter as calibre_highlighter, SyntaxHighlighter
 from calibre.gui2.tweak_book.editor.themes import get_theme, highlight_to_char_format
 from calibre.gui2.tweak_book.editor.syntax.utils import format_for_pygments_token, NULL_FMT
+from polyglot.builtins import iteritems, range
 
 
 class QtHighlighter(QTextDocument):
@@ -46,7 +46,7 @@ class QtHighlighter(QTextDocument):
                     afs = ()
                 for af in afs:
                     start = dest_block.position() + af.start
-                    c.setPosition(start), c.setPosition(start + af.length, c.KeepAnchor)
+                    c.setPosition(start), c.setPosition(start + af.length, QTextCursor.MoveMode.KeepAnchor)
                     c.setCharFormat(af.format)
                 cursor.insertBlock()
                 cursor.setCharFormat(NULL_FMT)
@@ -59,7 +59,7 @@ class NullHighlighter(object):
         self.lines = text.splitlines()
 
     def copy_lines(self, lo, hi, cursor):
-        for i in xrange(lo, hi):
+        for i in range(lo, hi):
             cursor.insertText(self.lines[i])
             cursor.insertBlock()
 
@@ -83,7 +83,7 @@ class PygmentsHighlighter(object):
 
     def __init__(self, text, lexer):
         theme, cache = get_theme(tprefs['editor_theme']), {}
-        theme = {k:highlight_to_char_format(v) for k, v in theme.iteritems()}
+        theme = {k:highlight_to_char_format(v) for k, v in iteritems(theme)}
         theme[None] = NULL_FMT
 
         def fmt(token):
@@ -101,7 +101,7 @@ class PygmentsHighlighter(object):
                     continue
 
     def copy_lines(self, lo, hi, cursor):
-        for i in xrange(lo, hi):
+        for i in range(lo, hi):
             for fmt, text in self.lines[i]:
                 cursor.insertText(text, fmt)
             cursor.setCharFormat(NULL_FMT)
