@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,10 +8,10 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 import sys, os, re
 from functools import partial
 
-from PyQt5.Qt import (
+from qt.core import (
     QGridLayout, QToolButton, QIcon, QRadioButton, QMenu, QApplication, Qt,
     QSize, QWidget, QLabel, QStackedLayout, QPainter, QRect, QVBoxLayout,
-    QCursor, QEventLoop, QKeySequence, pyqtSignal, QTimer, QHBoxLayout)
+    QCursor, QEventLoop, QKeySequence, pyqtSignal, QTimer, QHBoxLayout, QDialogButtonBox)
 
 from calibre.ebooks.oeb.polish.container import Container
 from calibre.ebooks.oeb.polish.utils import guess_type
@@ -24,6 +23,7 @@ from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.gui2.widgets2 import HistoryLineEdit2
 from calibre.utils.filenames import samefile
 from calibre.utils.icu import numeric_sort_key
+from polyglot.builtins import iteritems, unicode_type, map
 
 
 class BusyWidget(QWidget):  # {{{
@@ -34,10 +34,10 @@ class BusyWidget(QWidget):  # {{{
         self.setLayout(l)
         l.addStretch(10)
         self.pi = ProgressIndicator(self, 128)
-        l.addWidget(self.pi, alignment=Qt.AlignHCenter)
+        l.addWidget(self.pi, alignment=Qt.AlignmentFlag.AlignHCenter)
         self.dummy = QLabel('<h2>\xa0')
         l.addSpacing(10)
-        l.addWidget(self.dummy, alignment=Qt.AlignHCenter)
+        l.addWidget(self.dummy, alignment=Qt.AlignmentFlag.AlignHCenter)
         l.addStretch(10)
         self.text = _('Calculating differences, please wait...')
 
@@ -50,9 +50,9 @@ class BusyWidget(QWidget):  # {{{
         f.setBold(True)
         f.setPointSize(20)
         p.setFont(f)
-        p.setPen(Qt.SolidLine)
+        p.setPen(Qt.PenStyle.SolidLine)
         r = QRect(0, self.dummy.geometry().top() + 10, self.geometry().width(), 150)
-        p.drawText(r, Qt.AlignHCenter | Qt.AlignTop | Qt.TextSingleLine, self.text)
+        p.drawText(r, Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignTop | Qt.TextFlag.TextSingleLine, self.text)
         p.end()
 # }}}
 
@@ -79,11 +79,11 @@ def changed_files(list_of_names1, list_of_names2, get_data1, get_data2):
     removals = list_of_names1 - common_names
     adds = set(list_of_names2 - common_names)
     adata, rdata = {a:get_data2(a) for a in adds}, {r:get_data1(r) for r in removals}
-    ahash = {a:hash(d) for a, d in adata.iteritems()}
-    rhash = {r:hash(d) for r, d in rdata.iteritems()}
+    ahash = {a:hash(d) for a, d in iteritems(adata)}
+    rhash = {r:hash(d) for r, d in iteritems(rdata)}
     renamed_names, removed_names, added_names = {}, set(), set()
-    for name, rh in rhash.iteritems():
-        for n, ah in ahash.iteritems():
+    for name, rh in iteritems(rhash):
+        for n, ah in iteritems(ahash):
             if ah == rh:
                 renamed_names[name] = n
                 adds.discard(n)
@@ -118,6 +118,8 @@ def get_decoded_raw(name):
                     enc = 'utf-8'
             else:
                 enc = force_encoding(raw, verbose=True)
+            if isinstance(enc, bytes):
+                enc = enc.decode('utf-8', 'ignore')
             try:
                 raw = raw.decode(enc)
             except (LookupError, ValueError):
@@ -129,7 +131,7 @@ def get_decoded_raw(name):
 
 
 def string_diff(left, right, left_syntax=None, right_syntax=None, left_name='left', right_name='right'):
-    left, right = unicode(left), unicode(right)
+    left, right = unicode_type(left), unicode_type(right)
     cache = Cache()
     cache.set_left(left_name, left), cache.set_right(right_name, right)
     changed_names = {} if left == right else {left_name:right_name}
@@ -139,7 +141,8 @@ def string_diff(left, right, left_syntax=None, right_syntax=None, left_name='lef
 def file_diff(left, right):
     (raw1, syntax1), (raw2, syntax2) = map(get_decoded_raw, (left, right))
     if type(raw1) is not type(raw2):
-        raw1, raw2 = open(left, 'rb').read(), open(right, 'rb').read()
+        with open(left, 'rb') as f1, open(right, 'rb') as f2:
+            raw1, raw2 = f1.read(), f2.read()
     cache = Cache()
     cache.set_left(left, raw1), cache.set_right(right, raw2)
     changed_names = {} if raw1 == raw2 else {left:right}
@@ -213,9 +216,9 @@ class Diff(Dialog):
         self.show_open_in_editor = show_open_in_editor
         self.revert_button_msg = revert_button_msg
         Dialog.__init__(self, _('Differences between books'), 'diff-dialog', parent=parent)
-        self.setWindowFlags(self.windowFlags() | Qt.WindowMinMaxButtonsHint)
+        self.setWindowFlags(self.windowFlags() | Qt.WindowType.WindowMinMaxButtonsHint)
         if show_as_window:
-            self.setWindowFlags(Qt.Window)
+            self.setWindowFlags(Qt.WindowType.Window)
         self.view.line_activated.connect(self.line_activated)
 
     def sizeHint(self):
@@ -239,34 +242,34 @@ class Diff(Dialog):
         r = l.rowCount()
         self.bp = b = QToolButton(self)
         b.setIcon(QIcon(I('back.png')))
-        b.clicked.connect(partial(self.view.next_change, -1))
+        connect_lambda(b.clicked, self, lambda self: self.view.next_change(-1))
         b.setToolTip(_('Go to previous change') + ' [p]')
-        b.setText(_('&Previous change')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        b.setText(_('&Previous change')), b.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 0)
 
         self.bn = b = QToolButton(self)
         b.setIcon(QIcon(I('forward.png')))
-        b.clicked.connect(partial(self.view.next_change, 1))
+        connect_lambda(b.clicked, self, lambda self: self.view.next_change(1))
         b.setToolTip(_('Go to next change') + ' [n]')
-        b.setText(_('&Next change')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        b.setText(_('&Next change')), b.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 1)
 
         self.search = s = HistoryLineEdit2(self)
         s.initialize('diff_search_history')
         l.addWidget(s, r, 2)
         s.setPlaceholderText(_('Search for text'))
-        s.returnPressed.connect(partial(self.do_search, False))
+        connect_lambda(s.returnPressed, self, lambda self: self.do_search(False))
         self.sbn = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-down.png')))
-        b.clicked.connect(partial(self.do_search, False))
+        connect_lambda(b.clicked, self, lambda self: self.do_search(False))
         b.setToolTip(_('Find next match'))
-        b.setText(_('Next &match')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        b.setText(_('Next &match')), b.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 3)
         self.sbp = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-up.png')))
-        b.clicked.connect(partial(self.do_search, True))
+        connect_lambda(b.clicked, self, lambda self: self.do_search(True))
         b.setToolTip(_('Find previous match'))
-        b.setText(_('P&revious match')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        b.setText(_('P&revious match')), b.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 4)
         self.lb = b = QRadioButton(_('Left panel'), self)
         b.setToolTip(_('Perform search in the left panel'))
@@ -277,9 +280,9 @@ class Diff(Dialog):
         b.setChecked(True)
         self.pb = b = QToolButton(self)
         b.setIcon(QIcon(I('config.png')))
-        b.setText(_('&Options')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        b.setText(_('&Options')), b.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
         b.setToolTip(_('Change how the differences are displayed'))
-        b.setPopupMode(b.InstantPopup)
+        b.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
         m = QMenu(b)
         b.setMenu(m)
         cm = self.cm = QMenu(_('Lines of context around each change'))
@@ -296,16 +299,16 @@ class Diff(Dialog):
         self.names = QLabel('')
         self.hl.addWidget(self.names, r)
 
-        self.bb.setStandardButtons(self.bb.Close)
+        self.bb.setStandardButtons(QDialogButtonBox.StandardButton.Close)
         if self.revert_button_msg is not None:
-            self.rvb = b = self.bb.addButton(self.revert_button_msg, self.bb.ActionRole)
+            self.rvb = b = self.bb.addButton(self.revert_button_msg, QDialogButtonBox.ButtonRole.ActionRole)
             b.setIcon(QIcon(I('edit-undo.png'))), b.setAutoDefault(False)
             b.clicked.connect(self.revert_requested)
             b.clicked.connect(self.reject)
-        self.bb.button(self.bb.Close).setDefault(True)
+        self.bb.button(QDialogButtonBox.StandardButton.Close).setDefault(True)
         self.hl.addWidget(self.bb, r)
 
-        self.view.setFocus(Qt.OtherFocusReason)
+        self.view.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def break_cycles(self):
         self.view = None
@@ -316,7 +319,7 @@ class Diff(Dialog):
                 pass
 
     def do_search(self, reverse):
-        text = unicode(self.search.text())
+        text = unicode_type(self.search.text())
         if not text.strip():
             return
         v = self.view.view.left if self.lb.isChecked() else self.view.view.right
@@ -351,8 +354,8 @@ class Diff(Dialog):
         self.stacks.setCurrentIndex(0)
         self.busy.setVisible(True)
         self.busy.pi.startAnimation()
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-        QApplication.processEvents(QEventLoop.ExcludeUserInputEvents | QEventLoop.ExcludeSocketNotifiers)
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
+        QApplication.processEvents(QEventLoop.ProcessEventsFlag.ExcludeUserInputEvents | QEventLoop.ProcessEventsFlag.ExcludeSocketNotifiers)
 
     def __exit__(self, *args):
         self.busy.pi.stopAnimation()
@@ -397,7 +400,7 @@ class Diff(Dialog):
 
     def dir_diff(self, left, right, identical_msg=None):
         with self:
-            identical = self.apply_diff(identical_msg or _('The directories are identical'), *dir_diff(left, right))
+            identical = self.apply_diff(identical_msg or _('The folders are identical'), *dir_diff(left, right))
             self.view.finalize()
         if identical:
             self.reject()
@@ -419,7 +422,7 @@ class Diff(Dialog):
         kwargs = lambda name: {'context':self.context, 'beautify':self.beautify, 'syntax':syntax_map.get(name, None)}
 
         if isinstance(changed_names, dict):
-            for name, other_name in sorted(changed_names.iteritems(), key=lambda x:numeric_sort_key(x[0])):
+            for name, other_name in sorted(iteritems(changed_names), key=lambda x:numeric_sort_key(x[0])):
                 args = (name, other_name, cache.left(name), cache.right(other_name))
                 add(args, kwargs(name))
         else:
@@ -435,25 +438,25 @@ class Diff(Dialog):
             args = (name, _('[%s was removed]') % name, cache.left(name), None)
             add(args, kwargs(name))
 
-        for name, new_name in sorted(renamed_names.iteritems(), key=lambda x:numeric_sort_key(x[0])):
+        for name, new_name in sorted(iteritems(renamed_names), key=lambda x:numeric_sort_key(x[0])):
             args = (name, new_name, None, None)
             add(args, kwargs(name))
 
     def keyPressEvent(self, ev):
         if not self.view.handle_key(ev):
-            if ev.key() in (Qt.Key_Enter, Qt.Key_Return):
+            if ev.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
                 return  # The enter key is used by the search box, so prevent it closing the dialog
-            if ev.key() == Qt.Key_Slash:
-                return self.search.setFocus(Qt.OtherFocusReason)
-            if ev.matches(QKeySequence.Copy):
+            if ev.key() == Qt.Key.Key_Slash:
+                return self.search.setFocus(Qt.FocusReason.OtherFocusReason)
+            if ev.matches(QKeySequence.StandardKey.Copy):
                 text = self.view.view.left.selected_text + self.view.view.right.selected_text
                 if text:
                     QApplication.clipboard().setText(text)
                 return
-            if ev.matches(QKeySequence.FindNext):
+            if ev.matches(QKeySequence.StandardKey.FindNext):
                 self.sbn.click()
                 return
-            if ev.matches(QKeySequence.FindPrevious):
+            if ev.matches(QKeySequence.StandardKey.FindPrevious):
                 self.sbp.click()
                 return
             return Dialog.keyPressEvent(self, ev)
@@ -490,8 +493,10 @@ def main(args=sys.argv):
     d = Diff(show_as_window=True)
     func = getattr(d, attr)
     QTimer.singleShot(0, lambda : func(left, right))
-    d.exec_()
+    d.show()
+    app.exec_()
     return 0
 
+
 if __name__ == '__main__':
     main()
