@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -31,6 +30,7 @@ class URLMap(object):
                 raise KeyError('The mapping %s is not available' % key)
             return ans
 
+
 _url_map = URLMap()
 
 
@@ -131,5 +131,6 @@ def write_tag_help():
     with open(P(base % 'opf3'), 'wb') as f:
         f.write(dump(get_opf3_tag_index()))
 
+
 if __name__ == '__main__':
     write_tag_help()
