@@ -1,17 +1,15 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import re, io, weakref, sys
-from cStringIO import StringIO
 
-from PyQt5.Qt import (
+from qt.core import (
     pyqtSignal, QVBoxLayout, QHBoxLayout, QPlainTextEdit, QLabel, QFontMetrics,
-    QSize, Qt, QApplication, QIcon)
+    QSize, Qt, QApplication, QIcon, QDialogButtonBox)
 
 from calibre.ebooks.oeb.polish.utils import apply_func_to_match_groups, apply_func_to_html_text
 from calibre.gui2 import error_dialog
@@ -23,14 +21,16 @@ from calibre.utils.config import JSONConfig
 from calibre.utils.icu import capitalize, upper, lower, swapcase
 from calibre.utils.titlecase import titlecase
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import iteritems, unicode_type
+from polyglot.io import PolyglotStringIO
 
 user_functions = JSONConfig('editor-search-replace-functions')
 
 
 def compile_code(src, name='<string>'):
-    if not isinstance(src, unicode):
-        match = re.search(r'coding[:=]\s*([-\w.]+)', src[:200])
-        enc = match.group(1) if match else 'utf-8'
+    if not isinstance(src, unicode_type):
+        match = re.search(br'coding[:=]\s*([-\w.]+)', src[:200])
+        enc = match.group(1).decode('utf-8') if match else 'utf-8'
         src = src.decode(enc)
     if not src or not src.strip():
         src = EMPTY_FUNC
@@ -41,7 +41,7 @@ def compile_code(src, name='<string>'):
     code = compile(src, name, 'exec')
 
     namespace = {}
-    exec code in namespace
+    exec(code, namespace)
     return namespace
 
 
@@ -67,8 +67,8 @@ class Function(object):
         self.match_index = 0
         self.boss = get_boss()
         self.data = {}
-        self.debug_buf = StringIO()
-        self.functions = {name:func.mod for name, func in functions().iteritems() if func.mod is not None}
+        self.debug_buf = PolyglotStringIO()
+        self.functions = {name:func.mod for name, func in iteritems(functions()) if func.mod is not None}
 
     def __hash__(self):
         return hash(self.name)
@@ -108,7 +108,7 @@ class DebugOutput(Dialog):
 
     def __init__(self, parent=None):
         Dialog.__init__(self, 'Debug output', 'sr-function-debug-output')
-        self.setAttribute(Qt.WA_DeleteOnClose, False)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, False)
 
     def setup_ui(self):
         self.l = l = QVBoxLayout(self)
@@ -116,12 +116,14 @@ class DebugOutput(Dialog):
         self.log_text = ''
         l.addWidget(t)
         l.addWidget(self.bb)
-        self.bb.setStandardButtons(self.bb.Close)
-        self.cb = b = self.bb.addButton(_('&Copy to clipboard'), self.bb.ActionRole)
+        self.bb.setStandardButtons(QDialogButtonBox.StandardButton.Close)
+        self.cb = b = self.bb.addButton(_('&Copy to clipboard'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.copy_to_clipboard)
         b.setIcon(QIcon(I('edit-copy.png')))
 
     def show_log(self, name, text):
+        if isinstance(text, bytes):
+            text = text.decode('utf-8', 'replace')
         self.setWindowTitle(_('Debug output from %s') % name)
         self.text.setPlainText(self.windowTitle() + '\n\n' + text)
         self.log_text = text
@@ -137,7 +139,7 @@ class DebugOutput(Dialog):
 
 
 def builtin_functions():
-    for name, obj in globals().iteritems():
+    for name, obj in iteritems(globals()):
         if name.startswith('replace_') and callable(obj) and hasattr(obj, 'imports'):
             yield obj
 
@@ -151,7 +153,7 @@ def functions(refresh=False):
         ans = _functions = {}
         for func in builtin_functions():
             ans[func.name] = Function(func.name, func=func)
-        for name, source in user_functions.iteritems():
+        for name, source in iteritems(user_functions):
             try:
                 f = Function(name, source=source)
             except Exception:
