@@ -1,15 +1,17 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 import importlib
 import os
 import sys
 import time
 
-from PyQt5.Qt import QIcon
+from qt.core import QIcon
+from qt.webengine import QWebEngineUrlScheme
 
-from calibre.constants import EDITOR_APP_UID, islinux, iswindows
+from calibre.constants import EDITOR_APP_UID, FAKE_PROTOCOL, islinux
+from calibre.ebooks.oeb.polish.check.css import shutdown as shutdown_css_check_pool
 from calibre.gui2 import (
     Application, decouple, set_gui_prefs, setup_gui_option_parser
 )
@@ -26,12 +28,13 @@ def option_parser():
             '''\
 %prog [opts] [path_to_ebook] [name_of_file_inside_book ...]
 
-Launch the calibre edit book tool. You can optionally also specify the names of
+Launch the calibre Edit book tool. You can optionally also specify the names of
 files inside the book which will be opened for editing automatically.
 '''
         )
     )
     setup_gui_option_parser(parser)
+    parser.add_option('--select-text', default=None, help=_('The text to select in the book when it is opened for editing'))
     return parser
 
 
@@ -52,18 +55,10 @@ def _run(args, notify=None):
     # Ensure we can continue to function if GUI is closed
     os.environ.pop('CALIBRE_WORKER_TEMP_DIR', None)
     reset_base_dir()
-
-    if iswindows:
-        # Ensure that all ebook editor instances are grouped together in the task
-        # bar. This prevents them from being grouped with viewer process when
-        # launched from within calibre, as both use calibre-parallel.exe
-        import ctypes
-        try:
-            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(
-                EDITOR_APP_UID
-            )
-        except Exception:
-            pass  # Only available on windows 7 and newer
+    scheme = QWebEngineUrlScheme(FAKE_PROTOCOL.encode('ascii'))
+    scheme.setSyntax(QWebEngineUrlScheme.Syntax.Host)
+    scheme.setFlags(QWebEngineUrlScheme.Flag.SecureScheme)
+    QWebEngineUrlScheme.registerScheme(scheme)
 
     # The following two lines are needed to prevent circular imports causing
     # errors during initialization of plugins that use the polish container
@@ -76,7 +71,7 @@ def _run(args, notify=None):
     opts, args = parser.parse_args(args)
     decouple('edit-book-'), set_gui_prefs(tprefs)
     override = 'calibre-edit-book' if islinux else None
-    app = Application(args, override_program_name=override, color_prefs=tprefs)
+    app = Application(args, override_program_name=override, color_prefs=tprefs, windows_app_uid=EDITOR_APP_UID)
     app.file_event_hook = EventAccumulator()
     app.load_builtin_fonts()
     app.setWindowIcon(QIcon(I('tweak.png')))
@@ -85,7 +80,7 @@ def _run(args, notify=None):
     main.show()
     app.shutdown_signal_received.connect(main.boss.quit)
     if len(args) > 1:
-        main.boss.open_book(args[1], edit_file=args[2:], clear_notify_data=False)
+        main.boss.open_book(args[1], edit_file=args[2:], clear_notify_data=False, search_text=opts.select_text)
     else:
         for path in reversed(app.file_event_hook.events):
             main.boss.open_book(path)
@@ -101,7 +96,10 @@ def _run(args, notify=None):
 
 
 def main(args=sys.argv):
-    _run(args)
+    try:
+        _run(args)
+    finally:
+        shutdown_css_check_pool()
 
 
 if __name__ == '__main__':
