@@ -47,30 +47,38 @@ def customize_remove_unused_css(name, parent, ans):
     l.addWidget(c)
     d.la2 = label('<span style="font-size:small; font-style: italic">' + _(
         'Remove all class attributes from the HTML that do not match any existing CSS rules'))
-    d.m = m = QCheckBox(_('Merge CSS rules with identical selectors'))
+    d.m = m = QCheckBox(_('Merge CSS rules with identical &selectors'))
     m.setChecked(tprefs['merge_identical_selectors'])
     l.addWidget(m)
     d.la3 = label('<span style="font-size:small; font-style: italic">' + _(
         'Merge CSS rules in the same stylesheet that have identical selectors.'
     ' Note that in rare cases merging can result in a change to the effective styling'
     ' of the book, so use with care.'))
-    d.p = p = QCheckBox(_('Merge CSS rules with identical properties'))
+    d.p = p = QCheckBox(_('Merge CSS rules with identical &properties'))
     p.setChecked(tprefs['merge_rules_with_identical_properties'])
     l.addWidget(p)
     d.la4 = label('<span style="font-size:small; font-style: italic">' + _(
         'Merge CSS rules in the same stylesheet that have identical properties.'
     ' Note that in rare cases merging can result in a change to the effective styling'
     ' of the book, so use with care.'))
+    d.p = p = QCheckBox(_('Remove &unreferenced style sheets'))
+    p.setChecked(tprefs['remove_unreferenced_sheets'])
+    l.addWidget(p)
+    d.la4 = label('<span style="font-size:small; font-style: italic">' + _(
+        'Remove stylesheets that are not referenced by any content.'
+    ))
 
     d.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
     d.l.addWidget(d.bb)
     d.bb.rejected.connect(d.reject)
     d.bb.accepted.connect(d.accept)
-    if d.exec_() != QDialog.DialogCode.Accepted:
-        raise Abort()
+    ret = d.exec_()
     ans['remove_unused_classes'] = tprefs['remove_unused_classes'] = c.isChecked()
     ans['merge_identical_selectors'] = tprefs['merge_identical_selectors'] = m.isChecked()
     ans['merge_rules_with_identical_properties'] = tprefs['merge_rules_with_identical_properties'] = p.isChecked()
+    ans['remove_unreferenced_sheets'] = tprefs['remove_unreferenced_sheets'] = p.isChecked()
+    if ret != QDialog.DialogCode.Accepted:
+        raise Abort()
 
 
 def get_customization(action, name, parent):
@@ -79,7 +87,7 @@ def get_customization(action, name, parent):
         if action == 'remove_unused_css':
             customize_remove_unused_css(name, parent, ans)
         elif action == 'upgrade_book':
-            ans['remove_ncx'] = question_dialog(
+            ans['remove_ncx'] = tprefs['remove_ncx'] = question_dialog(
                 parent, _('Remove NCX ToC file'),
                 _('Remove the legacy Table of Contents in NCX form?'),
                 _('This form of Table of Contents is superseded by the new HTML based Table of Contents.'
@@ -87,6 +95,7 @@ def get_customization(action, name, parent):
                   ' old devices that lack proper support for EPUB 3'),
                 skip_dialog_name='edit-book-remove-ncx',
                 skip_dialog_msg=_('Ask this question again in the future'),
+                skip_dialog_skipped_value=tprefs['remove_ncx'],
                 yes_text=_('Remove NCX'), no_text=_('Keep NCX')
             )
     except Abort:
