@@ -4,17 +4,18 @@
 
 
 import json
-import textwrap
 import time
 from collections import defaultdict
 from functools import partial
 from qt.core import (
-    QApplication, QByteArray, QHBoxLayout, QIcon, QLabel, QMenu, QSize, QSizePolicy,
-    QStackedLayout, Qt, QTimer, QToolBar, QUrl, QVBoxLayout, QWidget, pyqtSignal
+    QAction, QApplication, QByteArray, QHBoxLayout, QIcon, QLabel, QMenu, QSize,
+    QSizePolicy, QStackedLayout, Qt, QTimer, QToolBar, QUrl, QVBoxLayout, QWidget,
+    pyqtSignal
 )
-from qt.webengine import QWebEngineUrlSchemeHandler, QWebEngineUrlRequestJob, QWebEngineUrlRequestInfo
 from qt.webengine import (
-    QWebEnginePage, QWebEngineProfile, QWebEngineScript, QWebEngineView, QWebEngineSettings, QWebEngineContextMenuData
+    QWebEngineContextMenuData, QWebEnginePage, QWebEngineProfile, QWebEngineScript,
+    QWebEngineSettings, QWebEngineUrlRequestInfo, QWebEngineUrlRequestJob,
+    QWebEngineUrlSchemeHandler, QWebEngineView
 )
 from threading import Thread
 
@@ -25,7 +26,8 @@ from calibre.constants import (
 from calibre.ebooks.oeb.base import OEB_DOCS, XHTML_MIME, serialize
 from calibre.ebooks.oeb.polish.parsing import parse
 from calibre.gui2 import (
-    NO_URL_FORMATTING, error_dialog, is_dark_theme, safe_open_url
+    NO_URL_FORMATTING, QT_HIDDEN_CLEAR_ACTION, error_dialog, is_dark_theme,
+    safe_open_url
 )
 from calibre.gui2.palette import dark_color, dark_link_color, dark_text_color
 from calibre.gui2.tweak_book import TOP, actions, current_container, editors, tprefs
@@ -235,6 +237,60 @@ def uniq(vals):
     return tuple(x for x in vals if x not in seen and not seen_add(x))
 
 
+def get_editor_settings(tprefs):
+    dark = is_dark_theme()
+
+    def get_color(name, dark_val):
+        ans = tprefs[name]
+        if ans == 'auto' and dark:
+            ans = dark_val.name()
+        if ans in ('auto', 'unset'):
+            return None
+        return ans
+
+    return {
+        'is_dark_theme': dark,
+        'bg': get_color('preview_background', dark_color),
+        'fg': get_color('preview_foreground', dark_text_color),
+        'link': get_color('preview_link_color', dark_link_color),
+    }
+
+
+def create_dark_mode_script():
+    dark_mode_css = P('dark_mode.css', data=True, allow_user_override=False).decode('utf-8')
+    return create_script('dark-mode.js', '''
+    (function() {
+        var settings = JSON.parse(navigator.userAgent.split('|')[1]);
+        var dark_css = CSS;
+
+        function apply_body_colors(event) {
+            if (document.documentElement) {
+                if (settings.bg) document.documentElement.style.backgroundColor = settings.bg;
+                if (settings.fg) document.documentElement.style.color = settings.fg;
+            }
+            if (document.body) {
+                if (settings.bg) document.body.style.backgroundColor = settings.bg;
+                if (settings.fg) document.body.style.color = settings.fg;
+            }
+        }
+
+        function apply_css() {
+            var css = '';
+            if (settings.link) css += 'html > body :link, html > body :link * { color: ' + settings.link + ' !important; }';
+            if (settings.is_dark_theme) { css += dark_css; }
+            var style = document.createElement('style');
+            style.textContent = css;
+            document.documentElement.appendChild(style);
+            apply_body_colors();
+        }
+
+        apply_body_colors();
+        document.addEventListener("DOMContentLoaded", apply_css);
+    })();
+    '''.replace('CSS', json.dumps(dark_mode_css), 1),
+    injection_point=QWebEngineScript.InjectionPoint.DocumentCreation)
+
+
 def create_profile():
     ans = getattr(create_profile, 'ans', None)
     if ans is None:
@@ -246,42 +302,11 @@ def create_profile():
             compile_editor()
         js = P('editor.js', data=True, allow_user_override=False)
         cparser = P('csscolorparser.js', data=True, allow_user_override=False)
-        dark_mode_css = P('dark_mode.css', data=True, allow_user_override=False).decode('utf-8')
 
         insert_scripts(ans,
             create_script('csscolorparser.js', cparser),
             create_script('editor.js', js),
-            create_script('dark-mode.js', '''
-            (function() {
-                var settings = JSON.parse(navigator.userAgent.split('|')[1]);
-                var dark_css = CSS;
-
-                function apply_body_colors(event) {
-                    if (document.documentElement) {
-                        if (settings.bg) document.documentElement.style.backgroundColor = settings.bg;
-                        if (settings.fg) document.documentElement.style.color = settings.fg;
-                    }
-                    if (document.body) {
-                        if (settings.bg) document.body.style.backgroundColor = settings.bg;
-                        if (settings.fg) document.body.style.color = settings.fg;
-                    }
-                }
-
-                function apply_css() {
-                    var css = '';
-                    if (settings.link) css += 'html > body :link, html > body :link * { color: ' + settings.link + ' !important; }';
-                    if (settings.is_dark_theme) { css += dark_css; }
-                    var style = document.createElement('style');
-                    style.textContent = css;
-                    document.documentElement.appendChild(style);
-                    apply_body_colors();
-                }
-
-                apply_body_colors();
-                document.addEventListener("DOMContentLoaded", apply_css);
-            })();
-            '''.replace('CSS', json.dumps(dark_mode_css), 1),
-            injection_point=QWebEngineScript.InjectionPoint.DocumentCreation)
+            create_dark_mode_script(),
         )
         url_handler = UrlSchemeHandler(ans)
         ans.installUrlSchemeHandler(QByteArray(FAKE_PROTOCOL.encode('ascii')), url_handler)
@@ -400,22 +425,7 @@ class WebView(RestartingWebEngineView, OpenWithHandler):
         return self._size_hint
 
     def update_settings(self):
-        dark = is_dark_theme()
-
-        def get_color(name, dark_val):
-            ans = tprefs[name]
-            if ans == 'auto' and dark:
-                ans = dark_val.name()
-            if ans in ('auto', 'unset'):
-                return None
-            return ans
-
-        settings = {
-            'is_dark_theme': dark,
-            'bg': get_color('preview_background', dark_color),
-            'fg': get_color('preview_foreground', dark_text_color),
-            'link': get_color('preview_link_color', dark_link_color),
-        }
+        settings = get_editor_settings(tprefs)
         p = self._page.profile()
         ua = p.httpUserAgent().split('|')[0] + '|' + json.dumps(settings)
         p.setHttpUserAgent(ua)
@@ -562,6 +572,10 @@ class Preview(QWidget):
         self.current_sync_request = None
 
         self.search = HistoryLineEdit2(self)
+        self.search.setClearButtonEnabled(True)
+        ac = self.search.findChild(QAction, QT_HIDDEN_CLEAR_ACTION)
+        if ac is not None:
+            ac.triggered.connect(self.clear_clicked)
         self.search.initialize('tweak_book_preview_search')
         self.search.setPlaceholderText(_('Search in preview'))
         self.search.returnPressed.connect(self.find_next)
@@ -572,6 +586,9 @@ class Preview(QWidget):
             ac.triggered.connect(getattr(self, 'find_' + d))
             self.bar.addAction(ac)
 
+    def clear_clicked(self):
+        self.view._page.findText('')
+
     def find(self, direction):
         text = unicode_type(self.search.text())
         self.view._page.findText(text, (
@@ -708,9 +725,9 @@ class Preview(QWidget):
             self.refresh()
 
     def split_toggled(self, checked):
-        actions['split-in-preview'].setToolTip(textwrap.fill(_(
+        actions['split-in-preview'].setToolTip('<p>' + (_(
             'Abort file split') if checked else _(
-                'Split this file at a specified location.\n\nAfter clicking this button, click'
+                'Split this file at a specified location.<p>After clicking this button, click'
                 ' inside the preview panel above at the location you want the file to be split.')))
         if checked:
             self.split_start_requested.emit()
