@@ -885,16 +885,11 @@ class SavedSearches(QWidget):
         self.l = l = QVBoxLayout(self)
         self.setLayout(l)
 
-        self.h = h = QHBoxLayout()
         self.filter_text = ft = QLineEdit(self)
+        ft.setClearButtonEnabled(True)
         ft.textChanged.connect(self.do_filter)
         ft.setPlaceholderText(_('Filter displayed searches'))
-        h.addWidget(ft)
-        self.cft = cft = QToolButton(self)
-        cft.setToolTip(_('Clear filter')), cft.setIcon(QIcon(I('clear_left.png')))
-        cft.clicked.connect(ft.clear)
-        h.addWidget(cft)
-        l.addLayout(h)
+        l.addWidget(ft)
 
         self.h2 = h = QHBoxLayout()
         self.searches = searches = QListView(self)
@@ -1051,7 +1046,7 @@ class SavedSearches(QWidget):
 
     def stack_current_changed(self, index):
         visible = index == 0
-        for x in ('eb', 'ab', 'rb', 'upb', 'dnb', 'd2', 'filter_text', 'cft', 'd3', 'ib', 'eb2'):
+        for x in ('eb', 'ab', 'rb', 'upb', 'dnb', 'd2', 'filter_text', 'd3', 'ib', 'eb2'):
             getattr(self, x).setVisible(visible)
 
     @property
