@@ -1,21 +1,22 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-from PyQt5.Qt import (
-    QDialog, pyqtSignal, QIcon, QVBoxLayout, QDialogButtonBox, QStackedWidget,
-    QAction, QMenu, QTreeWidget, QTreeWidgetItem, QGridLayout, QWidget, Qt,
-    QSize, QStyledItemDelegate, QApplication, QTimer)
+from qt.core import (
+    QAction, QApplication, QDialog, QDialogButtonBox, QGridLayout, QIcon, QMenu,
+    QSize, QStackedWidget, QStyledItemDelegate, Qt, QTimer, QTreeWidget,
+    QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
+)
 
-from calibre.constants import plugins
 from calibre.ebooks.oeb.polish.toc import commit_toc, get_toc
-from calibre.gui2 import error_dialog
-from calibre.gui2.toc.main import TOCView, ItemEdit
-from calibre.gui2.tweak_book import current_container, TOP, actions, tprefs
+from calibre.gui2 import error_dialog, make_view_use_window_background
+from calibre.gui2.toc.main import ItemEdit, TOCView
+from calibre.gui2.tweak_book import TOP, actions, current_container, tprefs
+from calibre_extensions.progress_indicator import set_no_activate_on_click
+from polyglot.builtins import range, unicode_type
 
 
 class TOCEditor(QDialog):
@@ -42,17 +43,21 @@ class TOCEditor(QDialog):
         self.item_edit = ItemEdit(self, tprefs)
         s.addWidget(self.item_edit)
 
-        bb = self.bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        bb = self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         l.addWidget(bb)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
+        self.undo_button = b = bb.addButton(_('&Undo'), QDialogButtonBox.ButtonRole.ActionRole)
+        b.setToolTip(_('Undo the last action, if any'))
+        b.setIcon(QIcon(I('edit-undo.png')))
+        b.clicked.connect(self.toc_view.undo)
 
         self.read_toc()
 
         self.resize(950, 630)
         geom = tprefs.get('toc_editor_window_geom', None)
         if geom is not None:
-            self.restoreGeometry(bytes(geom))
+            QApplication.instance().safe_restore_geometry(self, bytes(geom))
 
     def add_new_item(self, item, where):
         self.item_edit(item, where)
@@ -100,7 +105,8 @@ class TOCEditor(QDialog):
         commit_toc(current_container(), toc, lang=self.toc_view.toc_lang,
                 uid=self.toc_view.toc_uid)
 
-DEST_ROLE = Qt.UserRole
+
+DEST_ROLE = Qt.ItemDataRole.UserRole
 FRAG_ROLE = DEST_ROLE + 1
 
 
@@ -123,18 +129,16 @@ class TOCViewer(QWidget):
         self.setLayout(l)
         l.setContentsMargins(0, 0, 0, 0)
 
-        self.view = QTreeWidget(self)
+        self.view = make_view_use_window_background(QTreeWidget(self))
         self.delegate = Delegate(self.view)
         self.view.setItemDelegate(self.delegate)
         self.view.setHeaderHidden(True)
         self.view.setAnimated(True)
-        self.view.setContextMenuPolicy(Qt.CustomContextMenu)
-        self.view.customContextMenuRequested.connect(self.show_context_menu, type=Qt.QueuedConnection)
+        self.view.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
+        self.view.customContextMenuRequested.connect(self.show_context_menu, type=Qt.ConnectionType.QueuedConnection)
         self.view.itemActivated.connect(self.emit_navigate)
         self.view.itemPressed.connect(self.item_pressed)
-        pi = plugins['progress_indicator'][0]
-        if hasattr(pi, 'set_no_activate_on_click'):
-            pi.set_no_activate_on_click(self.view)
+        set_no_activate_on_click(self.view)
         self.view.itemDoubleClicked.connect(self.emit_navigate)
         l.addWidget(self.view)
 
@@ -164,7 +168,7 @@ class TOCViewer(QWidget):
         self.build()
 
     def item_pressed(self, item):
-        if QApplication.mouseButtons() & Qt.LeftButton:
+        if QApplication.mouseButtons() & Qt.MouseButton.LeftButton:
             QTimer.singleShot(0, self.emit_navigate)
 
     def show_context_menu(self, pos):
@@ -175,20 +179,20 @@ class TOCViewer(QWidget):
         menu.addAction(self.refresh_action)
         menu.exec_(self.view.mapToGlobal(pos))
 
-    def iteritems(self, parent=None):
+    def iter_items(self, parent=None):
         if parent is None:
             parent = self.invisibleRootItem()
-        for i in xrange(parent.childCount()):
+        for i in range(parent.childCount()):
             child = parent.child(i)
             yield child
-            for gc in self.iteritems(parent=child):
+            for gc in self.iter_items(parent=child):
                 yield gc
 
     def emit_navigate(self, *args):
         item = self.view.currentItem()
         if item is not None:
-            dest = unicode(item.data(0, DEST_ROLE) or '')
-            frag = unicode(item.data(0, FRAG_ROLE) or '')
+            dest = unicode_type(item.data(0, DEST_ROLE) or '')
+            frag = unicode_type(item.data(0, FRAG_ROLE) or '')
             if not frag:
                 frag = TOP
             self.navigate_requested.emit(dest, frag)
@@ -209,7 +213,7 @@ class TOCViewer(QWidget):
                 node.setData(0, FRAG_ROLE, child.frag or '')
                 tt = _('File: {0}\nAnchor: {1}').format(
                     child.dest or '', child.frag or _('Top of file'))
-                node.setData(0, Qt.ToolTipRole, tt)
+                node.setData(0, Qt.ItemDataRole.ToolTipRole, tt)
                 process_node(child, node)
 
         self.view.clear()
@@ -223,4 +227,3 @@ class TOCViewer(QWidget):
     def update_if_visible(self):
         if self.isVisible():
             self.build()
-
