@@ -1,16 +1,15 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import shutil
 
-from PyQt5.Qt import (
+from qt.core import (
     QAbstractListModel, Qt, QModelIndex, QApplication, QWidget,
-    QGridLayout, QListView, QStyledItemDelegate, pyqtSignal, QPushButton, QIcon)
+    QGridLayout, QListView, QStyledItemDelegate, pyqtSignal, QPushButton, QIcon, QItemSelectionModel)
 
 from calibre.gui2 import error_dialog
 
@@ -45,14 +44,14 @@ class GlobalUndoHistory(QAbstractListModel):
     def rowCount(self, parent=ROOT):
         return len(self.states)
 
-    def data(self, index, role=Qt.DisplayRole):
-        if role == Qt.DisplayRole:
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.DisplayRole:
             return self.label_for_row(index.row())
-        if role == Qt.FontRole and index.row() == self.pos:
+        if role == Qt.ItemDataRole.FontRole and index.row() == self.pos:
             f = QApplication.instance().font()
             f.setBold(True)
             return f
-        if role == Qt.UserRole:
+        if role == Qt.ItemDataRole.UserRole:
             return self.states[index.row()]
         return None
 
@@ -218,7 +217,7 @@ class CheckpointView(QWidget):
         self.view.clearSelection()
         m = self.view.model()
         sm = self.view.selectionModel()
-        sm.select(m.index(m.pos), sm.ClearAndSelect)
+        sm.select(m.index(m.pos), QItemSelectionModel.SelectionFlag.ClearAndSelect)
         self.view.setCurrentIndex(m.index(m.pos))
 
     def double_clicked(self, index):
@@ -243,4 +242,3 @@ class CheckpointView(QWidget):
             return error_dialog(self, _('Cannot compare'), _(
                 'There is no point comparing the current state to itself'), show=True)
         self.compare_requested.emit(m.states[row].container)
-
