@@ -8,17 +8,17 @@ from collections import defaultdict
 from functools import lru_cache
 from itertools import chain
 from qt.core import (
-    QColor, QFont, QHBoxLayout, QIcon, QImage, QItemSelectionModel, QKeySequence,
-    QLabel, QMenu, QPainter, QPainterPath, QPixmap, QPushButton, QRect, QSizePolicy,
-    Qt, QTextEdit, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal,
-    QAbstractItemView, QDialog, QPalette, QStyle
+    QAbstractItemView, QColor, QDialog, QFont, QHBoxLayout, QIcon, QImage,
+    QItemSelectionModel, QKeySequence, QLabel, QMenu, QPainter, QPainterPath,
+    QPalette, QPixmap, QPushButton, QRect, QSizePolicy, QStyle, Qt, QTextCursor,
+    QTextEdit, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
 )
 
 from calibre.constants import (
     builtin_colors_dark, builtin_colors_light, builtin_decorations
 )
 from calibre.ebooks.epub.cfi.parse import cfi_sort_key
-from calibre.gui2 import error_dialog, is_dark_theme
+from calibre.gui2 import error_dialog, is_dark_theme, safe_open_url
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.library.annotations import (
     Details, Export as ExportBase, render_highlight_as_text, render_notes
@@ -141,8 +141,17 @@ class Export(ExportBase):
         lines = []
         as_markdown = fmt == 'md'
         link_prefix = link_prefix_for_location_links()
-        for hl in self.annotations:
-            render_highlight_as_text(hl, lines, as_markdown=as_markdown, link_prefix=link_prefix)
+        chapter_groups = {}
+        def_chap = (_('Unknown chapter'),)
+        for a in self.annotations:
+            toc_titles = a.get('toc_family_titles', def_chap)
+            chapter_groups.setdefault(toc_titles[0], []).append(a)
+        for chapter, group in chapter_groups.items():
+            if len(chapter_groups) > 1:
+                lines.append('### ' + chapter)
+                lines.append('')
+            for hl in group:
+                render_highlight_as_text(hl, lines, as_markdown=as_markdown, link_prefix=link_prefix)
         return '\n'.join(lines).strip()
 
 
@@ -175,12 +184,10 @@ class Highlights(QTreeWidget):
         self.context_menu = m = QMenu(self)
         if h is not None:
             m.addAction(QIcon(I('edit_input.png')), _('Modify this highlight'), self.edit_requested.emit)
-            m.addAction(QIcon(I('edit_input.png')), _('Edit notes for this highlight'), self.edit_notes_requested.emit)
+            m.addAction(QIcon(I('modified.png')), _('Edit notes for this highlight'), self.edit_notes_requested.emit)
             m.addAction(QIcon(I('trash.png')), ngettext(
                 'Delete this highlight', 'Delete selected highlights', len(self.selectedItems())
             ), self.delete_requested.emit)
-            if h.get('notes'):
-                m.addAction(QIcon(I('modified.png')), _('Edit notes for this highlight'), self.edit_notes_requested.emit)
         m.addSeparator()
         m.addAction(_('Expand all'), self.expandAll)
         m.addAction(_('Collapse all'), self.collapseAll)
@@ -190,8 +197,15 @@ class Highlights(QTreeWidget):
     def current_item_changed(self, current, previous):
         self.current_highlight_changed.emit(current.data(0, Qt.ItemDataRole.UserRole) if current is not None else None)
 
-    def load(self, highlights):
+    def load(self, highlights, preserve_state=False):
         s = self.style()
+        expanded_chapters = set()
+        if preserve_state:
+            root = self.invisibleRootItem()
+            for i in range(root.childCount()):
+                chapter = root.child(i)
+                if chapter.isExpanded():
+                    expanded_chapters.add(chapter.data(0, Qt.ItemDataRole.DisplayRole))
         icon_size = s.pixelMetric(QStyle.PixelMetric.PM_SmallIconSize, None, self)
         dpr = self.devicePixelRatioF()
         is_dark = is_dark_theme()
@@ -225,7 +239,7 @@ class Highlights(QTreeWidget):
             if tt:
                 section.setToolTip(0, tt)
             self.addTopLevelItem(section)
-            section.setExpanded(True)
+            section.setExpanded(not preserve_state or sec in expanded_chapters)
             for itemnum, h in enumerate(items):
                 txt = h.get('highlighted_text')
                 txt = txt.replace('\n', ' ')
@@ -259,7 +273,7 @@ class Highlights(QTreeWidget):
 
     def refresh(self, highlights):
         h = self.current_highlight
-        self.load(highlights)
+        self.load(highlights, preserve_state=True)
         if h is not None:
             idx = self.uuid_map.get(h['uuid'])
             if idx is not None:
@@ -365,6 +379,7 @@ class NotesEditDialog(Dialog):
         qte.setMinimumWidth(600)
         if self.initial_notes:
             qte.setPlainText(self.initial_notes)
+            qte.moveCursor(QTextCursor.MoveOperation.End)
         l.addWidget(qte)
         l.addWidget(self.bb)
 
@@ -380,7 +395,7 @@ class NotesDisplay(Details):
     def __init__(self, parent=None):
         Details.__init__(self, parent)
         self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Maximum)
-        self.anchorClicked.connect(self.edit_notes)
+        self.anchorClicked.connect(self.anchor_clicked)
         self.current_notes = ''
 
     def show_notes(self, text=''):
@@ -388,10 +403,17 @@ class NotesDisplay(Details):
         self.setVisible(bool(text))
         self.current_notes = text
         html = '\n'.join(render_notes(text))
-        self.setHtml('<div><a href="edit://moo" style="text-decoration: none">{}</a></div>{}'.format(_('Edit notes'), html))
+        self.setHtml('<div><a href="edit://moo">{}</a></div>{}'.format(_('Edit notes'), html))
+        self.document().setDefaultStyleSheet('a[href] { text-decoration: none }')
         h = self.document().size().height() + 2
         self.setMaximumHeight(h)
 
+    def anchor_clicked(self, qurl):
+        if qurl.scheme() == 'edit':
+            self.edit_notes()
+        else:
+            safe_open_url(qurl)
+
     def edit_notes(self):
         current_text = self.current_notes
         d = NotesEditDialog(current_text, self)
@@ -405,6 +427,7 @@ class HighlightsPanel(QWidget):
     request_highlight_action = pyqtSignal(object, object)
     web_action = pyqtSignal(object, object)
     toggle_requested = pyqtSignal()
+    notes_edited_signal = pyqtSignal(object, object)
 
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
@@ -454,6 +477,7 @@ class HighlightsPanel(QWidget):
         if h is not None:
             h['notes'] = text
             self.web_action.emit('set-notes-in-highlight', h)
+            self.notes_edited_signal.emit(h['uuid'], text)
 
     def set_tooltips(self, rmap):
         a = rmap.get('create_annotation')
