@@ -1,27 +1,34 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+# License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
-__license__ = 'GPL v3'
-__copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, subprocess, cPickle, sys
+import os
+import subprocess
+import sys
 from threading import Thread
 
-from PyQt5.Qt import (
-    QFormLayout, QLineEdit, QToolButton, QHBoxLayout, QLabel, QIcon, QPrinter,
-    QPageSize, QComboBox, QDoubleSpinBox, QCheckBox, QProgressDialog, QTimer)
+from qt.core import (
+    QCheckBox, QDoubleSpinBox, QFormLayout, QHBoxLayout, QIcon, QLabel, QDialog,
+    QLineEdit, QPageSize, QProgressDialog, QTimer, QToolButton, QVBoxLayout
+)
 
-from calibre import sanitize_file_name2
-from calibre.ptempfile import PersistentTemporaryFile
+from calibre import sanitize_file_name
 from calibre.ebooks.conversion.plugins.pdf_output import PAPER_SIZES
-from calibre.gui2 import elided_text, error_dialog, choose_save_file, Application, open_local_file, dynamic
+from calibre.gui2 import (
+    Application, choose_save_file, dynamic, elided_text, error_dialog,
+    open_local_file
+)
+from calibre.gui2.widgets import PaperSizes
 from calibre.gui2.widgets2 import Dialog
-from calibre.gui2.viewer.main import vprefs
-from calibre.utils.icu import numeric_sort_key
-from calibre.utils.ipc.simple_worker import start_pipe_worker
+from calibre.ptempfile import PersistentTemporaryFile
+from calibre.utils.config import JSONConfig
 from calibre.utils.filenames import expanduser
+from calibre.utils.ipc.simple_worker import start_pipe_worker
+from calibre.utils.serialize import msgpack_dumps, msgpack_loads
+
+
+vprefs = JSONConfig('viewer')
 
 
 class PrintDialog(Dialog):
@@ -30,12 +37,15 @@ class PrintDialog(Dialog):
 
     def __init__(self, book_title, parent=None, prefs=vprefs):
         self.book_title = book_title
-        self.default_file_name = sanitize_file_name2(book_title[:75] + '.pdf')
+        self.default_file_name = sanitize_file_name(book_title[:75] + '.pdf')
         self.paper_size_map = {a:getattr(QPageSize, a.capitalize()) for a in PAPER_SIZES}
         Dialog.__init__(self, _('Print to PDF'), 'print-to-pdf', prefs=prefs, parent=parent)
 
     def setup_ui(self):
-        self.l = l = QFormLayout(self)
+        self.vl = vl = QVBoxLayout(self)
+        self.l = l = QFormLayout()
+        vl.addLayout(l)
+        l.setContentsMargins(0, 0, 0, 0)
         l.addRow(QLabel(_('Print %s to a PDF file') % elided_text(self.book_title)))
         self.h = h = QHBoxLayout()
         self.file_name = f = QLineEdit(self)
@@ -53,14 +63,9 @@ class PrintDialog(Dialog):
         w = QLabel(_('&File:'))
         l.addRow(w, h), w.setBuddy(f)
 
-        self.paper_size = ps = QComboBox(self)
-        ps.addItems([a.upper() for a in sorted(self.paper_size_map, key=numeric_sort_key)])
-        previous_size = vprefs.get('print-to-pdf-page-size', None)
-        if previous_size not in self.paper_size_map:
-            previous_size = (QPrinter().pageLayout().pageSize().name() or '').lower()
-        if previous_size not in self.paper_size_map:
-            previous_size = 'a4'
-        ps.setCurrentIndex(ps.findText(previous_size.upper()))
+        self.paper_size = ps = PaperSizes(self)
+        ps.initialize()
+        ps.set_value_for_config = vprefs.get('print-to-pdf-page-size', None)
         l.addRow(_('Paper &size:'), ps)
         tmap = {
                 'left':_('&Left margin:'),
@@ -80,23 +85,24 @@ class PrintDialog(Dialog):
         pnum.setChecked(vprefs.get('print-to-pdf-page-numbers', True))
         l.addRow(pnum)
 
-        self.show_file = sf = QCheckBox(_('Open PDF file after printing'), self)
+        self.show_file = sf = QCheckBox(_('&Open PDF file after printing'), self)
         sf.setChecked(vprefs.get('print-to-pdf-show-file', True))
         l.addRow(sf)
 
-        l.addRow(self.bb)
+        vl.addStretch(10)
+        vl.addWidget(self.bb)
 
     @property
     def data(self):
         fpath = self.file_name.text().strip()
         head, tail = os.path.split(fpath)
-        tail = sanitize_file_name2(tail)
+        tail = sanitize_file_name(tail)
         fpath = tail
         if head:
             fpath = os.path.join(head, tail)
         ans = {
             'output': fpath,
-            'paper_size': self.paper_size.currentText().lower(),
+            'paper_size': self.paper_size.get_value_for_config,
             'page_numbers':self.pnum.isChecked(),
             'show_file':self.show_file.isChecked(),
         }
@@ -143,7 +149,7 @@ class DoPrint(Thread):
         try:
             with PersistentTemporaryFile('print-to-pdf-log.txt') as f:
                 p = self.worker = start_pipe_worker('from calibre.gui2.viewer.printing import do_print; do_print()', stdout=f, stderr=subprocess.STDOUT)
-                p.stdin.write(cPickle.dumps(self.data, -1)), p.stdin.flush(), p.stdin.close()
+                p.stdin.write(msgpack_dumps(self.data)), p.stdin.flush(), p.stdin.close()
                 rc = p.wait()
                 if rc != 0:
                     f.seek(0)
@@ -158,9 +164,20 @@ class DoPrint(Thread):
 
 
 def do_print():
-    data = cPickle.loads(sys.stdin.read())
+    from calibre.customize.ui import plugin_for_input_format
+    stdin = getattr(sys.stdin, 'buffer', sys.stdin)
+    data = msgpack_loads(stdin.read())
+    ext = data['input'].lower().rpartition('.')[-1]
+    input_plugin = plugin_for_input_format(ext)
+    if input_plugin is None:
+        raise ValueError('Not a supported file type: {}'.format(ext.upper()))
     args = ['ebook-convert', data['input'], data['output'], '--paper-size', data['paper_size'], '--pdf-add-toc',
-            '--disable-remove-fake-margins', '--disable-font-rescaling', '--page-breaks-before', '/', '--chapter-mark', 'none', '-vv']
+            '--disable-remove-fake-margins', '--chapter-mark', 'none', '-vv']
+    if input_plugin.is_image_collection:
+        args.append('--no-process')
+    else:
+        args.append('--disable-font-rescaling')
+        args.append('--page-breaks-before=/')
     if data['page_numbers']:
         args.append('--pdf-page-numbers')
     for edge in 'left top right bottom'.split():
@@ -208,7 +225,7 @@ class Printing(QProgressDialog):
 def print_book(path_to_book, parent=None, book_title=None):
     book_title = book_title or os.path.splitext(os.path.basename(path_to_book))[0]
     d = PrintDialog(book_title, parent)
-    if d.exec_() == d.Accepted:
+    if d.exec_() == QDialog.DialogCode.Accepted:
         data = d.data
         data['input'] = path_to_book
         t = DoPrint(data)
@@ -219,4 +236,5 @@ def print_book(path_to_book, parent=None, book_title=None):
 if __name__ == '__main__':
     app = Application([])
     print_book(sys.argv[-1])
+
     del app
