@@ -17,7 +17,7 @@ from qt.core import (
 from threading import Thread
 
 from calibre import prints
-from calibre.constants import ismacos
+from calibre.constants import ismacos, iswindows
 from calibre.customize.ui import available_input_formats
 from calibre.db.annotations import merge_annotations
 from calibre.gui2 import choose_files, error_dialog, sanitize_env_vars
@@ -192,6 +192,7 @@ class EbookViewer(MainWindow):
         self.web_view.highlights_changed.connect(self.highlights_changed)
         self.web_view.edit_book.connect(self.edit_book)
         self.actions_toolbar.initialize(self.web_view, self.search_dock.toggleViewAction())
+        at.update_action_state(False)
         self.setCentralWidget(self.web_view)
         self.loading_overlay = LoadingOverlay(self)
         self.restore_state()
@@ -199,6 +200,7 @@ class EbookViewer(MainWindow):
         self.dock_visibility_changed()
         self.highlights_widget.request_highlight_action.connect(self.web_view.highlight_action)
         self.highlights_widget.web_action.connect(self.web_view.generic_action)
+        self.highlights_widget.notes_edited_signal.connect(self.notes_edited)
         if continue_reading:
             self.continue_reading()
         self.setup_mouse_auto_hide()
@@ -289,6 +291,7 @@ class EbookViewer(MainWindow):
 
     def toggle_full_screen(self):
         self.set_full_screen(not self.isFullScreen())
+
     # }}}
 
     # Docks (ToC, Bookmarks, Lookup, etc.) {{{
@@ -363,11 +366,11 @@ class EbookViewer(MainWindow):
         # annotations will be saved in book file on exit
         self.save_annotations(in_book_file=False)
 
-    def goto_cfi(self, cfi):
-        self.web_view.goto_cfi(cfi)
+    def goto_cfi(self, cfi, add_to_history=False):
+        self.web_view.goto_cfi(cfi, add_to_history=add_to_history)
 
     def bookmark_activated(self, cfi):
-        self.goto_cfi(cfi)
+        self.goto_cfi(cfi, add_to_history=True)
 
     def view_image(self, name):
         path = get_path_for_name(name)
@@ -410,11 +413,13 @@ class EbookViewer(MainWindow):
     def show_loading_message(self, msg):
         if msg:
             self.loading_overlay(msg)
+            self.actions_toolbar.update_action_state(False)
         else:
             if not hasattr(self, 'initial_loading_performace_reported'):
                 performance_monitor('loading finished')
                 self.initial_loading_performace_reported = True
             self.loading_overlay.hide()
+            self.actions_toolbar.update_action_state(True)
 
     def show_error(self, title, msg, details):
         self.loading_overlay.hide()
@@ -459,7 +464,10 @@ class EbookViewer(MainWindow):
             self.load_ebook(entry['pathtoebook'])
 
     def load_ebook(self, pathtoebook, open_at=None, reload_book=False):
+        if '.' not in os.path.basename(pathtoebook):
+            pathtoebook = os.path.abspath(os.path.realpath(pathtoebook))
         performance_monitor('Load of book started', reset=True)
+        self.actions_toolbar.update_action_state(False)
         self.web_view.show_home_page_on_ready = False
         if open_at:
             self.pending_open_at = open_at
@@ -500,6 +508,7 @@ class EbookViewer(MainWindow):
         open_at, self.pending_open_at = self.pending_open_at, None
         self.web_view.clear_caches()
         if not ok:
+            self.actions_toolbar.update_action_state(False)
             self.setWindowTitle(self.base_window_title)
             tb = as_unicode(data['tb'].strip(), errors='replace')
             tb = re.split(r'^calibre\.gui2\.viewer\.convert_book\.ConversionFailure:\s*', tb, maxsplit=1, flags=re.M)[-1]
@@ -643,6 +652,16 @@ class EbookViewer(MainWindow):
         self.highlights_widget.refresh(highlights)
         self.save_annotations()
 
+    def notes_edited(self, uuid, notes):
+        for h in self.current_book_data['annotations_map']['highlight']:
+            if h.get('uuid') == uuid:
+                h['notes'] = notes
+                h['timestamp'] = utcnow().isoformat()
+                break
+        else:
+            return
+        self.save_annotations()
+
     def edit_book(self, file_name, progress_frac, selected_text):
         import subprocess
 
@@ -688,6 +707,10 @@ class EbookViewer(MainWindow):
         if state:
             self.restoreState(state, self.MAIN_WINDOW_STATE_VERSION)
             self.inspector_dock.setVisible(False)
+            if not get_session_pref('restore_docks', True):
+                for dock_def in self.dock_defs.values():
+                    d = getattr(self, '{}_dock'.format(dock_def.name.partition('-')[0]))
+                    d.setVisible(False)
 
     def quit(self):
         self.close()
@@ -739,11 +762,16 @@ class EbookViewer(MainWindow):
         t.start()
 
     def eventFilter(self, obj, ev):
-        if ev.type() == QEvent.Type.MouseMove:
+        et = ev.type()
+        if et == QEvent.Type.MouseMove:
             if self.cursor_hidden:
                 self.cursor_hidden = False
                 QApplication.instance().restoreOverrideCursor()
             self.hide_cursor_timer.start()
+        elif et == QEvent.Type.FocusIn:
+            if iswindows and obj and obj.objectName() == 'EbookViewerClassWindow' and self.isFullScreen():
+                # See https://bugs.launchpad.net/calibre/+bug/1918591
+                self.web_view.repair_after_fullscreen_switch()
         return False
 
     def hide_cursor(self):
