@@ -274,6 +274,7 @@ class ViewerBridge(Bridge):
     speak_simple_text = from_js(object)
     tts = from_js(object, object)
     edit_book = from_js(object, object, object)
+    show_book_folder = from_js()
 
     create_view = to_js()
     start_book_load = to_js()
@@ -290,6 +291,7 @@ class ViewerBridge(Bridge):
     generic_action = to_js()
     show_search_result = to_js()
     prepare_for_close = to_js()
+    repair_after_fullscreen_switch = to_js()
     viewer_font_size_changed = to_js()
     tts_event = to_js()
 
@@ -535,6 +537,7 @@ class WebView(RestartingWebEngineView):
         self.bridge.close_prep_finished.connect(self.close_prep_finished)
         self.bridge.highlights_changed.connect(self.highlights_changed)
         self.bridge.edit_book.connect(self.edit_book)
+        self.bridge.show_book_folder.connect(self.show_book_folder)
         self.bridge.open_url.connect(safe_open_url)
         self.bridge.speak_simple_text.connect(self.tts.speak_simple_text)
         self.bridge.tts.connect(self.tts.action)
@@ -646,8 +649,8 @@ class WebView(RestartingWebEngineView):
     def goto_toc_node(self, node_id):
         self.execute_when_ready('goto_toc_node', node_id)
 
-    def goto_cfi(self, cfi):
-        self.execute_when_ready('goto_cfi', cfi)
+    def goto_cfi(self, cfi, add_to_history=False):
+        self.execute_when_ready('goto_cfi', cfi, bool(add_to_history))
 
     def notify_full_screen_state_change(self, in_fullscreen_mode):
         self.execute_when_ready('full_screen_state_changed', in_fullscreen_mode)
@@ -736,3 +739,10 @@ class WebView(RestartingWebEngineView):
 
     def tts_settings_changed(self, ui_settings):
         self.execute_when_ready('tts_event', 'configured', ui_settings)
+
+    def show_book_folder(self):
+        path = os.path.dirname(os.path.abspath(set_book_path.pathtoebook))
+        safe_open_url(QUrl.fromLocalFile(path))
+
+    def repair_after_fullscreen_switch(self):
+        self.execute_when_ready('repair_after_fullscreen_switch')
