@@ -1,17 +1,16 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import cStringIO, sys
-from binascii import hexlify, unhexlify
+import sys
 from functools import partial
 from threading import Thread
 
-from PyQt5.Qt import (
+from qt.core import (
     QWidget, pyqtSignal, QDialog, Qt, QLabel, QLineEdit, QDialogButtonBox,
     QGridLayout, QCheckBox, QIcon, QVBoxLayout, QPushButton, QPlainTextEdit,
     QHBoxLayout)
@@ -20,6 +19,9 @@ from calibre import prints
 from calibre.gui2.wizard.send_email_ui import Ui_Form
 from calibre.utils.smtp import config as smtp_prefs
 from calibre.gui2 import error_dialog, question_dialog
+from polyglot.builtins import unicode_type
+from polyglot.binary import as_hex_unicode, from_hex_unicode
+from polyglot.io import PolyglotStringIO
 
 
 class TestEmail(QDialog):
@@ -40,23 +42,27 @@ class TestEmail(QDialog):
             self.to.setText(pa)
         self.test_button = b = QPushButton(_('&Test'), self)
         b.clicked.connect(self.start_test)
-        self.test_done.connect(self.on_test_done, type=Qt.QueuedConnection)
+        self.test_done.connect(self.on_test_done, type=Qt.ConnectionType.QueuedConnection)
         self.h = h = QHBoxLayout()
         h.addWidget(le), h.addWidget(b)
         l.addLayout(h)
         if opts.relay_host:
             self.la = la = QLabel(_('Using: %(un)s:%(pw)s@%(host)s:%(port)s and %(enc)s encryption')%
-                    dict(un=opts.relay_username, pw=unhexlify(opts.relay_password).decode('utf-8'),
+                    dict(un=opts.relay_username, pw=from_hex_unicode(opts.relay_password),
                         host=opts.relay_host, port=opts.relay_port, enc=opts.encryption))
             l.addWidget(la)
         self.log = QPlainTextEdit(self)
         l.addWidget(self.log)
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Close)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Close)
         bb.rejected.connect(self.reject), bb.accepted.connect(self.accept)
         l.addWidget(bb)
 
     def start_test(self, *args):
-        self.log.setPlainText(_('Sending mail, please wait...'))
+        if not self.to.text().strip():
+            return error_dialog(self, _('No email address'), _(
+                'No email address to send mail to has been specified. You'
+                ' must specify a To: address before running the test.'), show=True)
+        self.log.setPlainText(_('Sending email, please wait...'))
         self.test_button.setEnabled(False)
         t = Thread(target=self.run_test, name='TestEmailSending')
         t.daemon = True
@@ -64,7 +70,7 @@ class TestEmail(QDialog):
 
     def run_test(self):
         try:
-            tb = self.test_func(unicode(self.to.text())) or _('Mail successfully sent')
+            tb = self.test_func(unicode_type(self.to.text())) or _('Email successfully sent')
         except Exception:
             import traceback
             tb = traceback.format_exc()
@@ -83,7 +89,7 @@ class RelaySetup(QDialog):
 
         self.l = l = QGridLayout()
         self.setLayout(l)
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
         self.tl = QLabel(('<p>'+_('Setup sending email using') +
@@ -111,9 +117,9 @@ class RelaySetup(QDialog):
                 self.ptoggle = QCheckBox(_('&Show password'), self)
                 l.addWidget(self.ptoggle, r, 2)
                 self.ptoggle.stateChanged.connect(
-                        lambda s: self.password.setEchoMode(self.password.Normal if s == Qt.Checked else self.password.Password))
+                        lambda s: self.password.setEchoMode(QLineEdit.EchoMode.Normal if s == Qt.CheckState.Checked else QLineEdit.EchoMode.Password))
         self.username.setText(service['username'])
-        self.password.setEchoMode(self.password.Password)
+        self.password.setEchoMode(QLineEdit.EchoMode.Password)
         self.bl = QLabel('<p>' + _(
             'If you plan to use email to send books to your Kindle, remember to'
             ' add your %s email address to the allowed email addresses in your '
@@ -126,7 +132,7 @@ class RelaySetup(QDialog):
         self.service = service
 
     def accept(self):
-        un = unicode(self.username.text())
+        un = unicode_type(self.username.text())
         if self.service.get('at_in_username', False) and '@' not in un:
             return error_dialog(self, _('Incorrect username'),
                     _('%s needs the full email address as your username') %
@@ -158,7 +164,7 @@ class SendEmail(QWidget, Ui_Form):
             self.relay_username.setText(opts.relay_username)
         self.relay_username.textChanged.connect(self.changed)
         if opts.relay_password:
-            self.relay_password.setText(unhexlify(opts.relay_password).decode('utf-8'))
+            self.relay_password.setText(from_hex_unicode(opts.relay_password))
         self.relay_password.textChanged.connect(self.changed)
         getattr(self, 'relay_'+opts.encryption.lower()).setChecked(True)
         self.relay_tls.toggled.connect(self.changed)
@@ -168,8 +174,8 @@ class SendEmail(QWidget, Ui_Form):
             button.clicked.connect(partial(self.create_service_relay, x))
         self.relay_show_password.stateChanged.connect(
          lambda state : self.relay_password.setEchoMode(
-             self.relay_password.Password if
-             state == 0 else self.relay_password.Normal))
+             QLineEdit.EchoMode.Password if
+             state == 0 else QLineEdit.EchoMode.Normal))
         self.test_email_button.clicked.connect(self.test_email)
 
     def changed(self, *args):
@@ -188,7 +194,7 @@ class SendEmail(QWidget, Ui_Form):
     def test_email_settings(self, to):
         opts = smtp_prefs().parse()
         from calibre.utils.smtp import sendmail, create_mail
-        buf = cStringIO.StringIO()
+        buf = PolyglotStringIO()
         debug_out = partial(prints, file=buf)
         oout, oerr = sys.stdout, sys.stderr
         sys.stdout = sys.stderr = buf
@@ -199,7 +205,7 @@ class SendEmail(QWidget, Ui_Form):
             sendmail(msg, from_=opts.from_, to=[to],
                 verbose=3, timeout=30, relay=opts.relay_host,
                 username=opts.relay_username, debug_output=debug_out,
-                password=unhexlify(opts.relay_password).decode('utf-8'),
+                password=from_hex_unicode(opts.relay_password),
                 encryption=opts.encryption, port=opts.relay_port)
         except:
             import traceback
@@ -217,7 +223,9 @@ class SendEmail(QWidget, Ui_Form):
                     'port': 587,
                     'username': '@gmx.com',
                     'url': 'www.gmx.com',
-                    'extra': '',
+                    'extra': _('Before using this account to send mail, you must enable the'
+                        ' "Enable access to this account via POP3 and IMAP" option in GMX'
+                        ' under More > E-mail Settings > POP3 & IMAP.'),
                     'at_in_username': True,
                 },
                 'gmail': {
@@ -230,7 +238,7 @@ class SendEmail(QWidget, Ui_Form):
                         'Google recently deliberately broke their email sending protocol (SMTP) support in'
                         ' an attempt to force everyone to use their web interface so they can'
                         ' show you more ads. They are trying to claim that SMTP is insecure,'
-                        ' that is incorrect and simply an excuse. To use a gmail account'
+                        ' that is incorrect and simply an excuse. To use a Gmail account'
                         ' you will need to "allow less secure apps" as described'
                         ' <a href="https://support.google.com/accounts/answer/6010255">here</a>.'),
                     'at_in_username': True,
@@ -242,14 +250,14 @@ class SendEmail(QWidget, Ui_Form):
                     'username': '',
                     'url': 'www.hotmail.com',
                     'extra': _('If you are setting up a new'
-                        ' hotmail account, Microsoft requires that you '
+                        ' Hotmail account, Microsoft requires that you '
                         ' verify your account periodically, before it'
                         ' will let calibre send email.'),
                     'at_in_username': True,
                 }
         }[service]
         d = RelaySetup(service, self)
-        if d.exec_() != d.Accepted:
+        if d.exec_() != QDialog.DialogCode.Accepted:
             return
         self.relay_username.setText(d.username.text())
         self.relay_password.setText(d.password.text())
@@ -259,14 +267,14 @@ class SendEmail(QWidget, Ui_Form):
         self.relay_tls.setChecked(True)
 
     def set_email_settings(self, to_set):
-        from_ = unicode(self.email_from.text()).strip()
+        from_ = unicode_type(self.email_from.text()).strip()
         if to_set and not from_:
             error_dialog(self, _('Bad configuration'),
                          _('You must set the From email address')).exec_()
             return False
-        username = unicode(self.relay_username.text()).strip()
-        password = unicode(self.relay_password.text()).strip()
-        host = unicode(self.relay_host.text()).strip()
+        username = unicode_type(self.relay_username.text()).strip()
+        password = unicode_type(self.relay_password.text()).strip()
+        host = unicode_type(self.relay_host.text()).strip()
         enc_method = ('TLS' if self.relay_tls.isChecked() else 'SSL'
                 if self.relay_ssl.isChecked() else 'NONE')
         if host:
@@ -276,16 +284,16 @@ class SendEmail(QWidget, Ui_Form):
                             _('You must either set both the username <b>and</b> password for '
                             'the mail server or no username and no password at all.')).exec_()
                 return False
-            if not (username and password) and not question_dialog(self,
-                    _('Are you sure?'),
-                _('No username and password set for mailserver. Most '
-                    ' mailservers need a username and password. Are you sure?')):
-                    return False
+            if not (username and password) and not question_dialog(
+                    self, _('Are you sure?'),
+                    _('No username and password set for mailserver. Most '
+                      ' mailservers need a username and password. Are you sure?')):
+                return False
         conf = smtp_prefs()
         conf.set('from_', from_)
         conf.set('relay_host', host if host else None)
         conf.set('relay_port', self.relay_port.value())
         conf.set('relay_username', username if username else None)
-        conf.set('relay_password', hexlify(password.encode('utf-8')))
+        conf.set('relay_password', as_hex_unicode(password))
         conf.set('encryption', enc_method)
         return True
