@@ -1,4 +1,5 @@
-from __future__ import with_statement
+
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
@@ -7,11 +8,10 @@ __docformat__ = 'restructuredtext en'
 Wrapper for multi-threaded access to a single sqlite database connection. Serializes
 all calls.
 '''
-import sqlite3 as sqlite, traceback, time, uuid, sys, os
-import repr as reprlib
+
+import sqlite3 as sqlite, traceback, time, uuid, os
 from sqlite3 import IntegrityError, OperationalError
 from threading import Thread
-from Queue import Queue
 from threading import RLock
 from datetime import datetime
 from functools import partial
@@ -19,16 +19,18 @@ from functools import partial
 from calibre.ebooks.metadata import title_sort, author_to_author_sort
 from calibre.utils.date import parse_date, isoformat, local_tz, UNDEFINED_DATE
 from calibre import isbytestring, force_unicode
-from calibre.constants import iswindows, DEBUG, plugins
+from calibre.constants import iswindows, DEBUG, plugins_loc
 from calibre.utils.icu import sort_key
+from calibre_extensions import speedup as _c_speedup
 from calibre import prints
+from polyglot.builtins import cmp, native_string_type, unicode_type
+from polyglot import reprlib
+from polyglot.queue import Queue
 
 from dateutil.tz import tzoffset
 
 global_lock = RLock()
 
-_c_speedup = plugins['speedup'][0]
-
 
 def _c_convert_timestamp(val):
     if not val:
@@ -67,6 +69,7 @@ def _py_convert_timestamp(val):
         return parse_date(val, as_utc=False)
     return None
 
+
 convert_timestamp = _py_convert_timestamp if _c_speedup is None else \
                     _c_convert_timestamp
 
@@ -74,23 +77,25 @@ convert_timestamp = _py_convert_timestamp if _c_speedup is None else \
 def adapt_datetime(dt):
     return isoformat(dt, sep=' ')
 
+
 sqlite.register_adapter(datetime, adapt_datetime)
-sqlite.register_converter('timestamp', convert_timestamp)
+sqlite.register_converter(native_string_type('timestamp'), convert_timestamp)
 
 
 def convert_bool(val):
     return val != '0'
 
+
 sqlite.register_adapter(bool, lambda x : 1 if x else 0)
-sqlite.register_converter('bool', convert_bool)
-sqlite.register_converter('BOOL', convert_bool)
+sqlite.register_converter(native_string_type('bool'), convert_bool)
+sqlite.register_converter(native_string_type('BOOL'), convert_bool)
 
 
 class DynamicFilter(object):
 
     def __init__(self, name):
         self.name = name
-        self.ids = frozenset([])
+        self.ids = frozenset()
 
     def __call__(self, id_):
         return int(id_ in self.ids)
@@ -111,9 +116,14 @@ class Concatenate(object):
             self.ans.append(value)
 
     def finalize(self):
-        if not self.ans:
-            return None
-        return self.sep.join(self.ans)
+        try:
+            if not self.ans:
+                return None
+            return self.sep.join(self.ans)
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            raise
 
 
 class SortedConcatenate(object):
@@ -128,9 +138,14 @@ class SortedConcatenate(object):
             self.ans[ndx] = value
 
     def finalize(self):
-        if len(self.ans) == 0:
-            return None
-        return self.sep.join(map(self.ans.get, sorted(self.ans.keys())))
+        try:
+            if len(self.ans) == 0:
+                return None
+            return self.sep.join(map(self.ans.get, sorted(self.ans.keys())))
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            raise
 
 
 class SortedConcatenateBar(SortedConcatenate):
@@ -148,10 +163,15 @@ class IdentifiersConcat(object):
         self.ans = []
 
     def step(self, key, val):
-        self.ans.append(u'%s:%s'%(key, val))
+        self.ans.append('%s:%s'%(key, val))
 
     def finalize(self):
-        return ','.join(self.ans)
+        try:
+            return ','.join(self.ans)
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            raise
 
 
 class AumSortedConcatenate(object):
@@ -165,13 +185,18 @@ class AumSortedConcatenate(object):
             self.ans[ndx] = ':::'.join((author, sort, link))
 
     def finalize(self):
-        keys = self.ans.keys()
-        l = len(keys)
-        if l == 0:
-            return None
-        if l == 1:
-            return self.ans[keys[0]]
-        return ':#:'.join([self.ans[v] for v in sorted(keys)])
+        try:
+            keys = tuple(self.ans)
+            l = len(keys)
+            if l == 0:
+                return None
+            if l == 1:
+                return self.ans[keys[0]]
+            return ':#:'.join([self.ans[v] for v in sorted(keys)])
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            raise
 
 
 class Connection(sqlite.Connection):
@@ -214,21 +239,20 @@ def icu_collator(s1, s2):
 def load_c_extensions(conn, debug=DEBUG):
     try:
         conn.enable_load_extension(True)
-        ext_path = os.path.join(sys.extensions_location, 'sqlite_custom.'+
+        ext_path = os.path.join(plugins_loc, 'sqlite_custom.'+
                 ('pyd' if iswindows else 'so'))
         conn.load_extension(ext_path)
         conn.enable_load_extension(False)
         return True
     except Exception as e:
         if debug:
-            print 'Failed to load high performance sqlite C extension'
-            print e
+            print('Failed to load high performance sqlite C extension')
+            print(e)
     return False
 
 
 def do_connect(path, row_factory=None):
-    conn = sqlite.connect(path, factory=Connection,
-                                detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
+    conn = sqlite.connect(path, factory=Connection)
     conn.execute('pragma cache_size=-5000')
     encoding = conn.execute('pragma encoding').fetchone()[0]
     conn.create_aggregate('sortconcat', 2, SortedConcatenate)
@@ -239,15 +263,15 @@ def do_connect(path, row_factory=None):
     conn.row_factory = sqlite.Row if row_factory else (lambda cursor, row : list(row))
     conn.create_aggregate('concat', 1, Concatenate)
     conn.create_aggregate('aum_sortconcat', 4, AumSortedConcatenate)
-    conn.create_collation('PYNOCASE', partial(pynocase,
+    conn.create_collation(native_string_type('PYNOCASE'), partial(pynocase,
         encoding=encoding))
     conn.create_function('title_sort', 1, title_sort)
     conn.create_function('author_to_author_sort', 1,
             _author_to_author_sort)
-    conn.create_function('uuid4', 0, lambda : str(uuid.uuid4()))
+    conn.create_function('uuid4', 0, lambda : unicode_type(uuid.uuid4()))
     # Dummy functions for dynamically created filters
     conn.create_function('books_list_filter', 1, lambda x: 1)
-    conn.create_collation('icucollate', icu_collator)
+    conn.create_collation(native_string_type('icucollate'), icu_collator)
     return conn
 
 
@@ -297,7 +321,7 @@ class DBThread(Thread):
                                 break
                             except OperationalError as err:
                                 # Retry if unable to open db file
-                                e = str(err)
+                                e = unicode_type(err)
                                 if 'unable to open' not in e or i == 2:
                                     if 'unable to open' in e:
                                         prints('Unable to open database for func',
@@ -317,7 +341,7 @@ class DatabaseException(Exception):
     def __init__(self, err, tb):
         tb = '\n\t'.join(('\tRemote'+tb).splitlines())
         try:
-            msg = unicode(err) +'\n' + tb
+            msg = unicode_type(err) +'\n' + tb
         except:
             msg = repr(err) + '\n' + tb
         Exception.__init__(self, msg)
@@ -338,7 +362,7 @@ def proxy(fn):
             ok, res = self.proxy.results.get()
             if not ok:
                 if isinstance(res[0], IntegrityError):
-                    raise IntegrityError(unicode(res[0]))
+                    raise IntegrityError(unicode_type(res[0]))
                 raise DatabaseException(*res)
             return res
     return run
@@ -409,5 +433,4 @@ def connect(dbpath, row_factory=None):
 def test():
     c = sqlite.connect(':memory:')
     if load_c_extensions(c, True):
-        print 'Loaded C extension successfully'
-
+        print('Loaded C extension successfully')
