@@ -1,12 +1,10 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import cPickle
 from collections import namedtuple
 
 from calibre.utils.localization import canonicalize_lang
@@ -19,7 +17,8 @@ ccodes, ccodemap, country_names = None, None, None
 def get_codes():
     global ccodes, ccodemap, country_names
     if ccodes is None:
-        data = cPickle.loads(P('localization/iso3166.pickle', allow_user_override=False, data=True))
+        from calibre.utils.serialize import msgpack_loads
+        data = msgpack_loads(P('localization/iso3166.calibre_msgpack', allow_user_override=False, data=True))
         ccodes, ccodemap, country_names = data['codes'], data['three_map'], data['names']
     return ccodes, ccodemap
 
@@ -39,5 +38,3 @@ def parse_lang_code(raw):
         else:
             cc = ccodemap.get(q, None)
     return DictionaryLocale(lc, cc)
-
-
