@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -11,7 +10,9 @@ import sys, glob, os, tempfile, re, codecs
 from lxml import etree
 
 from calibre.constants import config_dir
+from calibre.utils.xml_parse import safe_xml_fromstring
 from calibre.utils.zipfile import ZipFile
+from polyglot.builtins import iteritems
 
 NS_MAP = {
     'oor': "http://openoffice.org/2001/registry",
@@ -26,7 +27,7 @@ BUILTIN_LOCALES = {'en-US', 'en-GB', 'es-ES'}
 def parse_xcu(raw, origin='%origin%'):
     ' Get the dictionary and affix file names as well as supported locales for each dictionary '
     ans = {}
-    root = etree.fromstring(raw)
+    root = safe_xml_fromstring(raw)
 
     for node in XPath('//prop[@oor:name="Format"]/value[text()="DICT_SPELL"]/../..')(root):
         value = XPath('descendant::prop[@oor:name="Locations"]/value')(node)
@@ -45,8 +46,8 @@ def parse_xcu(raw, origin='%origin%'):
 def convert_to_utf8(dic_data, aff_data, errors='strict'):
     m = re.search(br'^SET\s+(\S+)$', aff_data[:2048], flags=re.MULTILINE)
     if m is not None:
-        enc = m.group(1)
-        if enc.upper() not in (b'UTF-8', b'UTF8'):
+        enc = m.group(1).decode('ascii', 'replace')
+        if enc.upper() not in ('UTF-8', 'UTF8'):
             try:
                 codecs.lookup(enc)
             except LookupError:
@@ -68,7 +69,7 @@ def import_from_libreoffice_source_tree(source_path):
     base = P('dictionaries', allow_user_override=False)
     want_locales = set(BUILTIN_LOCALES)
 
-    for (dic, aff), locales in dictionaries.iteritems():
+    for (dic, aff), locales in iteritems(dictionaries):
         c = set(locales) & want_locales
         if c:
             locale = tuple(c)[0]
@@ -123,10 +124,10 @@ def import_from_oxt(source_path, name, dest_dir=None, prefix='dic-'):
                     key = key[3:]
                 return zf.open(key.lstrip('/')).read()
 
-        root = etree.fromstring(zf.open('META-INF/manifest.xml').read())
+        root = safe_xml_fromstring(zf.open('META-INF/manifest.xml').read())
         xcu = XPath('//manifest:file-entry[@manifest:media-type="application/vnd.sun.star.configuration-data"]')(root)[0].get(
             '{%s}full-path' % NS_MAP['manifest'])
-        for (dic, aff), locales in parse_xcu(zf.open(xcu).read(), origin='').iteritems():
+        for (dic, aff), locales in iteritems(parse_xcu(zf.open(xcu).read(), origin='')):
             dic, aff = dic.lstrip('/'), aff.lstrip('/')
             d = tempfile.mkdtemp(prefix=prefix, dir=dest_dir)
             locales = uniq([x for x in map(fill_country_code, locales) if parse_lang_code(x).countrycode])
@@ -143,5 +144,6 @@ def import_from_oxt(source_path, name, dest_dir=None, prefix='dic-'):
             num += 1
     return num
 
+
 if __name__ == '__main__':
     import_from_libreoffice_source_tree(sys.argv[-1])
