@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -11,8 +10,9 @@ from threading import Event
 
 class BonJour(object):  # {{{
 
-    def __init__(self, name='Books in calibre', service_type='_calibre._tcp', path='/opds', add_hostname=True):
+    def __init__(self, name='Books in calibre', service_type='_calibre._tcp', path='/opds', add_hostname=True, wait_for_stop=True):
         self.service_name = name
+        self.wait_for_stop = wait_for_stop
         self.service_type = service_type
         self.add_hostname = add_hostname
         self.path = path
@@ -27,14 +27,8 @@ class BonJour(object):  # {{{
         ip_address, port = loop.bound_address[:2]
         self.zeroconf_ip_address = zipa = verify_ipV4_address(ip_address) or get_external_ip()
         prefix = loop.opts.url_prefix or ''
-        # The Zeroconf module requires everything to be bytestrings
-
-        def enc(x):
-            if not isinstance(x, bytes):
-                x = x.encode('ascii')
-            return x
         mdns_services = (
-            (enc(self.service_name), enc(self.service_type), port, {b'path':enc(prefix + self.path)}),
+            (self.service_name, self.service_type, port, {'path':prefix + self.path}),
         )
         if self.shutdown.is_set():
             return
@@ -48,6 +42,6 @@ class BonJour(object):  # {{{
 
         self.shutdown.wait()
         for s in mdns_services:
-            unpublish(*s, add_hostname=self.add_hostname)
+            unpublish(*s, add_hostname=self.add_hostname, wait_for_stop=self.wait_for_stop)
         self.stopped.set()
 # }}}
