@@ -1,38 +1,53 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, httplib, hashlib, uuid, struct, repr as reprlib, time
+import errno
+import hashlib
+import os
+import struct
+import uuid
 from collections import namedtuple
-from io import BytesIO, DEFAULT_BUFFER_SIZE
-from itertools import chain, repeat, izip_longest
-from operator import itemgetter
 from functools import wraps
-from future_builtins import map
+from io import DEFAULT_BUFFER_SIZE, BytesIO
+from itertools import chain, repeat
+from operator import itemgetter
 
-from calibre import guess_type, force_unicode
-from calibre.constants import __version__, plugins
-from calibre.srv.loop import WRITE
+from calibre import force_unicode, guess_type
+from calibre.constants import __version__
 from calibre.srv.errors import HTTPSimpleResponse
 from calibre.srv.http_request import HTTPRequest, read_headers
-from calibre.srv.sendfile import file_metadata, sendfile_to_socket_async, CannotSendfile, SendfileInterrupted
+from calibre.srv.loop import WRITE
 from calibre.srv.utils import (
-    MultiDict, http_date, HTTP1, HTTP11, socket_errors_socket_closed,
-    sort_q_values, get_translator_for_lang, Cookie)
-from calibre.utils.speedups import ReadOnlyFileBuffer
+    HTTP1, HTTP11, Cookie, MultiDict, fast_now_strftime, get_translator_for_lang,
+    http_date, socket_errors_socket_closed, sort_q_values
+)
 from calibre.utils.monotonic import monotonic
+from calibre.utils.speedups import ReadOnlyFileBuffer
+from polyglot import http_client, reprlib
+from polyglot.builtins import (
+    error_message, iteritems, itervalues, map, reraise, string_or_bytes,
+    unicode_type
+)
 
 Range = namedtuple('Range', 'start stop size')
-MULTIPART_SEPARATOR = uuid.uuid4().hex.decode('ascii')
+MULTIPART_SEPARATOR = uuid.uuid4().hex
+if isinstance(MULTIPART_SEPARATOR, bytes):
+    MULTIPART_SEPARATOR = MULTIPART_SEPARATOR.decode('ascii')
 COMPRESSIBLE_TYPES = {'application/json', 'application/javascript', 'application/xml', 'application/oebps-package+xml'}
-zlib, zlib2_err = plugins['zlib2']
-if zlib2_err:
-    raise RuntimeError('Failed to laod the zlib2 module with error: ' + zlib2_err)
-del zlib2_err
+import zlib
+from itertools import zip_longest
+
+
+def file_metadata(fileobj):
+    try:
+        fd = fileobj.fileno()
+        return os.fstat(fd)
+    except Exception:
+        pass
 
 
 def header_list_to_file(buf):  # {{{
@@ -210,21 +225,24 @@ class RequestData(object):  # {{{
     username = None
 
     def __init__(self, method, path, query, inheaders, request_body_file, outheaders, response_protocol,
-                 static_cache, opts, remote_addr, remote_port, translator_cache, tdir):
+                 static_cache, opts, remote_addr, remote_port, is_trusted_ip, translator_cache,
+                 tdir, forwarded_for, request_original_uri=None):
 
         (self.method, self.path, self.query, self.inheaders, self.request_body_file, self.outheaders,
          self.response_protocol, self.static_cache, self.translator_cache) = (
             method, path, query, inheaders, request_body_file, outheaders,
             response_protocol, static_cache, translator_cache
         )
-        self.remote_addr, self.remote_port = remote_addr, remote_port
+
+        self.remote_addr, self.remote_port, self.is_trusted_ip = remote_addr, remote_port, is_trusted_ip
+        self.forwarded_for = forwarded_for
+        self.request_original_uri = request_original_uri
         self.opts = opts
-        self.status_code = httplib.OK
+        self.status_code = http_client.OK
         self.outcookie = Cookie()
         self.lang_code = self.gettext_func = self.ngettext_func = None
         self.set_translator(self.get_preferred_language())
         self.tdir = tdir
-        self.allowed_book_ids = {}
 
     def generate_static_output(self, name, generator, content_type='text/html; charset=UTF-8'):
         ans = self.static_cache.get(name)
@@ -237,8 +255,7 @@ class RequestData(object):  # {{{
 
     def filesystem_file_with_custom_etag(self, output, *etag_parts):
         etag = hashlib.sha1()
-        string = type('')
-        tuple(map(lambda x:etag.update(string(x)), etag_parts))
+        tuple(map(lambda x:etag.update(unicode_type(x).encode('utf-8')), etag_parts))
         return ETaggedFile(output, etag.hexdigest())
 
     def filesystem_file_with_constant_etag(self, output, etag_as_hexencoded_string):
@@ -279,8 +296,8 @@ class RequestData(object):  # {{{
         if lang_code != self.lang_code:
             found, lang, t = self.get_translator(lang_code)
             self.lang_code = lang
-            self.gettext_func = t.ugettext
-            self.ngettext_func = t.ungettext
+            self.gettext_func = t.gettext
+            self.ngettext_func = t.ngettext
 # }}}
 
 
@@ -302,7 +319,10 @@ class ReadableOutput(object):
 def filesystem_file_output(output, outheaders, stat_result):
     etag = getattr(output, 'etag', None)
     if etag is None:
-        etag = hashlib.sha1(type('')(stat_result.st_mtime) + force_unicode(output.name or '')).hexdigest()
+        oname = output.name or ''
+        if not isinstance(oname, string_or_bytes):
+            oname = unicode_type(oname)
+        etag = hashlib.sha1((unicode_type(stat_result.st_mtime) + force_unicode(oname)).encode('utf-8')).hexdigest()
     else:
         output = output.output
     etag = '"%s"' % etag
@@ -346,7 +366,7 @@ class GeneratedOutput(object):
 class StaticOutput(object):
 
     def __init__(self, data):
-        if isinstance(data, type('')):
+        if isinstance(data, unicode_type):
             data = data.encode('utf-8')
         self.data = data
         self.etag = '"%s"' % hashlib.sha1(data).hexdigest()
@@ -364,20 +384,18 @@ class HTTPConnection(HTTPRequest):
             end = buf.tell()
             buf.seek(pos)
         limit = end - pos
-        if limit == 0:
+        if limit <= 0:
             return True
         if self.use_sendfile and not isinstance(buf, (BytesIO, ReadOnlyFileBuffer)):
+            limit = min(limit, 2 ** 30)
             try:
-                sent = sendfile_to_socket_async(buf, pos, limit, self.socket)
-            except CannotSendfile:
-                self.use_sendfile = False
-                return False
-            except SendfileInterrupted:
-                return False
-            except IOError as e:
+                sent = os.sendfile(self.socket.fileno(), buf.fileno(), pos, limit)
+            except OSError as e:
                 if e.errno in socket_errors_socket_closed:
                     self.ready = self.use_sendfile = False
                     return False
+                if e.errno in (errno.EAGAIN, errno.EINTR):
+                    return False
                 raise
             finally:
                 self.last_activity = monotonic()
@@ -387,24 +405,25 @@ class HTTPConnection(HTTPRequest):
                 self.use_sendfile = self.ready = False
                 raise IOError('sendfile() failed to write any bytes to the socket')
         else:
-            sent = self.send(buf.read(min(limit, self.send_bufsize)))
+            data = buf.read(min(limit, self.send_bufsize))
+            sent = self.send(data)
         buf.seek(pos + sent)
-        return buf.tell() == end
+        return buf.tell() >= end
 
     def simple_response(self, status_code, msg='', close_after_response=True, extra_headers=None):
         if self.response_protocol is HTTP1:
             # HTTP/1.0 has no 413/414/303 codes
             status_code = {
-                httplib.REQUEST_ENTITY_TOO_LARGE:httplib.BAD_REQUEST,
-                httplib.REQUEST_URI_TOO_LONG:httplib.BAD_REQUEST,
-                httplib.SEE_OTHER:httplib.FOUND
+                http_client.REQUEST_ENTITY_TOO_LARGE:http_client.BAD_REQUEST,
+                http_client.REQUEST_URI_TOO_LONG:http_client.BAD_REQUEST,
+                http_client.SEE_OTHER:http_client.FOUND
             }.get(status_code, status_code)
 
         self.close_after_response = close_after_response
         msg = msg.encode('utf-8')
         ct = 'http' if self.method == 'TRACE' else 'plain'
         buf = [
-            '%s %d %s' % (self.response_protocol, status_code, httplib.responses[status_code]),
+            '%s %d %s' % (self.response_protocol, status_code, http_client.responses[status_code]),
             "Content-Length: %s" % len(msg),
             "Content-Type: text/%s; charset=UTF-8" % ct,
             "Date: " + http_date(),
@@ -412,7 +431,7 @@ class HTTPConnection(HTTPRequest):
         if self.close_after_response and self.response_protocol is HTTP11:
             buf.append("Connection: close")
         if extra_headers is not None:
-            for h, v in extra_headers.iteritems():
+            for h, v in iteritems(extra_headers):
                 buf.append('%s: %s' % (h, v))
         buf.append('')
         buf = [(x + '\r\n').encode('ascii') for x in buf]
@@ -425,13 +444,14 @@ class HTTPConnection(HTTPRequest):
     def prepare_response(self, inheaders, request_body_file):
         if self.method == 'TRACE':
             msg = force_unicode(self.request_line, 'utf-8') + '\n' + inheaders.pretty()
-            return self.simple_response(httplib.OK, msg, close_after_response=False)
+            return self.simple_response(http_client.OK, msg, close_after_response=False)
         request_body_file.seek(0)
         outheaders = MultiDict()
         data = RequestData(
             self.method, self.path, self.query, inheaders, request_body_file,
             outheaders, self.response_protocol, self.static_cache, self.opts,
-            self.remote_addr, self.remote_port, self.translator_cache, self.tdir
+            self.remote_addr, self.remote_port, self.is_trusted_ip,
+            self.translator_cache, self.tdir, self.forwarded_for, self.request_original_uri
         )
         self.queue_job(self.run_request_handler, data)
 
@@ -441,28 +461,31 @@ class HTTPConnection(HTTPRequest):
 
     def send_range_not_satisfiable(self, content_length):
         buf = [
-            '%s %d %s' % (self.response_protocol, httplib.REQUESTED_RANGE_NOT_SATISFIABLE, httplib.responses[httplib.REQUESTED_RANGE_NOT_SATISFIABLE]),
+            '%s %d %s' % (
+                self.response_protocol,
+                http_client.REQUESTED_RANGE_NOT_SATISFIABLE,
+                http_client.responses[http_client.REQUESTED_RANGE_NOT_SATISFIABLE]),
             "Date: " + http_date(),
             "Content-Range: bytes */%d" % content_length,
         ]
         response_data = header_list_to_file(buf)
-        self.log_access(status_code=httplib.REQUESTED_RANGE_NOT_SATISFIABLE, response_size=response_data.sz)
+        self.log_access(status_code=http_client.REQUESTED_RANGE_NOT_SATISFIABLE, response_size=response_data.sz)
         self.response_ready(response_data)
 
     def send_not_modified(self, etag=None):
         buf = [
-            '%s %d %s' % (self.response_protocol, httplib.NOT_MODIFIED, httplib.responses[httplib.NOT_MODIFIED]),
+            '%s %d %s' % (self.response_protocol, http_client.NOT_MODIFIED, http_client.responses[http_client.NOT_MODIFIED]),
             "Content-Length: 0",
             "Date: " + http_date(),
         ]
         if etag is not None:
             buf.append('ETag: ' + etag)
         response_data = header_list_to_file(buf)
-        self.log_access(status_code=httplib.NOT_MODIFIED, response_size=response_data.sz)
+        self.log_access(status_code=http_client.NOT_MODIFIED, response_size=response_data.sz)
         self.response_ready(response_data)
 
     def report_busy(self):
-        self.simple_response(httplib.SERVICE_UNAVAILABLE)
+        self.simple_response(http_client.SERVICE_UNAVAILABLE)
 
     def job_done(self, ok, result):
         if not ok:
@@ -475,8 +498,8 @@ class HTTPConnection(HTTPRequest):
                     eh['WWW-Authenticate'] = e.authenticate
                 if e.log:
                     self.log.warn(e.log)
-                return self.simple_response(e.http_code, msg=e.message or '', close_after_response=e.close_connection, extra_headers=eh)
-            raise etype, e, tb
+                return self.simple_response(e.http_code, msg=error_message(e) or '', close_after_response=e.close_connection, extra_headers=eh)
+            reraise(etype, e, tb)
 
         data, output = result
         output = self.finalize_output(output, data, self.method is HTTP1)
@@ -501,10 +524,10 @@ class HTTPConnection(HTTPRequest):
         if ct.startswith('text/') and 'charset=' not in ct:
             outheaders.set('Content-Type', ct + '; charset=UTF-8', replace_all=True)
 
-        buf = [HTTP11 + (' %d ' % data.status_code) + httplib.responses[data.status_code]]
-        for header, value in sorted(outheaders.iteritems(), key=itemgetter(0)):
+        buf = [HTTP11 + (' %d ' % data.status_code) + http_client.responses[data.status_code]]
+        for header, value in sorted(iteritems(outheaders), key=itemgetter(0)):
             buf.append('%s: %s' % (header, value))
-        for morsel in data.outcookie.itervalues():
+        for morsel in itervalues(data.outcookie):
             morsel['version'] = '1'
             x = morsel.output()
             if isinstance(x, bytes):
@@ -522,11 +545,14 @@ class HTTPConnection(HTTPRequest):
     def log_access(self, status_code, response_size=None, username=None):
         if self.access_log is None:
             return
-        if not self.opts.log_not_found and status_code == httplib.NOT_FOUND:
+        if not self.opts.log_not_found and status_code == http_client.NOT_FOUND:
             return
-        line = '%s port-%s %s %s "%s" %s %s' % (
-            self.remote_addr, self.remote_port, username or '-',
-            time.strftime('%d/%b/%Y:%H:%M:%S %z'),
+        ff = self.forwarded_for
+        if ff:
+            ff = '[%s] ' % ff
+        line = '%s port-%s %s%s %s "%s" %s %s' % (
+            self.remote_addr, self.remote_port, ff or '', username or '-',
+            fast_now_strftime('%d/%b/%Y:%H:%M:%S %z'),
             force_unicode(self.request_line or '', 'utf-8'),
             status_code, ('-' if response_size is None else response_size))
         self.access_log(line)
@@ -546,7 +572,9 @@ class HTTPConnection(HTTPRequest):
             self.reset_state()
             return
         if isinstance(output, ReadableOutput):
-            self.use_sendfile = output.use_sendfile and self.opts.use_sendfile and sendfile_to_socket_async is not None
+            self.use_sendfile = output.use_sendfile and self.opts.use_sendfile and hasattr(os, 'sendfile') and self.ssl_context is None
+            # sendfile() does not work with SSL sockets since encryption has to
+            # be done in userspace
             if output.ranges is not None:
                 if isinstance(output.ranges, Range):
                     r = output.ranges
@@ -604,12 +632,13 @@ class HTTPConnection(HTTPRequest):
                 self.set_state(WRITE, self.write_iter, output)
 
     def reset_state(self):
-        self.connection_ready()
-        self.ready = not self.close_after_response
+        ready = not self.close_after_response
         self.end_send_optimization()
+        self.connection_ready()
+        self.ready = ready
 
     def report_unhandled_exception(self, e, formatted_traceback):
-        self.simple_response(httplib.INTERNAL_SERVER_ERROR)
+        self.simple_response(http_client.INTERNAL_SERVER_ERROR)
 
     def finalize_output(self, output, request, is_http1):
         none_match = parse_if_none_match(request.inheaders.get('If-None-Match', ''))
@@ -619,7 +648,7 @@ class HTTPConnection(HTTPRequest):
                 if self.method in ('GET', 'HEAD'):
                     self.send_not_modified(output.etag)
                 else:
-                    self.simple_response(httplib.PRECONDITION_FAILED)
+                    self.simple_response(http_client.PRECONDITION_FAILED)
                 return
 
         opts = self.opts
@@ -628,12 +657,17 @@ class HTTPConnection(HTTPRequest):
         if stat_result is not None:
             output = filesystem_file_output(output, outheaders, stat_result)
             if 'Content-Type' not in outheaders:
-                mt = guess_type(output.name)[0]
+                output_name = output.name
+                if not isinstance(output_name, string_or_bytes):
+                    output_name = unicode_type(output_name)
+                mt = guess_type(output_name)[0]
                 if mt:
                     if mt in {'text/plain', 'text/html', 'application/javascript', 'text/css'}:
                         mt += '; charset=UTF-8'
                     outheaders['Content-Type'] = mt
-        elif isinstance(output, (bytes, type(''))):
+                else:
+                    outheaders['Content-Type'] = 'application/octet-stream'
+        elif isinstance(output, string_or_bytes):
             output = dynamic_output(output, outheaders)
         elif hasattr(output, 'read'):
             output = ReadableOutput(output)
@@ -646,10 +680,10 @@ class HTTPConnection(HTTPRequest):
         ct = outheaders.get('Content-Type', '').partition(';')[0]
         compressible = (not ct or ct.startswith('text/') or ct.startswith('image/svg') or
                         ct.partition(';')[0] in COMPRESSIBLE_TYPES)
-        compressible = (compressible and request.status_code == httplib.OK and
+        compressible = (compressible and request.status_code == http_client.OK and
                         (opts.compress_min_size > -1 and output.content_length >= opts.compress_min_size) and
                         acceptable_encoding(request.inheaders.get('Accept-Encoding', '')) and not is_http1)
-        accept_ranges = (not compressible and output.accept_ranges is not None and request.status_code == httplib.OK and
+        accept_ranges = (not compressible and output.accept_ranges is not None and request.status_code == http_client.OK and
                         not is_http1)
         ranges = get_ranges(request.inheaders.get('Range'), output.content_length) if output.accept_ranges and self.method in ('GET', 'HEAD') else None
         if_range = (request.inheaders.get('If-Range') or '').strip()
@@ -666,7 +700,7 @@ class HTTPConnection(HTTPRequest):
             if self.method in ('GET', 'HEAD'):
                 self.send_not_modified(output.etag)
             else:
-                self.simple_response(httplib.PRECONDITION_FAILED)
+                self.simple_response(http_client.PRECONDITION_FAILED)
             return
 
         output.ranges = None
@@ -697,8 +731,8 @@ class HTTPConnection(HTTPRequest):
                 size = sum(map(len, range_parts)) + sum(r.size + 4 for r in ranges)
                 outheaders.set('Content-Length', '%d' % size, replace_all=True)
                 outheaders.set('Content-Type', 'multipart/byteranges; boundary=' + MULTIPART_SEPARATOR, replace_all=True)
-                output.ranges = izip_longest(ranges, range_parts)
-            request.status_code = httplib.PARTIAL_CONTENT
+                output.ranges = zip_longest(ranges, range_parts)
+            request.status_code = http_client.PARTIAL_CONTENT
         return output
 
 
