@@ -1,33 +1,41 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import ssl, socket, select, os, traceback
-from io import BytesIO
-from Queue import Empty, Full
+import ipaddress
+import os
+import select
+import socket
+import ssl
+import traceback
+from contextlib import suppress
 from functools import partial
+from io import BytesIO
 
 from calibre import as_unicode
+from calibre.constants import iswindows
 from calibre.ptempfile import TemporaryDirectory
 from calibre.srv.errors import JobQueueFull
-from calibre.srv.pool import ThreadPool, PluginPool
-from calibre.srv.opts import Options
 from calibre.srv.jobs import JobsManager
+from calibre.srv.opts import Options
+from calibre.srv.pool import PluginPool, ThreadPool
 from calibre.srv.utils import (
-    socket_errors_socket_closed, socket_errors_nonblocking, HandleInterrupt,
-    socket_errors_eintr, start_cork, stop_cork, DESIRED_SEND_BUFFER_SIZE,
-    create_sock_pair)
-from calibre.utils.socket_inheritance import set_socket_inherit
+    DESIRED_SEND_BUFFER_SIZE, HandleInterrupt, create_sock_pair, socket_errors_eintr,
+    socket_errors_nonblocking, socket_errors_socket_closed, start_cork, stop_cork
+)
 from calibre.utils.logging import ThreadSafeLog
-from calibre.utils.monotonic import monotonic
 from calibre.utils.mdns import get_external_ip
+from calibre.utils.monotonic import monotonic
+from calibre.utils.socket_inheritance import set_socket_inherit
+from polyglot.builtins import iteritems, unicode_type
+from polyglot.queue import Empty, Full
 
 READ, WRITE, RDWR, WAIT = 'READ', 'WRITE', 'RDWR', 'WAIT'
-WAKEUP, JOB_DONE = bytes(bytearray(xrange(2)))
+WAKEUP, JOB_DONE = b'\0', b'\x01'
+IPPROTO_IPV6 = getattr(socket, "IPPROTO_IPV6", 41)
 
 
 class ReadBuffer(object):  # {{{
@@ -118,6 +126,33 @@ class ReadBuffer(object):  # {{{
     # }}}
 
 
+class BadIPSpec(ValueError):
+    pass
+
+
+def parse_trusted_ips(spec):
+    for part in as_unicode(spec).split(','):
+        part = part.strip()
+        try:
+            if '/' in part:
+                yield ipaddress.ip_network(part)
+            else:
+                yield ipaddress.ip_address(part)
+        except Exception as e:
+            raise BadIPSpec(_('{0} is not a valid IP address/network, with error: {1}').format(part, e))
+
+
+def is_ip_trusted(remote_addr, trusted_ips):
+    for tip in trusted_ips:
+        if hasattr(tip, 'hosts'):
+            if remote_addr in tip:
+                return True
+        else:
+            if tip == remote_addr:
+                return True
+    return False
+
+
 class Connection(object):  # {{{
 
     def __init__(self, socket, opts, ssl_context, tdir, addr, pool, log, access_log, wakeup):
@@ -125,16 +160,22 @@ class Connection(object):  # {{{
         try:
             self.remote_addr = addr[0]
             self.remote_port = addr[1]
+            self.parsed_remote_addr = ipaddress.ip_address(as_unicode(self.remote_addr))
         except Exception:
-            # In case addr is None, which can occassionally happen
-            self.remote_addr = self.remote_port = None
+            # In case addr is None, which can occasionally happen
+            self.remote_addr = self.remote_port = self.parsed_remote_addr = None
+        self.is_trusted_ip = bool(self.opts.local_write and getattr(self.parsed_remote_addr, 'is_loopback', False))
+        if not self.is_trusted_ip and self.opts.trusted_ips and self.parsed_remote_addr is not None:
+            self.is_trusted_ip = is_ip_trusted(self.parsed_remote_addr, self.opts.trusted_ips)
         self.orig_send_bufsize = self.send_bufsize = 4096
         self.tdir = tdir
-        self.ssl_context = ssl_context
         self.wait_for = READ
         self.response_started = False
         self.read_buffer = ReadBuffer()
         self.handle_event = None
+        self.ssl_context = ssl_context
+        self.ssl_handshake_done = False
+        self.ssl_terminated = False
         if self.ssl_context is not None:
             self.ready = False
             self.socket = self.ssl_context.wrap_socket(socket, server_side=True, do_handshake_on_connect=False)
@@ -177,17 +218,21 @@ class Connection(object):  # {{{
         except ssl.SSLWantWriteError:
             self.set_state(WRITE, self.do_ssl_handshake)
         else:
+            self.ssl_handshake_done = True
             self.connection_ready()
 
     def send(self, data):
         try:
-            ret = self.socket.send(data)
+            ret = self.socket.send(data) if self.ssl_context is None else self.socket.write(data)
             self.last_activity = monotonic()
             return ret
+        except ssl.SSLWantWriteError:
+            return 0
         except socket.error as e:
             if e.errno in socket_errors_nonblocking or e.errno in socket_errors_eintr:
                 return 0
             elif e.errno in socket_errors_socket_closed:
+                self.log.error('Failed to send all data in state:', self.state_description, 'with error:', e)
                 self.ready = False
                 return 0
             raise
@@ -208,6 +253,8 @@ class Connection(object):  # {{{
                 self.ready = False
                 return b''
             return data
+        except ssl.SSLWantReadError:
+            return b''
         except socket.error as e:
             if e.errno in socket_errors_nonblocking or e.errno in socket_errors_eintr:
                 return b''
@@ -231,6 +278,8 @@ class Connection(object):  # {{{
                 self.ready = False
                 return 0
             return bytes_read
+        except ssl.SSLWantReadError:
+            return 0
         except socket.error as e:
             if e.errno in socket_errors_nonblocking or e.errno in socket_errors_eintr:
                 return 0
@@ -247,6 +296,25 @@ class Connection(object):  # {{{
                 # a closed connection is indicated by signaling
                 # a read condition, and having recv() return 0.
                 self.ready = False
+        except ssl.SSLWantReadError:
+            return
+        except socket.error as e:
+            if e.errno in socket_errors_nonblocking or e.errno in socket_errors_eintr:
+                return
+            if e.errno in socket_errors_socket_closed:
+                self.ready = False
+                return
+            raise
+
+    def drain_ssl_buffer(self):
+        try:
+            self.read_buffer.recv_from(self.socket)
+        except ssl.SSLWantReadError:
+            return
+        except ssl.SSLError as e:
+            self.log.error('Error while reading SSL data from client: %s' % as_unicode(e))
+            self.ready = False
+            return
         except socket.error as e:
             if e.errno in socket_errors_nonblocking or e.errno in socket_errors_eintr:
                 return
@@ -316,6 +384,8 @@ class ServerLoop(object):
         self.ready = False
         self.handler = handler
         self.opts = opts or Options()
+        if self.opts.trusted_ips:
+            self.opts.trusted_ips = tuple(parse_trusted_ips(self.opts.trusted_ips))
         self.log = log or ThreadSafeLog(level=ThreadSafeLog.DEBUG)
         self.jobs_manager = JobsManager(self.opts, self.log)
         self.access_log = access_log
@@ -332,6 +402,7 @@ class ServerLoop(object):
         if self.opts.ssl_certfile is not None and self.opts.ssl_keyfile is not None:
             self.ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
             self.ssl_context.load_cert_chain(certfile=self.opts.ssl_certfile, keyfile=self.opts.ssl_keyfile)
+            self.ssl_context.set_servername_callback(self.on_ssl_servername)
 
         self.pre_activated_socket = None
         if self.opts.allow_socket_preallocation:
@@ -345,8 +416,29 @@ class ServerLoop(object):
         self.pool = ThreadPool(self.log, self.job_completed, count=self.opts.worker_count)
         self.plugin_pool = PluginPool(self, plugins)
 
+    def on_ssl_servername(self, socket, server_name, ssl_context):
+        c = self.connection_map.get(socket.fileno())
+        if getattr(c, 'ssl_handshake_done', False):
+            c.ready = False
+            c.ssl_terminated = True
+            # We do not allow client initiated SSL renegotiation
+            return ssl.ALERT_DESCRIPTION_NO_RENEGOTIATION
+
     def create_control_connection(self):
-        self.control_in, self.control_out = create_sock_pair()
+        if iswindows:
+            self.control_in, self.control_out = create_sock_pair()
+        else:
+            r, w = os.pipe()
+            os.set_blocking(r, False)
+            os.set_blocking(w, True)
+            self.control_in =  open(w, 'wb')
+            self.control_out = open(r, 'rb')
+
+    def close_control_connection(self):
+        with suppress(Exception):
+            self.control_in.close()
+        with suppress(Exception):
+            self.control_out.close()
 
     def __str__(self):
         return "%s(%r)" % (self.__class__.__name__, self.bind_address)
@@ -387,9 +479,7 @@ class ServerLoop(object):
         if not self.socket:
             raise socket.error(msg)
 
-    def serve_forever(self):
-        """ Listen for incoming connections. """
-
+    def initialize_socket(self):
         if self.pre_activated_socket is None:
             try:
                 self.do_bind()
@@ -408,18 +498,19 @@ class ServerLoop(object):
             self.pre_activated_socket = None
             self.setup_socket()
 
+    def serve(self):
         self.connection_map = {}
         self.socket.listen(min(socket.SOMAXCONN, 128))
         self.bound_address = ba = self.socket.getsockname()
         if isinstance(ba, tuple):
-            ba = ':'.join(map(type(''), ba))
+            ba = ':'.join(map(unicode_type, ba))
         self.pool.start()
         with TemporaryDirectory(prefix='srv-') as tdir:
             self.tdir = tdir
-            self.ready = True
             if self.LISTENING_MSG:
                 self.log(self.LISTENING_MSG, ba)
             self.plugin_pool.start()
+            self.ready = True
 
             while self.ready:
                 try:
@@ -433,6 +524,11 @@ class ServerLoop(object):
                     self.log.exception('Error in ServerLoop.tick')
             self.shutdown()
 
+    def serve_forever(self):
+        """ Listen for incoming connections. """
+        self.initialize_socket()
+        self.serve()
+
     def setup_socket(self):
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
@@ -442,8 +538,7 @@ class ServerLoop(object):
         if (hasattr(socket, 'AF_INET6') and self.socket.family == socket.AF_INET6 and
                 self.bind_address[0] in ('::', '::0', '::0.0.0.0')):
             try:
-                self.socket.setsockopt(
-                    socket.IPPROTO_IPV6, socket.IPV6_V6ONLY, 0)
+                self.socket.setsockopt(IPPROTO_IPV6, socket.IPV6_V6ONLY, 0)
             except (AttributeError, socket.error):
                 # Apparently, the socket option is not available in
                 # this machine's TCP stack
@@ -459,8 +554,9 @@ class ServerLoop(object):
 
     def tick(self):
         now = monotonic()
-        read_needed, write_needed, readable, remove = [], [], [], []
-        for s, conn in self.connection_map.iteritems():
+        read_needed, write_needed, readable, remove, close_needed = [], [], [], [], []
+        has_ssl = self.ssl_context is not None
+        for s, conn in iteritems(self.connection_map):
             if now - conn.last_activity > self.opts.timeout:
                 if conn.handle_timeout():
                     conn.last_activity = now
@@ -468,18 +564,30 @@ class ServerLoop(object):
                     remove.append((s, conn))
                     continue
             wf = conn.wait_for
-            if wf is READ:
-                (readable if conn.read_buffer.has_data else read_needed).append(s)
+            if wf is READ or wf is RDWR:
+                if wf is RDWR:
+                    write_needed.append(s)
+                if conn.read_buffer.has_data:
+                    readable.append(s)
+                else:
+                    if has_ssl:
+                        conn.drain_ssl_buffer()
+                        if conn.ready:
+                            (readable if conn.read_buffer.has_data else read_needed).append(s)
+                        else:
+                            close_needed.append((s, conn))
+                    else:
+                        read_needed.append(s)
             elif wf is WRITE:
                 write_needed.append(s)
-            elif wf is RDWR:
-                write_needed.append(s)
-                (readable if conn.read_buffer.has_data else read_needed).append(s)
 
         for s, conn in remove:
             self.log('Closing connection because of extended inactivity: %s' % conn.state_description)
             self.close(s, conn)
 
+        for x, conn in close_needed:
+            self.close(s, conn)
+
         if readable:
             writable = []
         else:
@@ -494,7 +602,7 @@ class ServerLoop(object):
                 # e.args[0]
                 if getattr(e, 'errno', e.args[0]) in socket_errors_eintr:
                     return
-                for s, conn in tuple(self.connection_map.iteritems()):
+                for s, conn in tuple(iteritems(self.connection_map)):
                     try:
                         select.select([s], [], [], 0)
                     except (select.error, socket.error) as e:
@@ -525,24 +633,36 @@ class ServerLoop(object):
                             self.close(s, conn)
             except Exception as e:
                 ignore.add(s)
-                self.log.exception('Unhandled exception in state: %s' % conn.state_description)
-                if conn.ready:
-                    if conn.response_started:
-                        self.close(s, conn)
-                    else:
-                        try:
-                            conn.report_unhandled_exception(e, traceback.format_exc())
-                        except Exception:
-                            self.close(s, conn)
-                else:
-                    self.log.error('Error in SSL handshake, terminating connection: %s' % as_unicode(e))
+                ssl_terminated = getattr(conn, 'ssl_terminated', False)
+                if ssl_terminated:
+                    self.log.warn('Client tried to initiate SSL renegotiation, closing connection')
                     self.close(s, conn)
+                else:
+                    self.log.exception('Unhandled exception in state: %s' % conn.state_description)
+                    if conn.ready:
+                        if conn.response_started:
+                            self.close(s, conn)
+                        else:
+                            try:
+                                conn.report_unhandled_exception(e, traceback.format_exc())
+                            except Exception:
+                                self.close(s, conn)
+                    else:
+                        self.log.error('Error in SSL handshake, terminating connection: %s' % as_unicode(e))
+                        self.close(s, conn)
+
+    def write_to_control(self, what):
+        if iswindows:
+            self.control_in.sendall(what)
+        else:
+            self.control_in.write(what)
+            self.control_in.flush()
 
     def wakeup(self):
-        self.control_in.sendall(WAKEUP)
+        self.write_to_control(WAKEUP)
 
     def job_completed(self):
-        self.control_in.sendall(JOB_DONE)
+        self.write_to_control(JOB_DONE)
 
     def dispatch_job_results(self):
         while True:
@@ -572,14 +692,14 @@ class ServerLoop(object):
                         if self.ssl_context is not None:
                             yield s, conn, RDWR
             elif s == control:
+                f = self.control_out.recv if iswindows else self.control_out.read
                 try:
-                    c = self.control_out.recv(1)
-                except socket.error:
+                    c = f(1)
+                except (socket.error, OSError) as e:
                     if not self.ready:
                         return
-                    self.log.error('Control socket raised an error, resetting')
-                    self.create_control_connection()
-                    continue
+                    self.log.error('Control connection raised an error:', e)
+                    raise
                 if c == JOB_DONE:
                     for s, conn, event in self.dispatch_job_results():
                         yield s, conn, event
@@ -588,8 +708,8 @@ class ServerLoop(object):
                 elif not c:
                     if not self.ready:
                         return
-                    self.log.error('Control socket failed to recv(), resetting')
-                    self.create_control_connection()
+                    self.log.error('Control connection failed to read after signalling ready')
+                    raise Exception('Control connection failed to read, something bad happened')
             else:
                 yield s, self.connection_map[s], READ
         for s in writable:
@@ -613,13 +733,11 @@ class ServerLoop(object):
 
     def shutdown(self):
         self.jobs_manager.shutdown()
-        try:
+        with suppress(socket.error):
             if getattr(self, 'socket', None):
                 self.socket.close()
                 self.socket = None
-        except socket.error:
-            pass
-        for s, conn in tuple(self.connection_map.iteritems()):
+        for s, conn in tuple(iteritems(self.connection_map)):
             self.close(s, conn)
         wait_till = monotonic() + self.opts.shutdown_timeout
         for pool in (self.plugin_pool, self.pool):
@@ -665,7 +783,12 @@ class EchoLine(Connection):  # {{{
 # }}}
 
 
-if __name__ == '__main__':
+def main():
+    print('Starting Echo server')
     s = ServerLoop(EchoLine)
-    with HandleInterrupt(s.wakeup):
+    with HandleInterrupt(s.stop):
         s.serve_forever()
+
+
+if __name__ == '__main__':
+    main()
