@@ -1,11 +1,9 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
-import socket, os, struct, errno
-from base64 import standard_b64encode
+
+import socket, os, struct, errno, numbers
 from collections import deque, namedtuple
 from functools import partial
 from hashlib import sha1
@@ -16,6 +14,8 @@ from calibre.srv.web_socket import (
     PING, PONG, PROTOCOL_ERROR, CONTINUATION, INCONSISTENT_DATA, CONTROL_CODES)
 from calibre.utils.monotonic import monotonic
 from calibre.utils.socket_inheritance import set_socket_inherit
+from polyglot.builtins import range, unicode_type
+from polyglot.binary import as_base64_unicode
 
 HANDSHAKE_STR = '''\
 GET / HTTP/1.1\r
@@ -34,7 +34,7 @@ class WSClient(object):
         self.timeout = timeout
         self.socket = socket.create_connection(('localhost', port), timeout)
         set_socket_inherit(self.socket, False)
-        self.key = standard_b64encode(os.urandom(8))
+        self.key = as_base64_unicode(os.urandom(8))
         self.socket.sendall(HANDSHAKE_STR.format(self.key).encode('ascii'))
         self.read_buf = deque()
         self.read_upgrade_response()
@@ -63,9 +63,10 @@ class WSClient(object):
         if rl != b'HTTP/1.1 101 Switching Protocols\r\n':
             raise ValueError('Server did not respond with correct switching protocols line')
         headers = read_headers(partial(next, lines))
-        key = standard_b64encode(sha1(self.key + GUID_STR).digest())
+        key = as_base64_unicode(sha1((self.key + GUID_STR).encode('ascii')).digest())
         if headers.get('Sec-WebSocket-Accept') != key:
-            raise ValueError('Server did not respond with correct key in Sec-WebSocket-Accept')
+            raise ValueError('Server did not respond with correct key in Sec-WebSocket-Accept: {} != {}'.format(
+                key, headers.get('Sec-WebSocket-Accept')))
 
     def recv(self, max_amt):
         if self.read_buf:
@@ -131,7 +132,7 @@ class WSClient(object):
     def write_message(self, msg, chunk_size=None):
         if isinstance(msg, tuple):
             opcode, msg = msg
-            if isinstance(msg, type('')):
+            if isinstance(msg, unicode_type):
                 msg = msg.encode('utf-8')
             return self.write_frame(1, opcode, msg)
         w = MessageWriter(msg, self.mask, chunk_size)
@@ -146,7 +147,7 @@ class WSClient(object):
         self.socket.sendall(frame)
 
     def write_close(self, code, reason=b''):
-        if isinstance(reason, type('')):
+        if isinstance(reason, unicode_type):
             reason = reason.encode('utf-8')
         self.write_frame(1, CLOSE, struct.pack(b'!H', code) + reason)
 
@@ -182,11 +183,11 @@ class WebSocketTest(BaseTest):
 
         expected_messages, expected_controls = [], []
         for ex in expected:
-            if isinstance(ex, type('')):
+            if isinstance(ex, unicode_type):
                 ex = TEXT, ex
             elif isinstance(ex, bytes):
                 ex = BINARY, ex
-            elif isinstance(ex, int):
+            elif isinstance(ex, numbers.Integral):
                 ex = ex, b''
             if ex[0] in CONTROL_CODES:
                 expected_controls.append(ex)
@@ -194,7 +195,12 @@ class WebSocketTest(BaseTest):
                 expected_messages.append(ex)
         if send_close:
             client.write_close(close_code, close_reason)
-        messages, control_frames = client.read_messages()
+        try:
+            messages, control_frames = client.read_messages()
+        except ConnectionAbortedError:
+            if expected_messages or expected_controls or send_close:
+                raise
+            return
         self.ae(expected_messages, messages)
         self.assertGreaterEqual(len(control_frames), 1)
         self.ae(expected_controls, control_frames[:-1])
@@ -213,11 +219,10 @@ class WebSocketTest(BaseTest):
             for q in (b'', b'\xfe' * 125, b'\xfe' * 126, b'\xfe' * 127, b'\xfe' * 128, b'\xfe' * 65535, b'\xfe' * 65536):
                 simple_test([q], [q])
 
-            for payload in ['', 'ping', b'\x00\xff\xfe\xfd\xfc\xfb\x00\xff', b"\xfe" * 125]:
+            for payload in [b'', b'ping', b'\x00\xff\xfe\xfd\xfc\xfb\x00\xff', b"\xfe" * 125]:
                 simple_test([(PING, payload)], [(PONG, payload)])
 
-            with server.silence_log:
-                simple_test([(PING, 'a'*126)], close_code=PROTOCOL_ERROR, send_close=False)
+            simple_test([(PING, 'a'*126)], close_code=PROTOCOL_ERROR, send_close=False)
 
             for payload in (b'', b'pong'):
                 simple_test([(PONG, payload)], [])
@@ -225,45 +230,45 @@ class WebSocketTest(BaseTest):
             fragments = 'Hello-µ@ßöä üàá-UTF-8!!'.split()
             nc = struct.pack(b'!H', NORMAL_CLOSE)
 
-            with server.silence_log:
-                # It can happen that the server detects bad data and closes the
-                # connection before the client has finished sending all
-                # messages, so ignore failures to send packets.
-                isf_test = partial(simple_test, ignore_send_failures=True)
-                for rsv in xrange(1, 7):
-                    isf_test([{'rsv':rsv, 'opcode':BINARY}], [], close_code=PROTOCOL_ERROR, send_close=False)
-                for opcode in (3, 4, 5, 6, 7, 11, 12, 13, 14, 15):
-                    isf_test([{'opcode':opcode}], [], close_code=PROTOCOL_ERROR, send_close=False)
-
-                for opcode in (PING, PONG):
-                    isf_test([
-                        {'opcode':opcode, 'payload':'f1', 'fin':0}, {'opcode':opcode, 'payload':'f2'}
-                    ], close_code=PROTOCOL_ERROR, send_close=False)
-                isf_test([(CLOSE, nc + b'x'*124)], send_close=False, close_code=PROTOCOL_ERROR)
-
-                for fin in (0, 1):
-                    isf_test([{'opcode':0, 'fin': fin, 'payload':b'non-continuation frame'}, 'some text'], close_code=PROTOCOL_ERROR, send_close=False)
+            # It can happen that the server detects bad data and closes the
+            # connection before the client has finished sending all
+            # messages, so ignore failures to send packets.
+            isf_test = partial(simple_test, ignore_send_failures=True)
+            for rsv in range(1, 7):
+                isf_test([{'rsv':rsv, 'opcode':BINARY}], [], close_code=PROTOCOL_ERROR, send_close=False)
+            for opcode in (3, 4, 5, 6, 7, 11, 12, 13, 14, 15):
+                isf_test([{'opcode':opcode}], [], close_code=PROTOCOL_ERROR, send_close=False)
 
+            for opcode in (PING, PONG):
                 isf_test([
-                    {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':CONTINUATION, 'payload':fragments[1]}, {'opcode':0, 'fin':0}
-                ], [''.join(fragments)], close_code=PROTOCOL_ERROR, send_close=False)
-
-                isf_test([
-                    {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':TEXT, 'payload':fragments[1]},
+                    {'opcode':opcode, 'payload':'f1', 'fin':0}, {'opcode':opcode, 'payload':'f2'}
                 ], close_code=PROTOCOL_ERROR, send_close=False)
+            isf_test([(CLOSE, nc + b'x'*124)], send_close=False, close_code=PROTOCOL_ERROR)
 
-                frags = []
-                for payload in (b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5', b'\xed\xa0\x80', b'\x80\x65\x64\x69\x74\x65\x64'):
-                    frags.append({'opcode':(CONTINUATION if frags else TEXT), 'fin':1 if len(frags) == 2 else 0, 'payload':payload})
-                isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False)
+            for fin in (0, 1):
+                isf_test([{'opcode':0, 'fin': fin, 'payload':b'non-continuation frame'}, 'some text'], close_code=PROTOCOL_ERROR, send_close=False)
 
-                frags, q = [], b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5\xed\xa0\x80\x80\x65\x64\x69\x74\x65\x64'
-                for i, b in enumerate(q):
-                    frags.append({'opcode':(TEXT if i == 0 else CONTINUATION), 'fin':1 if i == len(q)-1 else 0, 'payload':b})
-                isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False, ignore_send_failures=True)
+            isf_test([
+                {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':CONTINUATION, 'payload':fragments[1]}, {'opcode':0, 'fin':0}
+            ], [''.join(fragments)], close_code=PROTOCOL_ERROR, send_close=False)
 
-                for q in (b'\xce', b'\xce\xba\xe1'):
-                    isf_test([{'opcode':TEXT, 'payload':q}], close_code=INCONSISTENT_DATA, send_close=False)
+            isf_test([
+                {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':TEXT, 'payload':fragments[1]},
+            ], close_code=PROTOCOL_ERROR, send_close=False)
+
+            frags = []
+            for payload in (b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5', b'\xed\xa0\x80', b'\x80\x65\x64\x69\x74\x65\x64'):
+                frags.append({'opcode':(CONTINUATION if frags else TEXT), 'fin':1 if len(frags) == 2 else 0, 'payload':payload})
+            isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False)
+
+            frags, q = [], b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5\xed\xa0\x80\x80\x65\x64\x69\x74\x65\x64'
+            for i in range(len(q)):
+                b = q[i:i+1]
+                frags.append({'opcode':(TEXT if i == 0 else CONTINUATION), 'fin':1 if i == len(q)-1 else 0, 'payload':b})
+            isf_test(frags, close_code=INCONSISTENT_DATA, send_close=False, ignore_send_failures=True)
+
+            for q in (b'\xce', b'\xce\xba\xe1'):
+                isf_test([{'opcode':TEXT, 'payload':q}], close_code=INCONSISTENT_DATA, send_close=False)
 
             simple_test([
                 {'opcode':TEXT, 'payload':fragments[0], 'fin':0}, {'opcode':CONTINUATION, 'payload':fragments[1]}
@@ -287,9 +292,10 @@ class WebSocketTest(BaseTest):
             simple_test([
                 {'opcode':TEXT, 'fin':0}, {'opcode':CONTINUATION, 'fin':0, 'payload':'x'}, {'opcode':CONTINUATION},], ['x'])
 
-            for q in (b'\xc2\xb5', b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5',  "Hello-µ@ßöäüàá-UTF-8!!".encode('utf-8')):
+            for q in (b'\xc2\xb5', b'\xce\xba\xe1\xbd\xb9\xcf\x83\xce\xbc\xce\xb5', "Hello-µ@ßöäüàá-UTF-8!!".encode('utf-8')):
                 frags = []
-                for i, b in enumerate(q):
+                for i in range(len(q)):
+                    b = q[i:i+1]
                     frags.append({'opcode':(TEXT if i == 0 else CONTINUATION), 'fin':1 if i == len(q)-1 else 0, 'payload':b})
                 simple_test(frags, [q.decode('utf-8')])
 
@@ -314,3 +320,8 @@ class WebSocketTest(BaseTest):
                 sz *= 1024
                 t, b = 'a'*sz, b'a'*sz
                 simple_test([t, b], [t, b])
+
+
+def find_tests():
+    import unittest
+    return unittest.defaultTestLoader.loadTestsFromTestCase(WebSocketTest)
