@@ -1,29 +1,28 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
 
-import httplib, os, weakref, socket
-from base64 import standard_b64encode
+import os
+import socket
+import weakref
+from calibre_extensions.speedup import utf8_decode, websocket_mask as fast_mask
 from collections import deque
 from hashlib import sha1
-from Queue import Queue, Empty
-from struct import unpack_from, pack, error as struct_error
+from struct import error as struct_error, pack, unpack_from
 from threading import Lock
 
 from calibre import as_unicode
-from calibre.constants import plugins
-from calibre.srv.loop import ServerLoop, HandleInterrupt, WRITE, READ, RDWR, Connection
 from calibre.srv.http_response import HTTPConnection, create_http_handler
+from calibre.srv.loop import (
+    RDWR, READ, WRITE, Connection, HandleInterrupt, ServerLoop
+)
 from calibre.srv.utils import DESIRED_SEND_BUFFER_SIZE
 from calibre.utils.speedups import ReadOnlyFileBuffer
-speedup, err = plugins['speedup']
-if not speedup:
-    raise RuntimeError('Failed to load speedup module with error: ' + err)
-fast_mask, utf8_decode = speedup.websocket_mask, speedup.utf8_decode
-del speedup, err
+from polyglot import http_client
+from polyglot.binary import as_base64_unicode
+from polyglot.builtins import unicode_type
+from polyglot.queue import Empty, Queue
 
 HANDSHAKE_STR = (
     "HTTP/1.1 101 Switching Protocols\r\n"
@@ -179,7 +178,7 @@ class ReadFrame(object):  # {{{
 
 
 def create_frame(fin, opcode, payload, mask=None, rsv=0):
-    if isinstance(payload, type('')):
+    if isinstance(payload, unicode_type):
         payload = payload.encode('utf-8')
     l = len(payload)
     header_len = 2 + (0 if l < 126 else 2 if 126 <= l <= 65535 else 8) + (0 if mask is None else 4)
@@ -208,7 +207,7 @@ class MessageWriter(object):
 
     def __init__(self, buf, mask=None, chunk_size=None):
         self.buf, self.data_type, self.mask = buf, BINARY, mask
-        if isinstance(buf, type('')):
+        if isinstance(buf, unicode_type):
             self.buf, self.data_type = ReadOnlyFileBuffer(buf.encode('utf-8')), TEXT
         elif isinstance(buf, bytes):
             self.buf = ReadOnlyFileBuffer(buf)
@@ -235,6 +234,7 @@ class MessageWriter(object):
         return ReadOnlyFileBuffer(create_frame(fin, opcode, raw, self.mask))
 # }}}
 
+
 conn_id = 0
 
 
@@ -285,11 +285,11 @@ class WebSocketConnection(HTTPConnection):
         except Exception:
             ver_ok = False
         if not ver_ok:
-            return self.simple_response(httplib.BAD_REQUEST, 'Unsupported WebSocket protocol version: %s' % ver)
+            return self.simple_response(http_client.BAD_REQUEST, 'Unsupported WebSocket protocol version: %s' % ver)
         if self.method != 'GET':
-            return self.simple_response(httplib.BAD_REQUEST, 'Invalid WebSocket method: %s' % self.method)
+            return self.simple_response(http_client.BAD_REQUEST, 'Invalid WebSocket method: %s' % self.method)
 
-        response = HANDSHAKE_STR % standard_b64encode(sha1(key + GUID_STR).digest())
+        response = HANDSHAKE_STR % as_base64_unicode(sha1((key + GUID_STR).encode('utf-8')).digest())
         self.optimize_for_sending_packet()
         self.socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         self.set_state(WRITE, self.upgrade_connection_to_ws, ReadOnlyFileBuffer(response.encode('ascii')), inheaders)
@@ -422,7 +422,7 @@ class WebSocketConnection(HTTPConnection):
         self.set_ws_state()
 
     def websocket_close(self, code=NORMAL_CLOSE, reason=b''):
-        if isinstance(reason, type('')):
+        if isinstance(reason, unicode_type):
             reason = reason.encode('utf-8')
         self.stop_reading = True
         reason = reason[:123]
@@ -487,7 +487,7 @@ class WebSocketConnection(HTTPConnection):
         ''' Useful for streaming handlers that want to break up messages into
         frames themselves. Note that these frames will be interleaved with
         control frames, so they should not be too large. '''
-        opcode = (TEXT if isinstance(data, type('')) else BINARY) if is_first else CONTINUATION
+        opcode = (TEXT if isinstance(data, unicode_type) else BINARY) if is_first else CONTINUATION
         fin = 1 if is_last else 0
         frame = create_frame(fin, opcode, data)
         with self.cf_lock:
@@ -496,7 +496,7 @@ class WebSocketConnection(HTTPConnection):
     def send_websocket_ping(self, data=b''):
         ''' Send a PING to the remote client, it should reply with a PONG which
         will be sent to the handle_websocket_pong callback in your handler. '''
-        if isinstance(data, type('')):
+        if isinstance(data, unicode_type):
             data = data.encode('utf-8')
         frame = create_frame(True, PING, data)
         with self.cf_lock:
@@ -562,6 +562,7 @@ def run_echo_server():
     with HandleInterrupt(s.wakeup):
         s.serve_forever()
 
+
 if __name__ == '__main__':
     # import cProfile
     # cProfile.runctx('r()', {'r':run_echo_server}, {}, filename='stats.profile')
