@@ -38,17 +38,17 @@ class BuildTest(unittest.TestCase):
 
     @unittest.skipUnless(islinux, 'DBUS only used on linux')
     def test_dbus(self):
-        import dbus
+        from jeepney.io.blocking import open_dbus_connection
         if 'DBUS_SESSION_BUS_ADDRESS' in os.environ:
-            bus = dbus.SystemBus()
-            self.assertTrue(bus.list_names(), 'Failed to list names on the system bus')
-            bus = dbus.SessionBus()
-            self.assertTrue(bus.list_names(), 'Failed to list names on the session bus')
+            bus = open_dbus_connection(bus='SYSTEM')
+            bus.close()
+            bus = open_dbus_connection(bus='SESSION')
+            bus.close()
             del bus
 
     def test_loaders(self):
         import importlib
-        ldr = importlib.import_module('calibre').__spec__.loader
+        ldr = importlib.import_module('calibre').__spec__.loader.get_resource_reader()
         self.assertIn('ebooks', ldr.contents())
         try:
             raw = ldr.open_resource('__init__.py').read()
@@ -106,6 +106,8 @@ class BuildTest(unittest.TestCase):
 
     def test_zeroconf(self):
         import zeroconf as z, ifaddr
+        from calibre.devices.smart_device_app.driver import monkeypatch_zeroconf
+        monkeypatch_zeroconf()
         del z
         del ifaddr
 
@@ -303,7 +305,7 @@ class BuildTest(unittest.TestCase):
         # hard-coded paths of the Qt installation should work. If they do not,
         # then it is a distro problem.
         fmts = set(map(lambda x: x.data().decode('utf-8'), QImageReader.supportedImageFormats()))  # no2to3
-        testf = {'jpg', 'png', 'svg', 'ico', 'gif'}
+        testf = {'jpg', 'png', 'svg', 'ico', 'gif', 'webp'}
         self.assertEqual(testf.intersection(fmts), testf, "Qt doesn't seem to be able to load some of its image plugins. Available plugins: %s" % fmts)
         data = P('images/blank.png', allow_user_override=False, data=True)
         img = image_from_data(data)
@@ -478,6 +480,8 @@ def find_tests():
     ans.addTests(find_tests())
     from calibre.spell.dictionary import find_tests
     ans.addTests(find_tests())
+    from calibre.db.tests.fts import find_tests
+    ans.addTests(find_tests())
     return ans
 
 
