@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -9,9 +8,9 @@ __docformat__ = 'restructuredtext en'
 
 import threading
 from functools import wraps
-from future_builtins import map
 
-from calibre.constants import plugins
+from calibre_extensions.freetype import FreeType as _FreeType
+from polyglot.builtins import map, unicode_type
 
 
 class ThreadingViolation(Exception):
@@ -30,8 +29,6 @@ def same_thread(func):
         return func(self, *args, **kwargs)
     return check_thread
 
-FreeTypeError = getattr(plugins['freetype'][0], 'FreeTypeError', Exception)
-
 
 class Face(object):
 
@@ -51,7 +48,7 @@ class Face(object):
         '''
         Returns True if all the characters in text have glyphs in this font.
         '''
-        if not isinstance(text, unicode):
+        if not isinstance(text, unicode_type):
             raise TypeError('%r is not a unicode object'%text)
         if has_non_printable_chars:
             from calibre.utils.fonts.utils import get_printable_characters
@@ -61,7 +58,7 @@ class Face(object):
 
     @same_thread
     def glyph_ids(self, text):
-        if not isinstance(text, unicode):
+        if not isinstance(text, unicode_type):
             raise TypeError('%r is not a unicode object'%text)
         for char in text:
             yield self.face.glyph_id(ord(char))
@@ -71,14 +68,8 @@ class FreeType(object):
 
     def __init__(self):
         self.start_thread = threading.current_thread()
-        ft, ft_err = plugins['freetype']
-        if ft_err:
-            raise RuntimeError('Failed to load FreeType module with error: %s'
-                    % ft_err)
-        self.ft = ft.FreeType()
+        self.ft = _FreeType()
 
     @same_thread
     def load_font(self, data):
         return Face(self.ft.load_font(data))
-
-
