@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -12,11 +11,13 @@ from struct import calcsize, unpack, unpack_from
 from collections import namedtuple
 
 from calibre.utils.fonts.utils import get_font_names2, get_font_characteristics
+from polyglot.builtins import range, unicode_type
 
 
 class UnsupportedFont(ValueError):
     pass
 
+
 FontCharacteristics = namedtuple('FontCharacteristics',
     'weight, is_italic, is_bold, is_regular, fs_type, panose, width, is_oblique, is_wws, os2_version')
 FontNames = namedtuple('FontNames',
@@ -47,7 +48,7 @@ class FontMetadata(object):
         elif wt == 700:
             wt = 'bold'
         else:
-            wt = type(u'')(wt)
+            wt = unicode_type(wt)
         self.font_weight = wt
 
         self.font_stretch = ('ultra-condensed', 'extra-condensed',
@@ -70,7 +71,7 @@ class FontMetadata(object):
         sz = calcsize(table_record)
         self.tables = {}
         block = f.read(sz * num_tables)
-        for i in xrange(num_tables):
+        for i in range(num_tables):
             table_tag, table_checksum, table_offset, table_length = \
                     unpack_from(table_record, block, i*sz)
             self.tables[table_tag.lower()] = (table_offset, table_length,
@@ -112,6 +113,7 @@ class FontMetadata(object):
             ans[f] = getattr(self.characteristics, f)
         return ans
 
+
 if __name__ == '__main__':
     import sys
     with open(sys.argv[-1], 'rb') as f:
