@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -12,10 +11,11 @@ from collections import defaultdict
 from threading import Thread
 
 from calibre import walk, prints, as_unicode
-from calibre.constants import (config_dir, iswindows, isosx, plugins, DEBUG,
+from calibre.constants import (config_dir, iswindows, ismacos, DEBUG,
         isworker, filesystem_encoding)
 from calibre.utils.fonts.metadata import FontMetadata, UnsupportedFont
 from calibre.utils.icu import sort_key
+from polyglot.builtins import itervalues, unicode_type, filter
 
 
 class NoFonts(ValueError):
@@ -100,14 +100,18 @@ def fc_list():
 
 def font_dirs():
     if iswindows:
-        winutil, err = plugins['winutil']
-        if err:
-            raise RuntimeError('Failed to load winutil: %s'%err)
-        try:
-            return [winutil.special_folder_path(winutil.CSIDL_FONTS)]
-        except ValueError:
-            return [r'C:\Windows\Fonts']
-    if isosx:
+        from calibre_extensions import winutil
+        paths = {os.path.normcase(r'C:\Windows\Fonts')}
+        for which in (winutil.CSIDL_FONTS, winutil.CSIDL_LOCAL_APPDATA, winutil.CSIDL_APPDATA):
+            try:
+                path = winutil.special_folder_path(which)
+            except ValueError:
+                continue
+            if which != winutil.CSIDL_FONTS:
+                path = os.path.join(path, r'Microsoft\Windows\Fonts')
+            paths.add(os.path.normcase(path))
+        return list(paths)
+    if ismacos:
         return [
                 '/Library/Fonts',
                 '/System/Library/Fonts',
@@ -153,14 +157,14 @@ def path_significance(path, folders):
 
 def build_families(cached_fonts, folders, family_attr='font-family'):
     families = defaultdict(list)
-    for f in cached_fonts.itervalues():
+    for f in itervalues(cached_fonts):
         if not f:
             continue
         lf = icu_lower(f.get(family_attr) or '')
         if lf:
             families[lf].append(f)
 
-    for fonts in families.itervalues():
+    for fonts in itervalues(families):
         # Look for duplicate font files and choose the copy that is from a
         # more significant font directory (prefer user directories over
         # system directories).
@@ -185,7 +189,7 @@ def build_families(cached_fonts, folders, family_attr='font-family'):
 
     font_family_map = dict.copy(families)
     font_families = tuple(sorted((f[0]['font-family'] for f in
-            font_family_map.itervalues()), key=sort_key))
+            itervalues(font_family_map)), key=sort_key))
     return font_family_map, font_families
 # }}}
 
@@ -266,7 +270,7 @@ class FontScanner(Thread):
         '''
         from calibre.utils.fonts.utils import (supports_text,
                 panose_to_css_generic_family, get_printable_characters)
-        if not isinstance(text, unicode):
+        if not isinstance(text, unicode_type):
             raise TypeError(u'%r is not unicode'%text)
         text = get_printable_characters(text)
         found = {}
@@ -280,7 +284,7 @@ class FontScanner(Thread):
             return False
 
         for family in self.find_font_families():
-            faces = filter(filter_faces, self.fonts_for_family(family))
+            faces = list(filter(filter_faces, self.fonts_for_family(family)))
             if not faces:
                 continue
             generic_family = panose_to_css_generic_family(faces[0]['panose'])
@@ -397,6 +401,7 @@ class FontScanner(Thread):
                 prints()
             prints()
 
+
 font_scanner = FontScanner()
 font_scanner.start()
 
@@ -406,5 +411,6 @@ def force_rescan():
     font_scanner.force_rescan()
     font_scanner.run()
 
+
 if __name__ == '__main__':
     font_scanner.dump_fonts()
