@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -16,6 +15,7 @@ from collections import OrderedDict
 from calibre.utils.fonts.utils import read_bmp_prefix
 from calibre.utils.fonts.sfnt import UnknownTable, max_power_of_two
 from calibre.utils.fonts.sfnt.errors import UnsupportedFont
+from polyglot.builtins import range
 
 
 def split_range(start_code, end_code, cmap):  # {{{
@@ -153,7 +153,7 @@ class BMPTable(object):
         ans = {}
         for i, ec in enumerate(self.end_count):
             sc = self.start_count[i]
-            for code in xrange(sc, ec+1):
+            for code in range(sc, ec+1):
                 ro = self.range_offset[i]
                 if ro == 0:
                     glyph_id = self.id_delta[i] + code
@@ -180,7 +180,7 @@ class CmapTable(UnknownTable):
         offset = 4
         sz = calcsize(b'>HHL')
         recs = []
-        for i in xrange(self.num_tables):
+        for i in range(self.num_tables):
             platform, encoding, table_offset = unpack_from(b'>HHL', self.raw,
                     offset)
             offset += sz
@@ -188,7 +188,7 @@ class CmapTable(UnknownTable):
 
         self.bmp_table = None
 
-        for i in xrange(len(recs)):
+        for i in range(len(recs)):
             platform, encoding, offset = recs[i]
             try:
                 next_offset = recs[i+1][-1]
@@ -207,8 +207,7 @@ class CmapTable(UnknownTable):
         if self.bmp_table is None:
             raise UnsupportedFont('This font has no Windows BMP cmap subtable.'
                     ' Most likely a special purpose font.')
-        chars = list(set(chars))
-        chars.sort()
+        chars = sorted(set(chars))
         ans = OrderedDict()
         for i, glyph_id in enumerate(self.bmp_table.get_glyph_ids(chars)):
             if glyph_id > 0:
@@ -229,8 +228,7 @@ class CmapTable(UnknownTable):
     def set_character_map(self, cmap):
         self.version, self.num_tables = 0, 1
         fmt = b'>7H'
-        codes = list(cmap.iterkeys())
-        codes.sort()
+        codes = sorted(cmap)
 
         if not codes:
             start_code = [0xffff]
@@ -256,9 +254,9 @@ class CmapTable(UnknownTable):
         id_delta = []
         id_range_offset = []
         glyph_index_array = []
-        for i in xrange(len(end_code)-1):  # skip the closing codes (0xffff)
-            indices = list(cmap[char_code] for char_code in xrange(start_code[i], end_code[i] + 1))
-            if indices == list(xrange(indices[0], indices[0] + len(indices))):
+        for i in range(len(end_code)-1):  # skip the closing codes (0xffff)
+            indices = list(cmap[char_code] for char_code in range(start_code[i], end_code[i] + 1))
+            if indices == list(range(indices[0], indices[0] + len(indices))):
                 # indices is a contiguous list
                 id_delta_temp = set_id_delta(indices[0] - start_code[i])
                 id_delta.append(id_delta_temp)
@@ -290,4 +288,3 @@ class CmapTable(UnknownTable):
         fmt = b'>4HL'
         offset = calcsize(fmt)
         self.raw = pack(fmt, self.version, self.num_tables, 3, 1, offset) + self.bmp_table
-
