@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -12,6 +11,7 @@ from struct import unpack_from, calcsize, pack, error as struct_error
 from calibre.utils.fonts.sfnt import (UnknownTable, FixedProperty,
         max_power_of_two)
 from calibre.utils.fonts.sfnt.errors import UnsupportedFont
+from polyglot.builtins import range
 
 
 class KernTable(UnknownTable):
@@ -30,7 +30,7 @@ class KernTable(UnknownTable):
             raise UnsupportedFont('kern table has version: %x'%self._version)
         offset = 4 if (self._version == 0) else 8
         tables = []
-        for i in xrange(self.num_tables):
+        for i in range(self.num_tables):
             if self._version == 0:
                 version, length, coverage = unpack_from(b'>3H', self.raw, offset)
                 table_format = version
@@ -57,7 +57,7 @@ class KernTable(UnknownTable):
         offset = calcsize(headerfmt + b'4H')
         entries = []
         entrysz = calcsize(b'>2Hh')
-        for i in xrange(npairs):
+        for i in range(npairs):
             try:
                 left, right, value = unpack_from(b'>2Hh', raw, offset)
             except struct_error:
@@ -87,4 +87,3 @@ class KernTable(UnknownTable):
             header = pack(headerfmt, length, coverage, tuple_index)
         return header + pack(b'>4H', npairs, search_range, entry_selector,
                 range_shift) + entries
-
