@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -11,6 +10,8 @@ import struct
 from io import BytesIO
 from collections import defaultdict
 
+from polyglot.builtins import iteritems, itervalues, unicode_type, range, as_bytes
+
 
 class UnsupportedFont(ValueError):
     pass
@@ -30,7 +31,7 @@ def is_truetype_font(raw):
 def get_tables(raw):
     num_tables = struct.unpack_from(b'>H', raw, 4)[0]
     offset = 4*3  # start of the table record entries
-    for i in xrange(num_tables):
+    for i in range(num_tables):
         table_tag, table_checksum, table_offset, table_length = struct.unpack_from(
                     b'>4s3L', raw, offset)
         yield (table_tag, raw[table_offset:table_offset+table_length], offset,
@@ -40,7 +41,7 @@ def get_tables(raw):
 
 def get_table(raw, name):
     ''' Get the raw table bytes for the specified table in the font '''
-    name = bytes(name.lower())
+    name = as_bytes(name.lower())
     for table_tag, table, table_index, table_offset, table_checksum in get_tables(raw):
         if table_tag.lower() == name:
             return table, table_index, table_offset, table_checksum
@@ -164,7 +165,7 @@ def decode_name_record(recs):
         return mac_names[0]
 
     # Use unicode names
-    for val in unicode_names.itervalues():
+    for val in itervalues(unicode_names):
         return val
 
     return None
@@ -181,7 +182,7 @@ def _get_font_names(raw, raw_is_table=False):
 
     records = defaultdict(list)
 
-    for i in xrange(count):
+    for i in range(count):
         try:
             platform_id, encoding_id, language_id, name_id, length, offset = \
                     struct.unpack_from(b'>6H', table, 6+i*12)
@@ -225,9 +226,9 @@ def get_all_font_names(raw, raw_is_table=False):
     records = _get_font_names(raw, raw_is_table)
     ans = {}
 
-    for name, num in {'family_name':1, 'subfamily_name':2, 'full_name':4,
+    for name, num in iteritems({'family_name':1, 'subfamily_name':2, 'full_name':4,
             'preferred_family_name':16, 'preferred_subfamily_name':17,
-            'wws_family_name':21, 'wws_subfamily_name':22}.iteritems():
+            'wws_family_name':21, 'wws_subfamily_name':22}):
         try:
             ans[name] = decode_name_record(records[num])
         except (IndexError, KeyError, ValueError):
@@ -396,7 +397,7 @@ def get_bmp_glyph_ids(table, bmp, codes):
 
 
 def get_glyph_ids(raw, text, raw_is_table=False):
-    if not isinstance(text, unicode):
+    if not isinstance(text, unicode_type):
         raise TypeError('%r is not a unicode object'%text)
     if raw_is_table:
         table = raw
@@ -406,7 +407,7 @@ def get_glyph_ids(raw, text, raw_is_table=False):
             raise UnsupportedFont('Not a supported font, has no cmap table')
     version, num_tables = struct.unpack_from(b'>HH', table)
     bmp_table = None
-    for i in xrange(num_tables):
+    for i in range(num_tables):
         platform_id, encoding_id, offset = struct.unpack_from(b'>HHL', table,
                 4 + (i*8))
         if platform_id == 3 and encoding_id == 1:
@@ -422,7 +423,7 @@ def get_glyph_ids(raw, text, raw_is_table=False):
 
 
 def supports_text(raw, text, has_only_printable_chars=False):
-    if not isinstance(text, unicode):
+    if not isinstance(text, unicode_type):
         raise TypeError('%r is not a unicode object'%text)
     if not has_only_printable_chars:
         text = get_printable_characters(text)
@@ -462,7 +463,7 @@ def test_glyph_ids():
 
 def test_supports_text():
     data = P('fonts/calibreSymbols.otf', data=True)
-    if not supports_text(data, '.★½'):
+    if not supports_text(data, '.★½⯨'):
         raise RuntimeError('Incorrectly returning that text is not supported')
     if supports_text(data, 'abc'):
         raise RuntimeError('Incorrectly claiming that text is supported')
@@ -472,10 +473,10 @@ def test_find_font():
     from calibre.utils.fonts.scanner import font_scanner
     abcd = '诶比西迪'
     family = font_scanner.find_font_for_text(abcd)[0]
-    print ('Family for Chinese text:', family)
+    print('Family for Chinese text:', family)
     family = font_scanner.find_font_for_text(abcd)[0]
     abcd = 'لوحة المفاتيح العربية'
-    print ('Family for Arabic text:', family)
+    print('Family for Arabic text:', family)
 
 
 def test():
@@ -486,13 +487,14 @@ def test():
 
 def main():
     import sys, os
-    for f in sys.argv[1:]:
-        print (os.path.basename(f))
-        raw = open(f, 'rb').read()
-        print (get_font_names(raw))
+    for arg in sys.argv[1:]:
+        print(os.path.basename(arg))
+        with open(arg, 'rb') as f:
+            raw = f.read()
+        print(get_font_names(raw))
         characs = get_font_characteristics(raw)
-        print (characs)
-        print (panose_to_css_generic_family(characs[5]))
+        print(characs)
+        print(panose_to_css_generic_family(characs[5]))
         verify_checksums(raw)
         remove_embed_restriction(raw)
 
