@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -11,9 +10,10 @@ import os, sys, atexit
 from itertools import product
 
 from calibre import prints, isbytestring
-from calibre.constants import plugins, filesystem_encoding
+from calibre.constants import filesystem_encoding
 from calibre.utils.fonts.utils import (is_truetype_font, get_font_names,
         get_font_characteristics)
+from polyglot.builtins import iteritems, unicode_type
 
 
 class WinFonts(object):
@@ -58,7 +58,7 @@ class WinFonts(object):
         return ft
 
     def fonts_for_family(self, family, normalize=True):
-        family = type(u'')(family)
+        family = unicode_type(family)
         ans = {}
         for weight, is_italic in product((self.w.FW_NORMAL, self.w.FW_BOLD), (False, True)):
             if family in self.app_font_families:
@@ -140,16 +140,15 @@ class WinFonts(object):
 
 
 def load_winfonts():
-    w, err = plugins['winfonts']
-    if w is None:
-        raise RuntimeError('Failed to load the winfonts module: %s'%err)
-    return WinFonts(w)
+    from calibre_extensions import winfonts
+    return WinFonts(winfonts)
 
 
 def test_ttf_reading():
-    for f in sys.argv[1:]:
-        raw = open(f).read()
-        print (os.path.basename(f))
+    for arg in sys.argv[1:]:
+        with open(arg, 'rb') as f:
+            raw = f.read()
+        print(os.path.basename(arg))
         get_font_characteristics(raw)
         print()
 
@@ -165,15 +164,16 @@ def test():
     else:
         w = load_winfonts()
 
-    print (w.w)
+    print(w.w)
     families = w.font_families()
-    print (families)
+    print(families)
 
     for family in families:
         prints(family + ':')
-        for font, data in w.fonts_for_family(family).iteritems():
+        for font, data in iteritems(w.fonts_for_family(family)):
             prints('  ', font, data[0], data[1], len(data[2]))
-        print ()
+        print()
+
 
 if __name__ == '__main__':
     test()
