@@ -1,34 +1,27 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import sys
-
-is_narrow_build = sys.maxunicode < 0x10ffff
-
 # Setup code {{{
 import codecs
+import sys
 
-from calibre.constants import plugins
 from calibre.utils.config_base import tweaks
+from calibre_extensions import icu as _icu
+from polyglot.builtins import cmp, filter, unicode_type
 
 _locale = _collator = _primary_collator = _sort_collator = _numeric_collator = _case_sensitive_collator = None
+cmp
 
-_none = u''
+_none = ''
 _none2 = b''
 _cmap = {}
 
-_icu, err = plugins['icu']
-if _icu is None:
-    raise RuntimeError('Failed to load icu with error: %s' % err)
-del err
 icu_unicode_version = getattr(_icu, 'unicode_version', None)
-_nmodes = {m:getattr(_icu, 'UNORM_'+m, None) for m in ('NFC', 'NFD', 'NFKC', 'NFKD', 'NONE', 'DEFAULT', 'FCD')}
+_nmodes = {m:getattr(_icu, m) for m in ('NFC', 'NFD', 'NFKC', 'NFKD')}
 
 # Ensure that the python internal filesystem and default encodings are not ASCII
 
@@ -38,6 +31,8 @@ def is_ascii(name):
         return codecs.lookup(name).name == b'ascii'
     except (TypeError, LookupError):
         return True
+
+
 try:
     if is_ascii(sys.getdefaultencoding()):
         _icu.set_default_encoding(b'utf-8')
@@ -66,7 +61,7 @@ def collator():
         try:
             _collator = _icu.Collator(_locale)
         except Exception as e:
-            print ('Failed to load collator for locale: %r with error %r, using English' % (_locale, e))
+            print('Failed to load collator for locale: %r with error %r, using English' % (_locale, e))
             _collator = _icu.Collator('en')
     return _collator
 
@@ -115,105 +110,94 @@ def case_sensitive_collator():
         _case_sensitive_collator.upper_first = True
     return _case_sensitive_collator
 
-# Templates that will be used to generate various concrete
-# function implementations based on different collators, to allow lazy loading
-# of collators, with maximum runtime performance
 
-_sort_key_template = '''
-def {name}(obj):
-    try:
-        try:
-            return {collator}.{func}(obj)
-        except AttributeError:
-            return {collator_func}().{func}(obj)
-    except TypeError:
-        if isinstance(obj, bytes):
-            try:
-                obj = obj.decode(sys.getdefaultencoding())
-            except ValueError:
-                return obj
-            return {collator}.{func}(obj)
-    return b''
-'''
+def make_sort_key_func(collator_function, func_name='sort_key'):
+    func = None
 
-_strcmp_template = '''
-def {name}(a, b):
-    try:
-        try:
-            return {collator}.{func}(a, b)
-        except AttributeError:
-            return {collator_func}().{func}(a, b)
-    except TypeError:
-        if isinstance(a, bytes):
-            try:
-                a = a.decode(sys.getdefaultencoding())
-            except ValueError:
-                return cmp(a, b)
-        elif a is None:
-            a = u''
-        if isinstance(b, bytes):
-            try:
-                b = b.decode(sys.getdefaultencoding())
-            except ValueError:
-                return cmp(a, b)
-        elif b is None:
-            b = u''
-        return {collator}.{func}(a, b)
-'''
+    def sort_key(a):
+        nonlocal func
+        if func is None:
+            func = getattr(collator_function(), func_name)
 
-_change_case_template = '''
-def {name}(x):
-    try:
         try:
-            return _icu.change_case(x, _icu.{which}, _locale)
-        except NotImplementedError:
+            return func(a)
+        except TypeError:
+            if isinstance(a, bytes):
+                try:
+                    a = a.decode(sys.getdefaultencoding())
+                except ValueError:
+                    return a
+                return func(a)
+        return b''
+
+    return sort_key
+
+
+def make_two_arg_func(collator_function, func_name='strcmp'):
+    func = None
+
+    def two_args(a, b):
+        nonlocal func
+        if func is None:
+            func = getattr(collator_function(), func_name)
+
+        try:
+            return func(a, b)
+        except TypeError:
+            if isinstance(a, bytes):
+                try:
+                    a = a.decode(sys.getdefaultencoding())
+                except Exception:
+                    return cmp(a, b)
+            elif a is None:
+                a = ''
+            if isinstance(b, bytes):
+                try:
+                    b = b.decode(sys.getdefaultencoding())
+                except Exception:
+                    return cmp(a, b)
+            elif b is None:
+                b = ''
+            return func(a, b)
+
+    return two_args
+
+
+def make_change_case_func(which):
+
+    def change_case(x):
+        try:
+            try:
+                return _icu.change_case(x, which, _locale)
+            except NotImplementedError:
+                pass
             collator()  # sets _locale
-            return _icu.change_case(x, _icu.{which}, _locale)
-    except TypeError:
-        if isinstance(x, bytes):
-            try:
-                x = x.decode(sys.getdefaultencoding())
-            except ValueError:
-                return x
-            return _icu.change_case(x, _icu.{which}, _locale)
-        raise
-'''
-
-
-def _make_func(template, name, **kwargs):
-    l = globals()
-    kwargs['name'] = name
-    kwargs['func'] = kwargs.get('func', 'sort_key')
-    exec template.format(**kwargs) in l
-    return l[name]
-
-
+            return _icu.change_case(x, which, _locale)
+        except TypeError:
+            if isinstance(x, bytes):
+                try:
+                    x = x.decode(sys.getdefaultencoding())
+                except ValueError:
+                    return x
+                return _icu.change_case(x, which, _locale)
+            raise
+    return change_case
 # }}}
 
+
 # ################ The string functions ########################################
-sort_key = _make_func(_sort_key_template, 'sort_key', collator='_sort_collator', collator_func='sort_collator')
+sort_key = make_sort_key_func(sort_collator)
+numeric_sort_key = make_sort_key_func(numeric_collator)
+primary_sort_key = make_sort_key_func(primary_collator)
+case_sensitive_sort_key = make_sort_key_func(case_sensitive_collator)
+collation_order = make_sort_key_func(sort_collator, 'collation_order')
 
-numeric_sort_key = _make_func(_sort_key_template, 'numeric_sort_key', collator='_numeric_collator', collator_func='numeric_collator')
-
-primary_sort_key = _make_func(_sort_key_template, 'primary_sort_key', collator='_primary_collator', collator_func='primary_collator')
-
-case_sensitive_sort_key = _make_func(_sort_key_template, 'case_sensitive_sort_key',
-                                     collator='_case_sensitive_collator', collator_func='case_sensitive_collator')
-
-collation_order = _make_func(_sort_key_template, 'collation_order', collator='_sort_collator', collator_func='sort_collator', func='collation_order')
-
-strcmp = _make_func(_strcmp_template, 'strcmp', collator='_sort_collator', collator_func='sort_collator', func='strcmp')
-
-case_sensitive_strcmp = _make_func(
-    _strcmp_template, 'case_sensitive_strcmp', collator='_case_sensitive_collator', collator_func='case_sensitive_collator', func='strcmp')
-
-primary_strcmp = _make_func(_strcmp_template, 'primary_strcmp', collator='_primary_collator', collator_func='primary_collator', func='strcmp')
-
-upper = _make_func(_change_case_template, 'upper', which='UPPER_CASE')
-
-lower = _make_func(_change_case_template, 'lower', which='LOWER_CASE')
-
-title_case = _make_func(_change_case_template, 'title_case', which='TITLE_CASE')
+strcmp = make_two_arg_func(sort_collator)
+case_sensitive_strcmp = make_two_arg_func(case_sensitive_collator)
+primary_strcmp = make_two_arg_func(primary_collator)
+upper = make_change_case_func(_icu.UPPER_CASE)
+lower = make_change_case_func(_icu.LOWER_CASE)
+title_case = make_change_case_func(_icu.TITLE_CASE)
 
 
 def capitalize(x):
@@ -222,31 +206,25 @@ def capitalize(x):
     except (IndexError, TypeError, AttributeError):
         return x
 
+
 try:
     swapcase = _icu.swap_case
 except AttributeError:  # For people running from source
     swapcase = lambda x:x.swapcase()
 
-find = _make_func(_strcmp_template, 'find', collator='_collator', collator_func='collator', func='find')
-
-primary_find = _make_func(_strcmp_template, 'primary_find', collator='_primary_collator', collator_func='primary_collator', func='find')
-
-contains = _make_func(_strcmp_template, 'contains', collator='_collator', collator_func='collator', func='contains')
-
-primary_contains = _make_func(_strcmp_template, 'primary_contains', collator='_primary_collator', collator_func='primary_collator', func='contains')
-
-startswith = _make_func(_strcmp_template, 'startswith', collator='_collator', collator_func='collator', func='startswith')
-
-primary_startswith = _make_func(_strcmp_template, 'primary_startswith', collator='_primary_collator', collator_func='primary_collator', func='startswith')
-
+find = make_two_arg_func(collator, 'find')
+primary_find = make_two_arg_func(primary_collator, 'find')
+contains = make_two_arg_func(collator, 'contains')
+primary_contains = make_two_arg_func(primary_collator, 'contains')
+startswith = make_two_arg_func(collator, 'startswith')
+primary_startswith = make_two_arg_func(primary_collator, 'startswith')
 safe_chr = _icu.chr
-
 ord_string = _icu.ord_string
 
 
 def character_name(string):
     try:
-        return _icu.character_name(unicode(string)) or None
+        return _icu.character_name(unicode_type(string)) or None
     except (TypeError, ValueError, KeyError):
         pass
 
@@ -263,7 +241,7 @@ def normalize(text, mode='NFC'):
     # that unless you have very good reasons not too. Also, it's speed
     # decreases on wide python builds, where conversion to/from ICU's string
     # representation is slower.
-    return _icu.normalize(_nmodes[mode], unicode(text))
+    return _icu.normalize(_nmodes[mode], unicode_type(text))
 
 
 def contractions(col=None):
@@ -290,9 +268,7 @@ def partition_by_first_letter(items, reverse=False, key=lambda x:x):
         c = icu_upper(key(item) or ' ')
         ordnum, ordlen = collation_order(c)
         if last_ordnum != ordnum:
-            if not is_narrow_build:
-                ordlen = 1
-            last_c = c[0:ordlen]
+            last_c = c[0:1]
             last_ordnum = ordnum
         try:
             ans[last_c].append(item)
@@ -300,15 +276,15 @@ def partition_by_first_letter(items, reverse=False, key=lambda x:x):
             ans[last_c] = [item]
     return ans
 
+
 # Return the number of unicode codepoints in a string
-string_length = _icu.string_length if is_narrow_build else len
+string_length = len
 
 # Return the number of UTF-16 codepoints in a string
-utf16_length = len if is_narrow_build else _icu.utf16_length
+utf16_length = _icu.utf16_length
 
 ################################################################################
 
 if __name__ == '__main__':
     from calibre.utils.icu_test import run
     run(verbosity=4)
-
