@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -26,6 +25,7 @@ class DirTooLarge(ValueError):
     def __init__(self, bdir):
         ValueError.__init__(self, 'The directory {0} is too large to monitor. Try increasing the value in /proc/sys/fs/inotify/max_user_watches'.format(bdir))
 
+
 _inotify = None
 
 
@@ -320,15 +320,16 @@ class INotifyTreeWatcher(INotify):
         self.modified = set()
         return ret
 
+
 if __name__ == '__main__':
     w = INotifyTreeWatcher(sys.argv[-1])
     w()
-    print ('Monitoring', sys.argv[-1], 'press Ctrl-C to stop')
+    print('Monitoring', sys.argv[-1], 'press Ctrl-C to stop')
     try:
         while w.wait():
             modified = w()
             for path in modified:
-                print (path or sys.argv[-1], 'changed')
+                print(path or sys.argv[-1], 'changed')
         raise SystemExit('inotify flaked out')
     except KeyboardInterrupt:
         pass
