@@ -1,21 +1,23 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, cPickle, sys
+import os, sys
 from threading import Thread
 from collections import namedtuple
-from Queue import Queue
+from multiprocessing.connection import Pipe
 
 from calibre import detect_ncpus, as_unicode, prints
 from calibre.constants import iswindows, DEBUG
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils import join_with_timeout
 from calibre.utils.ipc import eintr_retry_call
+from calibre.utils.serialize import pickle_dumps, pickle_loads
+from polyglot.builtins import iteritems, itervalues
+from polyglot.queue import Queue
 
 Job = namedtuple('Job', 'id module func args kwargs')
 Result = namedtuple('Result', 'value err traceback')
@@ -57,7 +59,7 @@ def get_stdout(process):
             raw = process.stdout.read(1)
             if raw:
                 try:
-                    sys.stdout.write(raw)
+                    sys.stdout.buffer.write(raw)
                 except EnvironmentError:
                     pass
             else:
@@ -66,11 +68,11 @@ def get_stdout(process):
             break
 
 
-def start_worker(code, name=''):
+def start_worker(code, pass_fds, name=''):
     from calibre.utils.ipc.simple_worker import start_pipe_worker
     if name:
         name = '-' + name
-    p = start_pipe_worker(code, **worker_kwargs)
+    p = start_pipe_worker(code, pass_fds=pass_fds, **worker_kwargs)
     if get_stdout_from_child:
         t = Thread(target=get_stdout, name='PoolWorkerGetStdout' + name, args=(p,))
         t.daemon = True
@@ -95,7 +97,7 @@ class Worker(object):
         self.name = name or ''
 
     def __call__(self, job):
-        eintr_retry_call(self.conn.send_bytes, cPickle.dumps(job, -1))
+        eintr_retry_call(self.conn.send_bytes, pickle_dumps(job))
         if job is not None:
             self.job_id = job.id
             t = Thread(target=self.recv, name='PoolWorker-'+self.name)
@@ -104,7 +106,7 @@ class Worker(object):
 
     def recv(self):
         try:
-            result = cPickle.loads(eintr_retry_call(self.conn.recv_bytes))
+            result = pickle_loads(eintr_retry_call(self.conn.recv_bytes))
             wr = WorkerResult(self.job_id, result, False, self)
         except Exception as err:
             import traceback
@@ -130,8 +132,7 @@ class Pool(Thread):
         self.results = Queue()
         self.tracker = Queue()
         self.terminal_failure = None
-        self.common_data = cPickle.dumps(None, -1)
-        self.worker_data = None
+        self.common_data = pickle_dumps(None)
         self.shutting_down = False
 
         self.start()
@@ -186,13 +187,15 @@ class Pool(Thread):
         self.shutdown_workers(wait_time=wait_time)
 
     def create_worker(self):
-        p = start_worker('from {0} import run_main, {1}; run_main({1})'.format(self.__class__.__module__, 'worker_main'))
+        a, b = Pipe()
+        with a:
+            cmd = 'from {0} import run_main, {1}; run_main({2!r}, {1})'.format(
+                self.__class__.__module__, 'worker_main', a.fileno())
+            p = start_worker(cmd, (a.fileno(),))
         sys.stdout.flush()
-        eintr_retry_call(p.stdin.write, self.worker_data)
-        p.stdin.flush(), p.stdin.close()
-        conn = eintr_retry_call(self.listener.accept)
-        w = Worker(p, conn, self.events, self.name)
-        if self.common_data != cPickle.dumps(None, -1):
+        p.stdin.close()
+        w = Worker(p, b, self.events, self.name)
+        if self.common_data != pickle_dumps(None):
             w.set_common_data(self.common_data)
         return w
 
@@ -208,10 +211,6 @@ class Pool(Thread):
             return False
 
     def run(self):
-        from calibre.utils.ipc.server import create_listener
-        self.auth_key = os.urandom(32)
-        self.address, self.listener = create_listener(self.auth_key)
-        self.worker_data = cPickle.dumps((self.address, self.auth_key), -1)
         if self.start_worker() is False:
             return
 
@@ -243,12 +242,12 @@ class Pool(Thread):
                 return False
             self.results.put(worker_result)
         else:
-            self.common_data = cPickle.dumps(event, -1)
+            self.common_data = pickle_dumps(event)
             if len(self.common_data) > MAX_SIZE:
                 self.cd_file = PersistentTemporaryFile('pool_common_data')
                 with self.cd_file as f:
                     f.write(self.common_data)
-                self.common_data = cPickle.dumps(File(f.name), -1)
+                self.common_data = pickle_dumps(File(f.name))
             for worker in self.available_workers:
                 try:
                     worker.set_common_data(self.common_data)
@@ -280,7 +279,7 @@ class Pool(Thread):
     def terminal_error(self):
         if self.shutting_down:
             return
-        for worker, job in self.busy_workers.iteritems():
+        for worker, job in iteritems(self.busy_workers):
             self.results.put(WorkerResult(job.id, Result(None, None, None), True, worker))
             self.tracker.task_done()
         while self.pending_jobs:
@@ -340,7 +339,7 @@ def worker_main(conn):
     common_data = None
     while True:
         try:
-            job = cPickle.loads(eintr_retry_call(conn.recv_bytes))
+            job = pickle_loads(eintr_retry_call(conn.recv_bytes))
         except EOFError:
             break
         except KeyboardInterrupt:
@@ -354,7 +353,9 @@ def worker_main(conn):
             break
         if not isinstance(job, Job):
             if isinstance(job, File):
-                common_data = cPickle.load(open(job.name, 'rb'))
+                with lopen(job.name, 'rb') as f:
+                    common_data = f.read()
+                common_data = pickle_loads(common_data)
             else:
                 common_data = job
             continue
@@ -374,7 +375,7 @@ def worker_main(conn):
             import traceback
             result = Result(None, as_unicode(err), traceback.format_exc())
         try:
-            eintr_retry_call(conn.send_bytes, cPickle.dumps(result, -1))
+            eintr_retry_call(conn.send_bytes, pickle_dumps(result))
         except EOFError:
             break
         except Exception:
@@ -385,16 +386,17 @@ def worker_main(conn):
     return 0
 
 
-def run_main(func):
-    from multiprocessing.connection import Client
-    from contextlib import closing
-    address, key = cPickle.loads(eintr_retry_call(sys.stdin.read))
-    with closing(Client(address, authkey=key)) as conn:
+def run_main(client_fd, func):
+    if iswindows:
+        from multiprocessing.connection import PipeConnection as Connection
+    else:
+        from multiprocessing.connection import Connection
+    with Connection(client_fd) as conn:
         raise SystemExit(func(conn))
 
 
 def test_write():
-    print ('Printing to stdout in worker')
+    print('Printing to stdout in worker')
 
 
 def test():
@@ -403,8 +405,8 @@ def test():
         while not p.results.empty():
             r = p.results.get()
             if not ignore_fail and r.is_terminal_failure:
-                print (r.result.err)
-                print (r.result.traceback)
+                print(r.result.err)
+                print(r.result.traceback)
                 raise SystemExit(1)
             ans[r.id] = r.result
         return ans
@@ -416,7 +418,7 @@ def test():
         p(i, 'def x(i):\n return 2*i', 'x', i)
         expected_results[i] = 2 * i
     p.wait_for_tasks(30)
-    results = {k:v.value for k, v in get_results(p).iteritems()}
+    results = {k:v.value for k, v in iteritems(get_results(p))}
     if results != expected_results:
         raise SystemExit('%r != %r' % (expected_results, results))
     p.shutdown(), p.join()
@@ -430,7 +432,7 @@ def test():
         p(i, 'def x(i, common_data=None):\n return common_data + i', 'x', i)
         expected_results[i] = 7 + i
     p.wait_for_tasks(30)
-    results = {k:v.value for k, v in get_results(p).iteritems()}
+    results = {k:v.value for k, v in iteritems(get_results(p))}
     if results != expected_results:
         raise SystemExit('%r != %r' % (expected_results, results))
     p.shutdown(), p.join()
@@ -452,7 +454,7 @@ def test():
         p(i, 'def x(i):\n return 1/0', 'x', i)
     p.wait_for_tasks(30)
     c = 0
-    for r in get_results(p).itervalues():
+    for r in itervalues(get_results(p)):
         c += 1
         if not r.traceback or 'ZeroDivisionError' not in r.traceback:
             raise SystemExit('Unexpected result: %s' % r)
@@ -482,4 +484,4 @@ def test():
         p(i, 'import time;\ndef x(i):\n time.sleep(10000)', 'x', i)
     p.shutdown(), p.join()
 
-    print ('Tests all passed!')
+    print('Tests all passed!')
