@@ -1,53 +1,54 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, cPickle, traceback, time, importlib
-from binascii import hexlify, unhexlify
-from multiprocessing.connection import Client
+import importlib
+import os
+import time
+import traceback
+from multiprocessing import Pipe
 from threading import Thread
-from contextlib import closing
 
 from calibre.constants import iswindows
 from calibre.utils.ipc import eintr_retry_call
 from calibre.utils.ipc.launch import Worker
+from calibre.utils.monotonic import monotonic
+from polyglot.builtins import environ_item, string_or_bytes, unicode_type
+
+if iswindows:
+    from multiprocessing.connection import PipeConnection as Connection
+else:
+    from multiprocessing.connection import Connection
 
 
 class WorkerError(Exception):
 
-    def __init__(self, msg, orig_tb=''):
+    def __init__(self, msg, orig_tb='', log_path=None):
         Exception.__init__(self, msg)
         self.orig_tb = orig_tb
+        self.log_path = log_path
 
 
 class ConnectedWorker(Thread):
 
-    def __init__(self, listener, args):
+    def __init__(self, conn, args):
         Thread.__init__(self)
         self.daemon = True
 
-        self.listener = listener
+        self.conn = conn
         self.args = args
         self.accepted = False
         self.tb = None
         self.res = None
 
     def run(self):
-        conn = tb = None
-        try:
-            conn = eintr_retry_call(self.listener.accept)
-        except:
-            tb = traceback.format_exc()
-        if conn is None:
-            self.tb = tb
-            return
         self.accepted = True
-        with closing(conn):
+        conn = self.conn
+        with conn:
             try:
                 eintr_retry_call(conn.send, self.args)
                 self.res = eintr_retry_call(conn.recv)
@@ -55,18 +56,15 @@ class ConnectedWorker(Thread):
                 self.tb = traceback.format_exc()
 
 
-class OffloadWorker(object):
+class OffloadWorker:
 
-    def __init__(self, listener, worker):
-        self.listener = listener
+    def __init__(self, conn, worker):
+        self.conn = conn
         self.worker = worker
-        self.conn = None
         self.kill_thread = t = Thread(target=self.worker.kill)
         t.daemon = True
 
     def __call__(self, module, func, *args, **kwargs):
-        if self.conn is None:
-            self.conn = eintr_retry_call(self.listener.accept)
         eintr_retry_call(self.conn.send, (module, func, args, kwargs))
         return eintr_retry_call(self.conn.recv)
 
@@ -90,16 +88,16 @@ class OffloadWorker(object):
         return self.worker.is_alive or self.kill_thread.is_alive()
 
 
-def communicate(ans, worker, listener, args, timeout=300, heartbeat=None,
+def communicate(ans, worker, conn, args, timeout=300, heartbeat=None,
         abort=None):
-    cw = ConnectedWorker(listener, args)
+    cw = ConnectedWorker(conn, args)
     cw.start()
-    st = time.time()
+    st = monotonic()
     check_heartbeat = callable(heartbeat)
 
     while worker.is_alive and cw.is_alive():
         cw.join(0.01)
-        delta = time.time() - st
+        delta = monotonic() - st
         if not cw.accepted and delta > min(10, timeout):
             break
         hung = not heartbeat() if check_heartbeat else delta > timeout
@@ -124,52 +122,87 @@ def communicate(ans, worker, listener, args, timeout=300, heartbeat=None,
 
 
 def create_worker(env, priority='normal', cwd=None, func='main'):
-    from calibre.utils.ipc.server import create_listener
-    auth_key = os.urandom(32)
-    address, listener = create_listener(auth_key)
-
     env = dict(env)
-    env.update({
-        'CALIBRE_WORKER_ADDRESS': hexlify(cPickle.dumps(listener.address, -1)),
-        'CALIBRE_WORKER_KEY': hexlify(auth_key),
-        'CALIBRE_SIMPLE_WORKER': 'calibre.utils.ipc.simple_worker:%s' % func,
-    })
+    a, b = Pipe()
+    with a:
+        env.update({
+            'CALIBRE_WORKER_FD': str(a.fileno()),
+            'CALIBRE_SIMPLE_WORKER': environ_item('calibre.utils.ipc.simple_worker:%s' % func),
+        })
 
-    w = Worker(env)
-    w(cwd=cwd, priority=priority)
-    return listener, w
+        w = Worker(env)
+        w(cwd=cwd, priority=priority, pass_fds=(a.fileno(),))
+    return b, w
 
 
 def start_pipe_worker(command, env=None, priority='normal', **process_args):
     import subprocess
-    from functools import partial
     w = Worker(env or {})
-    args = {'stdout':subprocess.PIPE, 'stdin':subprocess.PIPE, 'env':w.env}
+    args = {'stdout':subprocess.PIPE, 'stdin':subprocess.PIPE, 'env':w.env, 'close_fds': True}
     args.update(process_args)
-    if iswindows:
-        import win32process
-        priority = {
-                'high'   : win32process.HIGH_PRIORITY_CLASS,
-                'normal' : win32process.NORMAL_PRIORITY_CLASS,
-                'low'    : win32process.IDLE_PRIORITY_CLASS}[priority]
-        args['creationflags'] = win32process.CREATE_NO_WINDOW|priority
-    else:
-        def renice(niceness):
-            try:
-                os.nice(niceness)
-            except:
-                pass
-        niceness = {'normal' : 0, 'low'    : 10, 'high'   : 20}[priority]
-        args['preexec_fn'] = partial(renice, niceness)
-        args['close_fds'] = True
+    pass_fds = None
+    try:
+        if iswindows:
+            priority = {
+                    'high'   : subprocess.HIGH_PRIORITY_CLASS,
+                    'normal' : subprocess.NORMAL_PRIORITY_CLASS,
+                    'low'    : subprocess.IDLE_PRIORITY_CLASS}[priority]
+            args['creationflags'] = subprocess.CREATE_NO_WINDOW|priority
+            pass_fds = args.pop('pass_fds', None)
+            if pass_fds:
+                for fd in pass_fds:
+                    os.set_handle_inheritable(fd, True)
+                args['startupinfo'] = subprocess.STARTUPINFO(lpAttributeList={'handle_list':pass_fds})
+        else:
+            niceness = {'normal' : 0, 'low'    : 10, 'high'   : 20}[priority]
+            args['env']['CALIBRE_WORKER_NICENESS'] = str(niceness)
 
-    exe = w.executable
-    cmd = [exe] if isinstance(exe, basestring) else exe
-    p = subprocess.Popen(cmd + ['--pipe-worker', command], **args)
+        exe = w.executable
+        cmd = [exe] if isinstance(exe, string_or_bytes) else exe
+        p = subprocess.Popen(cmd + ['--pipe-worker', command], **args)
+    finally:
+        if iswindows and pass_fds:
+            for fd in pass_fds:
+                os.set_handle_inheritable(fd, False)
     return p
 
 
-def fork_job(mod_name, func_name, args=(), kwargs={}, timeout=300,  # seconds
+def two_part_fork_job(env=None, priority='normal', cwd=None):
+    env = env or {}
+    conn, w = create_worker(env, priority, cwd)
+
+    def run_job(
+        mod_name, func_name, args=(), kwargs=None, timeout=300,  # seconds
+        no_output=False, heartbeat=None, abort=None, module_is_source_code=False
+    ):
+        ans = {'result':None, 'stdout_stderr':None}
+        kwargs = kwargs or {}
+        try:
+            communicate(ans, w, conn, (mod_name, func_name, args, kwargs,
+                module_is_source_code), timeout=timeout, heartbeat=heartbeat,
+                abort=abort)
+        except WorkerError as e:
+            if not no_output:
+                e.log_path = w.log_path
+            raise
+        finally:
+            t = Thread(target=w.kill)
+            t.daemon=True
+            t.start()
+            if no_output:
+                try:
+                    os.remove(w.log_path)
+                except:
+                    pass
+        if not no_output:
+            ans['stdout_stderr'] = w.log_path
+        return ans
+    run_job.worker = w
+
+    return run_job
+
+
+def fork_job(mod_name, func_name, args=(), kwargs=None, timeout=300,  # seconds
         cwd=None, priority='normal', env={}, no_output=False, heartbeat=None,
         abort=None, module_is_source_code=False):
     '''
@@ -220,37 +253,24 @@ def fork_job(mod_name, func_name, args=(), kwargs={}, timeout=300,  # seconds
     path to a file that contains the stdout and stderr of the worker process.
     If you set no_output=True, then this will not be present.
     '''
-
-    ans = {'result':None, 'stdout_stderr':None}
-    listener, w = create_worker(env, priority, cwd)
-    try:
-        communicate(ans, w, listener, (mod_name, func_name, args, kwargs,
-            module_is_source_code), timeout=timeout, heartbeat=heartbeat,
-            abort=abort)
-    finally:
-        t = Thread(target=w.kill)
-        t.daemon=True
-        t.start()
-        if no_output:
-            try:
-                os.remove(w.log_path)
-            except:
-                pass
-    if not no_output:
-        ans['stdout_stderr'] = w.log_path
-    return ans
+    return two_part_fork_job(env, priority, cwd)(
+        mod_name, func_name, args=args, kwargs=kwargs, timeout=timeout,
+        no_output=no_output, heartbeat=heartbeat, abort=abort,
+        module_is_source_code=module_is_source_code
+    )
 
 
 def offload_worker(env={}, priority='normal', cwd=None):
-    listener, w = create_worker(env=env, priority=priority, cwd=cwd, func='offload')
-    return OffloadWorker(listener, w)
+    conn, w = create_worker(env=env, priority=priority, cwd=cwd, func='offload')
+    return OffloadWorker(conn, w)
 
 
 def compile_code(src):
-    import re, io
-    if not isinstance(src, unicode):
-        match = re.search(r'coding[:=]\s*([-\w.]+)', src[:200])
-        enc = match.group(1) if match else 'utf-8'
+    import io
+    import re
+    if not isinstance(src, unicode_type):
+        match = re.search(br'coding[:=]\s*([-\w.]+)', src[:200])
+        enc = match.group(1).decode('utf-8') if match else 'utf-8'
         src = src.decode(enc)
     # Python complains if there is a coding declaration in a unicode string
     src = re.sub(r'^#.*coding\s*[:=]\s*([-\w.]+)', '#', src, flags=re.MULTILINE)
@@ -260,15 +280,13 @@ def compile_code(src):
     namespace = {
             'time':time, 're':re, 'os':os, 'io':io,
     }
-    exec src in namespace
+    exec(src, namespace)
     return namespace
 
 
 def main():
     # The entry point for the simple worker process
-    address = cPickle.loads(unhexlify(os.environ['CALIBRE_WORKER_ADDRESS']))
-    key     = unhexlify(os.environ['CALIBRE_WORKER_KEY'])
-    with closing(Client(address, authkey=key)) as conn:
+    with Connection(int(os.environ['CALIBRE_WORKER_FD'])) as conn:
         args = eintr_retry_call(conn.recv)
         try:
             mod, func, args, kwargs, module_is_source_code = args
@@ -296,10 +314,8 @@ def main():
 
 def offload():
     # The entry point for the offload worker process
-    address = cPickle.loads(unhexlify(os.environ['CALIBRE_WORKER_ADDRESS']))
-    key     = unhexlify(os.environ['CALIBRE_WORKER_KEY'])
     func_cache = {}
-    with closing(Client(address, authkey=key)) as conn:
+    with Connection(int(os.environ['CALIBRE_WORKER_FD'])) as conn:
         while True:
             args = eintr_retry_call(conn.recv)
             if args is None:
@@ -324,4 +340,3 @@ def offload():
                 res['tb'] = traceback.format_exc()
 
             eintr_retry_call(conn.send, res)
-
