@@ -1,23 +1,27 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import absolute_import
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, locale, re, cStringIO, cPickle
+import os, locale, re, io
 from gettext import GNUTranslations, NullTranslations
 
+from polyglot.builtins import iteritems, unicode_type
+
 _available_translations = None
 
 
 def available_translations():
     global _available_translations
     if _available_translations is None:
-        stats = P('localization/stats.pickle', allow_user_override=False)
+        stats = P('localization/stats.calibre_msgpack', allow_user_override=False)
         if os.path.exists(stats):
-            stats = cPickle.load(open(stats, 'rb'))
+            from calibre.utils.serialize import msgpack_loads
+            with open(stats, 'rb') as f:
+                stats = msgpack_loads(f.read())
         else:
             stats = {}
         _available_translations = [x for x in stats if stats[x] > 0.1]
@@ -25,7 +29,7 @@ def available_translations():
 
 
 def get_system_locale():
-    from calibre.constants import iswindows, isosx, plugins
+    from calibre.constants import iswindows, ismacos
     lang = None
     if iswindows:
         try:
@@ -36,17 +40,26 @@ def get_system_locale():
                 lang = None
         except:
             pass  # Windows XP does not have the GetUserDefaultLocaleName fn
-    elif isosx:
+    elif ismacos:
+        from calibre_extensions.usbobserver import user_locale
         try:
-            lang = plugins['usbobserver'][0].user_locale() or None
-        except:
+            lang = user_locale() or None
+        except Exception:
             # Fallback to environment vars if something bad happened
             import traceback
             traceback.print_exc()
     if lang is None:
         try:
-            lang = locale.getdefaultlocale(['LANGUAGE', 'LC_ALL', 'LC_CTYPE',
-                                        'LC_MESSAGES', 'LANG'])[0]
+            envvars = ['LANGUAGE', 'LC_ALL', 'LC_CTYPE', 'LC_MESSAGES', 'LANG']
+            lang = locale.getdefaultlocale(envvars)[0]
+
+            # lang is None in two cases: either the environment variable is not
+            # set or it's "C". Stop looking for a language in the latter case.
+            if lang is None:
+                for var in envvars:
+                    if os.environ.get(var) == 'C':
+                        lang = 'en_US'
+                        break
         except:
             pass  # This happens on Ubuntu apparently
         if lang is None and 'LANG' in os.environ:  # Needed for OS X
@@ -60,6 +73,18 @@ def get_system_locale():
     return lang
 
 
+def sanitize_lang(lang):
+    if lang:
+        match = re.match('[a-z]{2,3}(_[A-Z]{2}){0,1}', lang)
+        if match:
+            lang = match.group()
+    if lang == 'zh':
+        lang = 'zh_CN'
+    if not lang:
+        lang = 'en'
+    return lang
+
+
 def get_lang():
     'Try to figure out what language to display the interface in'
     from calibre.utils.config_base import prefs
@@ -73,15 +98,7 @@ def get_lang():
         import traceback
         traceback.print_exc()
         lang = None
-    if lang:
-        match = re.match('[a-z]{2,3}(_[A-Z]{2}){0,1}', lang)
-        if match:
-            lang = match.group()
-    if lang == 'zh':
-        lang = 'zh_CN'
-    if not lang:
-        lang = 'en'
-    return lang
+    return sanitize_lang(lang)
 
 
 def is_rtl():
@@ -114,15 +131,31 @@ def get_all_translators():
         for lang in available_translations():
             mpath = get_lc_messages_path(lang)
             if mpath is not None:
-                buf = cStringIO.StringIO(zf.read(mpath + '/messages.mo'))
+                buf = io.BytesIO(zf.read(mpath + '/messages.mo'))
                 yield lang, GNUTranslations(buf)
 
 
-def get_single_translator(mpath):
+def get_single_translator(mpath, which='messages'):
     from zipfile import ZipFile
     with ZipFile(P('localization/locales.zip', allow_user_override=False), 'r') as zf:
-        buf = cStringIO.StringIO(zf.read(mpath + '/messages.mo'))
-        return GNUTranslations(buf)
+        path = '{}/{}.mo'.format(mpath, which)
+        data = zf.read(path)
+        buf = io.BytesIO(data)
+        try:
+            return GNUTranslations(buf)
+        except Exception as e:
+            import traceback
+            traceback.print_exc()
+            import hashlib
+            sig = hashlib.sha1(data).hexdigest()
+            raise ValueError('Failed to load translations for: {} (size: {} and signature: {}) with error: {}'.format(
+                path, len(data), sig, e))
+
+
+def get_iso639_translator(lang):
+    lang = sanitize_lang(lang)
+    mpath = get_lc_messages_path(lang) if lang else None
+    return get_single_translator(mpath, 'iso639') if mpath else None
 
 
 def get_translator(bcp_47_code):
@@ -134,6 +167,8 @@ def get_translator(bcp_47_code):
     lang = {'pt':'pt_BR', 'zh':'zh_CN'}.get(lang, lang)
     available = available_translations()
     found = True
+    if lang == 'en' or lang.startswith('en_'):
+        return found, lang, NullTranslations()
     if lang not in available:
         lang = {'pt':'pt_BR', 'zh':'zh_CN'}.get(parts[0], parts[0])
         if lang not in available:
@@ -142,83 +177,109 @@ def get_translator(bcp_47_code):
                 lang = 'en'
             found = False
     if lang == 'en':
-        return found, lang, NullTranslations()
+        return True, lang, NullTranslations()
     return found, lang, get_single_translator(lang)
 
 
 lcdata = {
-    u'abday': (u'Sun', u'Mon', u'Tue', u'Wed', u'Thu', u'Fri', u'Sat'),
-    u'abmon': (u'Jan', u'Feb', u'Mar', u'Apr', u'May', u'Jun', u'Jul', u'Aug', u'Sep', u'Oct', u'Nov', u'Dec'),
-    u'd_fmt': u'%m/%d/%Y',
-    u'd_t_fmt': u'%a %d %b %Y %r %Z',
-    u'day': (u'Sunday', u'Monday', u'Tuesday', u'Wednesday', u'Thursday', u'Friday', u'Saturday'),
-    u'mon': (u'January', u'February', u'March', u'April', u'May', u'June', u'July', u'August', u'September', u'October', u'November', u'December'),
-    u'noexpr': u'^[nN].*',
-    u'radixchar': u'.',
-    u't_fmt': u'%r',
-    u't_fmt_ampm': u'%I:%M:%S %p',
-    u'thousep': u',',
-    u'yesexpr': u'^[yY].*'
+    'abday': ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'),
+    'abmon': ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'),
+    'd_fmt': '%m/%d/%Y',
+    'd_t_fmt': '%a %d %b %Y %r %Z',
+    'day': ('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'),
+    'mon': ('January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'),
+    'noexpr': '^[nN].*',
+    'radixchar': '.',
+    't_fmt': '%r',
+    't_fmt_ampm': '%I:%M:%S %p',
+    'thousep': ',',
+    'yesexpr': '^[yY].*'
 }
 
 
 def load_po(path):
     from calibre.translations.msgfmt import make
-    buf = cStringIO.StringIO()
+    buf = io.BytesIO()
     try:
         make(path, buf)
     except Exception:
-        print (('Failed to compile translations file: %s, ignoring') % path)
+        print(('Failed to compile translations file: %s, ignoring') % path)
         buf = None
     else:
-        buf = cStringIO.StringIO(buf.getvalue())
+        buf = io.BytesIO(buf.getvalue())
     return buf
 
 
+def translator_for_lang(lang):
+    t = buf = iso639 = lcdata = None
+    if 'CALIBRE_TEST_TRANSLATION' in os.environ:
+        buf = load_po(os.path.expanduser(os.environ['CALIBRE_TEST_TRANSLATION']))
+
+    mpath = get_lc_messages_path(lang)
+    if buf is None and mpath and os.access(mpath + '.po', os.R_OK):
+        buf = load_po(mpath + '.po')
+
+    if mpath is not None:
+        from zipfile import ZipFile
+        with ZipFile(P('localization/locales.zip',
+            allow_user_override=False), 'r') as zf:
+            if buf is None:
+                buf = io.BytesIO(zf.read(mpath + '/messages.mo'))
+            if mpath == 'nds':
+                mpath = 'de'
+            isof = mpath + '/iso639.mo'
+            try:
+                iso639 = io.BytesIO(zf.read(isof))
+            except:
+                pass  # No iso639 translations for this lang
+            if buf is not None:
+                from calibre.utils.serialize import msgpack_loads
+                try:
+                    lcdata = msgpack_loads(zf.read(mpath + '/lcdata.calibre_msgpack'))
+                except:
+                    pass  # No lcdata
+
+    if buf is not None:
+        try:
+            t = GNUTranslations(buf)
+        except Exception:
+            import traceback
+            traceback.print_exc()
+            t = None
+        if iso639 is not None:
+            try:
+                iso639 = GNUTranslations(iso639)
+            except Exception:
+                iso639 = None
+            else:
+                if t is not None:
+                    t.add_fallback(iso639)
+
+    if t is None:
+        t = NullTranslations()
+
+    return {'translator': t, 'iso639_translator': iso639, 'lcdata': lcdata}
+
+
 def set_translators():
     global _lang_trans, lcdata
     # To test different translations invoke as
     # CALIBRE_OVERRIDE_LANG=de_DE.utf8 program
     lang = get_lang()
-    t = buf = iso639 = None
-
-    if 'CALIBRE_TEST_TRANSLATION' in os.environ:
-        buf = load_po(os.path.expanduser(os.environ['CALIBRE_TEST_TRANSLATION']))
 
     if lang:
-        mpath = get_lc_messages_path(lang)
-        if buf is None and mpath and os.access(mpath + '.po', os.R_OK):
-            buf = load_po(mpath + '.po')
-
-        if mpath is not None:
-            from zipfile import ZipFile
-            with ZipFile(P('localization/locales.zip',
-                allow_user_override=False), 'r') as zf:
-                if buf is None:
-                    buf = cStringIO.StringIO(zf.read(mpath + '/messages.mo'))
-                if mpath == 'nds':
-                    mpath = 'de'
-                isof = mpath + '/iso639.mo'
-                try:
-                    iso639 = cStringIO.StringIO(zf.read(isof))
-                except:
-                    pass  # No iso639 translations for this lang
-                if buf is not None:
-                    try:
-                        lcdata = cPickle.loads(zf.read(mpath + '/lcdata.pickle'))
-                    except:
-                        pass  # No lcdata
-
-    if buf is not None:
-        t = GNUTranslations(buf)
-        if iso639 is not None:
-            iso639 = _lang_trans = GNUTranslations(iso639)
-            t.add_fallback(iso639)
-
-    if t is None:
+        q = translator_for_lang(lang)
+        t = q['translator']
+        _lang_trans = q['iso639_translator']
+        if q['lcdata']:
+            lcdata = q['lcdata']
+    else:
         t = NullTranslations()
-
-    t.install(unicode=True, names=('ngettext',))
+    try:
+        set_translators.lang = t.info().get('language')
+    except Exception:
+        pass
+    t.install(names=('ngettext',))
     # Now that we have installed a translator, we have to retranslate the help
     # for the global prefs object as it was instantiated in get_lang(), before
     # the translator was installed.
@@ -226,10 +287,13 @@ def set_translators():
     prefs.retranslate_help()
 
 
+set_translators.lang = None
+
+
 _iso639 = None
 _extra_lang_codes = {
         'pt_BR' : _('Brazilian Portuguese'),
-        'en_GB' : _('English (UK)'),
+        'en_GB' : _('English (United Kingdom)'),
         'zh_CN' : _('Simplified Chinese'),
         'zh_TW' : _('Traditional Chinese'),
         'en'    : _('English'),
@@ -276,10 +340,10 @@ _extra_lang_codes = {
         'es_BO' : _('Spanish (Bolivia)'),
         'es_NI' : _('Spanish (Nicaragua)'),
         'es_CO' : _('Spanish (Colombia)'),
-        'de_AT' : _('German (AT)'),
-        'fr_BE' : _('French (BE)'),
-        'nl'    : _('Dutch (NL)'),
-        'nl_BE' : _('Dutch (BE)'),
+        'de_AT' : _('German (Austria)'),
+        'fr_BE' : _('French (Belgium)'),
+        'nl'    : _('Dutch (Netherlands)'),
+        'nl_BE' : _('Dutch (Belgium)'),
         'und'   : _('Unknown')
         }
 
@@ -298,7 +362,7 @@ if False:
     _('Select All')
     _('Copy Link')
     _('&Select All')
-    _('Copy &Link location')
+    _('Copy &Link Location')
     _('&Undo')
     _('&Redo')
     _('Cu&t')
@@ -310,6 +374,8 @@ if False:
     _('Fonts')
     _('&Step up')
     _('Step &down')
+    _('Close without Saving')
+    _('Close Tab')
 
 _lcase_map = {}
 for k in _extra_lang_codes:
@@ -319,40 +385,43 @@ for k in _extra_lang_codes:
 def _load_iso639():
     global _iso639
     if _iso639 is None:
-        ip = P('localization/iso639.pickle', allow_user_override=False)
-        with open(ip, 'rb') as f:
-            _iso639 = cPickle.load(f)
+        ip = P('localization/iso639.calibre_msgpack', allow_user_override=False, data=True)
+        from calibre.utils.serialize import msgpack_loads
+        _iso639 = msgpack_loads(ip)
+        if 'by_3' not in _iso639:
+            _iso639['by_3'] = _iso639['by_3t']
     return _iso639
 
 
-def get_language(lang):
-    translate = _
-    lang = _lcase_map.get(lang, lang)
-    if lang in _extra_lang_codes:
-        # The translator was not active when _extra_lang_codes was defined, so
-        # re-translate
-        return translate(_extra_lang_codes[lang])
+def get_iso_language(lang_trans, lang):
     iso639 = _load_iso639()
     ans = lang
     lang = lang.split('_')[0].lower()
     if len(lang) == 2:
         ans = iso639['by_2'].get(lang, ans)
     elif len(lang) == 3:
-        if lang in iso639['by_3b']:
-            ans = iso639['by_3b'][lang]
-        else:
-            ans = iso639['by_3t'].get(lang, ans)
-    try:
-        return _lang_trans.ugettext(ans)
-    except AttributeError:
-        return translate(ans)
+        if lang in iso639['by_3']:
+            ans = iso639['by_3'][lang]
+    return lang_trans(ans)
+
+
+def get_language(lang, gettext_func=None):
+    translate = gettext_func or _
+    lang = _lcase_map.get(lang, lang)
+    if lang in _extra_lang_codes:
+        # The translator was not active when _extra_lang_codes was defined, so
+        # re-translate
+        return translate(_extra_lang_codes[lang])
+    if gettext_func is None:
+        gettext_func = getattr(_lang_trans, 'gettext', translate)
+    return get_iso_language(gettext_func, lang)
 
 
 def calibre_langcode_to_name(lc, localize=True):
     iso639 = _load_iso639()
     translate = _ if localize else lambda x: x
     try:
-        return translate(iso639['by_3t'][lc])
+        return translate(iso639['by_3'][lc])
     except:
         pass
     return lc
@@ -361,7 +430,7 @@ def calibre_langcode_to_name(lc, localize=True):
 def canonicalize_lang(raw):
     if not raw:
         return None
-    if not isinstance(raw, unicode):
+    if not isinstance(raw, unicode_type):
         raw = raw.decode('utf-8', 'ignore')
     raw = raw.lower().strip()
     if not raw:
@@ -377,10 +446,8 @@ def canonicalize_lang(raw):
         if ans is not None:
             return ans
     elif len(raw) == 3:
-        if raw in iso639['by_3t']:
+        if raw in iso639['by_3']:
             return raw
-        if raw in iso639['3bto3t']:
-            return iso639['3bto3t'][raw]
 
     return iso639['name_map'].get(raw, None)
 
@@ -394,10 +461,20 @@ def lang_map():
     translate = _
     global _lang_map
     if _lang_map is None:
-        _lang_map = {k:translate(v) for k, v in iso639['by_3t'].iteritems()}
+        _lang_map = {k:translate(v) for k, v in iteritems(iso639['by_3'])}
     return _lang_map
 
 
+def lang_map_for_ui():
+    ans = getattr(lang_map_for_ui, 'ans', None)
+    if ans is None:
+        ans = lang_map().copy()
+        for x in ('zxx', 'mis', 'mul'):
+            ans.pop(x, None)
+        lang_map_for_ui.ans = ans
+    return ans
+
+
 def langnames_to_langcodes(names):
     '''
     Given a list of localized language names return a mapping of the names to 3
@@ -408,7 +485,7 @@ def langnames_to_langcodes(names):
     translate = _
     ans = {}
     names = set(names)
-    for k, v in iso639['by_3t'].iteritems():
+    for k, v in iteritems(iso639['by_3']):
         tv = translate(v)
         if tv in names:
             names.remove(tv)
@@ -439,21 +516,53 @@ def get_udc():
     return _udc
 
 
+def user_manual_stats():
+    stats = getattr(user_manual_stats, 'stats', None)
+    if stats is None:
+        import json
+        try:
+            stats = json.loads(P('user-manual-translation-stats.json', allow_user_override=False, data=True))
+        except EnvironmentError:
+            stats = {}
+        user_manual_stats.stats = stats
+    return stats
+
+
 def localize_user_manual_link(url):
     lc = lang_as_iso639_1(get_lang())
     if lc == 'en':
         return url
-    import json
-    try:
-        stats = json.loads(P('user-manual-translation-stats.json', allow_user_override=False, data=True))
-    except EnvironmentError:
-        return url
+    stats = user_manual_stats()
     if stats.get(lc, 0) < 0.3:
         return url
-    from urlparse import urlparse, urlunparse
+    from polyglot.urllib import urlparse, urlunparse
     parts = urlparse(url)
     path = re.sub(r'/generated/[a-z]+/', '/generated/%s/' % lc, parts.path or '')
     path = '/%s%s' % (lc, path)
     parts = list(parts)
     parts[2] = path
     return urlunparse(parts)
+
+
+def website_languages():
+    stats = getattr(website_languages, 'stats', None)
+    if stats is None:
+        try:
+            stats = frozenset(P('localization/website-languages.txt', allow_user_override=False, data=True).decode('utf-8').split())
+        except EnvironmentError:
+            stats = frozenset()
+        website_languages.stats = stats
+    return stats
+
+
+def localize_website_link(url):
+    lc = lang_as_iso639_1(get_lang())
+    langs = website_languages()
+    if lc == 'en' or lc not in langs:
+        return url
+    from polyglot.urllib import urlparse, urlunparse
+    parts = urlparse(url)
+    path = '/{}{}'.format(lc, parts.path)
+    parts = list(parts)
+    parts[2] = path
+    return urlunparse(parts)
