@@ -1,7 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -17,7 +16,9 @@ Tries to only use the local headers to extract data from the damaged zip file.
 import os, sys, zlib, shutil
 from struct import calcsize, unpack, pack
 from collections import namedtuple, OrderedDict
-from tempfile import SpooledTemporaryFile
+from calibre.ptempfile import SpooledTemporaryFile
+
+from polyglot.builtins import itervalues, getcwd
 
 HEADER_SIG = 0x04034b50
 HEADER_BYTE_SIG = pack(b'<L', HEADER_SIG)
@@ -32,6 +33,20 @@ LocalHeader = namedtuple('LocalHeader',
         'filename extra')
 
 
+if hasattr(sys, 'getwindowsversion'):
+    windows_reserved_filenames = (
+        'CON', 'PRN', 'AUX', 'CLOCK$', 'NUL' 'COM0', 'COM1', 'COM2', 'COM3',
+        'COM4', 'COM5', 'COM6', 'COM7', 'COM8', 'COM9' 'LPT0', 'LPT1', 'LPT2',
+        'LPT3', 'LPT4', 'LPT5', 'LPT6', 'LPT7', 'LPT8', 'LPT9')
+
+    def is_reserved_filename(x):
+        base = x.partition('.')[0].upper()
+        return base in windows_reserved_filenames
+else:
+    def is_reserved_filename(x):
+        return False
+
+
 def decode_arcname(name):
     if isinstance(name, bytes):
         from calibre.ebooks.chardet import detect
@@ -206,11 +221,18 @@ def _extractall(f, path=None, file_info=None):
             if not os.path.exists(bdir):
                 os.makedirs(bdir)
             dest = os.path.join(path, *parts)
-            with open(dest, 'wb') as o:
+            try:
+                df = open(dest, 'wb')
+            except EnvironmentError:
+                if is_reserved_filename(os.path.basename(dest)):
+                    raise ValueError('This ZIP file contains a file with a reserved filename'
+                            ' that cannot be processed on Windows: {}'.format(os.path.basename(dest)))
+                raise
+            with df:
                 if header.compression_method == ZIP_STORED:
-                    copy_stored_file(f, header.compressed_size, o)
+                    copy_stored_file(f, header.compressed_size, df)
                 else:
-                    copy_compressed_file(f, header.compressed_size, o)
+                    copy_compressed_file(f, header.compressed_size, df)
         else:
             f.seek(f.tell()+seekval)
 
@@ -225,7 +247,7 @@ def extractall(path_or_stream, path=None):
         f = open(f, 'rb')
         close_at_end = True
     if path is None:
-        path = os.getcwdu()
+        path = getcwd()
     pos = f.tell()
     try:
         _extractall(f, path)
@@ -242,14 +264,16 @@ class LocalZipFile(object):
         _extractall(stream, file_info=self.file_info)
         self.stream = stream
 
+    def _get_file_info(self, name):
+        fi = self.file_info.get(name)
+        if fi is None:
+            raise ValueError('This ZIP container has no file named: %s'%name)
+        return fi
+
     def open(self, name, spool_size=5*1024*1024):
         if isinstance(name, LocalHeader):
             name = name.filename
-        try:
-            offset, header = self.file_info.get(name)
-        except KeyError:
-            raise ValueError('This ZIP container has no file named: %s'%name)
-
+        offset, header = self._get_file_info(name)
         self.stream.seek(offset)
         dest = SpooledTemporaryFile(max_size=spool_size)
 
@@ -261,10 +285,7 @@ class LocalZipFile(object):
         return dest
 
     def getinfo(self, name):
-        try:
-            offset, header = self.file_info.get(name)
-        except KeyError:
-            raise ValueError('This ZIP container has no file named: %s'%name)
+        offset, header = self._get_file_info(name)
         return header
 
     def read(self, name, spool_size=5*1024*1024):
@@ -273,7 +294,7 @@ class LocalZipFile(object):
 
     def extractall(self, path=None):
         self.stream.seek(0)
-        _extractall(self.stream, path=(path or os.getcwdu()))
+        _extractall(self.stream, path=(path or getcwd()))
 
     def close(self):
         pass
@@ -283,7 +304,7 @@ class LocalZipFile(object):
         from calibre.utils.zipfile import ZipFile, ZipInfo
         replacements = {name:datastream}
         replacements.update(extra_replacements)
-        names = frozenset(replacements.keys())
+        names = frozenset(list(replacements.keys()))
         found = set()
 
         def rbytes(name):
@@ -294,7 +315,7 @@ class LocalZipFile(object):
 
         with SpooledTemporaryFile(max_size=100*1024*1024) as temp:
             ztemp = ZipFile(temp, 'w')
-            for offset, header in self.file_info.itervalues():
+            for offset, header in itervalues(self.file_info):
                 if header.filename in names:
                     zi = ZipInfo(header.filename)
                     zi.compress_type = header.compression_method
@@ -314,6 +335,6 @@ class LocalZipFile(object):
             shutil.copyfileobj(temp, zipstream)
             zipstream.flush()
 
+
 if __name__ == '__main__':
     extractall(sys.argv[-1])
-
