@@ -1,9 +1,8 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 import os
 from io import BytesIO
 
@@ -57,47 +56,43 @@ class Image(object):
     def to_qimage(self):
         return clone_image(self.img)
 
-    @dynamic_property
+    @property
     def type(self):
-        def fget(self):
-            if len(self.img.colorTable()) > 0:
-                return 'PaletteType'
-            return 'TrueColorType'
+        if len(self.img.colorTable()) > 0:
+            return 'PaletteType'
+        return 'TrueColorType'
 
-        def fset(self, t):
-            if t == 'GrayscaleType':
-                self.img = grayscale_image(self.img)
-            elif t == 'PaletteType':
-                self.img = quantize_image(self.img)
-        return property(fget=fget, fset=fset)
+    @type.setter
+    def type(self, t):
+        if t == 'GrayscaleType':
+            self.img = grayscale_image(self.img)
+        elif t == 'PaletteType':
+            self.img = quantize_image(self.img)
 
-    @dynamic_property
+    @property
     def format(self):
-        def fget(self):
-            return self.write_format or self.read_format
+        return self.write_format or self.read_format
 
-        def fset(self, val):
-            self.write_format = val
-        return property(fget=fget, fset=fset)
+    @format.setter
+    def format(self, val):
+        self.write_format = val
 
-    @dynamic_property
+    @property
     def colorspace(self):
-        def fget(self):
-            return 'RGBColorspace'
+        return 'RGBColorspace'
 
-        def fset(self, val):
-            raise NotImplementedError('Changing image colorspace is not supported')
-        return property(fget=fget, fset=fset)
+    @colorspace.setter
+    def colorspace(self, val):
+        raise NotImplementedError('Changing image colorspace is not supported')
 
-    @dynamic_property
+    @property
     def size(self):
-        def fget(self):
-            return self.img.width(), self.img.height()
+        return self.img.width(), self.img.height()
 
-        def fset(self, val):
-            w, h = val[:2]
-            self.img = resize_image(self.img, w, h)
-        return property(fget=fget, fset=fset)
+    @size.setter
+    def size(self, val):
+        w, h = val[:2]
+        self.img = resize_image(self.img, w, h)
 
     def save(self, path, format=None):
         if format is None:
