@@ -1,6 +1,7 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -46,6 +47,5 @@ def diff_hists(h1, h2):
         if k not in h2:
             h2[k] = 0
         if h1[k] != h2[k]:
-            print "%s: %d -> %d (%s%d)" % (
-                k, h1[k], h2[k], h2[k] > h1[k] and "+" or "", h2[k] - h1[k])
-
+            print("%s: %d -> %d (%s%d)" % (
+                k, h1[k], h2[k], h2[k] > h1[k] and "+" or "", h2[k] - h1[k]))
