@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import print_function
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -9,14 +9,16 @@ __docformat__ = 'restructuredtext en'
 import os, shutil, time, sys
 
 from calibre import isbytestring
-from calibre.constants import (iswindows, isosx, plugins, filesystem_encoding,
+from calibre.constants import (iswindows, ismacos, filesystem_encoding,
         islinux)
+from polyglot.builtins import unicode_type
 
 recycle = None
 
 if iswindows:
     from calibre.utils.ipc import eintr_retry_call
     from threading import Lock
+    from calibre_extensions import winutil
     recycler = None
     rlock = Lock()
 
@@ -27,16 +29,13 @@ if iswindows:
             recycler = start_pipe_worker('from calibre.utils.recycle_bin import recycler_main; recycler_main()')
 
     def recycle_path(path):
-        from win32com.shell import shell, shellcon
-        flags = (shellcon.FOF_ALLOWUNDO | shellcon.FOF_NOCONFIRMATION | shellcon.FOF_NOCONFIRMMKDIR | shellcon.FOF_NOERRORUI |
-                 shellcon.FOF_SILENT | shellcon.FOF_RENAMEONCOLLISION)
-        retcode, aborted = shell.SHFileOperation((0, shellcon.FO_DELETE, path, None, flags, None, None))
-        if retcode != 0 or aborted:
-            raise RuntimeError('Failed to delete: %r with error code: %d' % (path, retcode))
+        winutil.move_to_trash(unicode_type(path))
 
     def recycler_main():
+        stdin = getattr(sys.stdin, 'buffer', sys.stdin)
+        stdout = getattr(sys.stdout, 'buffer', sys.stdout)
         while True:
-            path = eintr_retry_call(sys.stdin.readline)
+            path = eintr_retry_call(stdin.readline)
             if not path:
                 break
             try:
@@ -46,23 +45,24 @@ if iswindows:
             try:
                 recycle_path(path)
             except:
-                eintr_retry_call(print, b'KO', file=sys.stdout)
-                sys.stdout.flush()
+                eintr_retry_call(stdout.write, b'KO\n')
+                stdout.flush()
                 try:
                     import traceback
                     traceback.print_exc()  # goes to stderr, which is the same as for parent process
                 except Exception:
                     pass  # Ignore failures to write the traceback, since GUI processes on windows have no stderr
             else:
-                eintr_retry_call(print, b'OK', file=sys.stdout)
-                sys.stdout.flush()
+                eintr_retry_call(stdout.write, b'OK\n')
+                stdout.flush()
 
     def delegate_recycle(path):
         if '\n' in path:
             raise ValueError('Cannot recycle paths that have newlines in them (%r)' % path)
         with rlock:
             start_recycler()
-            eintr_retry_call(print, path.encode('utf-8'), file=recycler.stdin)
+            recycler.stdin.write(path.encode('utf-8'))
+            recycler.stdin.write(b'\n')
             recycler.stdin.flush()
             # Theoretically this could be made non-blocking using a
             # thread+queue, however the original implementation was blocking,
@@ -86,14 +86,14 @@ if iswindows:
         path = os.path.abspath(path)  # Windows does not like recycling relative paths
         return delegate_recycle(path)
 
-elif isosx:
-    u = plugins['usbobserver'][0]
-    if hasattr(u, 'send2trash'):
-        def osx_recycle(path):
-            if isbytestring(path):
-                path = path.decode(filesystem_encoding)
-            u.send2trash(path)
-        recycle = osx_recycle
+elif ismacos:
+    from calibre_extensions.cocoa import send2trash
+
+    def osx_recycle(path):
+        if isbytestring(path):
+            path = path.decode(filesystem_encoding)
+        send2trash(path)
+    recycle = osx_recycle
 elif islinux:
     from calibre.utils.linux_trash import send2trash
 
@@ -149,4 +149,3 @@ def delete_tree(path, permanent=False):
                 import traceback
                 traceback.print_exc()
         delete_tree(path, permanent=True)
-
