@@ -1,20 +1,20 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2015, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import os, sys, time, ctypes
-from ctypes.wintypes import HLOCAL, LPCWSTR
+import os, sys, time, traceback
 from threading import Thread
 
-import winerror
 
 from calibre import guess_type, prints
 from calibre.constants import is64bit, isportable, isfrozen, __version__, DEBUG
 from calibre.utils.winreg.lib import Key, HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE
+from calibre.utils.lock import singleinstance
+from polyglot.builtins import iteritems, itervalues
+from calibre_extensions import winutil
 
 # See https://msdn.microsoft.com/en-us/library/windows/desktop/cc144154(v=vs.85).aspx
 
@@ -31,7 +31,7 @@ def default_programs():
 
         'ebook-edit.exe': {
             'icon_id':'editor_icon',
-            'description': _('The calibre e-book editor. It can be used to edit common e-book formats.'),
+            'description': _('The calibre E-book editor. It can be used to edit common e-book formats.'),
             'capability_name': 'Editor' + ('64bit' if is64bit else ''),
             'name': 'calibre Editor' + (' 64-bit' if is64bit else ''),
             'assoc_name': 'calibreEditor' + ('64bit' if is64bit else ''),
@@ -73,7 +73,7 @@ def check_allowed():
         raise NotAllowed('Not allowed to create associations for portable installs')
     if sys.getwindowsversion()[:2] < (6, 2):
         raise NotAllowed('Not allowed to create associations for windows versions older than Windows 8')
-    if b'CALIBRE_NO_DEFAULT_PROGRAMS' in os.environ:
+    if 'CALIBRE_NO_DEFAULT_PROGRAMS' in os.environ:
         raise NotAllowed('Disabled by the CALIBRE_NO_DEFAULT_PROGRAMS environment variable')
 
 
@@ -85,7 +85,10 @@ def create_prog_id(ext, prog_id, ext_map, exe):
         key.set('PerceivedType', 'Document')
         key.set(sub_key='DefaultIcon', value=exe+',0')
         key.set_default_value(r'shell\open\command', '"%s" "%%1"' % exe)
-        key.set('AllowSilentDefaultTakeOver')
+        # contrary to the msdn docs, this key prevents calibre programs
+        # from appearing in the initial open with list, see
+        # https://www.mobileread.com/forums/showthread.php?t=313668
+        # key.set('AllowSilentDefaultTakeOver')
 
     with Key(r'Software\Classes\.%s\OpenWithProgIDs' % ext) as key:
         key.set(prog_id)
@@ -102,48 +105,47 @@ def cap_path(data):
 def register():
     base = os.path.dirname(sys.executable)
 
-    for program, data in default_programs().iteritems():
+    for program, data in iteritems(default_programs()):
         data = data.copy()
         exe = os.path.join(base, program)
         capabilities_path = cap_path(data)
         ext_map = {ext.lower():guess_type('file.' + ext.lower())[0] for ext in extensions(program)}
-        ext_map = {ext:mt for ext, mt in ext_map.iteritems() if mt}
+        ext_map = {ext:mt for ext, mt in iteritems(ext_map) if mt}
         prog_id_map = {ext:progid_name(data['assoc_name'], ext) for ext in ext_map}
 
         with Key(capabilities_path) as key:
-            for k, v in {'ApplicationDescription':'description', 'ApplicationName':'name'}.iteritems():
+            for k, v in iteritems({'ApplicationDescription':'description', 'ApplicationName':'name'}):
                 key.set(k, data[v])
             key.set('ApplicationIcon', '%s,0' % exe)
             key.set_default_value(r'shell\open\command', '"%s" "%%1"' % exe)
 
             with Key('FileAssociations', root=key) as fak, Key('MimeAssociations', root=key) as mak:
-                # previous_associations = set(fak.itervalues())
-                for ext, prog_id in prog_id_map.iteritems():
+                # previous_associations = set(fak.values())
+                for ext, prog_id in iteritems(prog_id_map):
                     mt = ext_map[ext]
                     fak.set('.' + ext, prog_id)
                     mak.set(mt, prog_id)
-        for ext, prog_id in prog_id_map.iteritems():
+        for ext, prog_id in iteritems(prog_id_map):
             create_prog_id(ext, prog_id, ext_map, exe)
 
         with Key(r'Software\RegisteredApplications') as key:
             key.set(data['name'], capabilities_path)
 
-    from win32com.shell import shell, shellcon
-    shell.SHChangeNotify(shellcon.SHCNE_ASSOCCHANGED, shellcon.SHCNF_DWORD | shellcon.SHCNF_FLUSH, 0, 0)
+    winutil.notify_associations_changed()
 
 
 def unregister():
-    for program, data in default_programs().iteritems():
+    for program, data in iteritems(default_programs()):
         capabilities_path = cap_path(data).rpartition('\\')[0]
         ext_map = {ext.lower():guess_type('file.' + ext.lower())[0] for ext in extensions(program)}
-        ext_map = {ext:mt for ext, mt in ext_map.iteritems() if mt}
+        ext_map = {ext:mt for ext, mt in iteritems(ext_map) if mt}
         prog_id_map = {ext:progid_name(data['assoc_name'], ext) for ext in ext_map}
         with Key(r'Software\RegisteredApplications') as key:
             key.delete_value(data['name'])
         parent, sk = capabilities_path.rpartition('\\')[0::2]
         with Key(parent) as key:
             key.delete_tree(sk)
-        for ext, prog_id in prog_id_map.iteritems():
+        for ext, prog_id in iteritems(prog_id_map):
             with Key(r'Software\Classes\.%s\OpenWithProgIDs' % ext) as key:
                 key.delete_value(prog_id)
             with Key(r'Software\Classes') as key:
@@ -163,11 +165,9 @@ class Register(Thread):
         try:
             self.do_register()
         except Exception:
-            import traceback
             traceback.print_exc()
 
     def do_register(self):
-        from calibre.utils.lock import singleinstance
         try:
             check_allowed()
         except NotAllowed:
@@ -176,11 +176,11 @@ class Register(Thread):
             if self.prefs.get('windows_register_default_programs', None) != __version__:
                 self.prefs['windows_register_default_programs'] = __version__
                 if DEBUG:
-                    st = time.time()
+                    st = time.monotonic()
                     prints('Registering with default programs...')
                 register()
                 if DEBUG:
-                    prints('Registered with default programs in %.1f seconds' % (time.time() - st))
+                    prints('Registered with default programs in %.1f seconds' % (time.monotonic() - st))
 
     def __enter__(self):
         return self
@@ -195,15 +195,15 @@ def get_prog_id_map(base, key_path):
     desc, ans = None, {}
     try:
         k = Key(open_at=key_path, root=base)
-    except WindowsError as err:
-        if err.errno == winerror.ERROR_FILE_NOT_FOUND:
+    except OSError as err:
+        if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
             return desc, ans
         raise
     with k:
         desc = k.get_mui_string('ApplicationDescription')
         if desc is None:
             return desc, ans
-        for ext, prog_id in k.itervalues(sub_key='FileAssociations', get_data=True):
+        for ext, prog_id in k.values(sub_key='FileAssociations', get_data=True):
             ans[ext[1:].lower()] = prog_id
     return desc, ans
 
@@ -211,8 +211,8 @@ def get_prog_id_map(base, key_path):
 def get_open_data(base, prog_id):
     try:
         k = Key(open_at=r'Software\Classes\%s' % prog_id, root=base)
-    except WindowsError as err:
-        if err.errno == winerror.ERROR_FILE_NOT_FOUND:
+    except OSError as err:
+        if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
             return None, None, None
     with k:
         cmd = k.get(sub_key=r'shell\open\command')
@@ -223,36 +223,17 @@ def get_open_data(base, prog_id):
         return cmd, k.get(sub_key='DefaultIcon'), k.get_mui_string('FriendlyTypeName') or k.get()
 
 
-CommandLineToArgvW = ctypes.windll.shell32.CommandLineToArgvW
-CommandLineToArgvW.arg_types = [LPCWSTR, ctypes.POINTER(ctypes.c_int)]
-CommandLineToArgvW.restype = ctypes.POINTER(LPCWSTR)
-LocalFree = ctypes.windll.kernel32.LocalFree
-LocalFree.res_type = HLOCAL
-LocalFree.arg_types = [HLOCAL]
-
-
 def split_commandline(commandline):
     # CommandLineToArgvW returns path to executable if called with empty string.
     if not commandline.strip():
         return []
-    num = ctypes.c_int(0)
-    result_pointer = CommandLineToArgvW(commandline.lstrip(), ctypes.byref(num))
-    if not result_pointer:
-        raise ctypes.WinError()
-    result_array_type = LPCWSTR * num.value
-    result = [arg for arg in result_array_type.from_address(ctypes.addressof(result_pointer.contents))]
-    LocalFree(result_pointer)
-    return result
+    return list(winutil.parse_cmdline(commandline))
 
 
 def friendly_app_name(prog_id=None, exe=None):
     try:
-        from win32com.shell import shell, shellcon
-        a = shell.AssocCreate()
-        a.Init((shellcon.ASSOCF_INIT_BYEXENAME if exe else 0), exe or prog_id)
-        return a.GetString(shellcon.ASSOCF_REMAPRUNDLL, shellcon.ASSOCSTR_FRIENDLYAPPNAME)
+        return winutil.friendly_name(prog_id, exe)
     except Exception:
-        import traceback
         traceback.print_exc()
 
 
@@ -266,16 +247,15 @@ def find_programs(extensions):
     for base in (HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE):
         try:
             k = Key(open_at=r'Software\RegisteredApplications', root=base)
-        except WindowsError as err:
-            if err.errno == winerror.ERROR_FILE_NOT_FOUND:
+        except OSError as err:
+            if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
                 continue
             raise
         with k:
-            for name, key_path in k.itervalues(get_data=True):
+            for name, key_path in k.values(get_data=True):
                 try:
                     app_desc, prog_id_map = get_prog_id_map(base, key_path)
                 except Exception:
-                    import traceback
                     traceback.print_exc()
                     continue
                 for ext in extensions:
@@ -292,10 +272,10 @@ def find_programs(extensions):
     for ext in extensions:
         try:
             k = Key(open_at=r'Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.%s\OpenWithProgIDs' % ext, root=HKEY_CURRENT_USER)
-        except WindowsError as err:
-            if err.errno == winerror.ERROR_FILE_NOT_FOUND:
+        except OSError as err:
+            if err.winerror == winutil.ERROR_FILE_NOT_FOUND:
                 continue
-        for prog_id in k.itervalues():
+        for prog_id in itervalues(k):
             if prog_id and prog_id not in seen_prog_ids:
                 seen_prog_ids.add(prog_id)
                 cmdline, icon_resource, friendly_name = get_open_data(base, prog_id)
