@@ -1,6 +1,7 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -60,12 +61,12 @@ def create_bmp_from_dib(raw):
 
 
 def to_png(bmp):
-    from PyQt5.Qt import QImage, QByteArray, QBuffer
+    from qt.core import QImage, QByteArray, QBuffer, QIODevice
     i = QImage()
     if not i.loadFromData(bmp):
         raise ValueError('Invalid image data')
     ba = QByteArray()
     buf = QBuffer(ba)
-    buf.open(QBuffer.WriteOnly)
+    buf.open(QIODevice.OpenModeFlag.WriteOnly)
     i.save(buf, 'png')
-    return bytes(ba.data())
+    return ba.data()
