@@ -1,6 +1,6 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -14,9 +14,12 @@ from lxml import etree
 from lxml.builder import ElementMaker
 
 from calibre import force_unicode
+from calibre.utils.xml_parse import safe_xml_fromstring
+from calibre.constants import numeric_version
 from calibre.utils.iso8601 import parse_iso8601
 from calibre.utils.date import now as nowf, utcnow, local_tz, isoformat, EPOCH, UNDEFINED_DATE
 from calibre.utils.recycle_bin import delete_file
+from polyglot.builtins import iteritems, unicode_type
 
 NS = 'http://calibre-ebook.com/recipe_collection'
 E = ElementMaker(namespace=NS, nsmap={None:NS})
@@ -40,7 +43,7 @@ def serialize_recipe(urn, recipe_class):
 
     def attr(n, d):
         ans = getattr(recipe_class, n, d)
-        if isinstance(ans, str):
+        if isinstance(ans, bytes):
             ans = ans.decode('utf-8', 'replace')
         return ans
 
@@ -51,7 +54,7 @@ def serialize_recipe(urn, recipe_class):
     if ns is True:
         ns = 'yes'
     return E.recipe({
-        'id'                 : str(urn),
+        'id'                 : unicode_type(urn),
         'title'              : attr('title', _('Unknown')),
         'author'             : attr('__author__', default_author),
         'language'           : attr('language', 'und'),
@@ -64,7 +67,7 @@ def serialize_collection(mapping_of_recipe_classes):
     collection = E.recipe_collection()
     '''for u, x in mapping_of_recipe_classes.items():
         print 11111, u, repr(x.title)
-        if isinstance(x.title, str):
+        if isinstance(x.title, bytes):
             x.title.decode('ascii')
     '''
     for urn in sorted(mapping_of_recipe_classes.keys(),
@@ -78,7 +81,7 @@ def serialize_collection(mapping_of_recipe_classes):
             traceback.print_exc()
             continue
         collection.append(recipe)
-    collection.set('count', str(len(collection)))
+    collection.set('count', unicode_type(len(collection)))
     return etree.tostring(collection, encoding='utf-8', xml_declaration=True,
             pretty_print=True)
 
@@ -91,7 +94,7 @@ def serialize_builtin_recipes():
             try:
                 recipe_class = compile_recipe(stream.read())
             except:
-                print ('Failed to compile: %s'%f)
+                print('Failed to compile: %s'%f)
                 raise
         if recipe_class is not None:
             recipe_mapping['builtin:'+rid] = recipe_class
@@ -108,20 +111,21 @@ def get_custom_recipe_collection(*args):
             custom_recipes
     bdir = os.path.dirname(custom_recipes.file_path)
     rmap = {}
-    for id_, x in custom_recipes.iteritems():
+    for id_, x in iteritems(custom_recipes):
         title, fname = x
         recipe = os.path.join(bdir, fname)
         try:
-            recipe = open(recipe, 'rb').read().decode('utf-8')
+            with open(recipe, 'rb') as f:
+                recipe = f.read().decode('utf-8')
             recipe_class = compile_recipe(recipe)
             if recipe_class is not None:
                 rmap['custom:%s'%id_] = recipe_class
         except:
-            print 'Failed to load recipe from: %r'%fname
+            print('Failed to load recipe from: %r'%fname)
             import traceback
             traceback.print_exc()
             continue
-    return etree.fromstring(serialize_collection(rmap))
+    return safe_xml_fromstring(serialize_collection(rmap), recover=False)
 
 
 def update_custom_recipe(id_, title, script):
@@ -135,14 +139,14 @@ def update_custom_recipes(script_ids):
     bdir = os.path.dirname(custom_recipes.file_path)
     for id_, title, script in script_ids:
 
-        id_ = str(int(id_))
+        id_ = unicode_type(int(id_))
         existing = custom_recipes.get(id_, None)
 
         if existing is None:
             fname = custom_recipe_filename(id_, title)
         else:
             fname = existing[1]
-        if isinstance(script, unicode):
+        if isinstance(script, unicode_type):
             script = script.encode('utf-8')
 
         custom_recipes[id_] = (title, fname)
@@ -162,16 +166,16 @@ def add_custom_recipes(script_map):
     from calibre.web.feeds.recipes import custom_recipes, \
             custom_recipe_filename
     id_ = 1000
-    keys = tuple(map(int, custom_recipes.iterkeys()))
+    keys = tuple(map(int, custom_recipes))
     if keys:
         id_ = max(keys)+1
     bdir = os.path.dirname(custom_recipes.file_path)
     with custom_recipes:
-        for title, script in script_map.iteritems():
-            fid = str(id_)
+        for title, script in iteritems(script_map):
+            fid = unicode_type(id_)
 
             fname = custom_recipe_filename(fid, title)
-            if isinstance(script, unicode):
+            if isinstance(script, unicode_type):
                 script = script.encode('utf-8')
 
             custom_recipes[fid] = (title, fname)
@@ -186,7 +190,7 @@ def add_custom_recipes(script_map):
 
 def remove_custom_recipe(id_):
     from calibre.web.feeds.recipes import custom_recipes
-    id_ = str(int(id_))
+    id_ = unicode_type(int(id_))
     existing = custom_recipes.get(id_, None)
     if existing is not None:
         bdir = os.path.dirname(custom_recipes.file_path)
@@ -200,7 +204,7 @@ def remove_custom_recipe(id_):
 
 def get_custom_recipe(id_):
     from calibre.web.feeds.recipes import custom_recipes
-    id_ = str(int(id_))
+    id_ = unicode_type(int(id_))
     existing = custom_recipes.get(id_, None)
     if existing is not None:
         bdir = os.path.dirname(custom_recipes.file_path)
@@ -217,13 +221,21 @@ def download_builtin_recipe(urn):
     from calibre.utils.config_base import prefs
     from calibre.utils.https import get_https_resource_securely
     import bz2
-    return bz2.decompress(get_https_resource_securely(
+    recipe_source = bz2.decompress(get_https_resource_securely(
         'https://code.calibre-ebook.com/recipe-compressed/'+urn, headers={'CALIBRE-INSTALL-UUID':prefs['installation_uuid']}))
+    recipe_source = recipe_source.decode('utf-8')
+    from calibre.web.feeds.recipes import compile_recipe
+    recipe = compile_recipe(recipe_source)  # ensure the downloaded recipe is at least compile-able
+    if recipe is None:
+        raise ValueError('Failed to find recipe object in downloaded recipe: ' + urn)
+    if recipe.requires_version > numeric_version:
+        raise ValueError('Downloaded recipe for {} requires calibre >= {}'.format(urn, recipe.requires_version))
+    return recipe_source
 
 
 def get_builtin_recipe(urn):
     with zipfile.ZipFile(P('builtin_recipes.zip', allow_user_override=False), 'r') as zf:
-        return zf.read(urn+'.recipe')
+        return zf.read(urn+'.recipe').decode('utf-8')
 
 
 def get_builtin_recipe_by_title(title, log=None, download_recipe=False):
@@ -276,9 +288,9 @@ class SchedulerConfig(object):
         if os.access(self.conf_path, os.R_OK):
             with ExclusiveFile(self.conf_path) as f:
                 try:
-                    self.root = etree.fromstring(f.read())
+                    self.root = safe_xml_fromstring(f.read(), recover=False)
                 except:
-                    print 'Failed to read recipe scheduler config'
+                    print('Failed to read recipe scheduler config')
                     import traceback
                     traceback.print_exc()
         elif os.path.exists(old_conf_path):
@@ -391,7 +403,7 @@ class SchedulerConfig(object):
         elif typ == 'day/time':
             text = '%d:%d:%d'%schedule
         elif typ in ('days_of_week', 'days_of_month'):
-            dw = ','.join(map(str, map(int, schedule[0])))
+            dw = ','.join(map(unicode_type, map(int, schedule[0])))
             text = '%s:%d:%d'%(dw, schedule[1], schedule[2])
         else:
             raise ValueError('Unknown schedule type: %r'%typ)
@@ -540,8 +552,8 @@ class SchedulerConfig(object):
                         username, password = c[k]
                     except:
                         username = password = ''
-                    self.set_account_info(urn, unicode(username),
-                            unicode(password))
+                    self.set_account_info(urn, unicode_type(username),
+                            unicode_type(password))
                 except:
                     continue
         del c
