@@ -1,17 +1,12 @@
-#!/usr/local/bin/python2.7
+#!/usr/local/bin/python3.8
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import unittest
 
-try:
-    unicode
-except NameError:
-    unicode = str
 
 def jsonify(tokens):
     """Turn tokens into "JSON-compatible" data structures."""
@@ -24,6 +19,7 @@ def jsonify(tokens):
         else:
             yield token.type, token.value
 
+
 class BaseTest(unittest.TestCase):
 
     longMessage = True
@@ -34,10 +30,8 @@ class BaseTest(unittest.TestCase):
         """Test not complete error messages but only substrings."""
         self.ae(len(errors), len(expected_errors))
         for error, expected in zip(errors, expected_errors):
-            self.assertIn(expected, unicode(error))
+            self.assertIn(expected, type(u'')(error))
 
     def jsonify_declarations(self, rule):
         return [(decl.name, list(jsonify(decl.value)))
                 for decl in rule.declarations]
-
-
